/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"
#include "house.h"
#include "skills.h"
#include "map.h"
#include "dg_scripts.h"
#include "events.h"

/* external vars  */
extern struct room_data *world;
extern struct char_data *character_list;
extern struct descriptor_data *descriptor_list;
extern struct index_data *obj_index;
extern struct zone_data *zone_table;
extern int rev_dir[];
extern char *dirs[];
extern char *position_types[];
extern char *icon[];
extern struct map_icon_info map_info[];
extern struct str_app_type str_app[];

/* internal functs */
char *move_message(struct char_data *ch, struct char_data *people, int exitnumb, char *amsg);

/* external functs */

extern bool checkmovetrap(struct char_data *ch, int dir);
extern void smallmap_look(struct char_data *ch, int size, int dif_room);
extern void maplook(struct char_data *ch, sh_int size, int dif_room, int type);
int special(struct char_data *ch, int cmd, char *command, char *arg, int why_called);
int find_eq_pos(struct char_data *ch, struct obj_data *obj, char *argu);
void dismount_char(struct char_data *ch);
void mount_char(struct char_data *ch, struct char_data *mount);
int CoordsInRangeForInnerRoomTravel(struct char_data *ch, int dir);
int CoordsInRangeForInnerObjRoomTravel(struct char_data *ch, int dir);
void CoordsForRoomToRoom(struct char_data *ch, int room, int dir, int *x, int *y, int *z);
int CheckMoveDoors(struct char_data *ch, int dir);
int FindDoor(struct char_data *ch, int dir);
byte ProximityCheck(struct char_data *ch, struct obj_data *obj, struct char_data *target, int door);
int distance_from_shore(int room, int xcord, int ycord);
void stop_follower(struct char_data *ch);
void add_follower(struct char_data *ch, struct char_data *leader);
void char_to_object(struct char_data *ch, struct obj_data *obj);
void char_from_object(struct char_data *ch, struct obj_data *obj);
extern bool checkopen(struct char_data *ch, struct obj_data *obj);
extern void FallingChar(struct char_data *ch, const int kill_char);
EVENTFUNC(water_current_event);
void stop_memorizing(struct char_data *ch);

/* simple function to determine if char can walk on water */
int has_boat(struct char_data *ch)
{
  struct obj_data *obj;
  int i;

  if (IS_AFFECTED(ch, AFF_WATERWALK))
    return 1;

  /* non-wearable boats in inventory will do it */
  for (obj = ch->carrying; obj; obj = obj->next_content)
    if (GET_OBJ_TYPE(obj) == ITEM_BOAT && (find_eq_pos(ch, obj, NULL) < 0))
      return 1;

  /* and any boat you're wearing will do it too */
  for (i = 0; i < NUM_WEARS; i++)
    if (GET_EQ(ch, i) && GET_OBJ_TYPE(GET_EQ(ch, i)) == ITEM_BOAT)
      return 1;

  return 0;
}



/* do_simple_move assumes
 *    1. That there is no master and no followers.
 *    2. That the room exists.
 *
 *   Returns :
 *   1 : If succes.
 *   0 : If fail
 */
int do_simple_move(struct char_data *ch, int room, int dir, int need_specials_check)
{
  int was_in;

  /*
   * Check for special routines (North is 1 in command list, but 0 here) Note
   * -- only check if following; this avoids 'double spec-proc' bug
   */

  if (need_specials_check && special(ch, dir + 1, "", "", PROC_NORMAL))
    return 0;

  /* charmed? */
  if (IS_AFFECTED(ch, AFF_CHARM) && ch->master && ch->in_room == ch->master->in_room) {
    send_to_char("The thought of leaving your master makes you weep.\r\n", ch);
    act("$n bursts into tears.", FALSE, ch, 0, 0, TO_PROXIMITY);
    return 0;
  }
  /* just for messaging. Probably yank if we re-imp the following boat code */
  if (((SECT(ch->in_room) == SECT_WATER_NOSWIM) ||
       (SECT(room) == SECT_WATER_NOSWIM)) && GET_Z_CORD(ch) < 1)
    send_to_char("You step into the waters.\r\n", ch);

#if 0
  if (IS_SET(ROOM_FLAGS(ch->in_room), ROOM_ATRIUM)) {
    if (!House_can_enter(ch, world[room].number)) {
      send_to_char("That's private property -- no trespassing!\r\n", ch);
      return 0;
    }
  }
#endif

  /* see if an entry trigger disallows the move */
  if (!entry_mtrigger(ch))
    return 0;
  if (!enter_wtrigger(&world[EXIT(ch, dir)->to_room], ch, dir))
    return 0;

  sprintf(buf2, "$n moves %s.", dirs[dir]);
  act2(buf2, "You hear movement to your $d.",
       FALSE, ch, 0, 0,
       IS_AFFECTED(ch, AFF_SNEAK) ? TO_NOONE : TO_OTHERS,
       IS_AFFECTED4(ch, AFF4_SLNT_MOVE) ? TO_NOONE :
       TO_DISTANCE | char_noise_distance(ch));

  was_in = ch->in_room;

  char_from_room(ch);
  char_to_room(ch, room, rev_dir[dir]);

  if (!IS_AFFECTED(ch, AFF_SNEAK) && !GALLOPING(ch)) {
    if (!IS_NPC(ch) && GET_COND(ch, DRUNK) > 10)
      act("$n stumbles in drunkenly, looking very wasted.", TRUE, ch, 0, 0, TO_PROXIMITY);
    else
      act("$n approaches.", TRUE, ch, 0, 0, TO_PROXIMITY);
  }
  if (ch->desc != NULL && !GALLOPING(ch)) {
    smallmap_look(ch, view_distance(ch), -1);
    look_at_room(ch, 2);
  }
  if (IS_SET(ROOM_FLAGS(ch->in_room), ROOM_DEATH) && GET_LEVEL(ch) < LVL_IMMORT) {
    log_death_trap(ch);
    extract_char(ch);
    return 0;
  }
  entry_memory_mtrigger(ch);
  if (!greet_mtrigger(ch, dir)) {
    char_from_room(ch);
    char_to_room(ch, was_in, -1);
    smallmap_look(ch, view_distance(ch), -1);
    look_at_room(ch, 0);
  } else
    greet_memory_mtrigger(ch);

  return 1;
}

int convert_diagonals(struct char_data *ch, int dir)
{

  /* this is a diagonal kludge. Diagonals can raise such complexities.
     it will hopefully only be used in the wilderness, moving from one
     room to another. Rather than add complex and probably wrong math,
     we'll just move them a simple dir this one square */

  if (dir == SOUTHEAST && GET_X_CORD(ch) == GET_X_SIZE(ch->in_room) - 1)
    dir = SOUTH;
  else if (dir == SOUTHEAST && GET_Y_CORD(ch) == GET_Y_SIZE(ch->in_room) - 1)
    dir = EAST;
  else if (dir == SOUTHWEST && GET_X_CORD(ch) == GET_X_SIZE(ch->in_room) - 1)
    dir = SOUTH;
  else if (dir == SOUTHWEST && GET_Y_CORD(ch) == 0)
    dir = WEST;
  else if (dir == NORTHWEST && GET_X_CORD(ch) == 0)
    dir = NORTH;
  else if (dir == NORTHWEST && GET_Y_CORD(ch) == 0)
    dir = WEST;
  else if (dir == NORTHEAST && GET_X_CORD(ch) == 0)
    dir = NORTH;
  else if (dir == NORTHEAST && GET_Y_CORD(ch) == GET_Y_SIZE(ch->in_room) - 1)
    dir = EAST;

  return dir;
}

int CheckMoveDoors(struct char_data *ch, int dir)
{
  int room = 0;
  char Gbuf[256];

  /* return no if door didnt exist, was closed, or we were too far away */
  /* otherwise, move to door square and return yes */

  /* start of drunk addition */
  if (GET_COND(ch, DRUNK) > 0 && ((IS_NPC(ch) && number(0, 50) < 10)
   || (!IS_NPC(ch) && number(0, 50) < (10 + GET_COND(ch, DRUNK) > 0)))) {
    if (GET_COND(ch, DRUNK) > 10) {
      act("You drunkenly slam face-first into the 'exit' on your way $T.", FALSE, ch, NULL, dirs[dir], TO_CHAR);
      act("$n drunkenly slams face-first into the 'exit' on $s way $T.", FALSE, ch, NULL, dirs[dir], PROXIMITY);
      if (IS_PC(ch));
      damage(ch, NULL, 0, TYPE_UNDEFINED, -1, -1);
    } else {
      if (!IS_NPC(ch) && GET_COND(ch, DRUNK) > 10) {
	act("You stumble about aimlessly and fall down drunk.", TRUE, ch, NULL, dirs[dir], TO_CHAR);
	act("$n stumbles about aimlessly and falls down drunk.", TRUE, ch, NULL, dirs[dir], PROXIMITY);
	SET_POS(ch, POS_KNEELING + GET_STAT(ch));
      } else {
	act("You almost go $T, but suddenly realize that there's no exit there.", FALSE, ch, NULL, dirs[dir], TO_CHAR);
	act("$n looks like $e's about to go $T, but suddenly stops short and looks confused.", FALSE, ch, NULL, dirs[dir], PROXIMITY);
      }
    }
  }
  /* end of drunk addition */

  if (!EXIT(ch, dir) || EXIT(ch, dir)->to_room <= 1) {
    send_to_char("You can't go there.\r\n", ch);
    return 0;
  } else if (IS_SET(EXIT(ch, dir)->exit_info, EX_NOMOB) && !IS_PC(ch)) {
    send_to_char("You can't go there.\r\n", ch);
    return 0;
  } else if ((world[EXIT(ch, dir)->to_room].zone != world[ch->in_room].zone) &&
	     zone_table[world[EXIT(ch, dir)->to_room].zone].closed) {
    send_to_char("Sorry, that area is temporarily closed for remodeling.\r\n", ch);
    return 0;
  } else if (IS_SET(EXIT(ch, dir)->exit_info, EX_WALLED) &&
	     !IS_AFFECTED4(ch, AFF4_WRAITHFORM)) {
    send_to_char("You can't go there.\r\n", ch);
    return 0;
  } else if (IS_SET(EXIT(ch, dir)->exit_info, EX_CLOSED) &&
	     CAN_SEE_EXIT(ch, ch->in_room, dir) &&
	     !IS_AFFECTED4(ch, AFF4_WRAITHFORM)) {
    if (EXIT(ch, dir)->keyword) {
      sprintf(Gbuf, "The %s seems to be closed.\r\n", fname(EXIT(ch, dir)->keyword));
      send_to_char(Gbuf, ch);
      return 0;
    } else {
      send_to_char("It seems to be closed.\r\n", ch);
      return 0;
    }
    /* secret doors */
  } else if (IS_SET(EXIT(ch, dir)->exit_info, EX_CLOSED) &&
	     !CAN_SEE_EXIT(ch, ch->in_room, dir)
	     && !IS_AFFECTED4(ch, AFF4_WRAITHFORM)) {
    send_to_char("You can't go there.\r\n", ch);
    return 0;
  } else if (GET_DOOR_Z_CORD(ch->in_room, dir) != GET_Z_CORD(ch) && EXIT(ch, dir)->to_room > 1
	     && GET_Z_CORD(ch) >= GET_Z_SIZE(ch->in_room) + 1) {
    /* should be at roof height */
    if (do_simple_move(ch, EXIT(ch, dir)->to_room, dir, 1))
      return 1;
  } else if (GET_DOOR_Z_CORD(ch->in_room, dir) != GET_Z_CORD(ch) && !WILDERNESS(ch->in_room)
	     && !IS_AFFECTED4(ch, AFF4_WRAITHFORM) && EXIT(ch, dir)->exit_info >= EX_ISDOOR) {
    sprintf(Gbuf, "The %s is too far %s you to reach.\r\n", fname(EXIT(ch, dir)->keyword),
	    GET_DOOR_Z_CORD(ch->in_room, dir) > GET_Z_CORD(ch) ? "above" : "below");
    send_to_char(Gbuf, ch);
    return 0;
  } else {
    dir = convert_diagonals(ch, dir);
    room = FindDoor(ch, dir);
    if (room > 1)
      if (do_simple_move(ch, room, dir, 1))
	return 1;
  }
  send_to_char("You can't go there.\r\n", ch);
  return 0;
}

/* decides if coord char is heading for is occupyable */
int CantEnterCoord(struct char_data *ch, int dir)
{
  int newx, newy, size = 0;
  struct char_data *i, *i_next;
  struct obj_data *o, *o_next;
  newx = GET_X_CORD(ch);
  newy = GET_Y_CORD(ch);

  if (world[ch->in_room].map == NULL)
    return 0;
  switch (dir) {
  case NORTH:
    newx--;
    break;
  case NORTHEAST:
    newx--;
    newy++;
    break;
  case EAST:
    newy++;
    break;
  case SOUTHEAST:
    newx++;
    newy++;
    break;
  case SOUTH:
    newx++;
    break;
  case SOUTHWEST:
    newx++;
    newy--;
    break;
  case WEST:
    newy--;
    break;
  case NORTHWEST:
    newx--;
    newy--;
    break;
  }

  /* first off, check for out of room moves, and allow, since some other
     code will later check for that */
  if ((newx < 0) || (newx > GET_X_SIZE(ch->in_room)) ||
      (newy < 0) || (newy > GET_Y_SIZE(ch->in_room))) {
    return 0;
  }
  if ((IS_WALL(world[ch->in_room].map->map_grid[newx][newy]) ||
       world[ch->in_room].map->map_grid[newx][newy] == MAP_TREE ||
       world[ch->in_room].map->map_grid[newx][newy] == MAP_CLOSED_DOOR ||
       world[ch->in_room].map->map_grid[newx][newy] == MAP_RUBBLE ||
       world[ch->in_room].map->map_grid[newx][newy] == MAP_NOTBLANK ||
       UNPASSABLE_FURNITURE(ch, newx, newy) ||
  /* yank map_blank, since moving through a door always finds one */
       world[ch->in_room].map->map_grid[newx][newy] == MAP_ERROR) &&
      GET_Z_CORD(ch) < 4 && !IS_TRUSTED(ch)) {
    /* this will aid in error tracking.... 
       sprintf(buf, "&+CDEBUG:&n %s\r\n", map_info[world[ch->in_room].map->map_grid[newx][newy]].name);
       send_to_char(buf, ch);
     */
    return 1;
  }
  /* Now, we increase the check to out of bounds proofing */
  if ((newx <= 0) || (newx >= GET_X_SIZE(ch->in_room)) ||
      (newy <= 0) || (newy >= GET_Y_SIZE(ch->in_room))) {
    return 0;
  }
  /* diagonal wall walkthrough prevention */
  switch (dir) {
  case NORTHWEST:
    if (IS_WALL(world[ch->in_room].map->map_grid[newx - 1][newy]) &&
	IS_WALL(world[ch->in_room].map->map_grid[newx][newy - 1]))
      return 1;
    break;
  case NORTHEAST:
    if (IS_WALL(world[ch->in_room].map->map_grid[newx - 1][newy]) &&
	IS_WALL(world[ch->in_room].map->map_grid[newx][newy + 1]))
      return 1;
    break;
  case SOUTHWEST:
    if (IS_WALL(world[ch->in_room].map->map_grid[newx + 1][newy]) &&
	IS_WALL(world[ch->in_room].map->map_grid[newx][newy - 1]))
      return 1;
    break;
  case SOUTHEAST:
    if (IS_WALL(world[ch->in_room].map->map_grid[newx + 1][newy]) &&
	IS_WALL(world[ch->in_room].map->map_grid[newx][newy + 1]))
      return 1;
    break;
  }
  /* swimmers */
  if (GET_Z_CORD(ch) < 0 &&
      IS_WATER_CORD(ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch)))
    if (!IS_WATER_CORD(ch->in_room, newx, newy))
      return 1;

  /* waders */
  if (GET_Z_CORD(ch) == 0 &&
      !IS_WATER_CORD(ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch)))
    if (IS_WATER_CORD(ch->in_room, newx, newy))
      return 2;

  /* square size check. How many can fit in one spot? */
  if ((IS_SET(ROOM_FLAGS(ch->in_room), ROOM_INDOORS) ||
       world[ch->in_room].shape != ROOM_SHAPE_WILDERNESS)) {
    for (i = world[ch->in_room].people; i; i = i_next) {
      i_next = i->next_in_room;
      if (i == ch)
	continue;
      if (IS_TRUSTED(i))
	continue;
      if (GET_X_CORD(i) == newx && GET_Y_CORD(i) == newy &&
	  GET_Z_CORD(i) == GET_Z_CORD(ch)) {
	size += GET_SIZE(i);
      }
    }

    for (o = world[ch->in_room].contents; o; o = o_next) {
      o_next = o->next_content;
      if (GET_OBJ_X_CORD(o) == newx && GET_OBJ_Y_CORD(o) == newy &&
	  GET_OBJ_Z_CORD(o) == GET_Z_CORD(ch)) {
	size += (MIN(0, GET_OBJ_SIZE(o) - 2));	/* object sizes are smaller */
	if (UNPASSABLE_ITEM(o)) {
	  return 1;
	}
      }
    }
    // originally used size large, but too often its too crowded. Lets try giant
    if (GET_SIZE(ch) >= SIZE_GIANT && size == 0)
      return 0;			/* unoccupied, let them in */
    else if (size + GET_SIZE(ch) > SIZE_GIANT) {	/* current +
							 * ch size */
      if (!IS_TRUSTED(ch)) {
	send_to_char("It's too crowded! ", ch);		/* additional message */
	return 1;
      }
      send_to_char("It's too crowded, but you diet for a day to try and squeeze in.\n", ch);
    }
    /* I should check for following here, and stop them 1 square
       away, based on facing dir */
  }
  return 0;
}

/* Is the space we are heading for water? This isnt the same as IS_WATER_CORD! */
int HeadingForWaterCoord(struct char_data *ch, int dir)
{
  int newx, newy;
  newx = GET_X_CORD(ch);
  newy = GET_Y_CORD(ch);

  if (world[ch->in_room].map == NULL)
    return 0;

  switch (dir) {
  case NORTH:
    newx--;
    break;
  case NORTHEAST:
    newx--;
    newy++;
    break;
  case EAST:
    newy++;
    break;
  case SOUTHEAST:
    newx++;
    newy++;
    break;
  case SOUTH:
    newx++;
    break;
  case SOUTHWEST:
    newx++;
    newy--;
    break;
  case WEST:
    newy--;
    break;
  case NORTHWEST:
    newx--;
    newy--;
    break;
  }
  if (IS_WATER_CORD(ch->in_room, newx, newy) && newx < GET_X_SIZE(ch->in_room)
      && newx >= 0 && newy < GET_Y_SIZE(ch->in_room) && newy >= 0)
    return 1;

  return 0;
}

int CoordsInRangeForInnerRoomTravel(struct char_data *ch, int dir)
{
  int room, xroom, yroom, xchar, ychar, moved, cantenter;
  struct char_data *i, *i_next;
  int distance = 0;

  room = ch->in_room;
  xroom = world[room].x_size;
  yroom = world[room].y_size;
  xchar = GET_X_CORD(ch);
  ychar = GET_Y_CORD(ch);
  moved = 0;

  /* do a quick loop here, to see if a square is too crowded. Indoors
   * is approx 3' square, so allow only 2. Or 3 if size_tiny. Outdoors,
   * let's ignore any limits, as we're only really going to care during
   * combat, where we can always check again.
   */

  cantenter = CantEnterCoord(ch, dir);
  if (cantenter == 1)
    return -1;
  else if (cantenter == 2)
    send_to_char("You step into the waters.\r\n", ch);
  else if (checkmovetrap(ch, dir))
    return 1;

  switch (dir) {
  case NORTH:
    if (xchar >= 1 && (ychar >= 0 && ychar <= yroom - 1)) {
      GET_X_CORD(ch)--;
      SET_FACING(ch, NORTH);
      moved = 1;
    }
    break;
  case NORTHEAST:
    if (xchar >= 1 && ychar <= (yroom - 2)) {
      GET_X_CORD(ch)--;
      GET_Y_CORD(ch)++;
      SET_FACING(ch, NORTHEAST);
      moved = 1;
    }
    break;
  case EAST:
    if (ychar <= (yroom - 2) && (xchar >= 0 && xchar <= xroom - 1)) {
      GET_Y_CORD(ch)++;
      SET_FACING(ch, EAST);
      moved = 1;
    }
    break;
  case SOUTHEAST:
    if (ychar <= (yroom - 2) && xchar <= (xroom - 2)) {
      GET_X_CORD(ch)++;
      GET_Y_CORD(ch)++;
      SET_FACING(ch, SOUTHEAST);
      moved = 1;
    }
    break;
  case SOUTH:
    if (xchar <= (xroom - 2) && (ychar >= 0 && ychar <= yroom - 1)) {
      GET_X_CORD(ch)++;
      SET_FACING(ch, SOUTH);
      moved = 1;
    }
    break;
  case SOUTHWEST:
    if (xchar <= (xroom - 1) && ychar >= 1) {
      GET_X_CORD(ch)++;
      GET_Y_CORD(ch)--;
      SET_FACING(ch, SOUTHWEST);
      moved = 1;
    }
    break;
  case WEST:
    if (ychar >= 1 && (xchar >= 0 && xchar <= xroom - 1)) {
      GET_Y_CORD(ch)--;
      SET_FACING(ch, WEST);
      moved = 1;
    }
    break;
  case NORTHWEST:
    if (xchar >= 1 && ychar >= 1) {
      GET_X_CORD(ch)--;
      GET_Y_CORD(ch)--;
      SET_FACING(ch, NORTHWEST);
      moved = 1;
    }
    break;
  }
#if 0				/* pretty sure I dont need this anymore. 10/11/98 */
  /* fix any/all errors */
  if ((IS_SET(ROOM_FLAGS(ch->in_room), ROOM_INDOORS) ||
       world[ch->in_room].shape != ROOM_SHAPE_WILDERNESS) &&
      (GET_X_CORD(ch) == 0 || GET_X_CORD(ch) == xroom - 1
       || GET_Y_CORD(ch) == 0 || GET_Y_CORD(ch) == yroom - 1)) {
    GET_X_CORD(ch) = xchar;
    GET_Y_CORD(ch) = ychar;
    moved = 0;
  }
#endif
  if (GET_X_CORD(ch) < 0 || GET_X_CORD(ch) >= xroom) {
/* very spammy, look over later... 
   sprintf(buf, "%s was x_cord out of bounds in room %d", GET_NAME(ch), world[room].number);
   log(buf);
 */
    GET_X_CORD(ch) = (xroom / 2);
  }
  if (GET_Y_CORD(ch) < 0 || GET_Y_CORD(ch) >= yroom) {
/*
   sprintf(buf, "%s was y_cord out of bounds in room %d", GET_NAME(ch), world[room].number);
   log(buf);
 */
    GET_Y_CORD(ch) = (yroom / 2);
  }
  if (moved) {
    /* move mount */
    if (RIDING(ch)) {
      GET_X_CORD(RIDING(ch)) = GET_X_CORD(ch);
      GET_Y_CORD(RIDING(ch)) = GET_Y_CORD(ch);
      GET_Z_CORD(RIDING(ch)) = GET_Z_CORD(ch);
      SET_FACING(RIDING(ch), GET_FACING(ch));
    }
    /* look around */
    if (ch->desc != NULL && !GALLOPING(ch)) {
      smallmap_look(ch, view_distance(ch), -1);
      look_at_room(ch, 2);
    }
    /* loop folks to send messages out */
    send_to_char("----------\r\n", ch);
    for (i = world[ch->in_room].people; i; i = i_next) {
      i_next = i->next_in_room;
      if (RIDDEN_BY(ch) == i || RIDING(ch) == i)
	continue;
      if (!AWAKE(i) || GALLOPING(ch))
	continue;
      buf2[0] = '\0';
      /* can't see behind you */
      // no need to check the angle, as ac_can_see() does it
      if (CAN_SEE(i, ch) && SENDOK(ch)) {
	distance = DIST_NORM_SEE + GET_SIZE(ch) - SIZE_MEDIUM;
	if (i->watching && (i->watching->vict == ch))
	  distance = MAX(DIST_WATCH, distance);
	if (victWithinRangeofChar(ch, i, distance))
	  move_message(ch, i, dir, buf2);
      }
      /* now is when we discount ch == i, so its not shown twice */
      if (ch == i)
	continue;

      // if buf2 is an empty string, then move_message determined that
      // i can't see ch for some reason.  If thats the case, they
      // might still hear 'em.
      if (!buf2[0] && ac_can_hear(i, ch) && !IS_AFFECTED4(ch, AFF4_SLNT_MOVE)) {
	act("You hear movement to your $d.", FALSE, ch, 0, i, TO_VICT);
#if 0
	/* jumps CPU by 50% or so... */
	if (IS_NPC(i)) {
	  /* mobs will attempt to look that way. I'll call the actual command though,
	     so I can test for fighting, or other busy activities. */
	  sprintf(Gbuf, "turn %s", dirs[rev_dir[face_angle(angle_offset(ch, NULL, i, NULL))]]);
	  command_interpreter(i, Gbuf);
	}
#endif
      }
      if (RIDING(ch) && CAN_SEE(ch, i)) {
	act(buf2, TRUE, ch, VEHICLE(RIDING(ch)), i, TO_VICT);
      } else if (!IS_AFFECTED(ch, AFF_SNEAK) && CAN_SEE(ch, i)) {
	act(buf2, TRUE, ch, VEHICLE(ch), i, TO_VICT);
      } else if ((IS_TRUSTED(i) || IS_DETECTING(i, DETECTING_LIFE)) &&
		 CAN_SEE(ch, i)) {
	act(buf2, TRUE, ch, VEHICLE(ch), i, TO_VICT);
      }				/* check for crowds */
      if (ch != i && victWithinRangeofChar(ch, i, distance) && CAN_SEE(ch, i)) {
	if (distance == 0) {
	  act("You crowd in closer to $N.", TRUE, ch, 0, i, TO_CHAR);
	  act("$n crowds in closer to you.", TRUE, ch, 0, i, TO_VICT);
	}
      }
#if 0
      /* just doubles the view */
      distance = 1;
      if (ch != i && victWithinRangeofChar(ch, i, distance) && CAN_SEE(ch, i))
	act("$N is nearby.", TRUE, ch, 0, i, TO_CHAR);
#endif
    }				/* end of loop */

    return 1;
  }
  return 0;
}

int perform_move(struct char_data *ch, int dir, int need_specials_check)
{
  struct water_current_event_obj *wed;
  int was_in, moved_okay = 0, move_test = 0;
  struct follow_type *k, *next;
  struct obj_data *obj, *next_obj;
  extern int perform_drop(struct char_data *ch, struct obj_data *obj,
			  byte mode, char *sname, sh_int RDR);

  was_in = ch->in_room;

  if (ch == NULL || dir < 0 || dir >= NUM_OF_DIRS)
    return 0;

  if (IS_NPC(ch) && need_specials_check && special(ch, dir + 1, "", "", PROC_NORMAL))
    return 0;			/* we check mobs here, since they dont use do_simple_move */

/* fall is checked later. swim is something or other
   if ((GET_Z_CORD(ch) > 0) &&
   !IS_AFFECTED(ch, AFF_FLY) && !IS_TRUSTED(ch)) {
   send_to_char("But your feet arent touching the ground.\r\n", ch);
   return 0;
   } else if (!IS_TRUSTED(ch) && (GET_Z_CORD(ch) < 0 || (GET_Z_CORD(ch) == 0 && 
   IS_WATER_CORD(IN_ROOM(ch), GET_X_CORD(ch), GET_Y_CORD(ch))))) {
   send_to_char("You'll have to swim when in water.\r\n", ch);
   return 0;
   }

 */
  /*
     At this point, movement is possible. So drop everything in inventory,
     unless a mob or trusted.
   */

  if (IS_PC(ch) && !IS_TRUSTED(ch)) {
    for (obj = ch->carrying; obj; obj = next_obj) {
      next_obj = obj->next_content;
      perform_drop(ch, obj, SCMD_DROP, "drop", 0);
    }
  }
  if (!IS_NPC(ch)) {
    if (GET_COND(ch, DRUNK) > 10) {
      if (GET_COND(ch, DRUNK) > number(0, 50)) {
	act("You feel a little drunk.. not to mention kind of lost..",
	    TRUE, ch, NULL, NULL, TO_CHAR);
	act("$n looks a little drunk.. not to mention kind of lost..",
	    TRUE, ch, NULL, NULL, PROXIMITY);
	dir = number(0, NUM_OF_DIRS - 1);
      } else {
	act("You feel a little.. drunk..", FALSE, ch, NULL, NULL, TO_CHAR);
	act("$n looks a little.. drunk..", FALSE, ch, NULL, NULL, PROXIMITY);
      }
    }
  }
  /* move_test decides if we simply can't go there, or if we should
     continue checking other types of movement */
  move_test = CoordsInRangeForInnerRoomTravel(ch, dir);
  if (move_test == 1)
    moved_okay = 1;
  else if (move_test == -1) {
    send_to_char("You can't go there.\r\n", ch);
    return 0;
  } else if (CheckMoveDoors(ch, dir))
    moved_okay = 1;

  if (moved_okay && RIDDEN_BY(ch))
    perform_move(RIDDEN_BY(ch), dir, 1);

  if (moved_okay && ch->watchers) {
    for (k = ch->watchers; k; k = next) {
      next = k->next;
      // make sure they can still see...
      watch_movement(k->follower);
    }
  }
  if (moved_okay && ch->followers) {
    for (k = ch->followers; k; k = next) {
      next = k->next;
      if ((k->follower->in_room == was_in) &&
	  (GET_POS(k->follower) >= POS_STANDING)) {
	act("You follow $N.\r\n", FALSE, k->follower, 0, ch, TO_CHAR);
	perform_move(k->follower, dir, 1);
      }
    }
    return 1;
  }
  if (IS_NPC(ch) && moved_okay)
    GET_MOB_LASTMOVE(ch) = dir;
  if (moved_okay) {
    /* check water first */
    if (world[ch->in_room].current_speed > 0 &&
	GET_Z_CORD(ch) <= 0 && IS_WATER_CORD(ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch))) {
      CREATE(wed, struct water_current_event_obj, 1);
      wed->ch = ch;
      wed->obj = NULL;
      if (GET_ACTION(ch))
	event_cancel(GET_ACTION(ch));
      GET_ACTION(ch) = NULL;
      GET_ACTION(ch) = event_create(water_current_event, wed, BOUNDED(0, 110 - world[ch->in_room].current_speed + GET_STR(ch), 100));
    }
    return 1;
  } else
    return 0;
}

ACMD(do_gallop)
{
  int dir, x, count = 0;

  skip_spaces(&argument);
  if (!*argument) {
    send_to_char("Gallop which way?\r\n", ch);
    return;
  }
  if ((dir = search_block(argument, dirs, FALSE)) == -1) {
    send_to_char("That's not a direction.\r\n", ch);
    return;
  }
  /* how fast/far? */
  switch (SECT(ch->in_room)) {
  case SECT_INSIDE:
    send_to_char("No galloping in the house, my mother always used to say.\r\n", ch);
    break;
  case SECT_UNDERWATER:
    send_to_char("In water?!? You're lucky just to stay on!\r\n", ch);
    break;
  case SECT_FLYING:
    send_to_char("Your legs simply don't seem to increase your speed any.\r\n", ch);
    break;
  case SECT_ROAD:
    count = 12;
    break;
  case SECT_FIELD:
    count = 8;
    break;
  case SECT_WATER_SWIM:
  case SECT_WATER_NOSWIM:	/* prolly on a wilderness map */
  case SECT_CITY:
    count = 6;
    break;
  case SECT_FOREST:
  case SECT_HILLS:
  case SECT_CAVE:
    count = 3;
    break;
  case SECT_MOUNTAIN:
  case SECT_SWAMP:
    count = 2;
    break;
  default:
    count = 1;
    break;
  }
  if (WILDERNESS(ch->in_room))
    count *= 2;

  if (RIDING(ch))
    SET_BIT(AFF4_FLAGS(RIDING(ch)), AFF4_GALLOP);
  else if (IS_NPC(ch) && MOB_FLAGGED(ch, MOB_MOUNTABLE))
    SET_BIT(AFF4_FLAGS(ch), AFF4_GALLOP);
  else {
    send_to_char("That would look a bit foolish, don't you think?\r\n", ch);
    return;
  }
  /* save a final burst for the end, minus the flag */
  for (x = 0; x < count - 1; x++)
    perform_move(RIDING(ch), dir, 0);

  if (RIDING(ch))
    REMOVE_BIT(AFF4_FLAGS(RIDING(ch)), AFF4_GALLOP);
  else if (IS_NPC(ch) && MOB_FLAGGED(ch, MOB_MOUNTABLE))
    REMOVE_BIT(AFF4_FLAGS(ch), AFF4_GALLOP);

  /* a final one, without the bitset, so we see the map */
  perform_move(RIDING(ch), dir, 1);	/* make proc checks only once */
}

ACMD(do_move)
{
  struct obj_data *furniture = NULL;
  int circle_dir;

  extern char fends_off(struct char_data *defender, struct char_data *advancer);
  /*
   * This is basically a mapping of cmd numbers to perform_move indices.
   * It cannot be done in perform_move because perform_move is called
   * by other functions which do not require the remapping.
   */
  if (FURNITURE(ch)) {
    furniture = FURNITURE(ch);
    sprintf(buf, "$n gets out of $p.");
    act(buf, TRUE, ch, furniture, 0, TO_ROOM);
    char_from_object(ch, FURNITURE(ch));
    SET_POS(ch, POS_STANDING + STAT_NORMAL);
  }
  if (PRF_FLAGGED(ch, PRF_TURN) && (GET_FACING(ch) != cmd - 1) &&
      (cmd <= NUM_OF_DIRS)) {
    sprintf(buf, "You turn %s.\r\n", dirs[cmd - 1]);
    send_to_char(buf, ch);
    SET_FACING(ch, cmd - 1);
  } else if (subcmd > NUM_OF_DIRS) { // its how we tell fight/nonfight movement
    if (!IS_FIGHTING(ch)) {
      send_to_char("You may only do that in combat.\r\n", ch);
      return;
    }
    if (subcmd == SCMD_ADVANCE) {
      if (!fends_off(FIGHTING(ch), ch)) {
	act("$n advances towards $N.", FALSE, ch, NULL, FIGHTING(ch), TO_OTHERS);
	act("You advance towards $N.", TRUE, ch, NULL, FIGHTING(ch), TO_CHAR);
	act("$n advances towards you.", FALSE, ch, NULL, FIGHTING(ch), TO_VICT);
	perform_move(ch, GET_FACING(ch), 0);
      } else {
	act("$n tries to advance but is fended off by $N.", FALSE, ch, NULL, FIGHTING(ch), TO_OTHERS);
	act("You try to advance but are fended off by $N.", TRUE, ch, NULL, FIGHTING(ch), TO_CHAR);
	act("$n tries to advance but you fend $m off.", TRUE, ch, NULL, FIGHTING(ch), TO_VICT);
      }
    } else if (subcmd == SCMD_CIRCLE) {
      act("$n attempts to circle behind $N.", FALSE, ch, NULL, FIGHTING(ch), TO_OTHERS);
      act("You attempt to circle behind $N.", TRUE, ch, NULL, FIGHTING(ch), TO_CHAR);
      /* chance of noticing them */
      if (GET_INT(FIGHTING(ch)) > (GET_DEX(ch) + number(-3, 3)))
	act("$n attempts to circle behind you.", FALSE, ch, NULL, FIGHTING(ch), TO_VICT);
      else {
	perform_move(ch, rev_dir[GET_FACING(FIGHTING(ch))], 0);
      }
      circle_dir = angleBetweenCharAndVict(ch, FIGHTING(ch));
      sprintf(buf2, "Turning %s to face %s\r\n", dirs[circle_dir], GET_NAME(FIGHTING(ch)));
      send_to_char(buf2, ch);
      SET_FACING(ch, circle_dir);
    } else if (subcmd == SCMD_ROLL) {
      if ((GET_SIZE(ch) > GET_SIZE(FIGHTING(ch))) || (GET_DEX(FIGHTING(ch)) > (GET_DEX(ch) + number(-3, 3)))) {
	act("You drop to the ground, trying to roll past $N, but do not succeed.", TRUE, ch, NULL, FIGHTING(ch), TO_CHAR);
	act("$n drops to the ground trying to roll past you.", FALSE, ch, NULL, FIGHTING(ch), TO_VICT);
	SET_POS(ch, GET_STAT(ch) + POS_KNEELING);
      } else {
	act("You drop to the ground, and roll right past $N, standing up behind $M.", TRUE, ch, NULL, FIGHTING(ch), TO_CHAR);
	act("$n drops to the ground and rolls behind you.", FALSE, ch, NULL, FIGHTING(ch), TO_VICT);
	act("$n drops to the ground and rolls behind $N.", FALSE, ch, NULL, FIGHTING(ch), TO_OTHERS);
	perform_move(ch, GET_FACING(ch), 0);
      }
      circle_dir = angleBetweenCharAndVict(ch, FIGHTING(ch));
      sprintf(buf2, "Turning %s to face %s\r\n", dirs[circle_dir], GET_NAME(FIGHTING(ch)));
      send_to_char(buf2, ch);
      SET_FACING(ch, circle_dir);
    } else if (subcmd == SCMD_RETREAT) {
      send_to_char("You retreat.\r\n", ch);
      perform_move(ch, rev_dir[GET_FACING(ch)], 0);
    }
  } else
    perform_move(ch, cmd - 1, 0);
}

ACMD(do_turn)
{
  int dir;

  skip_spaces(&argument);
  if (!*argument) {
    sprintf(buf, "You are currently facing the %s.\r\n", dirs[GET_FACING(ch)]);
    send_to_char(buf, ch);
    return;
  }
  if (!str_cmp(argument, "on")) {
    SET_BIT(PRF_FLAGS(ch), PRF_TURN);
    send_to_char("If you are not facing the direction you typed, you will \
turn, but not move.\r\n", ch);
    return;
  }
  if (!str_cmp(argument, "off")) {
    REMOVE_BIT(PRF_FLAGS(ch), PRF_TURN);
    send_to_char("You will move in the direction you typed, whether or not \
you are facing that direction.\r\n", ch);
    return;
  }
  if ((dir = search_block(argument, dirs, FALSE)) == -1) {
    send_to_char("That's not a direction.\r\n", ch);
    return;
  }
  SET_FACING(ch, dir);
  sprintf(buf, "You turn to the %s.\r\n", dirs[dir]);
  send_to_char(buf, ch);
}

int find_door(struct char_data *ch, char *type, char *dir, char *cmdname)
{
  int door;

  if (*dir) {			/* a direction was specified */
    if ((door = search_block(dir, dirs, FALSE)) == -1) {	/* Partial Match */
      send_to_char("That's not a direction.\r\n", ch);
      return -1;
    }
    if (EXIT(ch, door))
      if (EXIT(ch, door)->keyword)
	if (isname(type, EXIT(ch, door)->keyword))
	  return door;
	else {
	  sprintf(buf2, "I see no %s there.\r\n", type);
	  send_to_char(buf2, ch);
	  return -1;
      } else
	return door;
    else {
      send_to_char("I really don't see how you can close anything there.\r\n", ch);
      return -1;
    }
  } else {			/* try to locate the keyword */
    if (!*type) {
      sprintf(buf2, "What is it you want to %s?\r\n", cmdname);
      send_to_char(buf2, ch);
      return -1;
    }
    for (door = 0; door < NUM_OF_DIRS; door++)
      if (EXIT(ch, door))
	if (EXIT(ch, door)->keyword)
	  if (ProximityCheck(ch, NULL, NULL, door))
	    if (isname(type, EXIT(ch, door)->keyword))
	      return door;
    if (str_cmp(cmdname, "hit")) {	/* do_hit will handle this */
      sprintf(buf2, "You don't see %s %s here close enough to %s.\r\n", AN(type), type, cmdname);
      send_to_char(buf2, ch);
    }
    return -1;
  }
}


int has_key(struct char_data *ch, int key)
{
  struct obj_data *o;

  for (o = ch->carrying; o; o = o->next_content)
    if (GET_OBJ_VNUM(o) == key)
      return 1;

  if (GET_EQ(ch, WEAR_HOLD))
    if (GET_OBJ_VNUM(GET_EQ(ch, WEAR_HOLD)) == key)
      return 1;

  return 0;
}



#define NEED_OPEN	1
#define NEED_CLOSED	2
#define NEED_UNLOCKED	4
#define NEED_LOCKED	8

char *cmd_door[] =
{
  "hit",
  "open",
  "close",
  "unlock",
  "lock",
  "pick"
};

const int flags_door[] =
{
  0,
  NEED_CLOSED | NEED_UNLOCKED,
  NEED_OPEN,
  NEED_CLOSED | NEED_LOCKED,
  NEED_CLOSED | NEED_UNLOCKED,
  NEED_CLOSED | NEED_LOCKED
};


void do_doorcmd(struct char_data *ch, struct obj_data *obj, int door, int scmd)
{
  int other_room = 0;
  struct room_direction_data *back = 0;

  sprintf(buf, "$n %ss ", cmd_door[scmd]);
  if (!obj && ((other_room = EXIT(ch, door)->to_room) != NOWHERE))
    if ((back = world[other_room].dir_option[rev_dir[door]]))
      if (back->to_room != ch->in_room)
	back = 0;

  /* trigger check */
  if (obj && scmd == SCMD_OPEN && !open_otrigger(obj, ch))
    return;
  else if (obj && scmd == SCMD_OPEN && checkopen(ch, obj))
    return;

  switch (scmd) {
  case SCMD_OPEN:
  case SCMD_CLOSE:
    OPEN_DOOR(ch->in_room, obj, door);
    if (back)
      OPEN_DOOR(other_room, obj, rev_dir[door]);
    send_to_char(OK, ch);
    if (!obj)
      maplook(ch, view_distance(ch), -1, TO_CHAR);
    break;
  case SCMD_UNLOCK:
  case SCMD_LOCK:
    LOCK_DOOR(ch->in_room, obj, door);
    if (back)
      LOCK_DOOR(other_room, obj, rev_dir[door]);
    send_to_char("*Click*\r\n", ch);
    break;
  case SCMD_PICK:
    LOCK_DOOR(ch->in_room, obj, door);
    if (back)
      LOCK_DOOR(other_room, obj, rev_dir[door]);
    send_to_char("The lock quickly yields to your skills.\r\n", ch);
    strcpy(buf, "$n skillfully picks the lock on ");
    break;
  }

  /* Notify the room */
  sprintf(buf + strlen(buf), "%s%s.", ((obj) ? "" : "the "), (obj) ? "$p" :
	  (EXIT(ch, door)->keyword ? "$F" : "door"));
  if (!(obj) || (obj->in_room != NOWHERE))
    act(buf, FALSE, ch, obj, obj ? 0 : EXIT(ch, door)->keyword, TO_PROXIMITY);

  /* Notify the other room */
  if ((scmd == SCMD_OPEN || scmd == SCMD_CLOSE) && back) {
    sprintf(buf, "The %s is %s%s from the other side.\r\n",
	 (back->keyword ? fname(back->keyword) : "door"), cmd_door[scmd],
	    (scmd == SCMD_CLOSE) ? "d" : "ed");
    if (world[EXIT(ch, door)->to_room].people) {
      act(buf, FALSE, world[EXIT(ch, door)->to_room].people, 0, 0, TO_PROXIMITY);
      act(buf, FALSE, world[EXIT(ch, door)->to_room].people, 0, 0, TO_CHAR);
    }
  }
}


int ok_pick(struct char_data *ch, int keynum, int pickproof, int scmd)
{
  int percent;

  percent = number(1, 101);

  if (scmd == SCMD_PICK) {
    if (keynum < 0)
      send_to_char("Odd - you can't seem to find a keyhole.\r\n", ch);
    else if (pickproof)
      send_to_char("It resists your attempts to pick it.\r\n", ch);
    else if (percent > GET_SKILL(ch, SKILL_PICK_LOCK))
      send_to_char("You failed to pick the lock.\r\n", ch);
    else
      return (1);
    return (0);
  }
  return (1);
}



ACMD(do_gen_door)
{
  int door = -1, keynum;
  char type[MAX_INPUT_LENGTH], dir[MAX_INPUT_LENGTH];
  struct obj_data *obj = NULL;
  struct char_data *victim = NULL;

  skip_spaces(&argument);
  if (!*argument) {
    sprintf(buf, "%s what?\r\n", cmd_door[subcmd]);
    send_to_char(CAP(buf), ch);
    return;
  }
  two_arguments(argument, type, dir);
  if (!generic_find(type, FIND_OBJ_INV | FIND_OBJ_ROOM, ch, &victim, &obj))
    door = find_door(ch, type, dir, cmd_door[subcmd]);

  if ((obj) || (door >= 0)) {
    /* secret door, dont give it away */
    /*if((door >= 0) && !CAN_SEE_EXIT(ch, ch->in_room, dir)){
       send_to_char("Open what?\r\n");
       return;
       } */
    /* See if they are close enough to do this */
    if (!ProximityCheck(ch, obj, NULL, door)) {
      /* standing in the doorway? */
      if (GET_X_CORD(ch) == 0 || GET_X_CORD(ch) == GET_X_SIZE(ch->in_room) - 1
	  || GET_Y_CORD(ch) == 0 || GET_Y_CORD(ch) == GET_Y_SIZE(ch->in_room) - 1) {
	send_to_char("This is not a very good spot for doing that.", ch);
	return;
      } else {
	act("You're not close enough to $F that!", FALSE, ch, 0, cmd_door[subcmd], TO_CHAR);
	return;
      }
    }
    keynum = DOOR_KEY(ch->in_room, obj, door);
    if (!(DOOR_IS_OPENABLE(ch->in_room, obj, door)))
      act("You can't $F that!", FALSE, ch, 0, cmd_door[subcmd], TO_CHAR);
    else if (!DOOR_IS_OPEN(ch->in_room, obj, door) &&
	     IS_SET(flags_door[subcmd], NEED_OPEN))
      send_to_char("But it's already closed!\r\n", ch);
    else if (!DOOR_IS_CLOSED(ch->in_room, obj, door) &&
	     IS_SET(flags_door[subcmd], NEED_CLOSED))
      send_to_char("But it's currently open!\r\n", ch);
    else if (!(DOOR_IS_LOCKED(ch->in_room, obj, door)) &&
	     IS_SET(flags_door[subcmd], NEED_LOCKED))
      send_to_char("Oh.. it wasn't locked, after all..\r\n", ch);
    else if (!(DOOR_IS_UNLOCKED(ch->in_room, obj, door)) &&
	     IS_SET(flags_door[subcmd], NEED_UNLOCKED))
      send_to_char("It seems to be locked.\r\n", ch);
    else if (!has_key(ch, keynum) && !IS_TRUSTED(ch) &&
	     ((subcmd == SCMD_LOCK) || (subcmd == SCMD_UNLOCK)))
      send_to_char("You don't seem to have the proper key.\r\n", ch);
    else if (ok_pick(ch, keynum, DOOR_IS_PICKPROOF(ch->in_room, obj, door), subcmd))
      do_doorcmd(ch, obj, door, subcmd);
  }
  return;
}



ACMD(do_enter)
{
  int door, room;

  one_argument(argument, buf);

  if (*buf) {			/* an argument was supplied, search for door
				 * keyword */
    for (door = 0; door < NUM_OF_DIRS; door++)
      if (EXIT(ch, door))
	if (EXIT(ch, door)->keyword)
	  if (!str_cmp(EXIT(ch, door)->keyword, buf)) {
	    room = FindDoor(ch, door);
	    if (room > 1)
	      perform_move(ch, door, 1);
	    return;
	  }
    sprintf(buf2, "There is no %s here.\r\n", buf);
    send_to_char(buf2, ch);
  } else if (IS_SET(ROOM_FLAGS(ch->in_room), ROOM_INDOORS))
    send_to_char("You are already indoors.\r\n", ch);
  else {
    /* try to locate an entrance */
    for (door = 0; door < NUM_OF_DIRS; door++)
      if (EXIT(ch, door))
	if (EXIT(ch, door)->to_room != NOWHERE)
	  if (!IS_SET(EXIT(ch, door)->exit_info, EX_CLOSED) &&
	      IS_SET(ROOM_FLAGS(EXIT(ch, door)->to_room), ROOM_INDOORS)) {
	    room = FindDoor(ch, door);
	    if (room > 1)
	      perform_move(ch, door, 1);
	    return;
	  }
    send_to_char("You can't seem to find anything to enter.\r\n", ch);
  }
}


ACMD(do_leave)
{
  int door;

  if (!IS_SET(ROOM_FLAGS(ch->in_room), ROOM_INDOORS))
    send_to_char("You are outside.. where do you want to go?\r\n", ch);
  else {
    for (door = 0; door < NUM_OF_DIRS; door++)
      if (EXIT(ch, door))
	if (EXIT(ch, door)->to_room != NOWHERE)
	  if (!IS_SET(EXIT(ch, door)->exit_info, EX_CLOSED) &&
	    !IS_SET(ROOM_FLAGS(EXIT(ch, door)->to_room), ROOM_INDOORS)) {
	    perform_move(ch, door, 1);
	    return;
	  }
    send_to_char("I see no obvious exits to the outside.\r\n", ch);
  }
}

void furniture_ok(struct char_data *ch, char *arg, char *text, int pos)
{
  struct char_data *tch, *next_tch;
  struct obj_data *furniture;
  int num = 0;

  if (FURNITURE(ch))
    return;			/* they are already in something? furniture_position_change()
				   should have handled that one... */

  /* Can the char see the object requested? */
  if (!(furniture = get_obj_in_list_cord(ch, arg, world[ch->in_room].contents))) {
    sprintf(buf, "You don't see %s %s here.\r\n", AN(arg), arg);
    send_to_char(buf, ch);
    return;
  }
  if (GET_X_CORD(ch) != GET_OBJ_X_CORD(furniture) ||
      GET_Y_CORD(ch) != GET_OBJ_Y_CORD(furniture) ||
      GET_Z_CORD(ch) != GET_OBJ_Z_CORD(furniture)) {
    sprintf(buf, "You aren't close enough to %s %s.\r\n", AN(arg), arg);
    send_to_char(buf, ch);
    return;
  }
  /* Make sure they can actually sit on the object */
  if ((GET_OBJ_TYPE(furniture) != ITEM_FURNITURE) ||
      (pos < GET_OBJ_VAL(furniture, 0))) {
    sprintf(buf, "%s isn't for %s on!\r\n", furniture->short_description,
	    position_types[(int) pos]);
    CAP(buf);
    send_to_char(buf, ch);
    return;
  }
  /* Check if there's room on this piece of furniture */
  for (tch = furniture->people; tch; tch = next_tch) {
    next_tch = tch->next_in_furniture;
    num++;
  }

  /* Mob only check */
  /* This can be modified for size in your MUD */
  if (GET_OBJ_VAL(furniture, 1) == 0 ||
      ((!(IS_MOB(ch))) && GET_OBJ_VAL(furniture, 3) == -1)) {
    sprintf(buf, "You can't %s on that!\r\n", text);
    send_to_char(buf, ch);
    return;
  }
  /* Tell them why they can't fit on */
  if (GET_OBJ_VAL(furniture, 1) == num) {
    if (num == 1) {
      tch = furniture->people;
      sprintf(buf, "%s is already %s on %s.\r\n", GET_NAME(tch),
	      position_types[(int) GET_POS(tch)],
	      furniture->short_description);
      sprintf(buf2, "%s just tried to %s on you!\r\n", GET_NAME(ch), text);
    } else
      sprintf(buf, "There are already %d people using %s.\r\n", num,
	      furniture->short_description);
    if (tch)
      send_to_char(buf2, tch);
    send_to_char(buf, ch);
    return;
  }
  SET_POS(ch, pos + GET_STAT(ch));
  char_to_object(ch, furniture);

  sprintf(buf, "You %s down on $p.", text);
  act(buf, FALSE, ch, furniture, 0, TO_CHAR | TO_SLEEP);

  sprintf(buf, "$n %ss down on $p.", text);
  act(buf, TRUE, ch, furniture, 0, TO_ROOM);
}

/* sitting on bed, they now wish to lay down   */
/* either allow and message it, or remove them */
void furniture_position_change(struct char_data *ch, char *arg, char *text, int pos)
{
  struct obj_data *furniture;

  const char *pos_mess, *how;

  if (pos > GET_POS(ch))
    pos_mess = "up";
  else
    pos_mess = "down";

  if (GET_POS(ch) == POS_STANDING)
    how = "from";
  else
    how = "on";

  furniture = FURNITURE(ch);
  if (!furniture) {
    sprintf(buf, "This is odd, %s %s is no longer here!.\r\n", AN(arg), arg);
    send_to_char(buf, ch);
    log(buf);
    FURNITURE(ch) = NULL;
    return;
  }
  /* Make sure they can actually sit on the object */
  if (pos < GET_OBJ_VAL(furniture, 0)) {
    sprintf(buf, "You can't %s %s %s $p.", text, pos_mess, how);
    act(buf, FALSE, ch, furniture, 0, TO_CHAR | TO_SLEEP);
    sprintf(buf, "$n gets out of $p and %s %s.", text, pos_mess);
    act(buf, TRUE, ch, furniture, 0, TO_ROOM);
    char_from_object(ch, FURNITURE(ch));
    return;
  }
  sprintf(buf, "You %s %s %s $p.", text, pos_mess, how);
  act(buf, FALSE, ch, furniture, 0, TO_CHAR | TO_SLEEP);

  sprintf(buf, "$n %ss %s %s $p.", text, pos_mess, how);
  act(buf, TRUE, ch, furniture, 0, TO_ROOM);
}

ACMD(do_recline)
{
  one_argument(argument, arg);

  if (FURNITURE(ch))
    furniture_position_change(ch, arg, "lie", POS_PRONE);

  if (!*arg) {
    switch (GET_STAT(ch)) {
    case STAT_DEAD:
    case STAT_DYING:
    case STAT_INCAP:
      send_to_char("Posture would seem to be far down on your list of problems right now!\r\n", ch);
      return;
      break;
    case STAT_SLEEPING:
      send_to_char("You dream of laying down.\r\n", ch);
      return;
      break;
    case STAT_RESTING:
    case STAT_NORMAL:
      if (IS_AFFECTED4(ch, AFF4_KNOCKED_OUT)) {
	send_to_char("You should probably worry more about becoming conscious again.\r\n", ch);
	return;
      }
      switch (GET_POS(ch)) {
      case POS_PRONE:
	send_to_char("You are already laying down.\r\n", ch);
	return;
	break;
      case POS_KNEELING:
	if (IS_AFFECTED2(ch, AFF2_MINOR_PARALYSIS) || IS_AFFECTED2(ch, AFF2_MAJOR_PARALYSIS)) {
	  send_to_char("You can't even twitch, much less lay down!\r\n", ch);
	  return;
	}
	act("You recline.", FALSE, ch, 0, 0, TO_CHAR);
	act("$n eases off $s knees and lays down.", TRUE, ch, 0, 0, TO_ROOM);
	break;
      case POS_SITTING:
	if (IS_AFFECTED2(ch, AFF2_MINOR_PARALYSIS) || IS_AFFECTED2(ch, AFF2_MAJOR_PARALYSIS)) {
	  send_to_char("You can't even twitch, much less change position!\r\n", ch);
	  return;
	}
	act("You stop sitting around and lay down.", FALSE, ch, 0, 0, TO_CHAR);
	act("$n stops sitting around and lays down.", TRUE, ch, 0, 0, TO_ROOM);
	break;
      case POS_STANDING:
	if (IS_AFFECTED2(ch, AFF2_MINOR_PARALYSIS) || IS_AFFECTED2(ch, AFF2_MAJOR_PARALYSIS)) {
	  send_to_char("You can't even twitch, much less change position!\r\n", ch);
	  return;
	}
	act("You drop to your belly.", FALSE, ch, 0, 0, TO_CHAR);
	act("$n drops flat to the ground.", TRUE, ch, 0, 0, TO_ROOM);
	break;
      }
      break;
    }
    SET_POS(ch, GET_STAT(ch) + POS_PRONE);
  } else
    furniture_ok(ch, arg, "lies", POS_PRONE);
}

ACMD(do_kneel)
{
  one_argument(argument, arg);

  if (FURNITURE(ch))
    furniture_position_change(ch, arg, "kneels", POS_KNEELING);

  if (!*arg) {
    switch (GET_STAT(ch)) {
    case STAT_DEAD:
    case STAT_DYING:
    case STAT_INCAP:
      send_to_char("Posture would seem to be far down on your list of problems right now!\r\n", ch);
      return;
      break;
    case STAT_SLEEPING:
      send_to_char("You seem to be having a dream of infancy.\r\n", ch);
      return;
      break;
    case STAT_RESTING:
    case STAT_NORMAL:
      if (IS_AFFECTED4(ch, AFF4_KNOCKED_OUT)) {
	send_to_char("You should probably worry more about becoming conscious again.\r\n", ch);
	return;
      }
      switch (GET_POS(ch)) {
      case POS_PRONE:
	if (IS_AFFECTED2(ch, AFF2_MINOR_PARALYSIS) || IS_AFFECTED2(ch, AFF2_MAJOR_PARALYSIS)) {
	  send_to_char("You can't even twitch, much less get to your knees!\r\n", ch);
	  return;
	}
	act("You swing up onto your knees.", FALSE, ch, 0, 0, TO_CHAR);
	act("$n levers up onto $s knees.", TRUE, ch, 0, 0, TO_ROOM);
	break;
      case POS_KNEELING:
	send_to_char("You are kneeling already.\r\n", ch);
	return;
	break;
      case POS_SITTING:
	if (IS_AFFECTED2(ch, AFF2_MINOR_PARALYSIS) || IS_AFFECTED2(ch, AFF2_MAJOR_PARALYSIS)) {
	  send_to_char("You can't even twitch, much less change position!\r\n", ch);
	  return;
	}
	act("You move from your butt to your knees.", FALSE, ch, 0, 0, TO_CHAR);
	act("$n stops sitting around and gets to $s knees.", TRUE, ch, 0, 0, TO_ROOM);
	break;
      case POS_STANDING:
	if (IS_AFFECTED2(ch, AFF2_MINOR_PARALYSIS) || IS_AFFECTED2(ch, AFF2_MAJOR_PARALYSIS)) {
	  send_to_char("You can't even twitch, much less kneel!\r\n", ch);
	  return;
	}
	act("You kneel.", FALSE, ch, 0, 0, TO_CHAR);
	act("$n settles to $s knees.", TRUE, ch, 0, 0, TO_ROOM);
	break;
      }
      break;
    }
    SET_POS(ch, GET_STAT(ch) + POS_KNEELING);
  } else
    furniture_ok(ch, arg, "kneels", POS_KNEELING);
}

ACMD(do_stand)
{
  one_argument(argument, arg);

  if (FURNITURE(ch)) {
    char_from_object(ch, FURNITURE(ch));
    FURNITURE(ch) = NULL;
  }
  if (!*arg) {
    if (GET_POS(ch) == POS_STANDING) {
      act("You are already standing.", FALSE, ch, 0, 0, TO_CHAR);
      return;
    }
    switch (GET_STAT(ch)) {
    case STAT_DEAD:
    case STAT_DYING:
    case STAT_INCAP:
      send_to_char("Posture would seem to be far down on your list of problems right now!\r\n", ch);
      return;
    case STAT_SLEEPING:
      send_to_char("You dream of sleepwalking.\r\n", ch);
      return;
    case STAT_RESTING:
    case STAT_NORMAL:
      if (IS_AFFECTED4(ch, AFF4_KNOCKED_OUT)) {
	send_to_char("You should probably worry more about becoming conscious again.\r\n", ch);
	return;
      }
      if (IS_AFFECTED2(ch, AFF2_MINOR_PARALYSIS) ||
	  IS_AFFECTED2(ch, AFF2_MAJOR_PARALYSIS)) {
	send_to_char("You can't even twitch, much less stand up!\r\n", ch);
	return;
      }
      if (IS_AFFECTED2(ch, AFF2_STUNNED) || IS_AFFECTED4(ch, AFF4_BOUND)) {
	/* maaaybeee */
	if (number(1, GET_DEX(ch)) < number(5, 25)) {
	  send_to_char("You stagger about, then fall sprawling!\r\n", ch);
	  SET_POS(ch, GET_STAT(ch) + POS_PRONE);
	  return;
	} else {
	  send_to_char("You manage to unsteadily get to your feet.\r\n", ch);
	  act("$n staggers about, but manages to get to $s feet.", TRUE, ch, 0, 0, TO_ROOM);
	}
      } else {
	switch (GET_POS(ch)) {
	case POS_PRONE:
	  act("You clamber to your feet.", FALSE, ch, 0, 0, TO_CHAR);
	  act("$n clambers to $s feet.", TRUE, ch, 0, 0, TO_ROOM);
	  break;
	case POS_KNEELING:
	  act("You rise to your feet.", FALSE, ch, 0, 0, TO_CHAR);
	  act("$n rises to $s feet.", TRUE, ch, 0, 0, TO_ROOM);
	  break;
	case POS_SITTING:
	  act("You clamber to your feet.", FALSE, ch, 0, 0, TO_CHAR);
	  act("$n clambers to $s feet.", TRUE, ch, 0, 0, TO_ROOM);
	  break;
	case POS_STANDING:
	  act("You are already standing.", FALSE, ch, 0, 0, TO_CHAR);
	  return;
	}
	break;
      }
      break;
    }

    if (IS_AFFECTED(ch, AFF_MEMORIZING)) {
      stop_memorizing(ch);
    }
    SET_POS(ch, POS_STANDING + STAT_NORMAL);

  } else
    furniture_ok(ch, arg, "stand", POS_STANDING);
}


ACMD(do_sit)
{
  one_argument(argument, arg);

  if (FURNITURE(ch))
    furniture_position_change(ch, arg, "sit", POS_SITTING);

  if (!*arg) {
    switch (GET_STAT(ch)) {
    case STAT_DEAD:
    case STAT_DYING:
    case STAT_INCAP:
      send_to_char("Posture would seem to be far down on your list of problems right now!\r\n", ch);
      return;
      break;
    case STAT_SLEEPING:
      send_to_char("You dream of sitting up.\r\n", ch);
      return;
      break;
    case STAT_RESTING:
    case STAT_NORMAL:
      if (IS_AFFECTED4(ch, AFF4_KNOCKED_OUT)) {
	send_to_char("You should probably worry more about becoming conscious again.\r\n", ch);
	return;
      }
      switch (GET_POS(ch)) {
      case POS_PRONE:
	if (IS_AFFECTED2(ch, AFF2_MINOR_PARALYSIS) ||
	    IS_AFFECTED2(ch, AFF2_MAJOR_PARALYSIS)) {
	  send_to_char("You can't even twitch, much less sit up!\r\n", ch);
	  return;
	}
	act("You sit up.", FALSE, ch, 0, 0, TO_CHAR);
	act("$n sits up.", TRUE, ch, 0, 0, TO_ROOM);
	break;
      case POS_KNEELING:
	if (IS_AFFECTED2(ch, AFF2_MINOR_PARALYSIS) ||
	    IS_AFFECTED2(ch, AFF2_MAJOR_PARALYSIS)) {
	  send_to_char("You can't even twitch, much less sit!\r\n", ch);
	  return;
	}
	act("You get off your knees and sit.", FALSE, ch, 0, 0, TO_CHAR);
	act("$n drops back off $s knees and sits.", TRUE, ch, 0, 0, TO_ROOM);
	break;
      case POS_SITTING:
	send_to_char("You are sitting already.\r\n", ch);
	return;
	break;
      case POS_STANDING:
	if (IS_AFFECTED2(ch, AFF2_MINOR_PARALYSIS) || IS_AFFECTED2(ch, AFF2_MAJOR_PARALYSIS)) {
	  send_to_char("You can't even twitch, much less sit down!\r\n", ch);
	  return;
	}
	act("You sit down.", FALSE, ch, 0, 0, TO_CHAR);
	act("$n sits down.", TRUE, ch, 0, 0, TO_ROOM);
	break;
      }
      break;
    }
    SET_POS(ch, GET_STAT(ch) + POS_SITTING);
  } else
    furniture_ok(ch, arg, "sit", POS_SITTING);
}


ACMD(do_rest)
{
  switch (GET_STAT(ch)) {
  case STAT_RESTING:
    send_to_char("You are already resting.\r\n", ch);
    return;
    break;
  case STAT_SLEEPING:
    send_to_char("You dream of relaxing.\r\n", ch);
    return;
    break;
  case STAT_DEAD:
  case STAT_DYING:
  case STAT_INCAP:
    send_to_char("Just wait a bit, you'll soon be VERY relaxed.\r\n", ch);
    return;
    break;
  case STAT_NORMAL:
    if (FIGHTING(ch)) {
      send_to_char("Resting now will most likely lead to your final rest!\r\n", ch);
      return;
    }
    if (IS_AFFECTED4(ch, AFF4_BOUND)) {
      send_to_char("Your bonds prevent you from really getting comfortable.\r\n", ch);
      return;
    }
    if (IS_AFFECTED4(ch, AFF4_KNOCKED_OUT)) {
      send_to_char("You should probably worry more about becoming conscious again.\r\n", ch);
      return;
    }
    if (IS_AFFECTED2(ch, AFF2_MINOR_PARALYSIS) ||
	IS_AFFECTED2(ch, AFF2_MAJOR_PARALYSIS)) {
      send_to_char("You can't even twitch, much less relax!\r\n", ch);
      return;
    }
    switch (GET_POS(ch)) {
    case POS_PRONE:
      send_to_char("You close your eyes and relax.\r\n", ch);
      act("You see some of the tension leave $n's body.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case POS_KNEELING:
      send_to_char("You slump and relax your posture.\r\n", ch);
      act("$n relaxes a bit.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case POS_SITTING:
      send_to_char("You wiggle to find the most comfortable position.\r\n", ch);
      act("$n relaxes a bit.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case POS_STANDING:
      send_to_char("You sit down and relax.\r\n", ch);
      act("$n sits down in a comfortable spot.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    }
    break;
  }
  SET_POS(ch, MIN(POS_SITTING, GET_POS(ch)) + STAT_RESTING);
}

ACMD(do_alert)
{
  switch (GET_STAT(ch)) {
  case STAT_DEAD:
  case STAT_DYING:
  case STAT_INCAP:
    send_to_char("Just wait a bit, you'll soon be VERY relaxed.\r\n", ch);
    return;
    break;
  case STAT_SLEEPING:
    send_to_char("You dream of being alert.\r\n", ch);
    return;
    break;
  case STAT_NORMAL:
    send_to_char("You are already about as tense as you can get.\r\n", ch);
    return;
    break;
  case STAT_RESTING:
    if (IS_AFFECTED4(ch, AFF4_KNOCKED_OUT)) {
      send_to_char("Your brain is on strike, please try again later.\r\n", ch);
      return;
    }
    if (IS_AFFECTED2(ch, AFF2_STUNNED)) {
      send_to_char("Your head is spinning, you can barely tell which way is up!\r\n", ch);
      return;
    }
    if (FIGHTING(ch))
      send_to_char("Excellent Idea!  You might actually survive!\r\n", ch);

    switch (GET_POS(ch)) {
    case POS_PRONE:
      send_to_char("You stop relaxing and try to become more aware of your surroundings.\r\n", ch);
      break;
    case POS_KNEELING:
      send_to_char("You straighten up a bit.\r\n", ch);
      act("$n straightens up a bit.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case POS_SITTING:
      send_to_char("You sit up straight and start to pay attention.\r\n", ch);
      act("$n sits at attention.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case POS_STANDING:
      send_to_char("You tense up and become more alert.\r\n", ch);
      break;
    }
    break;
  }
  SET_POS(ch, GET_POS(ch) + STAT_NORMAL);
}

ACMD(do_sleep)
{
  switch (GET_STAT(ch)) {
  case STAT_SLEEPING:
    send_to_char("You are already fast asleep.\r\n", ch);
    return;
    break;
  case STAT_DEAD:
  case STAT_DYING:
  case STAT_INCAP:
    send_to_char("Just wait a bit, you're about to start a LONG sleep.\r\n", ch);
    return;
    break;
  case STAT_RESTING:
  case STAT_NORMAL:
    if (IS_AFFECTED4(ch, AFF4_BOUND)) {
      send_to_char("Your bonds prevent you from really getting comfortable.\r\n", ch);
      return;
    }
    if (IS_AFFECTED4(ch, AFF4_KNOCKED_OUT)) {
      send_to_char("You should probably worry more about becoming conscious again.\r\n", ch);
      return;
    }
    if (IS_AFFECTED2(ch, AFF2_MINOR_PARALYSIS) ||
	IS_AFFECTED2(ch, AFF2_MAJOR_PARALYSIS)) {
      send_to_char("You can't even twitch, much less relax!\r\n", ch);
      return;
    }
    switch (GET_POS(ch)) {
    case POS_PRONE:
      send_to_char("You drift off to sleep.\r\n", ch);
      break;
    case POS_KNEELING:
      send_to_char("You slump and relax your posture.\r\n", ch);
      act("$n seems to have fallen asleep.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case POS_SITTING:
      send_to_char("You nod off.\r\n", ch);
      act("$n seems to have fallen asleep.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case POS_STANDING:
      send_to_char("You fall asleep.\r\n", ch);
      act("$n has fallen asleep on his feet, this should be fun.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    }
    break;
  }

  SET_POS(ch, GET_POS(ch) + STAT_SLEEPING);
}


ACMD(do_wake)
{
  struct char_data *vict;
  int self = 0;

  one_argument(argument, arg);
  if (*arg) {
    if (GET_STAT(ch) == STAT_SLEEPING)
      send_to_char("Maybe you should wake yourself up first.\r\n", ch);
    else if ((vict = get_char_room_vis(ch, arg)) == NULL)
      send_to_char(NOPERSON, ch);
    else if (vict == ch)
      self = 1;
    else if (GET_STAT(vict) > STAT_SLEEPING)
      act("$E is already awake.", FALSE, ch, 0, vict, TO_CHAR);
    else if (IS_AFFECTED(vict, AFF_SLEEP))
      act("You can't wake $N up!", FALSE, ch, 0, vict, TO_CHAR);
    else if (GET_STAT(vict) < STAT_SLEEPING)
      act("$E's in pretty bad shape!", FALSE, ch, 0, vict, TO_CHAR);
    else {
      act("You wake $N up.", FALSE, ch, 0, vict, TO_CHAR);
      act("You are awakened by $n.", FALSE, ch, 0, vict, TO_VICT | TO_SLEEP);
      SET_POS(ch, GET_POS(ch) + STAT_NORMAL);
    }
    if (!self)
      return;
  }
  if (IS_AFFECTED(ch, AFF_SLEEP))
    send_to_char("You can't wake up!\r\n", ch);
  else if (GET_STAT(ch) > STAT_SLEEPING)
    send_to_char("You are already awake...\r\n", ch);
  else {
    send_to_char("You awaken.\r\n", ch);
    act("$n awakens.", TRUE, ch, 0, 0, TO_PROXIMITY);
    SET_POS(ch, GET_POS(ch) + STAT_NORMAL);
  }
}


ACMD(do_follow)
{
  struct char_data *leader;

  one_argument(argument, buf);

  if (*buf) {
    if (!(leader = get_char_room_vis(ch, buf))) {
      send_to_char(NOPERSON, ch);
      return;
    }
  } else {
    send_to_char("Whom do you wish to follow?\r\n", ch);
    return;
  }

  if (ch->master == leader) {
    act("You are already following $N.", FALSE, ch, 0, leader, TO_CHAR);
    return;
  }
  if (IS_AFFECTED(ch, AFF_CHARM) && (ch->master)) {
    act("But you only feel like following $N!", FALSE, ch, 0, ch->master, TO_CHAR);
  } else {			/* Not Charmed follow person */
    if (leader == ch) {
      if (!ch->master) {
	send_to_char("You are already following yourself.\r\n", ch);
	return;
      }
      stop_follower(ch);
    } else {
      if (circle_follow(ch, leader)) {
	act("Sorry, but following in loops is not allowed.", FALSE, ch, 0, 0, TO_CHAR);
	return;
      }
      if (ch->master)
	stop_follower(ch);
      add_follower(ch, leader);
    }
  }
}


ACMD(do_mount)
{
  int distance = 1;
  char argu[MAX_INPUT_LENGTH];
  struct char_data *vict;

  one_argument(argument, argu);

  if (!argu || !*argu) {
    send_to_char("Mount who?\r\n", ch);
    return;
  } else if (!(vict = get_char_room_vis(ch, argu))) {
    send_to_char("There is no-one by that name here.\r\n", ch);
    return;
  } else if (!victWithinRangeofChar(ch, vict, distance)) {
    send_to_char("That would be one hell of a jump. Move closer.\r\n", ch);
    return;
  } else if (!IS_NPC(vict) && GET_LEVEL(ch) < LVL_IMMORT) {
    send_to_char("Ehh... no.\r\n", ch);
    return;
  } else if (RIDING(ch) || RIDDEN_BY(ch)) {
    send_to_char("You are already mounted.\r\n", ch);
    return;
  } else if (RIDING(vict) || RIDDEN_BY(vict)) {
    send_to_char("It is already mounted.\r\n", ch);
    return;
  } else if (!IS_TRUSTED(ch) && IS_NPC(vict) && !MOB_FLAGGED(vict, MOB_MOUNTABLE) &&
	     GET_RACE(vict) != RACE_HORSE) {
    send_to_char("You can't mount that!\r\n", ch);
    return;
    /*} else if (!GET_SKILL(ch, SKILL_MOUNT) && !IS_TRUSTED(ch)) {
       send_to_char("First you need to learn *how* to mount.\r\n", ch);
       return; */
  } else if (!GET_SKILL(ch, SKILL_RIDING_LAND) && !IS_TRUSTED(ch)) {
    send_to_char("First you must learn *how* to ride.\r\n", ch);
    return;
  } else if (GET_SKILL(ch, SKILL_MOUNT) <= number(1, 101) && !IS_TRUSTED(ch)) {
    act("You try to mount $N, but slip and fall off.", FALSE, ch, 0, vict, TO_CHAR);
    act("$n tries to mount you, but slips and falls off.", FALSE, ch, 0, vict, TO_VICT);
    act("$n tries to mount $N, but slips and falls off.", TRUE, ch, 0, vict, TO_NOTVICT);
    damage(ch, ch, dice(1, 2), -1, -1, -1);
    return;
  } else if (ch == vict) {
    send_to_char("Um, interesting, but heh, no.", ch);
    return;
  }
  act("You mount $N.", FALSE, ch, 0, vict, TO_CHAR);
  act("$n mounts you.", FALSE, ch, 0, vict, TO_VICT);
  act("$n mounts $N.", TRUE, ch, 0, vict, TO_NOTVICT);
  mount_char(ch, vict);

  if (IS_TRUSTED(ch))
    return;

  if (IS_NPC(vict) && !AFF_FLAGGED(vict, AFF_TAMED) && GET_SKILL(ch, SKILL_MOUNT) <= number(1, 101)) {
    act("$N suddenly bucks upwards, throwing you violently to the ground!", FALSE, ch, 0, vict, TO_CHAR);
    act("$n is thrown to the ground as $N violently bucks!", TRUE, ch, 0, vict, TO_NOTVICT);
    act("You buck violently and throw $n to the ground.", FALSE, ch, 0, vict, TO_VICT);
    dismount_char(ch);
    damage(ch, ch, dice(1, 3), -1, -1, -1);
  }
}


ACMD(do_dismount)
{
  if (!RIDING(ch)) {
    send_to_char("You aren't even riding anything.\r\n", ch);
    return;
  } else if (SECT(ch->in_room) == SECT_WATER_NOSWIM && !has_boat(ch)) {
    send_to_char("Yah, right, and then drown...\r\n", ch);
    return;
  }
  act("You dismount $N.", FALSE, ch, 0, RIDING(ch), TO_CHAR);
  act("$n dismounts from you.", FALSE, ch, 0, RIDING(ch), TO_VICT);
  act("$n dismounts $N.", TRUE, ch, 0, RIDING(ch), TO_NOTVICT);
  dismount_char(ch);
}


ACMD(do_buck)
{
  if (!RIDDEN_BY(ch)) {
    send_to_char("You're not even being ridden!\r\n", ch);
    return;
  } else if (AFF_FLAGGED(ch, AFF_TAMED)) {
    send_to_char("But you're tamed!\r\n", ch);
    return;
  }
  act("You quickly buck, throwing $N to the ground.", FALSE, ch, 0, RIDDEN_BY(ch), TO_CHAR);
  act("$n quickly bucks, throwing you to the ground.", FALSE, ch, 0, RIDDEN_BY(ch), TO_VICT);
  act("$n quickly bucks, throwing $N to the ground.", FALSE, ch, 0, RIDDEN_BY(ch), TO_NOTVICT);
  SET_POS(RIDDEN_BY(ch), POS_PRONE + GET_STAT(ch));
  dismount_char(ch);

  if (number(0, 4)) {
    send_to_char("You hit the ground hard!\r\n", ch);
    damage(ch, ch, dice(2, 4), -1, -1, -1);
  }
  // you might want to call set_fighting() or some non-sense here if you
  // want the mount to attack the unseated rider or vice-versa.
}


ACMD(do_tame)
{
  char argu[MAX_INPUT_LENGTH];
  struct char_data *vict;

  one_argument(argument, argu);

  if (!argu || !*argu) {
    send_to_char("Tame who?\r\n", ch);
    return;
  } else if (!(vict = get_char_room_vis(ch, argu))) {
    send_to_char("They're not here.\r\n", ch);
    return;
  } else if (!IS_TRUSTED(ch) && IS_NPC(vict) && !MOB_FLAGGED(vict, MOB_MOUNTABLE)) {
    send_to_char("You can't do that to them.\r\n", ch);
    return;
  } else if (!IS_TRUSTED(ch) && !GET_SKILL(ch, SKILL_TAME)) {
    send_to_char("You don't even know how to tame something.\r\n", ch);
    return;
  } else if (!IS_NPC(vict) && !IS_TRUSTED(ch)) {
    send_to_char("You can't do that.\r\n", ch);
    return;
  } else if (GET_SKILL(ch, SKILL_TAME) <= number(1, 101) && !IS_TRUSTED(ch)) {
    send_to_char("You fail to tame it.\r\n", ch);
    return;
  }
  SET_BIT(AFF_FLAGS(ch), AFF_TAMED);

  act("You tame $N.", FALSE, ch, 0, vict, TO_CHAR);
  act("$n tames you.", FALSE, ch, 0, vict, TO_VICT);
  act("$n tames $N.", FALSE, ch, 0, vict, TO_NOTVICT);
}

  /* need to add some door flags here, like EX_LARGE */
int FindDoor(struct char_data *ch, int dir)
{

  if (world[ch->in_room].dir_option[dir] &&
      world[ch->in_room].dir_option[dir]->to_room > 1 &&
    (!IS_SET(world[ch->in_room].dir_option[dir]->exit_info, EX_ISDOOR) &&
     !IS_SET(world[ch->in_room].dir_option[dir]->exit_info, EX_STAIRS) &&
     !IS_SET(world[ch->in_room].dir_option[dir]->exit_info, EX_LADDER)))
    return world[ch->in_room].dir_option[dir]->to_room;
  else if (world[ch->in_room].dir_option[dir] &&
	   world[ch->in_room].dir_option[dir]->to_room > 1 &&
     (IS_SET(world[ch->in_room].dir_option[dir]->exit_info, EX_STAIRS) ||
      IS_SET(world[ch->in_room].dir_option[dir]->exit_info, EX_LADDER)) &&
	   GET_Y_CORD(ch) == world[ch->in_room].dir_option[dir]->y_cord &&
	   GET_X_CORD(ch) == world[ch->in_room].dir_option[dir]->x_cord &&
	   GET_Z_CORD(ch) == world[ch->in_room].dir_option[dir]->z_cord)
    return world[ch->in_room].dir_option[dir]->to_room;
  else if (world[ch->in_room].dir_option[dir] &&
	   world[ch->in_room].dir_option[dir]->to_room > 1 &&
     (GET_Y_CORD(ch) + 1 == world[ch->in_room].dir_option[dir]->y_cord ||
      GET_Y_CORD(ch) == world[ch->in_room].dir_option[dir]->y_cord ||
      GET_Y_CORD(ch) - 1 == world[ch->in_room].dir_option[dir]->y_cord) &&
     (GET_X_CORD(ch) + 1 == world[ch->in_room].dir_option[dir]->x_cord ||
      GET_X_CORD(ch) == world[ch->in_room].dir_option[dir]->x_cord ||
      GET_X_CORD(ch) - 1 == world[ch->in_room].dir_option[dir]->x_cord) &&
	   GET_Z_CORD(ch) == world[ch->in_room].dir_option[dir]->z_cord)
    return world[ch->in_room].dir_option[dir]->to_room;
  else
    return 0;
}

/* Checks if ch is close enough to manipulate either 
 * an obj, a mob/player, or a door. Use only one, NULL the
 * unneeded ones. Returns 1 if okay, 0 if not.
 */
byte
ProximityCheck(struct char_data * ch, struct obj_data * obj, struct char_data * target, int door)
{
  int room = 0;

  if (!ch)
    return 0;

  if (target) {
    if (GET_Z_CORD(ch) == GET_Z_CORD(target) &&
	(GET_X_CORD(ch) == GET_X_CORD(target) ||
	 GET_X_CORD(ch) + 1 == GET_X_CORD(target) ||
	 GET_X_CORD(ch) - 1 == GET_X_CORD(target)) &&
	(GET_Y_CORD(ch) == GET_Y_CORD(target) ||
	 GET_Y_CORD(ch) + 1 == GET_Y_CORD(target) ||
	 GET_Y_CORD(ch) - 1 == GET_Y_CORD(target)))
      return 1;
  } else if (door > -1) {
    /* If we get a room returned, assume we're valid location, 
     * and just return a hearty aye-aye cap'n
     */
    room = FindDoor(ch, door);
    if (room > 1)
      return 1;
  } else if (obj) {
    if (obj->carried_by || obj->worn_by)
      return 1;
    if (GET_Z_CORD(ch) == GET_OBJ_Z_CORD(obj) &&
	(GET_X_CORD(ch) == GET_OBJ_X_CORD(obj) ||
	 GET_X_CORD(ch) + 1 == GET_OBJ_X_CORD(obj) ||
	 GET_X_CORD(ch) - 1 == GET_OBJ_X_CORD(obj)) &&
	(GET_Y_CORD(ch) == GET_OBJ_Y_CORD(obj) ||
	 GET_Y_CORD(ch) + 1 == GET_OBJ_Y_CORD(obj) ||
	 GET_Y_CORD(ch) - 1 == GET_OBJ_Y_CORD(obj)))
      return 1;
  }
  return 0;
}

ACMD(do_fly)
{
  struct char_data *rider = NULL;
  struct follow_type *k, *next_dude;
  int saw_map = 0, sect, oldz, newz, to_room;

  /* mounted chars can use this to order their mounts as well */
  if (RIDING(ch)) {
    ch = RIDING(ch);		/* simply change who were dealing with */
    rider = RIDDEN_BY(ch);
  }
  one_argument(argument, buf);

  if (!*buf) {
    send_to_char("Fly what, where why or who?!?\r\n", ch);
    return;
  }
  if (!affected_by_spell(ch, SPELL_FLY) && !IS_AFFECTED(ch, AFF_FLY) &&
      !IS_TRUSTED(ch) && !(IS_AFFECTED(ch, AFF_LEVITATE) && !str_cmp(buf, "land"))) {
    send_to_char("You don't have the means to fly!\r\n", ch);
    if (rider)
      send_to_char("Your mount hasn't the means to fly, and you cannot do so whilst mounted.\r\n", rider);
    return;
  }
  if ((world[ch->in_room].number < 100000 || world[ch->in_room].number > 150099)
      && !OUTSIDE(ch) && !str_cmp(buf, "up")) {
    send_to_char("You must be outdoors to fly.\r\n", ch);
    if (rider)
      send_to_char("You must be outdoors to fly.\r\n", ch);
    return;
  }
  if (GET_Z_CORD(ch) < 0) {
    send_to_char("Umm, try surfacing first.\r\n", ch);
    if (rider)
      send_to_char("Umm, try surfacing first.\r\n", ch);
    return;
  }
  sect = world[ch->in_room].sector_type;

  oldz = GET_Z_CORD(ch);

  if (!str_cmp(buf, "up")) {
    /* only worry about outside maxes, as ceilings check other apps */
    if (GET_Z_CORD(ch) > 50 && !IS_TRUSTED(ch) && !IS_NPC(ch) && OUTSIDE(ch)) {
      send_to_char("The wind currents are much too strong at that altitude.\r\n", ch);
      return;
    } else if (GET_Z_CORD(ch) == GET_Z_SIZE(ch->in_room) && !EXIT(ch, UP) && !OUTSIDE(ch)) {
      send_to_char("You're already at the ceiling!\r\n", ch);
      return;
    } else if (GET_Z_CORD(ch) == GET_Z_SIZE(ch->in_room) && EXIT(ch, UP)) {
      if (EXIT(ch, UP)->exit_info && (IS_SET(EXIT(ch, UP)->exit_info, EX_STAIRS)
			|| IS_SET(EXIT(ch, UP)->exit_info, EX_LADDER))) {
	perform_move(ch, UP, 1);
	GET_Z_CORD(ch) = 0;
      } else if (EXIT(ch, UP)->to_room > 1) {
	to_room = EXIT(ch, UP)->to_room;
	char_from_room(ch);
	char_to_room(ch, to_room, UP);
	send_to_char("You fly up out of the room.\r\n", ch);
	smallmap_look(ch, view_distance(ch), -1);
      }
      return;
    } else {
      GET_Z_CORD(ch)++;
      act("You fly up higher.", FALSE, ch, 0, 0, TO_CHAR);
      if (rider) {
	act("You fly up higher.", FALSE, ch, 0, rider, TO_VICT);
      }
      act("$n flys up higher.", TRUE, ch, 0, 0, TO_PROXIMITY3);
#if 0
      /* this is used to fly up and out of a city */
      if (!IS_MAP_ROOM(ch->in_room) && GET_Z_CORD(ch) > 2 && OUTSIDE(ch))
	if (real_room0(maproom_of_zone(world[zone->real_bottom].zone))) {
	  char_from_room(ch);
	  char_to_room(ch,
	  real_room0(maproom_of_zone(world[zone->real_bottom].zone)), -4);

	  saw_map = 1;
	  if (rider) {
	    char_from_room(rider);
	    char_to_room(rider, ch->in_room, -4);
	    saw_map = 1;
	  }
	}
#endif
    }
  } else if (!str_cmp(buf, "down")) {
    /* if there's already a down exit here, let's move em down it */
    if (EXIT(ch, DOWN)) {
      perform_move(ch, 6, 1);
      GET_Z_CORD(ch) = 0;
      return;
    }
    if (GET_Z_CORD(ch) < 1 || (GET_Z_CORD(ch) == GET_Z_SIZE(ch->in_room) + 1)) {	/* rooftop */
      send_to_char("Umm, you've landed already, bucko.\r\n", ch);
      return;
    } else {
      act("$n flys down lower.", TRUE, ch, 0, 0, TO_PROXIMITY3);
      GET_Z_CORD(ch) -= 1;
      act("You fly down lower.", FALSE, ch, 0, 0, TO_CHAR);
      if (rider)
	act("You fly down lower.", FALSE, ch, 0, rider, TO_VICT);
      act("$n flys down lower.", TRUE, ch, 0, 0, TO_PROXIMITY3);
    }
  } else if (!str_cmp(buf, "land")) {
    if (GET_Z_CORD(ch) < 1 || (GET_Z_CORD(ch) == GET_Z_SIZE(ch->in_room) + 1)) {	/* rooftop */
      send_to_char("Umm, you've landed already, bucko.\r\n", ch);
      return;
    } else {
      act("$n flys down lower.", TRUE, ch, 0, 0, TO_PROXIMITY3);
      GET_Z_CORD(ch) = 0;
      act("You land to the ground.", FALSE, ch, 0, 0, TO_CHAR);
      if (rider)
	act("You land to the ground.", FALSE, ch, 0, rider, TO_VICT);
      act("$n lands $s flight.", TRUE, ch, 0, 0, TO_PROXIMITY3);
    }
  }
  /* flying mounts, et al */
  if (RIDDEN_BY(ch))
    GET_Z_CORD(RIDDEN_BY(ch)) = GET_Z_CORD(ch);
  /* Followers */
  if (ch->followers) {
    newz = GET_Z_CORD(ch);
    GET_Z_CORD(ch) = oldz;

    for (k = ch->followers; k; k = next_dude) {
      next_dude = k->next;
      if ((affected_by_spell(k->follower, SPELL_FLY) ||
	   IS_AFFECTED(k->follower, AFF_FLY)) &&
	  (k->follower->in_room == ch->in_room) &&
	  (GET_Z_CORD(k->follower) == GET_Z_CORD(ch)) &&
	  !IS_FIGHTING(k->follower) && (GET_POS(k->follower) == POS_STANDING) &&
	  CAN_SEE(k->follower, ch)) {
	GET_Z_CORD(k->follower) = newz;
	act("$N follows you.", TRUE, ch, 0, k->follower, TO_CHAR);
	act("$N follows $n.", TRUE, ch, 0, k->follower, TO_NOTVICT);
	act("You follow $n.", TRUE, ch, 0, k->follower, TO_VICT);
      }
    }

    GET_Z_CORD(ch) = newz;
  }
  if (!saw_map) {
    if (rider)
      smallmap_look(rider, view_distance(rider), -1);
    smallmap_look(ch, view_distance(ch), -1);
  }
}


ACMD(do_swim)
{
  struct char_data *rider = NULL;
  struct follow_type *k, *next_dude;
  int saw_map = 0, sect;

  /* mounted chars can use this to order their mounts as well */
  if (RIDING(ch)) {
    ch = RIDING(ch);		/* simply change who were dealing with */
    rider = RIDDEN_BY(ch);
  }
  one_argument(argument, buf);
  if (!*buf) {
    send_to_char("Swim what, where why or who?!?\r\n", ch);
    return;
  }
  if (!IS_WATER_CORD(ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch))) {
    if (GET_Z_CORD(ch) < 0)	/* in case of errors */
      GET_Z_CORD(ch) = 0;
    send_to_char("You must be in water to swim.\r\n", ch);
    return;
  }
  if (GET_Z_CORD(ch) > 0) {
    send_to_char("Umm, try landing first.\r\n", ch);
    return;
  }
  sect = world[ch->in_room].sector_type;

  if (!str_cmp(buf, "up")) {
    /* if there's already an up exit here, let's move em up it */
    if (EXIT(ch, UP)) {
      perform_move(ch, 5, 1);
      GET_Z_CORD(ch) = 0;
      return;
    }
    if (GET_Z_CORD(ch) == 0) {
      send_to_char("You've already surfaced doofus!\r\n", ch);
      return;
    } else {
      act("$n swims up higher.", TRUE, ch, 0, 0, TO_PROXIMITY);
      GET_Z_CORD(ch)++;
      act("You swim up higher.", FALSE, ch, 0, 0, TO_CHAR);
      if (rider) {
	act("You swim up higher.", FALSE, ch, 0, rider, TO_VICT);
      }
      act("$n swims up higher.", TRUE, ch, 0, 0, TO_PROXIMITY);
    }
  } else if (!str_cmp(buf, "down")) {
    /* if there's already a down exit here, let's move em to it */
    if (EXIT(ch, DOWN)) {
      perform_move(ch, 6, 1);
      GET_Z_CORD(ch) = 0;
      return;
    }
    if (GET_Z_CORD(ch) == -(distance_from_shore(ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch)))) {
      send_to_char("Umm, you've reached the floor already, bucko.\r\n", ch);
      return;
    } else {
      act("$n swims down lower.", TRUE, ch, 0, 0, TO_PROXIMITY);
      GET_Z_CORD(ch) -= 1;
      act("You swim down lower.", FALSE, ch, 0, 0, TO_CHAR);
      if (rider)
	act("You swim down lower.", FALSE, ch, 0, rider, TO_VICT);
      act("$n swims down lower.", TRUE, ch, 0, 0, TO_PROXIMITY);
    }
  } else if (!str_cmp(buf, "surface")) {
    if (GET_Z_CORD(ch) > -1) {
      send_to_char("Umm, you've surfaced already, bucko.\r\n", ch);
      return;
    } else {
      act("$n swims down lower.", TRUE, ch, 0, 0, TO_PROXIMITY);
      GET_Z_CORD(ch) = 0;
      act("You swim to the surface.", FALSE, ch, 0, 0, TO_CHAR);
      if (rider)
	act("You swim to the surface.", FALSE, ch, 0, rider, TO_VICT);
      act("$n bobs to the surface of the water.", TRUE, ch, 0, 0, TO_PROXIMITY);
    }
  }
  /* swiming mounts, et al */
  if (RIDDEN_BY(ch))
    GET_Z_CORD(RIDDEN_BY(ch)) = GET_Z_CORD(ch);
  /* Followers */
  if (ch->followers)
    for (k = ch->followers; k; k = next_dude) {
      next_dude = k->next;
      if (IS_NPC(k->follower) && k->follower->in_room == ch->in_room) {
	act("$N follows $n.", TRUE, ch, 0, k->follower, TO_PROXIMITY);
	GET_Z_CORD(k->follower) = GET_Z_CORD(ch);
	act("$N follows you.", TRUE, ch, 0, k->follower, TO_CHAR);
	act("$N follows $n.", TRUE, ch, 0, k->follower, TO_PROXIMITY);
	act("You follow $n.", TRUE, ch, 0, k->follower, TO_VICT);
      }
    }
  if (!saw_map) {
    if (rider)
      smallmap_look(rider, view_distance(rider), -1);
    smallmap_look(ch, view_distance(ch), -1);
  }
}


/* send out to_char messages now, the rest get returned */
char *move_message(struct char_data *ch, struct char_data *people, int exitnumb, char *amsg)
{
  char tmp[512];

  if (!ch || !people) {
    strcpy(amsg, "leave_message error #283\r\n");
    return amsg;
  }
  if (RIDING(ch)) {
    if (SECT(ch->in_room) == SECT_UNDERWATER || GET_Z_CORD(ch) < 0) {
      sprintf(amsg, "Your mount swims %s", dirs[exitnumb]);
      if (VEHICLE(RIDING(ch)))
	strcat(amsg + strlen(amsg), " pulling $p behind.\n");
      else
	strcat(amsg + strlen(amsg), ".\n");
      act(amsg, FALSE, ch, VEHICLE(RIDING(ch)), 0, TO_CHAR);
      sprintf(amsg, "%s swims %s on ", PERS(ch, people, FALSE), dirs[exitnumb]);
    } else if (GET_Z_CORD(RIDING(ch)) > 0) {
      sprintf(amsg, "You fly %s on %s", dirs[exitnumb], CAN_SEE(ch, RIDING(ch)) ?
	      GET_NAME(RIDING(ch)) : "something");
      if (VEHICLE(RIDING(ch)))
	strcat(amsg + strlen(amsg), " pulling $p behind.\n");
      else
	strcat(amsg + strlen(amsg), ".\n");
      act(amsg, FALSE, ch, VEHICLE(RIDING(ch)), 0, TO_CHAR);
      sprintf(amsg, "%s flys %s on ", PERS(ch, people, FALSE), dirs[exitnumb]);
    } else {
      sprintf(amsg, "You ride %s on %s", dirs[exitnumb], GET_NAME(RIDING(ch)));
      if (VEHICLE(RIDING(ch)))
	strcat(amsg + strlen(amsg), " pulling $p behind.\n");
      else
	strcat(amsg + strlen(amsg), ".\n");
      act(amsg, FALSE, ch, VEHICLE(RIDING(ch)), 0, TO_CHAR);
      sprintf(amsg, "%s rides %s on ", PERS(ch, people, FALSE), dirs[exitnumb]);
    }
  } else if (!IS_NPC(ch) && GET_COND(ch, DRUNK) > 10) {
    sprintf(amsg, "You stumble off drunkenly on $s way %s.", dirs[exitnumb]);
    act(amsg, FALSE, ch, 0, 0, TO_CHAR);
    sprintf(amsg, "$n stumbles off drunkenly on $s way %s.", dirs[exitnumb]);
  } else {
    /* build name */
    if (ch == people)
      strcpy(amsg, "You");
    else if (IS_TRUSTED(ch) && GET_NAME(ch))
      strcpy(amsg, GET_NAME(ch));
    else
      strcpy(amsg, PERS(ch, people, TRUE));

    strcat(amsg, " ");

    /* add verb and direction */
    sprintf(amsg + strlen(amsg), "%s",
	    SECT(ch->in_room) == SECT_UNDERWATER ? "swim" :
	    GET_Z_CORD(ch) < 0 ? "swim" :
	    GET_Z_CORD(ch) == 0 && IS_WATER_CORD(IN_ROOM(ch), GET_X_CORD(ch), GET_Y_CORD(ch)) ? "wade" :
	    GET_Z_CORD(ch) > 0 ? "fly" :
	    LEVITATE(ch, exitnumb) ? "float" :
	    IS_SLIME(ch) ? "ooze" :
	    carried_load_percent(ch) >= 75 ? "stagger" :
	    SNEAK(ch) ? "sneak" :
	    GET_RACE(ch) == RACE_SNAKE ? "slither" :
	    GET_POS(ch) <= POS_SITTING ? "crawl" :
	    GET_RACE(ch) == RACE_HORSE ? "trot" :
	    GET_SIZE(ch) == SIZE_TINY ? "scurrie" :
	    GET_SIZE(ch) >= SIZE_HUGE ? "rumble" :
	    "walk");

    if (ch != people)
      strcat(amsg, "s");
    sprintf(amsg + strlen(amsg), " %s", dirs[exitnumb]);
    /* add goodies */
    if (GET_Z_CORD(ch) != GET_Z_CORD(people)) {
      if (GET_Z_CORD(ch) > GET_Z_CORD(people))
	strcat(amsg, " above you");
      else
	strcat(amsg, " below you");
    }
    /* add suffix or end it */
    if (VEHICLE(ch))
      strcat(amsg, " pulling $p behind.");
    else if (!RIDING(ch))
      strcat(amsg, ".");
  }
  if (RIDING(ch)) {
    sprintf(tmp, "%s riding on %s", amsg, CAN_SEE(people, RIDING(ch)) ?
	    GET_NAME(RIDING(ch)) : "something");
    if (VEHICLE(RIDING(ch)))
      sprintf(tmp + strlen(tmp), " pulling $p behind.");
    else
      sprintf(tmp + strlen(tmp), ".");
    strcpy(amsg, tmp);
  }
  return amsg;
}


int CoordsInRangeForInnerObjRoomTravel(struct char_data *ch, int dir)
{
  int room, xroom, yroom, xchar, ychar, moved, cantenter;

  room = ch->in_room;
  xroom = world[room].x_size;
  yroom = world[room].y_size;
  xchar = GET_X_CORD(ch);
  ychar = GET_Y_CORD(ch);
  moved = 0;

  /* do a quick loop here, to see if a square is too crowded. Indoors
   * is approx 3' square, so allow only 2. Or 3 if size_tiny. Outdoors,
   * let's ignore any limits, as we're only really going to care during
   * combat, where we can always check again.
   */

  cantenter = CantEnterCoord(ch, dir);
  if (cantenter == 1)
    return -1;
  else if (cantenter == 2)
    return -2;
  else if (checkmovetrap(ch, dir))
    return 1;

  /* Check the ground conditions */
  FallingChar(ch, TRUE);

  switch (dir) {
  case NORTH:
    if (xchar >= 1 && (ychar >= 0 && ychar <= yroom - 1)) {
      GET_X_CORD(ch)--;
      SET_FACING(ch, NORTH);
      moved = 1;
    }
    break;
  case NORTHEAST:
    if (xchar >= 1 && ychar <= (yroom - 2)) {
      GET_X_CORD(ch)--;
      GET_Y_CORD(ch)++;
      SET_FACING(ch, NORTHEAST);
      moved = 1;
    }
    break;
  case EAST:
    if (ychar <= (yroom - 2) && (xchar >= 0 && xchar <= xroom - 1)) {
      GET_Y_CORD(ch)++;
      SET_FACING(ch, EAST);
      moved = 1;
    }
    break;
  case SOUTHEAST:
    if (ychar <= (yroom - 2) && xchar <= (xroom - 2)) {
      GET_X_CORD(ch)++;
      GET_Y_CORD(ch)++;
      SET_FACING(ch, SOUTHEAST);
      moved = 1;
    }
    break;
  case SOUTH:
    if (xchar <= (xroom - 2) && (ychar >= 0 && ychar <= yroom - 1)) {
      GET_X_CORD(ch)++;
      SET_FACING(ch, SOUTH);
      moved = 1;
    }
    break;
  case SOUTHWEST:
    if (xchar <= (xroom - 1) && ychar >= 1) {
      GET_X_CORD(ch)++;
      GET_Y_CORD(ch)--;
      SET_FACING(ch, SOUTHWEST);
      moved = 1;
    }
    break;
  case WEST:
    if (ychar >= 1 && (xchar >= 0 && xchar <= xroom - 1)) {
      GET_Y_CORD(ch)--;
      SET_FACING(ch, WEST);
      moved = 1;
    }
    break;
  case NORTHWEST:
    if (xchar >= 1 && ychar >= 1) {
      GET_X_CORD(ch)--;
      GET_Y_CORD(ch)--;
      SET_FACING(ch, NORTHWEST);
      moved = 1;
    }
    break;
  }
  if (GET_X_CORD(ch) < 0 || GET_X_CORD(ch) >= xroom)
    GET_X_CORD(ch) = (xroom / 2);
  if (moved) {
    /* this will handle pits within a room */
    FallingChar(ch, TRUE);
    return 1;
  }
  return 0;
}


void CoordsForRoomToRoom(struct char_data *ch, int room, int dir, int *x, int *y, int *z)
{
  int from_dir = -1, modifier = 0, mod2 = 0;
  room_rnum previous = 0;

  from_dir = rev_dir[(int) dir];

  previous = world[room].dir_option[from_dir] ? world[room].dir_option[from_dir]->to_room : 0;
  if (previous) {
    if (from_dir == NORTH || from_dir == SOUTH) {
      modifier = VIRTUAL_EXIT(previous, dir) ? GET_DOOR_Y_CORD(previous, dir) : 0;
      mod2 = VIRTUAL_EXIT(room, from_dir) ? GET_DOOR_Y_CORD(room, from_dir) : 0;
    }				//if

    else if (from_dir == EAST || from_dir == WEST) {
      modifier = VIRTUAL_EXIT(previous, dir) ? GET_DOOR_X_CORD(previous, dir) : 0;
      mod2 = VIRTUAL_EXIT(room, from_dir) ? GET_DOOR_X_CORD(room, from_dir) : 0;
    }				//else if

  }				//if
  /* a door coord of -1 means we won't alter that stat. It's probably
   * a non-door wilderness move
   */
  if (from_dir > -1) {
    switch (from_dir) {
    case NORTH:
      *x = 1;
      *y = mod2 + (GET_Y_CORD(ch) - modifier);
      break;
    case SOUTH:
      *x = GET_X_SIZE(room) - 2;
      *y = mod2 + (GET_Y_CORD(ch) - modifier);
      break;
    case EAST:
      *x = mod2 + (GET_X_CORD(ch) - modifier);
      *y = GET_Y_SIZE(room) - 2;
      break;
    case WEST:
      *x = mod2 + (GET_X_CORD(ch) - modifier);
      *y = 1;
      break;
    default:
      /* some kind of error, just randomize */
      *x = number(1, GET_X_SIZE(room) - 2);
      *y = number(1, GET_Y_SIZE(room) - 2);
      break;
    }				//switch

    /* The doors may not be at the same height, so let's equalize */
    if (VIRTUAL_EXIT(room, from_dir))
      *z = GET_DOOR_Z_CORD(room, from_dir);

    /* if it was a real door, ensure they remain in door square */
    if ((IS_SET(ROOM_FLAGS(room), ROOM_INDOORS)) &&
	(GET_X_CORD(ch) == 0 || GET_X_CORD(ch) == GET_X_SIZE(room) - 1
     || GET_Y_CORD(ch) == 0 || GET_Y_CORD(ch) == GET_Y_SIZE(room) - 1)) {
      if (from_dir == NORTH || from_dir == SOUTH)
	*y = GET_DOOR_Y_CORD(room, from_dir);
      else if (from_dir == WEST || from_dir == EAST)
	*x = GET_DOOR_X_CORD(room, from_dir);
    }				//if

  }				//if

  else if (from_dir == -1) {
    *x = number(1, GET_X_SIZE(room) - 2);
    *y = number(1, GET_Y_SIZE(room) - 2);
  }				//else if

}				//CoordsForRoomtoRoom


ACMD(do_push)
{
  struct obj_data *obj;
  struct char_data *tmp_char;
  char _name[256], _direction[256];
  int dir = 0, o_x, o_y, ch_x, ch_y, ch_z, obj_x, obj_y, obj_z, movetest = 1,
   WithinRange = 0;
  room_rnum previous = 0, room = 0;

  if (!ch || !argument || !*argument || !cmd)
    return;

  if (!CMD_IS("push"))
    return;

  ch_x = o_x = GET_X_CORD(ch);
  ch_y = o_y = GET_Y_CORD(ch);
  ch_z = GET_Z_CORD(ch);

  two_arguments(argument, _name, _direction);

  dir = search_block(_direction, dirs, FALSE);

  generic_find(_name, FIND_OBJ_ROOM, ch, &tmp_char, &obj);

  if (!obj) {
    //check if char is standing at the edge of a room
    previous = ch->in_room;
    movetest = CoordsInRangeForInnerObjRoomTravel(ch, dir);

    if (movetest == 0) {
      //check if object is adjacent to char in another room by moving char to that room
      room = FindDoor(ch, dir);
      char_from_room(ch);
      char_to_room(ch, room, rev_dir[(int) dir]);

      //look for item
      generic_find(_name, FIND_OBJ_ROOM, ch, &tmp_char, &obj);

      if (!obj) {
	//put char back
	char_from_room(ch);
	put_char_to_room(ch, previous, dir, ch_x, ch_y, ch_z);
	GET_X_CORD(ch) = ch_x;
	GET_Y_CORD(ch) = ch_y;
	send_to_char("Push what?\r\n", ch);
	return;
      }				//if

      else {
	CoordsForRoomToRoom(ch, room, dir, &ch_x, &ch_y, &ch_z);
	char_from_room(ch);
	put_char_to_room(ch, room, dir, ch_x, ch_y, ch_z);

	if (ch_x == GET_OBJ_X_CORD(obj) && ch_y == GET_OBJ_Y_CORD(obj))
	  WithinRange = 1;
	else {			//return char

	  send_to_char("Push what?\r\n", ch);
	  char_from_room(ch);
	  put_char_to_room(ch, previous, dir, o_x, o_y, ch_z);
	  GET_X_CORD(ch) = o_x;
	  GET_Y_CORD(ch) = o_y;
	  return;
	}
      }
    }				//if

    else {
      char_from_room(ch);
      put_char_to_room(ch, previous, dir, ch_x, ch_y, ch_z);
      send_to_char("Push what?\r\n", ch);
      return;
    }				//else

  }				//if

  if (isname("ne", _direction))
    dir = 6;
  else if (isname("nw", _direction))
    dir = 7;
  else if (isname("se", _direction))
    dir = 8;
  else if (isname("sw", _direction))
    dir = 9;

  obj_x = GET_OBJ_X_CORD(obj);
  obj_y = GET_OBJ_Y_CORD(obj);
  obj_z = GET_OBJ_Z_CORD(obj);

  switch (dir) {
  case NORTH:
    ch_x--;
    obj_x--;
    break;
  case NORTHEAST:
    ch_x--;
    ch_y++;
    obj_x--;
    obj_y++;
    break;
  case EAST:
    ch_y++;
    obj_y++;
    break;
  case SOUTHEAST:
    ch_x++;
    ch_y++;
    obj_x++;
    obj_y++;
    break;
  case SOUTH:
    ch_x++;
    obj_x++;
    break;
  case SOUTHWEST:
    ch_x++;
    ch_y--;
    obj_x++;
    obj_y--;
    break;
  case WEST:
    ch_y--;
    obj_y--;
    break;
  case NORTHWEST:
    ch_x--;
    ch_y--;
    obj_x--;
    obj_y--;
    break;
  default:
    break;
  }				//switch

  if (distBetweenCharAndObj(ch, obj) > 1) {	//distance from obj check

    act("You need to get closer to push it.", FALSE, ch, obj, 0, TO_CHAR);
    return;
  }				//if

  if (dir == 4 || dir == 5 || dir < 0 || dir > 9) {
    act("Which direction do you want to push $p", FALSE, ch, obj, 0, TO_CHAR);
    return;
  }				//if

  if (!WithinRange && !WILDERNESS(IN_ROOM(ch)) && (GET_OBJ_X_CORD(obj) != ch_x || GET_OBJ_Y_CORD(obj) != ch_y)) {
    sprintf(_name, "You need to get behind $p to push it %s.", dirs[dir]);
    act(_name, FALSE, ch, obj, 0, TO_CHAR);
    return;
  }				//if

  if (IS_AFFECTED4(ch, AFF4_WRAITHFORM)) {
    send_to_char("You arent solid enough to push anything yet", ch);
    return;
  }
  previous = ch->in_room;
  room = obj->in_room;
  ch_x = GET_X_CORD(ch);
  ch_y = GET_Y_CORD(ch);
  ch_z = GET_Z_CORD(ch);

  // strength check
  if (GET_OBJ_WEIGHT(obj) >= 1000) {
    act("$p is much too heavy for you to push it.", FALSE, ch, obj, 0, TO_CHAR);
    return;
  }
  if (IS_CARRYING_W(ch) + GET_OBJ_WEIGHT(obj) > (CAN_CARRY_W(ch) * 2) + GET_WEIGHT(ch)) {
    act("$n pushes at a $p but it fails to move.", FALSE, ch, obj, 0, PROXIMITY);
    act("You push at $p but it fails to move.", FALSE, ch, obj, 0, TO_CHAR);
    return;
  }				//else if

  ch->in_room = obj->in_room;
  GET_X_CORD(ch) = GET_OBJ_X_CORD(obj);
  GET_Y_CORD(ch) = GET_OBJ_Y_CORD(obj);

  // check to see if object is pushed into another room
  movetest = CoordsInRangeForInnerObjRoomTravel(ch, dir);
  if (movetest == 0) {
    // check to see if exit is enterable
    if ((!EXIT(ch, dir) || EXIT(ch, dir)->to_room <= 1) ||
	(IS_SET(EXIT(ch, dir)->exit_info, EX_WALLED)) ||
	(IS_SET(EXIT(ch, dir)->exit_info, EX_CLOSED))) {
      send_to_char("You can't go there.\r\n", ch);
      return;
    }				//if

    sprintf(_name, "$n begins to push $p %s", dirs[dir]);
    act(_name, FALSE, ch, obj, 0, PROXIMITY);
    sprintf(_name, "You push $p %s", dirs[dir]);
    act(_name, FALSE, ch, obj, 0, TO_CHAR);

    // move object into exit
    dir = convert_diagonals(ch, dir);
    room = FindDoor(ch, dir);

    CoordsForRoomToRoom(ch, room, dir, &obj_x, &obj_y, &obj_z);

    /* return player to orinal position */
    char_from_room(ch);
    put_char_to_room(ch, obj->in_room, -1, ch_x, ch_y, ch_z);

    /* place object */
    obj_from_room(obj);
    put_obj_to_room(obj, room, obj_x, obj_y, obj_z);

    /* move char */
    perform_move(ch, dir, 0);
    return;
  }				//if

  char_from_room(ch);
  put_char_to_room(ch, obj->in_room, -1, GET_OBJ_X_CORD(obj), GET_OBJ_Y_CORD(obj), GET_OBJ_Z_CORD(obj));

  if (CantEnterCoord(ch, dir)) {	//invalid direction

    act("$n fails to move $p.", FALSE, ch, obj, 0, PROXIMITY);
    sprintf(_name, "You fail to push $p %s", dirs[dir]);
    act(_name, FALSE, ch, obj, 0, TO_CHAR);
    char_from_room(ch);
    put_char_to_room(ch, previous, -1, ch_x, ch_y, ch_z);
    return;
  }				//else if

  else {			//move object

    sprintf(_name, "$n begins to push $p %s", dirs[dir]);
    act(_name, FALSE, ch, obj, 0, PROXIMITY);
    sprintf(_name, "You push $p %s", dirs[dir]);
    act(_name, FALSE, ch, obj, 0, TO_CHAR);

    /* return player to orinal position */
    char_from_room(ch);
    put_char_to_room(ch, previous, -1, ch_x, ch_y, ch_z);

    /* place object */
    obj_from_room(obj);
    put_obj_to_room(obj, room, obj_x, obj_y, obj_z);

    /* move char */
    if (!WithinRange)
      perform_move(ch, dir, 0);
    else
      look_at_room(ch, 0);

    return;
  }				//else

  char_from_room(ch);
  put_char_to_room(ch, previous, -1, ch_x, ch_y, ch_z);
  return;
}				//push boulder

ACMD(do_climb)
{
  int newx, newy, x, y, percent, dir, slip = FALSE;
  char *Gbuf;

  percent = number(0, 101);

  skip_spaces(&argument);
  if (!*argument) {
    send_to_char("Climb what?\r\n", ch);
    return;
  }
  if (!GET_SKILL(ch, SKILL_CLIMB_WALL) && !IS_TRUSTED(ch)) {
    send_to_char("First you need to learn *how* to climb.\r\n", ch);
    return;
  } else if (percent > GET_SKILL(ch, SKILL_CLIMB_WALL) && !IS_TRUSTED(ch)) {
    /* they fail, so they fall */
    send_to_char("Oops, you slipped!\r\n", ch);
    if (GET_Z_CORD(ch) == 0)	/* If they were on the ground, how far up did they get? */
      GET_Z_CORD(ch) += number(0, 5);
    else
      slip = TRUE;		/* So we can drop them _after_ moving them to the next room */
    FallingChar(ch, TRUE);
    return;
  }
  for (x = -1; x <= 1; x++) {
    for (y = -1; y <= 1; y++) {

      /* cardinal dirs only. */
      if (x != 0 && y != 0)
	continue;

      /* what dir is the climb? We only care about the cardinals here */
      if (x == -1 && y == 0)
	dir = NORTH;
      else if (x == 1 && y == 0)
	dir = SOUTH;
      else if (x == 0 && y == -1)
	dir = WEST;
      else if (x == 0 && y == 1)
	dir = EAST;
      else
	dir = -1;

      if (GET_X_CORD(ch) + x < 0 || GET_X_CORD(ch) + x >= GET_X_SIZE(ch->in_room))
	newx = GET_X_CORD(ch);
      else
	newx = GET_X_CORD(ch) + x;
      if (GET_Y_CORD(ch) + y < 0 || GET_Y_CORD(ch) + y >= GET_Y_SIZE(ch->in_room))
	newy = GET_Y_CORD(ch);
      else
	newy = GET_Y_CORD(ch) + y;

      /* loop till we find the closest climbable square */
      if ((IS_WALL(world[ch->in_room].map->map_grid[newx][newy]) && !str_cmp(argument, "wall")) ||
	  (IS_WALL(world[ch->in_room].map->map_grid[newx][newy]) && GET_Z_CORD(ch) > 0 && !str_cmp(argument, "down")) ||
	  (world[ch->in_room].map->map_grid[newx][newy] == MAP_TREE && !str_cmp(argument, "tree")) ||
	  (world[ch->in_room].map->map_grid[newx][newy] == MAP_RUBBLE && !str_cmp(argument, "rubble"))) {
	/* move the char to it. This isnt possible with normal moves */
	GET_X_CORD(ch) = newx;
	GET_Y_CORD(ch) = newy;
	/* not infallible logic, but hey, it may work */
	if (!str_cmp(argument, "tree")) {
	  GET_Z_CORD(ch) = 5;	/* how large of trees do we have? */
	  Gbuf = strdup("top");
	} else if (!str_cmp(argument, "rubble")) {
	  GET_Z_CORD(ch) = 1;
	  Gbuf = strdup("top");
	} else if (dir > -1 && EXIT(ch, dir) && GET_DOOR_Z_CORD(ch->in_room, dir)) {
	  GET_Z_CORD(ch) = GET_DOOR_Z_CORD(ch->in_room, dir);
	  Gbuf = strdup(EXIT(ch, dir)->keyword);
	  /* need work here... how do we get them back down off the roof? */
	} else if (GET_Z_CORD(ch) == GET_Z_SIZE(ch->in_room) + 1 && !OUTSIDE(ch)) {
	  if ((GET_X_CORD(ch) <= 0 || (GET_X_CORD(ch) >= GET_X_SIZE(ch->in_room) - 1)
	       || (GET_Y_CORD(ch) <= 0) || (GET_Y_CORD(ch) >= GET_Y_SIZE(ch->in_room) - 1))) {
	    if (EXIT(ch, dir) && !IS_SET(ROOM_FLAGS(EXIT(ch, dir)->to_room), ROOM_INDOORS)) {
	      do_simple_move(ch, EXIT(ch, dir)->to_room, dir, 1);
	      if (slip) {
		FallingChar(ch, TRUE);
		return;
	      }
	      GET_Z_CORD(ch) = 0;
	      Gbuf = strdup("ground");
	    } else {
	      send_to_char("Try as you might, the rooftiles block your passage.\r\n", ch);
	      return;
	    }
	  } else {
	    send_to_char("Try as you might, the rooftiles prevent your passage.\r\n", ch);
	    return;
	  }
	} else if (GET_Z_CORD(ch) > GET_Z_SIZE(ch->in_room) + 1 && !OUTSIDE(ch)) {
	  /* GET_Z_CORD(ch) = GET_Z_SIZE(ch->in_room) + 1; */
	  /* problem is, I need to figure out which room it is, and how high the ceiling is */
	  GET_Z_CORD(ch) = 4;
	  Gbuf = strdup("roof");
	} else if (GET_Z_CORD(ch) > 0) {
	  GET_Z_CORD(ch) = 0;	/* Does this work for climbing down? */
	  Gbuf = strdup("floor");
	  /* move them off the wall tile */
	  if (GET_X_CORD(ch) <= 0)
	    GET_X_CORD(ch) = 1;
	  else if (GET_X_CORD(ch) >= GET_X_SIZE(ch->in_room) - 1)
	    GET_X_CORD(ch) = GET_X_SIZE(ch->in_room) - 2;
	  else if (GET_Y_CORD(ch) <= 0)
	    GET_Y_CORD(ch) = 1;
	  else if (GET_Y_CORD(ch) >= GET_Y_SIZE(ch->in_room) - 1)
	    GET_Y_CORD(ch) = GET_Y_SIZE(ch->in_room) - 2;
	} else if (!OUTSIDE(ch)) {
	  GET_Z_CORD(ch) = GET_Z_SIZE(ch->in_room);
	  Gbuf = strdup("ceiling");
	} else {
	  /* GET_Z_CORD(ch) = GET_Z_SIZE(ch->in_room) + 1; */
	  /* problem is, I need to figure out which room it is, and how high the ceiling is */
	  GET_Z_CORD(ch) = 4;
	  Gbuf = strdup("roof");
	}
	sprintf(buf, "You climb the %s to the %s.\r\n", UNCAP(argument), Gbuf);
	send_to_char(buf, ch);
	maplook(ch, view_distance(ch), -1, TO_CHAR);
	/* done, lets get out of here */
	return;
      }
    }
  }
  sprintf(buf, "Are you sure there's a %s to climb?\r\n", UNCAP(argument));
  send_to_char(buf, ch);
}

ACMD(do_knock)
{
  int door = -1;
  struct obj_data *obj = NULL;
  char type[MAX_INPUT_LENGTH], dir[MAX_INPUT_LENGTH];

  skip_spaces(&argument);
  if (!*argument) {
    send_to_char("Knock on what, exactly?", ch);
    return;
  }
  two_arguments(argument, type, dir);
  door = find_door(ch, type, dir, cmd_door[subcmd]);

  if (door < 0) {
    send_to_char("There's nothing like that here.\r\n", ch);
    return;
  }
  if (!ProximityCheck(ch, obj, NULL, door)) {
    send_to_char("You're not close enough to knock!", ch);
    return;
  }
  if (!(DOOR_IS_OPENABLE(ch->in_room, obj, door))) {
    act("You can't knock on that!", FALSE, ch, 0, 0, TO_CHAR);
    return;
  } else if (!DOOR_IS_CLOSED(ch->in_room, obj, door) &&
	     IS_SET(flags_door[subcmd], NEED_CLOSED)) {
    send_to_char("But it's currently open!\r\n", ch);
    return;
  }
  sprintf(buf, "$n knocks on the %s.", (!(EXIT(ch, door)->keyword) ?
			       "door" : fname(EXIT(ch, door)->keyword)));

  act(buf, FALSE, ch, 0, 0, TO_ROOM);

  sprintf(buf, "You knock on the %s.", (!(EXIT(ch, door)->keyword) ?
			       "door" : fname(EXIT(ch, door)->keyword)));

  act(buf, FALSE, ch, 0, 0, TO_CHAR);

  sprintf(buf, "There is a loud knock from the %s.",
	  (!(EXIT(ch, door)->keyword) ? "door" :
	   fname(EXIT(ch, door)->keyword)));

  send_to_room(buf, EXIT(ch, door)->to_room);
  return;
}
