
/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"
#include <values.h>

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"
#include "graph.h"
#include "skills.h"

/* Externals */
extern int top_of_world;
extern const char *dirs[];
extern struct room_data *world;
extern struct zone_data *zone_table;
extern ACMD(do_say);
extern struct char_data *character_list;
extern void move_closer(struct char_data *ch, struct char_data *vict, struct obj_data *obj);


#if 1
#define TRACK_THROUGH_DOORS
/*
 * You can define or not define TRACK_THOUGH_DOORS, above, depending on
 * whether or not you want track to find paths which lead through closed
 * or hidden doors. 
 */
static inline void bfs_dequeue(void);
static void bfs_clear_queue(void);

struct bfs_queue_struct {
  int room;
  int dir;
  int step_no;
  struct bfs_queue_struct *next;
};

static struct bfs_queue_struct *queue_head = 0, *queue_tail = 0;



static inline void bfs_enqueue(int room, int dir, int step)
{
  struct bfs_queue_struct *curr;

  CREATE(curr, struct bfs_queue_struct, 1);

  curr->room = room;
  curr->dir = (ubyte) dir;
  curr->step_no = step;
  curr->next = 0;

  if (queue_tail) {
    queue_tail->next = curr;
    queue_tail = curr;
  } else
    queue_head = queue_tail = curr;
}

static inline void bfs_dequeue(void)
{
  struct bfs_queue_struct *curr;

  curr = queue_head;

  if (!(queue_head = queue_head->next))
    queue_tail = 0;
  FREE((char *) curr);
}

static void bfs_clear_queue(void)
{
  while (queue_head)
    bfs_dequeue();
}

/*
 * find_first_step: given a source room and a target room, find the first
 * step on the shortest path from the source to the target.
 * 
 * can_fly should be TRUE if a path can go over WATER_NOSWIM and NOGROUND
 * rooms.
 * 
 * ttl_steps will result in being the total number of step required to get 
 * to the destination. (assuming its possible)
 * 
 * Intended usage: in mobile_activity, give a mob a dir to go if they're
 * tracking another mob or a PC.  Or, a 'track' skill for PCs. 
 */

/*
 * NOTE: this uses REAL room numbers!! 
 */

inline byte
 find_first_step(int src, int target, int can_fly, int is_ship, int wagon_type, int *ttl_steps)
{
  int curr_dir;
  int curr_room;
  ulong i = 0U;

  if (src < 0 || src > top_of_world || target < 0 || target > top_of_world) {
    log("SYSERR: Illegal value passed to find_first_step (graph.c)");
    return BFS_ERROR;
  }
  if (src == target) {
    return BFS_ALREADY_THERE;
  }
  /*
   * clear marks first 
   */
  for (curr_room = 0; curr_room <= top_of_world; curr_room++)
    UNMARK(curr_room);

  MARK(src);

  /*
   * first, enqueue the first steps, saving which direction we're going. 
   */
  for (curr_dir = 0; curr_dir < NUM_EXITS; curr_dir++) {
    if (wagon_type) {
      if (wagon_type == 1 && VALID_WAGON_EDGE(src, curr_dir)) {
	MARK(TOROOM(src, curr_dir));
	bfs_enqueue(TOROOM(src, curr_dir), curr_dir, 1);
      }
    } else if (is_ship ? VALID_SHIP_EDGE(src, curr_dir) : VALID_EDGE(src, curr_dir) &&
	       (can_fly || !NEEDS_FLY(src, curr_dir))) {
      MARK(TOROOM(src, curr_dir));
      bfs_enqueue(TOROOM(src, curr_dir), curr_dir, 1);
    }
  }
  /*
   * now, do the classic BFS. 
   */
  while (queue_head) {
    if (i++ > BFS_MAX_ROOMS) {
      /*
       * ack!  we searched too many rooms.. just give up 
       */
      bfs_clear_queue();
      return BFS_ERROR;
    }
    if (queue_head->room == target) {
      curr_dir = queue_head->dir;
      *ttl_steps = queue_head->step_no;
      bfs_clear_queue();
      return curr_dir;
    } else {
      for (curr_dir = 0; curr_dir < NUM_EXITS; curr_dir++) {
	if (wagon_type) {
	  if (wagon_type == 1 && VALID_WAGON_EDGE(queue_head->room, curr_dir)) {
	    MARK(TOROOM(queue_head->room, curr_dir));
	    bfs_enqueue(TOROOM(queue_head->room, curr_dir), queue_head->dir,
			queue_head->step_no + 1);
	  }
	} else if (is_ship ? VALID_SHIP_EDGE(queue_head->room, curr_dir) :
		   VALID_EDGE(queue_head->room, curr_dir) &&
		   (can_fly || !NEEDS_FLY(queue_head->room, curr_dir))) {
	  MARK(TOROOM(queue_head->room, curr_dir));
	  bfs_enqueue(TOROOM(queue_head->room, curr_dir), queue_head->dir,
		      queue_head->step_no + 1);
	}
      }
      bfs_dequeue();
    }
  }
  return BFS_NO_PATH;
}
#else
struct bfs_queue_struct {
  int room;
  char dir;
  struct bfs_queue_struct *next;
};

static struct bfs_queue_struct *queue_head = 0, *queue_tail = 0;

void bfs_enqueue(int room, int dir)
{
  struct bfs_queue_struct *curr;

  CREATE(curr, struct bfs_queue_struct, 1);
  curr->room = room;
  curr->dir = dir;
  curr->next = 0;

  if (queue_tail) {
    queue_tail->next = curr;
    queue_tail = curr;
  } else
    queue_head = queue_tail = curr;
}


void bfs_dequeue(void)
{
  struct bfs_queue_struct *curr;

  curr = queue_head;

  if (!(queue_head = queue_head->next))
    queue_tail = 0;
  FREE(curr);
}


void bfs_clear_queue(void)
{
  while (queue_head)
    bfs_dequeue();
}


/* find_first_step: given a source room and a target room, find the first
   step on the shortest path from the source to the target.

   Intended usage: in mobile_activity, give a mob a dir to go if they're
   tracking another mob or a PC.  Or, a 'track' skill for PCs.
 */

int find_first_step(int src, int target)
{
  int curr_dir;
  int curr_room;

  if (src < 0 || src > top_of_world || target < 0 || target > top_of_world) {
    log("Illegal value passed to find_first_step (graph.c)");
    return BFS_ERROR;
  }
  if (src == target)
    return BFS_ALREADY_THERE;

  /* clear marks first */
  for (curr_room = 0; curr_room <= top_of_world; curr_room++)
    UNMARK(curr_room);

  MARK(src);

  /* first, enqueue the first steps, saving which direction we're going. */
  for (curr_dir = 0; curr_dir < NUM_OF_DIRS; curr_dir++)
    if (VALID_EDGE(src, curr_dir)) {
      MARK(TOROOM(src, curr_dir));
      bfs_enqueue(TOROOM(src, curr_dir), curr_dir);
    }
  /* now, do the classic BFS. */
  while (queue_head) {
    if (queue_head->room == target) {
      curr_dir = queue_head->dir;
      bfs_clear_queue();
      return curr_dir;
    } else {
      for (curr_dir = 0; curr_dir < NUM_OF_DIRS; curr_dir++)
	if (VALID_EDGE(queue_head->room, curr_dir)) {
	  MARK(TOROOM(queue_head->room, curr_dir));
	  bfs_enqueue(TOROOM(queue_head->room, curr_dir), queue_head->dir);
	}
      bfs_dequeue();
    }
  }

  return BFS_NO_PATH;
}
#endif

/************************************************************************
*  Functions and Commands which use the above fns		        *
************************************************************************/

ACMD(do_track)
{
  struct char_data *vict;
  int dir, num, num_steps;

  if (!GET_SKILL(ch, SKILL_TRACK) && !IS_TRUSTED(ch)) {
    send_to_char("You have no idea how.\r\n", ch);
    return;
  }
  one_argument(argument, arg);
  if (!*arg) {
    send_to_char("Whom are you trying to track?\r\n", ch);
    return;
  }
  if (!(vict = get_char_vis(ch, arg))) {
    send_to_char("No-one around by that name.\r\n", ch);
    return;
  }
  if (IS_AFFECTED(vict, AFF_NOTRACK)) {
    send_to_char("You sense no trail.\r\n", ch);
    return;
  }
#if 0
  dir = find_first_step(ch->in_room, vict->in_room);
#else
  dir = find_first_step(ch->in_room, vict->in_room, 1, 0, 0, &num_steps);
#endif
  switch (dir) {
  case BFS_ERROR:
    send_to_char("Hmm.. something seems to be wrong.\r\n", ch);
    break;
  case BFS_ALREADY_THERE:
    send_to_char("You're in visual range now.\r\n", ch);
    break;
  case BFS_NO_PATH:
    sprintf(buf, "You can't sense a trail to %s from here.\r\n",
	    HMHR(vict));
    send_to_char(buf, ch);
    break;
  default:
    num = number(0, 101);	/* 101% is a complete failure */
    if ((GET_SKILL(ch, SKILL_TRACK) < num) && !IS_TRUSTED(ch))
      do {
	dir = number(0, NUM_OF_DIRS - 1);
      } while (!CAN_GO(ch, dir));
    sprintf(buf, "You sense a trail %s from here!\r\n", dirs[dir]);
    send_to_char(buf, ch);
    break;
  }
}


void hunt_victim(struct char_data *ch)
{
  int dir, num_steps;
  byte found;
  struct char_data *tmp;

  if (!ch || !HUNTING(ch))
    return;

  /* make sure the char still exists */
  for (found = 0, tmp = character_list; tmp && !found; tmp = tmp->next)
    if (HUNTING(ch) == tmp)
      found = 1;

  if (!found) {
    do_say(ch, "Damn!  My prey is gone!!", 0, 0);
    HUNTING(ch) = 0;
    return;
  }
#if 0
  dir = find_first_step(ch->in_room, HUNTING(ch)->in_room);
#else
  dir = find_first_step(ch->in_room, HUNTING(ch)->in_room, 1, 0, 0, &num_steps);
#endif
  if (dir == BFS_ALREADY_THERE) {
    /* in same room now, get close */
    move_closer(ch, HUNTING(ch), NULL);
  } else if (dir < 0) {
    sprintf(buf, "Damn!  Lost %s!", HMHR(HUNTING(ch)));
    do_say(ch, buf, 0, 0);
    HUNTING(ch) = 0;
    return;
  } else {
    perform_move(ch, dir, 1);
    if (ch->in_room == HUNTING(ch)->in_room)
      hit(ch, HUNTING(ch), TYPE_UNDEFINED);
    return;
  }
}
