/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "sysdep.h"
#include "conf.h"
#include <signal.h>

#include "strings.h"
#include "structs.h"
#include "utils.h"
#include "db.h"
#include "spells.h"
#include "interpreter.h"
#include "guild.h"
#include "handler.h"
#include "comm.h"


/*   external vars  */

extern struct guild_info *guild_index;
extern struct descriptor_data *descriptor_list;
extern char *class_abbrevs[];
extern FILE *player_fl;
extern int guild_top;
extern int mini_mud;

/* Local functions */
void prepare_colour_table(struct char_data *ch);
void update_guild_cross_reference(void);
void guild_save(int system, struct char_data *ch);
void write_guild(FILE * fl, int cnum);
void display_menu(struct char_data *ch, int cnum);
int is_guildnum_already_used(int cnum);
void update_cnum(int oldcnum, int newcnum);
void setup_new_guild(struct char_data *ch, int cnum);
void setup_guild_edit(struct char_data *ch, int cnum);
void cleanup_guild_edit(struct descriptor_data *d, int method);
void refresh_guild_index(void);
void display_rank_menu(struct descriptor_data *d, int num);
void display_colour_menu(struct descriptor_data *d);
void fix_illegal_cnum(void);
int debug_guild(char *what, struct char_data *ch);
int already_being_edited(int cnum);

/*
 * Clan levels
 */
const int GUILD_APPLY = 1;
const int GUILD_SOLDIER = 2;
const int GUILD_SARGEANT = 3;
const int GUILD_CAPTAIN = 4;
const int GUILD_RULER = 5;

/* Define cross reference table */
int cross_reference[9999];

/* Define quick reference for colours (global) */
struct guild_colours guild_colour[8];

/* Define guild editing table */
struct guild_editing guild_edit[10];


/*
 * The general player commands go here
 */
ACMD(do_guild)
{
  char subbcmd[MAX_INPUT_LENGTH], val_arg[MAX_INPUT_LENGTH];
  int players_guild = 0, players_guildnum = 0, ccmd = 0, value = 0,
   found = 0, temp = 0, temp2 = 0;
  struct char_data *vict = NULL;
  struct descriptor_data *pt;

  half_chop(argument, subbcmd, buf);
  strcpy(val_arg, buf);

  /* if (ch) belongs to a guild, find its number in the index array */
  if (!((PLAYERGUILDNUM(ch) == 0) && (GUILDRANK(ch) == 0)))
    for (ccmd = 0; ccmd <= guild_top; ccmd++)
      if (GUILDNUM(guild_index[ccmd]) == PLAYERGUILD(ch)) {
	players_guildnum = ccmd;
	players_guild = GUILDNUM(guild_index[ccmd]);
      }
  if (players_guild == 0)
    players_guild = -1;

  /* Prepare the colour table for this player */
  prepare_colour_table(ch);

  /* Update guild cross reference */
  update_guild_cross_reference();

  if (strcasecmp(subbcmd, "debug") == 0) {

    if (GET_LEVEL(ch) != LVL_IMPL) {
      send_to_char("Huh?!\r\n", ch);
      return;
    }
    /* Debug guild variables */
    if (debug_guild(val_arg, ch) == NO)
      send_to_char("Debug dump not written to disk.\r\n", ch);
    else
      send_to_char("Debug dump complete.\r\n", ch);
    return;
  }
  if (strcasecmp(subbcmd, "save") == 0) {
    /* Save guilds */
    if (!(GET_LEVEL(ch) >= LVL_GOD)) {
      send_to_char("Huh?!", ch);
      return;
    }
    guild_save(NO, ch);
    return;
  }
  if (strcasecmp(subbcmd, "edit") == 0) {
    if (!(GET_LEVEL(ch) == LVL_IMPL)) {
      send_to_char("Huh?!", ch);
      return;
    }
    value = atoi(val_arg);
    if (value == 0) {
      send_to_char("Zero is an illegal guild number.\r\n", ch);
      return;
    }
    if (already_being_edited(value) == YES) {
      send_to_char("Someone is already editing that guild number.\r\n", ch);
      return;
    }
    if (is_guildnum_already_used(value) == YES)
      setup_guild_edit(ch, value);
    else
      setup_new_guild(ch, value);

    STATE(ch->desc) = CON_GEDIT;

    act("$n starts editing guilds.", YES, ch, 0, 0, PROXIMITY);
    SET_BIT(PLR_FLAGS(ch), PLR_WRITING);

    return;
  }
  if (strcasecmp(subbcmd, "list") == 0) {
    /* Display title for list */
    send_to_char("[   # Clan Title                Clan Owner     ]\r\n", ch);
    /* List all guilds and owners */
    for (ccmd = 0; ccmd < guild_top; ccmd++) {
      sprintf(buf, " %4d %s%-25s %-15s\r\n",
	      GUILDNUM(guild_index[ccmd]), GUILDCOLOUR(ccmd),
	      GUILDNAME(guild_index[ccmd]),
	      GUILDOWNER(guild_index[ccmd]));
      send_to_char(buf, ch);
    }
    sprintf(buf, "\r\n[%d guilds displayed.]\r\n", (ccmd));
    send_to_char(buf, ch);

    return;
  }
  if (strcasecmp(subbcmd, "info") == 0) {
    /* Use index to retrieve information on guild */
    value = atoi(val_arg);
    if (value != 0) {
      for (ccmd = 0; ccmd < guild_top; ccmd++)
	if (GUILDNUM(guild_index[ccmd]) == value) {
	  found = 1;
	  temp2 = 0;

	  /* Find number of members in guild */
	  temp = GUILDPLAYERS(guild_index[ccmd]);

	  temp2 = 0;
	  /* Find number of guild members currently online */
	  for (pt = descriptor_list; pt; pt = pt->next)
	    if (!pt->connected && pt->character &&
		pt->character->player_specials->saved.guildnum == value)
	      temp2++;

	  /* Display info header */
	  sprintf(buf, "[%s", GUILDCOLOUR(ccmd));

	  sprintf(buf, "%s%-25s]\r\n", buf, GUILDNAME(guild_index[ccmd]));
	  send_to_char(buf, ch);

	  send_to_char("\r\n", ch);
	  sprintf(buf, "Owner: %-15s\r\n", GUILDOWNER(guild_index[ccmd]));
	  send_to_char(buf, ch);
	  sprintf(buf, "Clan Gold: %d\r\n", GUILDGOLD(guild_index[ccmd]));
	  send_to_char(buf, ch);
	  sprintf(buf, "Members currently online: %-3d\r\nTotal number of members: %-3d\r\n", temp2, temp);
	  send_to_char(buf, ch);
	  send_to_char("\r\n", ch);
	  send_to_char(GUILDDESC(guild_index[ccmd]), ch);
	  send_to_char("\r\n", ch);
	}
      if (found == 0) {
	send_to_char("Unable to find a guild with that number.\r\n", ch);
	return;
      }
    } else {
      send_to_char("Unable to find a guild with that number.\r\n", ch);
    }
    return;
  }
  if (strcasecmp(subbcmd, "resign") == 0) {
    temp2 = 0;

    /* This handles guild resignations */
    if ((PLAYERGUILDNUM(ch) == 0) && (GUILDRANK(ch) == 0)) {
      send_to_char("But you don't belong to any guild!\r\n", ch);
      return;
    }
    if (strcasecmp(GET_NAME(ch), GUILDOWNER(guild_index[players_guildnum])) == 0) {
      send_to_char("You can't resign from a guild you own!\r\n", ch);
      return;
    }
    PLAYERGUILD(ch) = 0;
    GUILDRANK(ch) = 0;

    /* Update number of members in guild */
    if (GUILDRANK(ch) != GUILD_APPLY)
      GUILDPLAYERS(guild_index[players_guildnum]) = GUILDPLAYERS(guild_index[players_guildnum]) - 1;

    send_to_char("You have resigned from your guild.\r\n", ch);

    return;
  }
  if (strcasecmp(subbcmd, "apply") == 0) {
    /* This handles guild applications */
    if (GUILDRANK(ch) != 0) {
      send_to_char("You must resign from the guild you currently belong to, first.\r\n", ch);
      return;
    }
    value = 0;
    found = 0;
    value = atoi(val_arg);
    if (value >= 0) {
      for (ccmd = 0; ((ccmd < guild_top) && (found != 1)); ccmd++)
	if (GUILDNUM(guild_index[ccmd]) == value) {
	  found = 1;

	  PLAYERGUILD(ch) = GUILDNUM(guild_index[ccmd]);
	  GUILDRANK(ch) = GUILD_APPLY;

	  sprintf(buf, "You have applied to the guild '%s'.\r\n",
		  GUILDNAME(guild_index[ccmd]));
	  send_to_char(buf, ch);
	}
      if (found != 1) {
	send_to_char("Unable to find that guild.\r\n", ch);
	return;
      }
    } else {
      send_to_char("Which guild number?!\r\n", ch);
      return;
    }

    return;
  }
  if (strcasecmp(subbcmd, "enlist") == 0) {
    /* This handles the enlistment of characters who have applied to your
     * guild.
     *
     * Implementors can force an enlistment.  So, when creating a new guild,
     * make sure your owner applies to join it.  Then the implementor must
     * do a 'guild enlist <character>' so that the owner belongs to the guild.
     */

    if (!(GET_LEVEL(ch) == LVL_IMPL)) {
      if ((GUILDRANK(ch) == 0) && (PLAYERGUILDNUM(ch) == 0)) {
	send_to_char("But you don't belong to any guild!\r\n", ch);
	return;
      }
      if (!(GUILDRANK(ch) >= GUILD_SARGEANT)) {
	send_to_char("You have insufficent rank in your guild to do this.\r\n", ch);
	return;
      }
      if ((vict = get_player_vis(ch, val_arg, 0))) {
	if (GUILDRANK(vict) != GUILD_APPLY) {
	  send_to_char("But that person isn't applying to any guild!\r\n", ch);
	  return;
	}
	if (PLAYERGUILD(vict) != PLAYERGUILD(ch)) {
	  send_to_char("But that person isn't applying to your guild!\r\n", ch);
	  return;
	}
	GUILDRANK(vict) = GUILD_SOLDIER;
	sprintf(buf, "You have been enlisted into the ranks of the '%s' guild.\r\n",
		GUILDNAME(guild_index[players_guildnum]));
	send_to_char(buf, vict);

	sprintf(buf, "You have enlisted '%s' into your guild.\r\n",
		GET_NAME(vict));
	send_to_char(buf, ch);

	SET_BIT(PRF_FLAGS(vict), PRF_GUILDTALK);

	/* Update number of guild members */

	GUILDPLAYERS(guild_index[players_guildnum])++;

      } else {
	send_to_char("But there is noone here by that name!\r\n", ch);
	return;
      }
    } else {

      if ((vict = get_player_vis(ch, val_arg, 0))) {
	if (GUILDRANK(vict) != GUILD_APPLY) {
	  send_to_char("But that person isn't applying to any guild!\r\n", ch);
	  return;
	}
	if (strcasecmp(GET_NAME(vict),
		     GUILDOWNER(guild_index[PLAYERGUILDNUM(vict)])) == 0)
	  GUILDRANK(vict) = GUILD_RULER;
	else
	  GUILDRANK(vict) = GUILD_SOLDIER;
	log("Done forcing enlistment");

	sprintf(buf, "You have forced the enlistment of '%s' into the guild '%s'.\r\n",
	   GET_NAME(vict), GUILDNAME(guild_index[PLAYERGUILDNUM(vict)]));
	send_to_char(buf, ch);

	sprintf(buf, "You have been enlisted into the guild '%s'.\r\n",
		GUILDNAME(guild_index[PLAYERGUILDNUM(vict)]));
	send_to_char(buf, vict);

	/* Update number of guild members */
	GUILDPLAYERS(guild_index[PLAYERGUILDNUM(vict)])++;
      } else {
	send_to_char("Unable to find a character of that name.\r\n", ch);
	return;
      }

    }
    return;
  }
  if (strcasecmp(subbcmd, "promote") == 0) {
    /* Clan promotions */
    if (GUILDRANK(ch) < GUILD_CAPTAIN) {
      send_to_char("You have insufficent rank to do that.\r\n", ch);
      return;
    }
    if ((vict = get_player_vis(ch, val_arg, 0))) {
      if (GUILDRANK(vict) > GUILDRANK(ch)) {
	send_to_char("You can't promote someone who ranks higher than you!\r\n", ch);
	return;
      }
      if ((GUILDRANK(vict) < GUILD_SOLDIER) ||
	  (PLAYERGUILD(vict) != PLAYERGUILD(ch))) {
	send_to_char("You can't promote someone who doesn't belong to your guild!\r\n", ch);
	return;
      }
      if (GUILDRANK(vict) == GUILD_RULER) {
	send_to_char("You can't promote this person any higher in rank!\r\n", ch);
	return;
      }
      GUILDRANK(vict)++;
      strcat(buf, "You have been promoted to ");
      if (GUILDRANK(vict) == GUILD_SARGEANT)
	strcat(buf, SARGEANTTITLE(guild_index[players_guildnum]));
      if (GUILDRANK(vict) == GUILD_CAPTAIN)
	strcat(buf, CAPTAINTITLE(guild_index[players_guildnum]));
      if (GUILDRANK(vict) == GUILD_RULER)
	strcat(buf, RULERTITLE(guild_index[players_guildnum]));

      sprintf(buf, "%s in your guild.\r\n", buf);
      send_to_char(buf, vict);

      sprintf(buf, "You have promoted '%s' to the rank of ", GET_NAME(vict));
      if (GUILDRANK(vict) == GUILD_SARGEANT)
	strcat(buf, SARGEANTTITLE(guild_index[players_guildnum]));
      if (GUILDRANK(vict) == GUILD_CAPTAIN)
	strcat(buf, CAPTAINTITLE(guild_index[players_guildnum]));
      if (GUILDRANK(vict) == GUILD_RULER)
	strcat(buf, RULERTITLE(guild_index[players_guildnum]));

      sprintf(buf, "%s in your guild.\r\n", buf);
      send_to_char(buf, ch);
    } else {
      send_to_char("Promote who?!\r\n", ch);
      return;
    }

    return;
  }
  if (strcasecmp(subbcmd, "demote") == 0) {
    /* Clan demotions */
    if (GUILDRANK(ch) == 0) {
      send_to_char("But you don't belong to any guild!\r\n", ch);
      return;
    }
    if (GUILDRANK(ch) < GUILD_CAPTAIN) {
      send_to_char("You have insufficent rank to do that.\r\n", ch);
      return;
    }
    if ((vict = get_player_vis(ch, val_arg, 0))) {
      if (GUILDRANK(vict) > GUILDRANK(ch)) {
	send_to_char("You can't demote someone who ranks higher than you!\r\n", ch);
	return;
      }
      if ((GUILDRANK(vict) < GUILD_SOLDIER) ||
	  (PLAYERGUILD(vict) != PLAYERGUILD(ch))) {
	send_to_char("You can't demote someone who doesn't belong to your guild!\r\n", ch);
	return;
      }
      if (GUILDRANK(vict) == GUILD_SOLDIER) {
	send_to_char("You can't demote this person any lower! Try guild boot <person>!\r\n", ch);
	return;
      }
      if (strcasecmp(GET_NAME(vict), GUILDOWNER(guild_index[players_guildnum])) == 0) {
	send_to_char("You can't demote the guild owner!\r\n", ch);
	return;
      }
      GUILDRANK(vict)--;
      strcat(buf, "You have been demoted to ");
      if (GUILDRANK(vict) == GUILD_SOLDIER)
	strcat(buf, SOLDIERTITLE(guild_index[players_guildnum]));
      if (GUILDRANK(vict) == GUILD_SARGEANT)
	strcat(buf, SARGEANTTITLE(guild_index[players_guildnum]));
      if (GUILDRANK(vict) == GUILD_CAPTAIN)
	strcat(buf, CAPTAINTITLE(guild_index[players_guildnum]));

      sprintf(buf, "%s in your guild.\r\n", buf);
      send_to_char(buf, vict);

      sprintf(buf, "You have demoted '%s' to the rank of ", GET_NAME(vict));
      if (GUILDRANK(vict) == GUILD_SOLDIER)
	strcat(buf, SOLDIERTITLE(guild_index[players_guildnum]));
      if (GUILDRANK(vict) == GUILD_CAPTAIN)
	strcat(buf, CAPTAINTITLE(guild_index[players_guildnum]));
      if (GUILDRANK(vict) == GUILD_SARGEANT)
	strcat(buf, SARGEANTTITLE(guild_index[players_guildnum]));

      sprintf(buf, "%s in your guild.\r\n", buf);
      send_to_char(buf, ch);
    } else {
      send_to_char("Demote who?!\r\n", ch);
      return;
    }

    return;
  }
  if (strcasecmp(subbcmd, "who") == 0) {
    /* List all guild members online, along with their guild rank */
    if ((PLAYERGUILDNUM(ch) == 0) && (GUILDRANK(ch) == 0)) {
      send_to_char("But you don't belong to any guild!\r\n", ch);
      return;
    }
    sprintf(buf, "Members of the guild '%s' currently online:\r\n",
	    GUILDNAME(guild_index[players_guildnum]));
    send_to_char(buf, ch);

    for (pt = descriptor_list; pt; pt = pt->next)
      if (!pt->connected && pt->character &&
	  PLAYERGUILD(pt->character) == PLAYERGUILD(ch)) {
	sprintf(buf, "[%d-%s]",
		GET_LEVEL(pt->character), CLASS_ABBR(pt->character));
	if (GUILDRANK(pt->character) == GUILD_APPLY)
	  sprintf(buf, "%s%-10s", buf, "(Applying)");
	if (GUILDRANK(pt->character) == GUILD_SOLDIER)
	  sprintf(buf, "%s%-10s", buf, SOLDIERTITLE(guild_index[players_guildnum]));
	if (GUILDRANK(pt->character) == GUILD_SARGEANT)
	  sprintf(buf, "%s%-10s", buf, SARGEANTTITLE(guild_index[players_guildnum]));
	if (GUILDRANK(pt->character) == GUILD_CAPTAIN)
	  sprintf(buf, "%s%-10s", buf, CAPTAINTITLE(guild_index[players_guildnum]));
	if (GUILDRANK(pt->character) == GUILD_RULER)
	  sprintf(buf, "%s%-10s", buf, RULERTITLE(guild_index[players_guildnum]));
	sprintf(buf, "%s] %s %s\r\n", buf, GET_NAME(pt->character), GET_TITLE(pt->character));
	send_to_char(buf, ch);
      }
    return;
  }
  if (strcasecmp(subbcmd, "say") == 0) {
    /* Clan say */
    if (GUILDRANK(ch) == 0) {
      send_to_char("But you don't belong to any guild!\r\n", ch);
      return;
    }
    if (GUILDRANK(ch) < GUILD_SOLDIER) {
      send_to_char("You have insufficent rank to do that!\r\n", ch);
      return;
    }
    if (!PRF_FLAGGED(ch, PRF_GUILDTALK)) {
      send_to_char("Try turning your guild talk channel on first, dork!\r\n", ch);
      return;
    }
    for (pt = descriptor_list; pt; pt = pt->next)
      if (!pt->connected && pt->character &&
	  PLAYERGUILD(pt->character) == PLAYERGUILD(ch) &&
      pt->character != ch && PRF_FLAGGED(pt->character, PRF_GUILDTALK)) {
	sprintf(buf, "%s says to the guild, '%s'\r\n", GET_NAME(ch), val_arg);
	send_to_char(buf, pt->character);
      }
    sprintf(buf, "You say to the guild, '%s'\r\n", val_arg);
    send_to_char(buf, ch);

    return;
  }
  if (strcasecmp(subbcmd, "channel") == 0) {
    if (PRF_FLAGGED(ch, PRF_GUILDTALK)) {
      TOGGLE_BIT(PRF_FLAGS(ch), PRF_GUILDTALK);
      send_to_char("Clan talk channel turned off.\r\n", ch);
    } else {
      TOGGLE_BIT(PRF_FLAGS(ch), PRF_GUILDTALK);
      send_to_char("Clan talk channel turned on.\r\n", ch);
    }
    return;
  }
  if (strcasecmp(subbcmd, "deposit") == 0) {
    /* This handles the process of depositing gold into the guild account */
    /* This may only be done in the guild room */

    if ((PLAYERGUILDNUM(ch) == 0) && (GUILDRANK(ch) == 0)) {
      send_to_char("But you don't belong to any guild!\r\n", ch);
      return;
    }
    if (GUILDRANK(ch) < GUILD_SOLDIER) {
      send_to_char("You have insufficent rank to do that!\r\n", ch);
      return;
    }
    if (ch->in_room != GUILDROOM(guild_index[players_guildnum])) {
      send_to_char("You can only do that in guild rooms!\r\n", ch);
      return;
    }
    value = atoi(val_arg);
    if (value != 0) {
      if (ch->points.gold < value) {
	send_to_char("But you don't have that much gold!\r\n", ch);
	return;
      }
      ch->points.gold = ch->points.gold - value;

      GUILDGOLD(guild_index[players_guildnum]) = GUILDGOLD(guild_index[players_guildnum]) + value;

      sprintf(buf, "%d gold deposited into guild account.\r\n", value);
      send_to_char(buf, ch);

    } else {
      send_to_char("Huh?\r\n", ch);
    }

    return;
  }
  if (strcasecmp(subbcmd, "withdraw") == 0) {
    /* This handles withdrawals from the guild account */
    if ((GUILDRANK(ch) == 0) && (PLAYERGUILDNUM(ch) == 0)) {
      send_to_char("But you don't belong to any guild!\r\n", ch);
      return;
    }
    value = atoi(val_arg);
    if (value > -1) {
      if (GUILDRANK(ch) < GUILD_RULER) {
	send_to_char("You have insufficent rank to do that!\r\n", ch);
	return;
      }
      if (ch->in_room != GUILDROOM(guild_index[players_guildnum])) {
	send_to_char("But you can only do that in your guild room!\r\n", ch);
	return;
      }
      if (GUILDGOLD(guild_index[players_guildnum]) < value) {
	send_to_char("But your guild doesn't have that much money!\r\n", ch);
	return;
      } else {
	GUILDGOLD(guild_index[players_guildnum]) = GUILDGOLD(guild_index[players_guildnum]) - value;
	ch->points.gold = ch->points.gold + value;
	sprintf(buf, "%d gold coins withdrawn from the guild account.\r\n", value);
	send_to_char(buf, ch);
      }

    } else {
      send_to_char("Huh?\r\n", ch);
      return;
    }

    return;
  }
  if (strcasecmp(subbcmd, "kick") == 0) {
    if ((GUILDRANK(ch) == 0) && (PLAYERGUILDNUM(ch) == 0)) {
      send_to_char("But you do not belong to any guild!\r\n", ch);
      return;
    }
    if (GUILDRANK(ch) < GUILD_RULER) {
      send_to_char("You have insufficent rank to do that!\r\n", ch);
      return;
    }
    if ((vict = get_player_vis(ch, val_arg, 0))) {
      if (PLAYERGUILD(vict) != PLAYERGUILD(ch)) {
	send_to_char("But that person doesn't belong to your guild!\r\n", ch);
	return;
      }
      if (strcasecmp(GUILDOWNER(guild_index[players_guildnum]), GET_NAME(vict)) == 0) {
	send_to_char("You can't kick the guild owner out!\r\n", ch);
	return;
      }
      GUILDRANK(vict) = 0;
      PLAYERGUILD(vict) = 0;

      sprintf(buf, "You have kicked '%s' from your guild.\r\n", GET_NAME(vict));
      send_to_char(buf, ch);
      sprintf(buf, "You have been kicked from the guild, '%s'.\r\n", GUILDNAME(guild_index[players_guildnum]));
      send_to_char(buf, vict);

      /* Update number of members in guild */
      if (GUILDRANK(vict) != GUILD_APPLY)
	GUILDPLAYERS(guild_index[players_guildnum])--;

    } else {
      send_to_char("Can't find that person.\r\n", ch);
      return;
    }
    return;
  }
  /* If no recognised subcommand given, display help */
  if (strcasecmp(subbcmd, "help") != 0) {
    send_to_char("Clan help:\r\n", ch);
    send_to_char("\r\n", ch);
    if (GUILDRANK(ch) == 0)
      send_to_char("apply, info, list.\r\n", ch);
    if (GUILDRANK(ch) == GUILD_APPLY)
      send_to_char("info, list, resign, who.\r\n", ch);
    if (GUILDRANK(ch) == GUILD_SOLDIER)
      send_to_char("channel, deposit, info, list, resign, say, who.\r\n", ch);
    if (GUILDRANK(ch) == GUILD_SARGEANT)
      send_to_char("channel, deposit, enlist, info, list, resign, say, who.\r\n", ch);
    if (GUILDRANK(ch) == GUILD_CAPTAIN)
      send_to_char("channel, demote, deposit, enlist, info, list, promote, resign, say, who.\r\n", ch);
    if (GUILDRANK(ch) == GUILD_RULER || GET_LEVEL(ch) == LVL_IMPL) {
      send_to_char("channel, demote, deposit, enlist, info, kick, list, promote, resign, say,\r\n", ch);
      send_to_char("withdraw, who.\r\n", ch);
    }
    if (GET_LEVEL(ch) == LVL_IMPL) {
      send_to_char("\r\nImplementor commands:\r\n", ch);
      send_to_char("debug, edit, save.\r\n", ch);
    }
    send_to_char("\r\n", ch);
    send_to_char("For further help type: guild help <command>\r\n", ch);
    return;
  } else {
    if (strcasecmp(val_arg, "debug") == 0) {
      send_to_char("GUILD DEBUG <type>\r\n", ch);
      send_to_char("\r\nDump debug information to debug file.\r\n", ch);
      send_to_char("Type 'guild debug' for more information.\r\n", ch);
      return;
    }
    if (strcasecmp(val_arg, "save") == 0) {
      send_to_char("GUILD SAVE\r\n", ch);
      send_to_char("\r\nSave guild data to guild files.\r\n", ch);
      return;
    }
    if (strcasecmp(val_arg, "edit") == 0) {
      send_to_char("GUILD EDIT <num>\r\n", ch);
      send_to_char("\r\nEnter full screen guild editor for guild <num>.\r\n", ch);
      send_to_char("If <num> doesn't exist, a guild will be created.\r\n", ch);
      return;
    }
    if (strcasecmp(val_arg, "apply") == 0) {
      send_to_char("GUILD APPLY <num>\r\n", ch);
      send_to_char("\r\n", ch);
      send_to_char("Apply to the guild with the number <num>.  To find out the number of a\r\n", ch);
      send_to_char("guild, use the GUILD INFO command.\r\n", ch);
      send_to_char("\r\nRelated topics: resign, enlist.\r\n", ch);
      return;
    }
    if (strcasecmp(val_arg, "kick") == 0) {
      send_to_char("GUILD KICK <player>\r\n", ch);
      send_to_char("\r\nThis kicks <player> from your guild.\r\n", ch);
      return;
    }
    if (strcasecmp(val_arg, "channel") == 0) {
      send_to_char("GUILD CHANNEL\r\n", ch);
      send_to_char("\r\nUsed to toggle the guild talk channel.  Similar to the gossip channel command.\r\n", ch);
      send_to_char("\r\nRelated topics: say.\r\n", ch);
      return;
    }
    if (strcasecmp(val_arg, "demote") == 0) {
      send_to_char("GUILD DEMOTE <player>\r\n", ch);
      send_to_char("\r\nThis command demotes <player> one rank in your guild.\r\n", ch);
      send_to_char("\r\nRelated topics: promote.\r\n", ch);
      return;
    }
    if (strcasecmp(val_arg, "deposit") == 0) {
      send_to_char("GUILD DEPOSIT <amount>\r\n", ch);
      send_to_char("\r\nThis command deposits <amount> of gold into the guild account.\r\n", ch);
      send_to_char("This can only be done in the guild room.\r\n", ch);
      send_to_char("\r\nRelated topics: withdraw.\r\n", ch);
      return;
    }
    if (strcasecmp(val_arg, "enlist") == 0) {
      send_to_char("GUILD ENLIST <player>\r\n", ch);
      send_to_char("\r\nThis command enlists <player> into your guild.\r\n", ch);
      send_to_char("<player> must be applying to your guild first, of course.\r\n", ch);
      if (GET_LEVEL(ch) == LVL_IMPL)
	send_to_char("\r\nAs an implementor you can force the enlisting of <player> into\r\nthe guild they are applying for whether you are in that guild or not.\r\n", ch);

      send_to_char("\r\nRelated topics: apply, resign.\r\n", ch);
      return;
    }
    if (strcasecmp(val_arg, "info") == 0) {
      send_to_char("GUILD INFO <num>\r\n", ch);
      send_to_char("\r\nThis command gives info on the guild <num>.  To get the number of a\r\n", ch);
      send_to_char("guild, use the GUILD LIST command.\r\n", ch);
      send_to_char("\r\nRelated topics: list.\r\n", ch);
      return;
    }
    if (strcasecmp(val_arg, "list") == 0) {
      send_to_char("GUILD LIST\r\n", ch);
      send_to_char("\r\nThis command lists all the existing guilds.\r\n", ch);
      send_to_char("\r\nRelated topics: info.\r\n", ch);
      return;
    }
    if (strcasecmp(val_arg, "promote") == 0) {
      send_to_char("GUILD PROMOTE <player>\r\n", ch);
      send_to_char("\r\nThis command promotes <player> a level in your guild.  Provided\r\n", ch);
      send_to_char("they already belong to your guild.\r\n", ch);
      send_to_char("\r\nRelated topics: demote, enlist.\r\n", ch);
      return;
    }
    if (strcasecmp(val_arg, "resign") == 0) {
      send_to_char("GUILD RESIGN\r\n", ch);
      send_to_char("\r\nThis command resigns you from your guild.\r\n", ch);
      send_to_char("\r\nRelated topics: apply, enlist.\r\n", ch);
      return;
    }
    if (strcasecmp(val_arg, "say") == 0) {
      send_to_char("GUILD SAY <message>\r\n", ch);
      send_to_char("\r\nThis command passes a message onto your fellow guild members.\r\n", ch);
      send_to_char("Much the same as GOSSIP.\r\n", ch);
      send_to_char("Abbreviation: | <message>\r\n", ch);
      send_to_char("\r\nRelated topics: channel.\r\n", ch);
      return;
    }
    if (strcasecmp(val_arg, "withdraw") == 0) {
      send_to_char("GUILD WITHDRAW <amount>\r\n", ch);
      send_to_char("\r\nThis withdraws <amount> of gold from the guild account.\r\n", ch);
      send_to_char("You must be in the guild room to do this.\r\n", ch);
      send_to_char("\r\nRelated topics: deposit.\r\n", ch);
      return;
    }
    if (strcasecmp(val_arg, "who") == 0) {
      send_to_char("GUILD WHO\r\n", ch);
      send_to_char("\r\nThis command is similar to WHO.  It displays all the guild members\r\n", ch);
      send_to_char("currently online.  As well as their rank in the guild.\r\n", ch);
      send_to_char("\r\nRelated topics: list, info.\r\n", ch);
      return;
    }
    send_to_char("Huh?\r\n", ch);
  }

};

ACMD(do_guildsay)
{
  struct descriptor_data *pt;

  /* Clan say */
  if (GUILDRANK(ch) == 0) {
    send_to_char("But you don't belong to any guild!\r\n", ch);
    return;
  }
  if (!PRF_FLAGGED(ch, PRF_GUILDTALK)) {
    send_to_char("Try turning your guild talk channel on first, dork!\r\n", ch);
    return;
  }
  for (pt = descriptor_list; pt; pt = pt->next)
    if (!pt->connected && pt->character &&
	PLAYERGUILD(pt->character) == PLAYERGUILD(ch) &&
      pt->character != ch && PRF_FLAGGED(pt->character, PRF_GUILDTALK)) {
      sprintf(buf, "%s says to the guild, '%s'\r\n", GET_NAME(ch), argument);
      send_to_char(buf, pt->character);
    }
  sprintf(buf, "You say to the guild, '%s'\r\n", argument);
  send_to_char(buf, ch);

};

/*
 * The following procedures attempt to fit into the db.c load procedures
 * to allow the dynamic creation, editing, and removal of guilds whilst
 * in game play.
 *
 * Most of these functions will be almost direct copies of the db.c
 * procedures.
 *
 */

void parse_guild(FILE * fl, int virtual_nr)
{
  static int guild_nr = 0;
  int t[10];
  char line[256];

  sprintf(buf2, "Parsing guild #%d", virtual_nr);
  log(buf2);

  guild_index[guild_nr].number = virtual_nr;
  guild_index[guild_nr].title = fread_string(fl, buf2);
  guild_index[guild_nr].description = fread_string(fl, buf2);
  guild_index[guild_nr].owner = fread_string(fl, buf2);
  guild_index[guild_nr].ranksoldiertitle = fread_string(fl, buf2);
  guild_index[guild_nr].ranksargeanttitle = fread_string(fl, buf2);
  guild_index[guild_nr].rankcaptaintitle = fread_string(fl, buf2);
  guild_index[guild_nr].rankrulertitle = fread_string(fl, buf2);

  if (!get_line(fl, line) || sscanf(line, " %d %d %d %d ", t, t + 1, t + 2, t + 3) != 4) {
    fprintf(stderr, "Format error in guild #%d\n", virtual_nr);
    raise(SIGSEGV);
  }
  guild_index[guild_nr].guildroom = t[0];
  guild_index[guild_nr].guildgold = t[1];
  guild_index[guild_nr].nummembers = t[2];
  guild_index[guild_nr].colour = t[3];
  guild_index[guild_nr].status = GUILD_OLD;

}

void prepare_colour_table(struct char_data *ch)
{
  guild_colour[0].colour_string = "&+W";
  guild_colour[1].colour_string = "&+g";
  guild_colour[2].colour_string = "&+r";
  guild_colour[3].colour_string = "&+B";
  guild_colour[4].colour_string = "&+M";
  guild_colour[5].colour_string = "&+Y";
  guild_colour[6].colour_string = "&+C";
  guild_colour[7].colour_string = "&+w";
}

void update_guild_cross_reference()
{
  int i = 0;

  for (i = 0; i < 9999; i++)
    cross_reference[i] = -1;

  for (i = 0; i < guild_top; i++)
    cross_reference[GUILDNUM(guild_index[i])] = i;

}

void guild_save(int system, struct char_data *ch)
{
  int i = 0, found = 0, last = 0, rec_count = 0, files = 0;
  FILE *fl, *index;
  char *index_file;
  char line[256];
  int reference[9999];

  if (mini_mud)
    index_file = MINDEX_FILE;
  else
    index_file = INDEX_FILE;

  sprintf(buf, "%s/%s", GUILD_PREFIX, index_file);

  if (!(index = fopen(buf, "r"))) {
    log("FUNCTION(guild_save): Unable to open guild index file");
    if (system == NO)
      send_to_char("ClanSave: Unable to open index file (ERROR)\r\n", ch);
    return;
  }
  fscanf(index, "%s\n", buf);
  while (*buf != '$') {
    sprintf(buf2, "%s/%s", GUILD_PREFIX, buf);
    if (!(fl = fopen(buf2, "r"))) {
      log("FUNCTION(guild_save): Unable to open sub file from index file");
      if (system == NO) {
	sprintf(buf, "ClanSave: Unable to open '%s' (ERROR)\r\n", buf2);
	send_to_char(buf, ch);
      }
      return;
    } else {
      rec_count = 0;

      while (found == 0) {
	if (!get_line(fl, line)) {
	  log("FUNCTION(guild_save): Format error encountered");
	  if (system == NO);
	  {
	    sprintf(buf, "ClanSave: Format error after guild #%d (ERROR)\r\n", i);
	    send_to_char(buf, ch);
	  }
	  raise(SIGSEGV);
	}
	if (*line == '$')
	  found = 1;

	if (*line == '#') {
	  last = i;

	  if (sscanf(line, "#%d", &i) != 1) {
	    log("FUNCTION(guild_save): Format error in guild file");
	    if (system == NO) {
	      sprintf(buf, "ClanSave: Format error after guild #%d\r\n", last);
	      send_to_char(buf, ch);
	    }
	    raise(SIGSEGV);
	  }
	  if (i >= 99999)
	    return;

	  reference[rec_count] = i;
	  rec_count++;
	}
      }

      /* Now that we have a list of the guild #'s in this guild file */
      /* We replace them with the updated records */
      fclose(fl);

      sprintf(buf2, "%s/%s", GUILD_PREFIX, buf);
      if (!(fl = fopen(buf2, "w"))) {
	log("FUNCTION(guild_save): Error truncuating guild file for rewrite");
	if (system == NO) {
	  sprintf(buf, "ClanSave: Unable to truncuate '%s' (ERROR)\r\n", buf2);
	  send_to_char(buf, ch);
	}
	return;
      }
      for (i = 0; i < rec_count; i++)
	write_guild(fl, reference[i]);

      /* New guild records go at the end of the first file in the index */
      if (files == 0)
	for (i = 0; i < guild_top; i++)
	  if (GUILDSTATUS(i) == GUILD_NEW) {
	    write_guild(fl, GUILDNUM(guild_index[i]));
	    GUILDSTATUS(i) = GUILD_OLD;
	  }
      fputs("$~\n", fl);
      fclose(fl);
    }

    fscanf(index, "%s\n", buf);
    files++;
  }

  log("Successfully saved guild details");

  if (system == NO)
    send_to_char("Clans saved.\r\n", ch);

}

void write_guild(FILE * fl, int cnum)
{
  int j = 0;

  log("Whew! Made it to the 'write_guild' function");

  update_guild_cross_reference();
  j = cross_reference[cnum];

  if (j == -1)
    return;

  fprintf(stderr, "Writing guild #%d which is '%s'\n", cnum, guild_index[j].title);


  fprintf(fl, "#%d\n", guild_index[j].number);
  fprintf(fl, "%s~\n", guild_index[j].title);
  fprintf(fl, "%s\n~\n", guild_index[j].description);
  fprintf(fl, "%s~\n", guild_index[j].owner);
  fprintf(fl, "%s~\n", guild_index[j].ranksoldiertitle);
  fprintf(fl, "%s~\n", guild_index[j].ranksargeanttitle);
  fprintf(fl, "%s~\n", guild_index[j].rankcaptaintitle);
  fprintf(fl, "%s~\n", guild_index[j].rankrulertitle);
  fprintf(fl, "%d %d %d %d\n", guild_index[j].guildroom,
	  guild_index[j].guildgold, guild_index[j].nummembers,
	  guild_index[j].colour);

}

void display_menu(struct char_data *ch, int cnum)
{

  send_to_char("\x1B[H\x1B[J [Editing guild record]\r\n\r\n", ch);

  sprintf(buf2, "Clan #%4d", cnum);
  sprintf(buf, "%54s\r\n", buf2);
  send_to_char(buf, ch);

  sprintf(buf2, "%s'%s'",
	  GUILDCOLOUR(cross_reference[cnum]),
	  GUILDNAME(guild_index[cross_reference[cnum]]));
  sprintf(buf, "%54s\r\n", buf2);
  send_to_char(buf, ch);

  sprintf(buf2, "Clan Gold : %d", GUILDGOLD(guild_index[cross_reference[cnum]]));
  sprintf(buf, "\r\n%50s\r\n", buf2);
  send_to_char(buf, ch);

  sprintf(buf2, "Clan Room : %d", GUILDROOM(guild_index[cross_reference[cnum]]));
  sprintf(buf, "%50s\r\n", buf2);
  send_to_char(buf, ch);

  sprintf(buf2, "Clan Owner : %s", GUILDOWNER(guild_index[cross_reference[cnum]]));
  sprintf(buf, "%56s\r\n", buf2);
  send_to_char(buf, ch);

  strcat(buf2, "1. Change guild number");
  sprintf(buf, "\r\n%59s\r\n", buf2);
  send_to_char(buf, ch);

  strcat(buf2, "2. Change guild name");
  sprintf(buf, "%57s\r\n", buf2);
  send_to_char(buf, ch);

  strcat(buf2, "3. Change guild owner");
  sprintf(buf, "%58s\r\n", buf2);
  send_to_char(buf, ch);

  strcat(buf2, "4. Change guild description");
  sprintf(buf, "%64s\r\n", buf2);
  send_to_char(buf, ch);

  strcat(buf2, "5. Change guild colour");
  sprintf(buf, "%59s\r\n", buf2);
  send_to_char(buf, ch);

  strcat(buf2, "6. Change guild gold");
  sprintf(buf, "%57s\r\n", buf2);
  send_to_char(buf, ch);

  strcat(buf2, "7. Change guild room");
  sprintf(buf, "%57s\r\n", buf2);
  send_to_char(buf, ch);

  strcat(buf2, "8. Change guild ranks");
  sprintf(buf, "%58s\r\n", buf2);
  send_to_char(buf, ch);

  strcat(buf2, "Q. Quit editing guild");
  sprintf(buf, "%58s\r\n", buf2);
  send_to_char(buf, ch);

  strcat(buf2, "D. Delete guild");
  sprintf(buf, "\r\n%52s\r\n", buf2);
  send_to_char(buf, ch);

}

/* Some generic tests */
int is_guildnum_already_used(int cnum)
{
  update_guild_cross_reference();

  if (cross_reference[cnum] >= 0)
    return YES;


  return NO;

}

int already_being_edited(int cnum)
{
  int i = 0;

  for (i = 0; i < 10; i++)
    if (GUILD_EDIT_GUILD(i) == cnum)
      return YES;

  return NO;
}

void update_cnum(int oldcnum, int newcnum)
{
  struct descriptor_data *pt;

  for (pt = descriptor_list; pt; pt = pt->next)
    if (!pt->connected && pt->character &&
	PLAYERGUILD(pt->character) == oldcnum)
      PLAYERGUILD(pt->character) = newcnum;

  guild_index[cross_reference[oldcnum]].number = newcnum;

  update_guild_cross_reference();
}

void setup_new_guild(struct char_data *ch, int cnum)
{
  int i = 0;
  int node_assigned = 0;

  for (i = 0; i < 10; i++)
    if ((GUILD_EDIT_DESC(i) == NULL) && (node_assigned == NO)) {
      GUILD_EDIT_DESC(i) = ch->desc;
      GUILD_EDIT_GUILD(i) = cnum;
      GUILD_EDIT_MODE(i) = GUILD_EDIT_MAIN_MENU;
      node_assigned = YES;
    }
  if (node_assigned == NO) {
    send_to_char("Sorry, but all editing slots are full.  Try again later.\r\n", ch);
    cleanup_guild_edit(ch->desc, CLEAN_ALL);
    return;
  }
  guild_index[guild_top].number = cnum;
  guild_index[guild_top].title = str_dup("New Clan");
  guild_index[guild_top].owner = str_dup(GET_NAME(ch));
  guild_index[guild_top].description = str_dup("A new guild");
  guild_index[guild_top].guildgold = 0;
  guild_index[guild_top].guildroom = 0;
  guild_index[guild_top].colour = GUILD_NORMAL;
  guild_index[guild_top].ranksoldiertitle = str_dup("Soldier");
  guild_index[guild_top].ranksargeanttitle = str_dup("Sargeant");
  guild_index[guild_top].rankcaptaintitle = str_dup("Captain");
  guild_index[guild_top].rankrulertitle = str_dup("Ruler");
  guild_index[guild_top].status = GUILD_NEW;

  guild_top++;

  update_guild_cross_reference();

  display_menu(ch, cnum);

}

void setup_guild_edit(struct char_data *ch, int cnum)
{
  int i = 0;
  int node_assigned = 0;

  for (i = 0; i < 10; i++)
    if ((GUILD_EDIT_DESC(i) == NULL) && (node_assigned == NO)) {
      GUILD_EDIT_DESC(i) = ch->desc;
      GUILD_EDIT_GUILD(i) = cnum;
      GUILD_EDIT_MODE(i) = GUILD_EDIT_MAIN_MENU;
      node_assigned = YES;
    }
  if (node_assigned == NO) {
    send_to_char("Sorry, but all editing slots are full.  Try again later.\r\n", ch);
    cleanup_guild_edit(ch->desc, CLEAN_ALL);
    return;
  }
  display_menu(ch, cnum);

}

void cleanup_guild_edit(struct descriptor_data *d, int method)
{
  int i = 0;

  for (i = 0; i < 10; i++)
    if (GUILD_EDIT_DESC(i) == d) {
      GUILD_EDIT_DESC(i) = NULL;
      GUILD_EDIT_MODE(i) = 0;
      if (method == CLEAN_ALL)
	GUILDSTATUS(cross_reference[GUILD_EDIT_GUILD(i)]) = GUILD_DELETE;
      GUILD_EDIT_GUILD(i) = 0;
    }
  refresh_guild_index();

  if (d->character) {
    STATE(d) = CON_PLAYING;
    REMOVE_BIT(PLR_FLAGS(d->character), PLR_WRITING);
    act("$n stops editing guilds.", YES, d->character, 0, 0, PROXIMITY);
  }
  if (method == CLEAN_ALL)
    fix_illegal_cnum();

}

void fix_illegal_cnum()
{
  /* Search out and destroy cnum's in player's records which no longer */
  /* exist in the guild_index array */
  int i = 0, valid = 0;
  struct descriptor_data *pt;

  for (pt = descriptor_list; pt; pt = pt->next) {
    for (i = 0; i < guild_top; i++)
      if (guild_index[i].number == PLAYERGUILD(pt->character))
	valid = YES;
    if (valid == NO) {
      sprintf(buf, "(ILLEGAL GUILD NUMBER FOUND) belonging to '%s' - #%d\n",
	      GET_NAME(pt->character), PLAYERGUILD(pt->character));
      fprintf(stderr, "%s", buf);
      PLAYERGUILD(pt->character) = -1;
      GUILDRANK(pt->character) = 0;
    }
    valid = NO;
  }

  log("Done fixing illegal guildnum links");

}

void refresh_guild_index()
{
  int i = 0;
  int j = 0;
  int k = 0;

  for (i = 0; i < guild_top; i++)
    if (GUILDSTATUS(i) == GUILD_DELETE) {

      k = guild_index[i].number;
      guild_index[i].number = -1;
      update_cnum(k, -1);
      if (i != (guild_top - 1))
	for (j = i; j < (guild_top - 1); j++)
	  guild_index[j] = guild_index[j + 1];
      guild_top--;
    }
  update_guild_cross_reference();

}

void parse_guild_edit(struct descriptor_data *d, char *arg)
{
  int i = 0;
  int num = -1;

  for (i = 0; i < 10; i++)
    if (GUILD_EDIT_DESC(i) == d)
      num = i;

  if (num == -1)
    return;

  switch (GUILD_EDIT_MODE(num)) {
  case GUILD_EDIT_MAIN_MENU:
    switch (*arg) {
    case 'd':
    case 'D':
      GUILD_EDIT_MODE(num) = GUILD_EDIT_CONFIRM_DELETE;
      send_to_char("\r\nAre you sure you wish to delete this guild? [Yy/Nn]", d->character);
      break;
    case 'q':
    case 'Q':
      GUILD_EDIT_MODE(num) = GUILD_EDIT_CONFIRM_SAVE;
      send_to_char("\r\nDo you wish to save this guild? [Yy/Nn]", d->character);
      break;
    case '1':
      GUILD_EDIT_MODE(num) = GUILD_EDIT_NUMBER;
      send_to_char("\r\nEnter new guild number: ", d->character);
      break;
    case '2':
      GUILD_EDIT_MODE(num) = GUILD_EDIT_NAME;
      send_to_char("\r\nEnter guild name: \r\n", d->character);
      break;
    case '3':
      GUILD_EDIT_MODE(num) = GUILD_EDIT_OWNER;
      send_to_char("\r\nEnter guild owners name: \r\n", d->character);
      break;
    case '4':
      GUILD_EDIT_MODE(num) = GUILD_EDIT_DESCRIPTION;
      SEND_TO_Q("\x1B[H\x1B[J", d);
      /*
       * Attempting the impossible - Using the inbuilt text editor */
      guild_index[cross_reference[GUILD_EDIT_GUILD(num)]].old_description = NULL;
      guild_index[cross_reference[GUILD_EDIT_GUILD(num)]].old_description =
	  str_dup(GUILDDESC(guild_index[cross_reference[GUILD_EDIT_GUILD(num)]]));
      SEND_TO_Q("Old guild description:\r\n\r\n", d);
      SEND_TO_Q(GUILDDESC(guild_index[cross_reference[GUILD_EDIT_GUILD(num)]]), d);
      SEND_TO_Q("\r\n\r\nEnter new description: \r\n\r\n", d);
      guild_index[cross_reference[GUILD_EDIT_GUILD(num)]].description = NULL;
      d->str = &guild_index[cross_reference[GUILD_EDIT_GUILD(num)]].description;
      d->max_str = 1000;
      break;
    case '5':
      GUILD_EDIT_MODE(num) = GUILD_EDIT_COLOUR;
      display_colour_menu(d);
      break;
    case '6':
      GUILD_EDIT_MODE(num) = GUILD_EDIT_GOLD;
      send_to_char("\r\nEnter new gold value: ", d->character);
      break;
    case '7':
      GUILD_EDIT_MODE(num) = GUILD_EDIT_GUILDROOM;
      send_to_char("\r\nEnter new guild room: ", d->character);
      break;
    case '8':
      GUILD_EDIT_MODE(num) = GUILD_EDIT_RANK_MENU;
      display_rank_menu(d, num);
      break;
    default:
      send_to_char("r\nInvalid command!", d->character);
      display_menu(d->character, GUILD_EDIT_GUILD(num));
      break;
    }
    return;

  case GUILD_EDIT_DESCRIPTION_FINISHED:
    switch (*arg) {
    case 'S':
    case 's':
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
      break;
    default:
      GUILDDESC(guild_index[cross_reference[GUILD_EDIT_GUILD(num)]]) =
	  str_dup(guild_index[cross_reference[GUILD_EDIT_GUILD(num)]].old_description);
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
      break;
    }
    break;

  case GUILD_EDIT_CONFIRM_DELETE:
    switch (*arg) {
    case 'Y':
    case 'y':
      GUILDSTATUS(cross_reference[GUILD_EDIT_GUILD(num)]) = GUILD_DELETE;
      refresh_guild_index();
      cleanup_guild_edit(d, CLEAN_NODE);
      send_to_char("\r\nClan deleted.\r\n", d->character);
      break;
    case 'N':
    case 'n':
      send_to_char("\r\nNOT deleted.\r\n", d->character);
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
      break;
    }
    break;

  case GUILD_EDIT_CONFIRM_SAVE:
    switch (*arg) {
    case 'Y':
    case 'y':
      cleanup_guild_edit(d, CLEAN_NODE);
      send_to_char("\r\nSaved.\r\n", d->character);
      break;
    case 'N':
    case 'n':
      cleanup_guild_edit(d, CLEAN_ALL);
      send_to_char("\r\nNOT saved.\r\n", d->character);
      break;
    default:
      send_to_char("\r\nInvalid command!", d->character);
      display_menu(d->character, GUILD_EDIT_GUILD(num));
      break;
    }
    return;

  case GUILD_EDIT_NUMBER:
    i = 0;
    i = atoi(arg);
    if (!(i <= 0)) {
      if (is_guildnum_already_used(i) == NO) {
	update_cnum(GUILD_EDIT_GUILD(num), i);
	GUILD_EDIT_GUILD(num) = i;
	GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
	display_menu(d->character, GUILD_EDIT_GUILD(num));
      } else {
	send_to_char("\r\nThat number already in use!\r\n", d->character);
	GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
	display_menu(d->character, GUILD_EDIT_GUILD(num));
      }
    } else {
      send_to_char("\r\nInvalid number!\r\n", d->character);
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
    }
    break;

  case GUILD_EDIT_NAME:
    GUILDNAME(guild_index[cross_reference[GUILD_EDIT_GUILD(num)]]) = str_dup(arg);
    GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
    display_menu(d->character, GUILD_EDIT_GUILD(num));
    break;

  case GUILD_EDIT_OWNER:
    /* No error checking here */
    GUILDOWNER(guild_index[cross_reference[GUILD_EDIT_GUILD(num)]]) = str_dup(arg);
    GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
    display_menu(d->character, GUILD_EDIT_GUILD(num));
    break;

  case GUILD_EDIT_GOLD:
    i = 0;
    i = atoi(arg);
    if (!(i <= 0)) {
      GUILDGOLD(guild_index[cross_reference[GUILD_EDIT_GUILD(num)]]) = i;
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
    } else {
      send_to_char("\r\nInvalid number!\r\n", d->character);
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
    }
    break;

  case GUILD_EDIT_GUILDROOM:
    /* No error checking */
    i = 0;
    i = atoi(arg);
    if (!(i <= 0)) {
      GUILDROOM(guild_index[cross_reference[GUILD_EDIT_GUILD(num)]]) = i;
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
    } else {
      send_to_char("\r\nInvalid number!\r\n", d->character);
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
    }
    break;

  case GUILD_EDIT_COLOUR:
    switch (*arg) {
    case 'w':
    case 'W':
      GUILDCOLOR(cross_reference[GUILD_EDIT_GUILD(num)]) = GUILD_WHITE;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      break;
    case 'n':
    case 'N':
      GUILDCOLOR(cross_reference[GUILD_EDIT_GUILD(num)]) = GUILD_NORMAL;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      break;
    case 'r':
    case 'R':
      GUILDCOLOR(cross_reference[GUILD_EDIT_GUILD(num)]) = GUILD_RED;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      break;
    case 'g':
    case 'G':
      GUILDCOLOR(cross_reference[GUILD_EDIT_GUILD(num)]) = GUILD_GREEN;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      break;
    case 'b':
    case 'B':
      GUILDCOLOR(cross_reference[GUILD_EDIT_GUILD(num)]) = GUILD_BLUE;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      break;
    case 'y':
    case 'Y':
      GUILDCOLOR(cross_reference[GUILD_EDIT_GUILD(num)]) = GUILD_YELLOW;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      break;
    case 'm':
    case 'M':
      GUILDCOLOR(cross_reference[GUILD_EDIT_GUILD(num)]) = GUILD_MAGENTA;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      break;
    case 'c':
    case 'C':
      GUILDCOLOR(cross_reference[GUILD_EDIT_GUILD(num)]) = GUILD_CYAN;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      break;
    default:
      send_to_char("Invalid colour!\r\n", d->character);
      display_menu(d->character, GUILD_EDIT_GUILD(num));
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      break;
    }
    display_menu(d->character, GUILD_EDIT_GUILD(num));
    GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
    break;

  case GUILD_EDIT_SOLDIER:
    SOLDIERTITLE(guild_index[cross_reference[GUILD_EDIT_GUILD(num)]]) =
	str_dup(arg);
    GUILD_EDIT_MODE(num) = GUILD_EDIT_RANK_MENU;
    display_rank_menu(d, num);
    break;

  case GUILD_EDIT_SARGEANT:
    SARGEANTTITLE(guild_index[cross_reference[GUILD_EDIT_GUILD(num)]]) =
	str_dup(arg);
    GUILD_EDIT_MODE(num) = GUILD_EDIT_RANK_MENU;
    display_rank_menu(d, num);
    break;

  case GUILD_EDIT_CAPTAIN:
    CAPTAINTITLE(guild_index[cross_reference[GUILD_EDIT_GUILD(num)]]) =
	str_dup(arg);
    GUILD_EDIT_MODE(num) = GUILD_EDIT_RANK_MENU;
    display_rank_menu(d, num);
    break;

  case GUILD_EDIT_RULER:
    RULERTITLE(guild_index[cross_reference[GUILD_EDIT_GUILD(num)]]) =
	str_dup(arg);
    GUILD_EDIT_MODE(num) = GUILD_EDIT_RANK_MENU;
    display_rank_menu(d, num);
    break;

  case GUILD_EDIT_RANK_MENU:
    switch (*arg) {
    case '1':
      GUILD_EDIT_MODE(num) = GUILD_EDIT_SOLDIER;
      send_to_char("\r\nEnter new soldier rank: \r\n", d->character);
      break;
    case '2':
      GUILD_EDIT_MODE(num) = GUILD_EDIT_SARGEANT;
      send_to_char("\r\nEnter new sargeant rank: \r\n", d->character);
      break;
    case '3':
      GUILD_EDIT_MODE(num) = GUILD_EDIT_CAPTAIN;
      send_to_char("\r\nEnter new captain rank: \r\n", d->character);
      break;
    case '4':
      GUILD_EDIT_MODE(num) = GUILD_EDIT_RULER;
      send_to_char("\r\nEnter new ruler rank: \r\n", d->character);
      break;
    case 'q':
    case 'Q':
      GUILD_EDIT_MODE(num) = GUILD_EDIT_MAIN_MENU;
      display_menu(d->character, GUILD_EDIT_GUILD(num));
      break;
    default:
      send_to_char("\r\nInvalid command!\r\n", d->character);
      display_rank_menu(d, num);
      break;
    }
    break;

  }

}

void display_colour_menu(struct descriptor_data *d)
{
  struct char_data *ch;

  ch = d->character;

  send_to_char("\r\n\x1B[H\x1B[J", ch);
  send_to_char("&+W(W)hite&n\r\n&+R(R)ed&n\r\n&+g(G)reen&n\r\n&+B(B)lue&n\r\n&+Y(Y)ellow&n\r\n&+M(M)agenta&n\r\n&+C(C)yan&n\r\n&+w(N)ormal&n\r\n", ch);
}

void display_rank_menu(struct descriptor_data *d, int num)
{
  struct char_data *ch;

  ch = d->character;

  sprintf(buf, "\x1B[H\x1B[J\r\nSoldier Rank : %s\r\n",
      SOLDIERTITLE(guild_index[cross_reference[GUILD_EDIT_GUILD(num)]]));
  sprintf(buf, "%sSargeant Rank: %s\r\n", buf,
     SARGEANTTITLE(guild_index[cross_reference[GUILD_EDIT_GUILD(num)]]));
  sprintf(buf, "%sCaptain Rank : %s\r\n", buf,
      CAPTAINTITLE(guild_index[cross_reference[GUILD_EDIT_GUILD(num)]]));
  sprintf(buf, "%sRuler Rank: %s\r\n", buf,
	RULERTITLE(guild_index[cross_reference[GUILD_EDIT_GUILD(num)]]));
  sprintf(buf, "%s\r\n1. Edit Soldier Rank\r\n", buf);
  sprintf(buf, "%s2. Edit Sargeant Rank\r\n", buf);
  sprintf(buf, "%s3. Edit Captain Rank\r\n", buf);
  sprintf(buf, "%s4. Edit Ruler Rank\r\n", buf);
  sprintf(buf, "%s\r\nQ. Return to main menu\r\n", buf);
  send_to_char(buf, ch);


}

/* Debug stuff goes here */
int debug_guild(char *what, struct char_data *ch)
{
  int i = 0, count = 0, debug_type = 0;
  time_t ct;
  char *tmstr;
  FILE *fl = NULL;

  /* Find type of debug request */
  if (strcasecmp(what, "cref") == 0)
    debug_type = GUILD_DEBUG_CREF;
  if (strcasecmp(what, "node") == 0)
    debug_type = GUILD_DEBUG_NODE;
  if (strcasecmp(what, "var") == 0)
    debug_type = GUILD_DEBUG_VAR;
  if (strcasecmp(what, "all") == 0)
    debug_type = GUILD_DEBUG_ALL;

  if (debug_type != GUILD_DEBUG_UNKNOWN) {
    /* Open debug file */
    sprintf(buf, "%s/DEBUG", GUILD_PREFIX);
    if (!(fl = fopen(buf, "a"))) {
      fprintf(stderr, "Error opening guild debug file '%s'!\n", buf);
      return NO;
    }
    /* DateTime stamp entry */
    ct = time(0);
    tmstr = asctime(localtime(&ct));
    *(tmstr + strlen(tmstr) - 1) = '\0';
    fprintf(fl, "%-19.19s :: Debug requested by user\n", tmstr);
    fprintf(fl, "\nDebug requested by: %s\n", GET_NAME(ch));
  }
  if ((debug_type == GUILD_DEBUG_VAR) || (debug_type == GUILD_DEBUG_ALL)) {
    /* Dump relevant global variables */
    fprintf(fl, "Debug request type: -GLOBAL VARIABLES-\n\n");

    fprintf(fl, "-GUILD STATUS-\n\n");
    fprintf(fl, "Clan # | Status\n");
    fprintf(fl, "-------|-------\n");

    for (i = 0; i < guild_top; i++)
      fprintf(fl, "  %-4d | %d    \n", guild_index[i].number, GUILDSTATUS(i));


    fprintf(fl, "\n\n");
  }
  if ((debug_type == GUILD_DEBUG_CREF) || (debug_type == GUILD_DEBUG_ALL)) {
    i = 0;
    /* Dump cross reference variable to debug file */
    fprintf(fl, "Debug request type: -CROSS REFERENCE-\n");

    fprintf(fl, "\nClan # | Index\n");
    fprintf(fl, "-------|------\n");
    for (i = 0; i < 9999; i++)
      if (cross_reference[i] != -1)
	fprintf(fl, "%-6d | %d\n", i, cross_reference[i]);
      else
	count++;
    fprintf(fl, "\n%d valid records, %d empty records.\n\n",
	    (9999 - count), count);
  }
  if ((debug_type == GUILD_DEBUG_NODE) || (debug_type == GUILD_DEBUG_ALL)) {
    count = 0;
    /* Dump guild editing nodes to debug file */
    fprintf(fl, "Debug request type: -GUILD EDIT NODES-\n");

    fprintf(fl, "\nNode # | Clan # | Mode   | Owner\n");
    fprintf(fl, "-------|--------|--------|-------------------\n");
    for (i = 0; i < 10; i++)
      if (GUILD_EDIT_DESC(i) != NULL) {
	sprintf(buf2, "    %-2d |   %-4d | ", i, GUILD_EDIT_GUILD(i));
	switch (GUILD_EDIT_MODE(i)) {
	case GUILD_EDIT_DESCRIPTION_FINISHED:
	  fprintf(fl, "%s-Desc- | %s\n", buf2,
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	case GUILD_EDIT_CONFIRM_SAVE:
	  fprintf(fl, "%sCon-Sv | %s\n", buf2,
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	case GUILD_EDIT_MAIN_MENU:
	  fprintf(fl, "%s-Main- | %s\n", buf2,
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	case GUILD_EDIT_NAME:
	  fprintf(fl, "%sClnNam | %s\n", buf2,
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	case GUILD_EDIT_NUMBER:
	  fprintf(fl, "%sClnNum | %s\n", buf2,
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	case GUILD_EDIT_GOLD:
	  fprintf(fl, "%sClnGld | %s\n", buf2,
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	case GUILD_EDIT_OWNER:
	  fprintf(fl, "%sClnOwn | %s\n", buf2,
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	case GUILD_EDIT_DESCRIPTION:
	  fprintf(fl, "%sClnDsc | %s\n", buf2,
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	case GUILD_EDIT_GUILDROOM:
	  fprintf(fl, "%sCln-Rm | %s\n", buf2,
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	case GUILD_EDIT_COLOUR:
	  fprintf(fl, "%sClnClr | %s\n", buf2,
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	case GUILD_EDIT_RANK_MENU:
	  fprintf(fl, "%sRnkMnu | %s\n", buf2,
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	case GUILD_EDIT_SELECT_COLOUR:
	  fprintf(fl, "%sSltClr | %s\n", buf2,
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	case GUILD_EDIT_SOLDIER:
	  fprintf(fl, "%sRnkSld | %s\n", buf2,
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	case GUILD_EDIT_SARGEANT:
	  fprintf(fl, "%sRnkSgt | %s\n", buf2,
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	case GUILD_EDIT_CAPTAIN:
	  fprintf(fl, "%sRnkCpt | %s\n", buf2,
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	case GUILD_EDIT_RULER:
	  fprintf(fl, "%sRnkRlr | %s\n", buf2,
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	default:
	  fprintf(fl, "%s%d | %s\n", buf2, GUILD_EDIT_MODE(i),
		  GET_NAME(GUILD_EDIT_DESC(i)->character));
	  break;
	}
      } else
	count++;

    fprintf(fl, "\n%d nodes used, %d free nodes.\n\n", (10 - count), count);
  }
  if (debug_type == GUILD_DEBUG_UNKNOWN) {
    send_to_char("guild debug <type>\r\n", ch);
    send_to_char("\r\nValid types:\r\n", ch);
    send_to_char("            cref          - cross reference variable\r\n", ch);
    send_to_char("            node          - guild edit nodes\r\n", ch);
    send_to_char("            var           - global variables\r\n", ch);
    send_to_char("            all           - all debug reports\r\n", ch);
    return NO;
  }
  fclose(fl);
  return YES;
}


/* (FIDO) End */
