/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"
#include <signal.h>

#include "structs.h"

#include "utils.h"
#include "comm.h"
#include "db.h"
#include "handler.h"
#include "interpreter.h"
#include "spells.h"
#include "events.h"
#include "quest.h"
#include "dg_scripts.h"
#include "map.h"

/* external vars */
extern int top_of_world;
extern struct room_data *world;
extern struct obj_data *object_list;
extern struct char_data *character_list;
extern struct index_data *mob_index;
extern struct index_data *obj_index;
extern struct descriptor_data *descriptor_list;
extern char *MENU;
extern struct aq_data *aquest_table;
extern int rev_dir[];
extern int dirs[];
extern char *kingdom_list[];
extern char *coin_names[];

/* external functions */
void free_char(struct char_data *ch);
void stop_fighting(struct char_data *ch);
void remove_follower(struct char_data *ch);
void clearMemory(struct char_data *ch);
void dismount_char(struct char_data *ch);
void event_cancel(struct event *event);
void set_saving_throws(struct char_data *ch);
struct char_data *find_char(int n);
void char_from_object(struct char_data *ch, struct obj_data *obj);
int CantEnterCoord(struct char_data *ch, int dir);
void maplook(struct char_data *ch, sh_int size, int dif_room, int type);
int HeadingForWaterCoord(struct char_data *ch, int dir);
void FallingChar(struct char_data *ch, const int kill_char);
void FallingObj(struct obj_data *obj);
void convert_prototype_to_random_item(struct char_data *ch, struct obj_data *obj, int room);

char *
 fname(char *namelist)
{
  static char holder[30];
  register char *point;

  if (!namelist || !*namelist)
    return ("BUG!!");
  for (point = holder; isalpha(*namelist); namelist++, point++)
    *point = *namelist;

  *point = '\0';

  return (holder);
}


int isname(char *str, char *namelist)
{
  register char *curname, *curstr;

  curname = namelist;
  for (;;) {
    for (curstr = str;; curstr++, curname++) {
      if (!*curstr && !isalpha(*curname))
	return (1);

      if (!*curname)
	return (0);

      if (!*curstr || *curname == ' ')
	break;

      if (LOWER(*curstr) != LOWER(*curname))
	break;
    }

    /* skip to next name */

    for (; isalpha(*curname); curname++);
    if (!*curname)
      return (0);
    curname++;			/* first char of new name */
  }
}

void obj_affect_modify(struct obj_data *obj, byte loc, sbyte mod, long bitv,
		long bitv2, long bitv3, long bitv4, long prot, long dete,
		       long cont, bool add)
{
  if (add) {
    SET_BIT(obj->obj_flags.bitvector, bitv);
    SET_BIT(obj->obj_flags.bitvector2, bitv2);
    SET_BIT(obj->obj_flags.bitvector3, bitv3);
    SET_BIT(obj->obj_flags.bitvector4, bitv4);
    SET_BIT(obj->obj_flags.protection, prot);
    SET_BIT(obj->obj_flags.detecting, dete);
    SET_BIT(obj->obj_flags.controlling, cont);
  } else {
    REMOVE_BIT(obj->obj_flags.bitvector, bitv);
    REMOVE_BIT(obj->obj_flags.bitvector2, bitv2);
    REMOVE_BIT(obj->obj_flags.bitvector3, bitv3);
    REMOVE_BIT(obj->obj_flags.bitvector4, bitv4);
    REMOVE_BIT(obj->obj_flags.protection, prot);
    REMOVE_BIT(obj->obj_flags.detecting, dete);
    REMOVE_BIT(obj->obj_flags.controlling, cont);
  }
}

void room_affect_modify(int room, long bitv, bool add)
{
}

void affect_modify(struct char_data *ch, byte loc, sbyte mod, long bitv,
		long bitv2, long bitv3, long bitv4, long prot, long dete,
		   long cont, bool add)
{
  int maxabil;

  if (add) {
    SET_BIT(AFF_FLAGS(ch), bitv);
    SET_BIT(AFF2_FLAGS(ch), bitv2);
    SET_BIT(AFF3_FLAGS(ch), bitv3);
    SET_BIT(AFF4_FLAGS(ch), bitv4);
    SET_BIT(PROT_FLAGS(ch), prot);
    SET_BIT(DETE_FLAGS(ch), dete);
    SET_BIT(CONT_FLAGS(ch), cont);
  } else {
    REMOVE_BIT(AFF_FLAGS(ch), bitv);
    REMOVE_BIT(AFF2_FLAGS(ch), bitv2);
    REMOVE_BIT(AFF3_FLAGS(ch), bitv3);
    REMOVE_BIT(AFF4_FLAGS(ch), bitv4);
    REMOVE_BIT(PROT_FLAGS(ch), prot);
    REMOVE_BIT(DETE_FLAGS(ch), dete);
    REMOVE_BIT(CONT_FLAGS(ch), cont);
    mod = -mod;
  }


  maxabil = (IS_NPC(ch) ? 25 : 25);

  switch (loc) {
  case APPLY_NONE:
    break;

  case APPLY_STR:
    GET_STR(ch) += mod;
    break;
  case APPLY_DEX:
    GET_DEX(ch) += mod;
    break;
  case APPLY_INT:
    GET_INT(ch) += mod;
    break;
  case APPLY_WIS:
    GET_WIS(ch) += mod;
    break;
  case APPLY_CON:
    GET_CON(ch) += mod;
    break;
  case APPLY_CHA:
    GET_CHA(ch) += mod;
    break;

  case APPLY_MAX_STR:
    if (mod < 26)
      GET_STR(ch) = mod;
    else {
      GET_STR(ch) = 18;
      GET_ADD(ch) = 100;
    }
    break;

  case APPLY_CLASS:
    /* ??? GET_CLASS(ch) += mod; */
    break;

  case APPLY_LEVEL:
    /* ??? GET_LEVEL(ch) += mod; */
    break;

  case APPLY_AGE:
    ch->player.time.birth -= (mod * SECS_PER_MUD_YEAR);
    break;

  case APPLY_CHAR_WEIGHT:
    GET_WEIGHT(ch) += mod;
    break;

  case APPLY_CHAR_HEIGHT:
    GET_HEIGHT(ch) += mod;
    break;

  case APPLY_HIT:
    GET_MAX_HIT(ch) += mod;
    break;

  case APPLY_MAGIC_RES:
    GET_MAGIC_RESISTANCE(ch) += mod;
    break;

  case APPLY_EXP:
    break;

  case APPLY_AC:
    GET_AC(ch) += mod;
    break;

  case APPLY_HITROLL:
    GET_HITROLL(ch) += mod;
    break;

  case APPLY_DAMROLL:
    GET_DAMROLL(ch) += mod;
    break;

  case APPLY_SAVING_PARA:
    GET_SAVE(ch, SAVING_PARA) += mod;
    break;

  case APPLY_SAVING_ROD:
    GET_SAVE(ch, SAVING_ROD) += mod;
    break;

  case APPLY_SAVING_PETRI:
    GET_SAVE(ch, SAVING_PETRI) += mod;
    break;

  case APPLY_SAVING_BREATH:
    GET_SAVE(ch, SAVING_BREATH) += mod;
    break;

  case APPLY_SAVING_SPELL:
    GET_SAVE(ch, SAVING_SPELL) += mod;
    break;

  case APPLY_SAVING_ALL:
    GET_SAVE(ch, SAVING_PARA) += mod;
    GET_SAVE(ch, SAVING_ROD) += mod;
    GET_SAVE(ch, SAVING_PETRI) += mod;
    GET_SAVE(ch, SAVING_BREATH) += mod;
    GET_SAVE(ch, SAVING_SPELL) += mod;
    break;

  default:
    vlogf("SYSERR: Unknown apply adjust attempt for %s(handler.c, affect_modify).", GET_NAME(ch));
    break;

  }				/* switch */
}



/* This updates a character by subtracting everything he is affected by */
/* restoring original abilities, and then affecting all again           */
void affect_total(struct char_data *ch)
{
  struct affected_type *af;
  int i, j, temp = 0;

  for (i = 0; i < NUM_WEARS; i++) {
    if (GET_EQ(ch, i) && !UNAFFECTABLE_WEAR_LOCATION(i))
      for (j = 0; j < MAX_OBJ_AFFECT; j++) {
	affect_modify(ch, GET_EQ(ch, i)->affected[j].location,
		      GET_EQ(ch, i)->affected[j].modifier,
		      GET_EQ(ch, i)->obj_flags.bitvector,
		      GET_EQ(ch, i)->obj_flags.bitvector2,
		      GET_EQ(ch, i)->obj_flags.bitvector3,
		      GET_EQ(ch, i)->obj_flags.bitvector4,
		      GET_EQ(ch, i)->obj_flags.protection,
		      GET_EQ(ch, i)->obj_flags.detecting,
		      GET_EQ(ch, i)->obj_flags.controlling,
		      FALSE);
      }
  }


  for (af = ch->affected; af; af = af->next) {
    affect_modify(ch, af->location, af->modifier, af->bitvector,
		  af->bitvector2, af->bitvector3, af->bitvector4,
		  af->protection, af->detecting, af->controlling, FALSE);
  }

  ch->aff_abils = ch->real_abils;
  set_saving_throws(ch);


  /* ok armor first */
  if (GET_EQ(ch, WEAR_BODY)) {
    if ((GET_OBJ_TYPE(GET_EQ(ch, WEAR_BODY)) == ITEM_ARMOR) &&
	IS_SET(GET_OBJ_EXTRA(GET_EQ(ch, WEAR_BODY)), ITEM_WHOLE_BODY)) {
      GET_AC(ch) = GET_OBJ_VAL(GET_EQ(ch, WEAR_BODY), 0);
      temp = 0;
    } else if ((GET_OBJ_TYPE(GET_EQ(ch, WEAR_BODY)) == ITEM_ARMOR) &&
	!IS_SET(GET_OBJ_EXTRA(GET_EQ(ch, WEAR_BODY)), ITEM_WHOLE_BODY)) {
      temp = temp - 50 + GET_OBJ_VAL(GET_EQ(ch, WEAR_BODY), 0) * 5;
    }
  }
  if (GET_EQ(ch, WEAR_LEGS)) {
    if (GET_OBJ_TYPE(GET_EQ(ch, WEAR_LEGS)) == ITEM_ARMOR) {
      temp = temp - 30 + GET_OBJ_VAL(GET_EQ(ch, WEAR_LEGS), 0) * 3;
    }
  }
  if (GET_EQ(ch, WEAR_ARMS)) {
    if (GET_OBJ_TYPE(GET_EQ(ch, WEAR_ARMS)) == ITEM_ARMOR) {
      temp = temp - 20 + GET_OBJ_VAL(GET_EQ(ch, WEAR_ARMS), 0) * 2;
    }
  }
  if (temp > 0)
    GET_AC(ch) = (int) (temp / 10);

  for (i = 0; i < NUM_WEARS; i++) {
    if (GET_EQ(ch, i))
      for (j = 0; j < MAX_OBJ_AFFECT; j++) {
	affect_modify(ch, GET_EQ(ch, i)->affected[j].location,
		      GET_EQ(ch, i)->affected[j].modifier,
		      GET_EQ(ch, i)->obj_flags.bitvector,
		      GET_EQ(ch, i)->obj_flags.bitvector2,
		      GET_EQ(ch, i)->obj_flags.bitvector3,
		      GET_EQ(ch, i)->obj_flags.bitvector4,
		      GET_EQ(ch, i)->obj_flags.protection,
		      GET_EQ(ch, i)->obj_flags.detecting,
		      GET_EQ(ch, i)->obj_flags.controlling,
		      TRUE);
      }
  }

  /* end armor check */

  for (af = ch->affected; af; af = af->next) {
    affect_modify(ch, af->location, af->modifier, af->bitvector,
		  af->bitvector2, af->bitvector3, af->bitvector4,
		  af->protection, af->detecting, af->controlling, TRUE);
  }

  for (i = 0; i < 5; i++) {
    if (GET_LEVEL(ch) > LVL_IMMORT)
      GET_SAVE(ch, i) = MAX(2, GET_SAVE(ch, i));
    else
      GET_SAVE(ch, i) = MAX(3, GET_SAVE(ch, i));
  }

  /* Make certain values are between 0..25, not < 0 and not > 25! */

  i = (IS_NPC(ch) ? 25 : 25);

  GET_DEX(ch) = MAX(0, MIN(GET_DEX(ch), i));
  GET_INT(ch) = MAX(0, MIN(GET_INT(ch), i));
  GET_WIS(ch) = MAX(0, MIN(GET_WIS(ch), i));
  GET_CON(ch) = MAX(0, MIN(GET_CON(ch), i));
  GET_STR(ch) = MAX(0, GET_STR(ch));

/*  if (IS_NPC(ch)) { */
  GET_STR(ch) = MIN(GET_STR(ch), i);
/*  } else {
   if (GET_STR(ch) > 18) {
   i = GET_ADD(ch) + ((GET_STR(ch) - 18) * 10);
   GET_ADD(ch) = MIN(i, 100);
   GET_STR(ch) = 18;
   }
   } */

  check_regen_rates(ch);	/* update regen rates (for age) */
}



/* Insert an affect_type in a char_data structure
   Automatically sets apropriate bits and apply's */
void affect_to_char(struct char_data *ch, struct affected_type *af)
{
  struct affected_type *affected_alloc, *hjp, *next;

  CREATE(affected_alloc, struct affected_type, 1);

  *affected_alloc = *af;

  for (hjp = ch->affected; hjp; hjp = next) {
    next = hjp->next;
    if (hjp->type == af->type)
      affect_remove(ch, hjp);
  }

  affected_alloc->next = ch->affected;
  ch->affected = affected_alloc;

  affect_modify(ch, af->location, af->modifier, af->bitvector,
		af->bitvector2, af->bitvector3, af->bitvector4,
		af->protection, af->detecting, af->controlling, TRUE);
  affect_total(ch);
}

/*
 * Remove an affected_type structure from a obj (called when duration
 * reaches zero). Pointer *af must never be NIL!  Frees mem and calls
 * affect_location_apply
 */
void obj_affect_remove(struct obj_data *obj, struct affected_type *af)
{
  struct affected_type *temp;

  assert(obj->spell_affects);

  obj_affect_modify(obj, af->location, af->modifier, af->bitvector,
		    af->bitvector2, af->bitvector3, af->bitvector4,
		  af->protection, af->detecting, af->controlling, FALSE);
  REMOVE_FROM_LIST(af, obj->spell_affects, next);
  FREE(af);
}

/*
 * Remove an affected_type structure from a room (called when duration
 * reaches zero). Pointer *af must never be NIL!  Frees mem and calls
 * affect_location_apply
 */
void room_affect_remove(int room, struct room_affected_type *af)
{
  struct room_affected_type *temp;
  assert(world[room].spell_affects);

  room_affect_modify(room, af->bitvector, FALSE);
  REMOVE_FROM_LIST(af, world[room].spell_affects, next);
  FREE(af->loc);
  FREE(af);
}

void affect_to_obj(struct obj_data *obj, struct affected_type *af)
{
  struct affected_type *affected_alloc, *hjp, *next;

  CREATE(affected_alloc, struct affected_type, 1);

  *affected_alloc = *af;

  for (hjp = obj->spell_affects; hjp; hjp = next) {
    next = hjp->next;
    if (hjp->type == af->type)
      obj_affect_remove(obj, hjp);
  }

  affected_alloc->next = obj->spell_affects;
  obj->spell_affects = affected_alloc;

  obj_affect_modify(obj, af->location, af->modifier, af->bitvector,
		    af->bitvector2, af->bitvector3, af->bitvector4,
		    af->protection, af->detecting, af->controlling, TRUE);
}

void affect_to_room(int room, struct room_affected_type *af)
{
  struct room_affected_type *affected_alloc;

  CREATE(affected_alloc, struct room_affected_type, 1);

  *affected_alloc = *af;

  affected_alloc->loc = NULL;
  CREATE(affected_alloc->loc, struct location_data, 1);
  *(affected_alloc->loc) = *(af->loc);

  affected_alloc->next = world[room].spell_affects;
  world[room].spell_affects = affected_alloc;

  room_affect_modify(room, af->bitvector, TRUE);
  sprintf(buf, " affect to room %d", world[room].spell_affects->loc->real_room);
  log(buf);
}


/*
 * Remove an affected_type structure from a char (called when duration
 * reaches zero). Pointer *af must never be NIL!  Frees mem and calls
 * affect_location_apply
 */
void affect_remove(struct char_data *ch, struct affected_type *af)
{
  struct affected_type *temp;

  assert(ch->affected);

  affect_modify(ch, af->location, af->modifier, af->bitvector,
		af->bitvector2, af->bitvector3, af->bitvector4,
		af->protection, af->detecting, af->controlling, FALSE);
  REMOVE_FROM_LIST(af, ch->affected, next);
  FREE(af);
  affect_total(ch);
}

/* Call affect_remove with every spell of spelltype "skill" */
void affect_from_char(struct char_data *ch, int type)
{
  struct affected_type *hjp, *next;

  for (hjp = ch->affected; hjp; hjp = next) {
    next = hjp->next;
    if (hjp->type == type)
      affect_remove(ch, hjp);
  }
}



/*
 * Return if a char is affected by a spell (SPELL_XXX), NULL indicates
 * not affected
 */
bool
affected_by_spell(struct char_data *ch, int type)
{
  struct affected_type *hjp;

  for (hjp = ch->affected; hjp; hjp = hjp->next)
    if (hjp->type == type)
      return TRUE;

  return FALSE;
}



void affect_join(struct char_data *ch, struct affected_type *af,
		 bool add_dur, bool avg_dur, bool add_mod, bool avg_mod)
{
  struct affected_type *hjp;
  bool found = FALSE;

  for (hjp = ch->affected; !found && hjp; hjp = hjp->next) {

    if ((hjp->type == af->type) && (hjp->location == af->location)) {
      if (add_dur)
	af->duration += hjp->duration;
      if (avg_dur)
	af->duration >>= 1;

      if (add_mod)
	af->modifier += hjp->modifier;
      if (avg_mod)
	af->modifier >>= 1;

      affect_remove(ch, hjp);
      affect_to_char(ch, af);
      found = TRUE;
    }
  }
  if (!found)
    affect_to_char(ch, af);
}


/* move a player out of a room */
void char_from_room(struct char_data *ch)
{
  struct char_data *temp;

  if (ch == NULL || ch->in_room == NOWHERE) {
    log("SYSERR: NULL or NOWHERE in handler.c, char_from_room");
    raise(SIGSEGV);
  }
  if (FIGHTING(ch) != NULL)
    stop_fighting(ch);

  if (GET_EQ(ch, WEAR_HOLD) != NULL)
    if (GET_OBJ_TYPE(GET_EQ(ch, WEAR_HOLD)) == ITEM_LIGHT)
      if (GET_OBJ_VAL(GET_EQ(ch, WEAR_HOLD), 2))	/* Light is ON */
	world[ch->in_room].light -= 5;
  if (GET_EQ(ch, WEAR_HOLD2) != NULL)
    if (GET_OBJ_TYPE(GET_EQ(ch, WEAR_HOLD2)) == ITEM_LIGHT)
      if (GET_OBJ_VAL(GET_EQ(ch, WEAR_HOLD2), 2))	/* Light is ON */
	world[ch->in_room].light -= 5;

  REMOVE_FROM_LIST(ch, world[ch->in_room].people, next_in_room);
  ch->in_room = NOWHERE;
  ch->next_in_room = NULL;
}


/* place a character in a room with XYZ cord */
/* be sure to also change char_to_room */
void put_char_to_room(struct char_data *ch, room_rnum room, sh_int from_dir, sh_int x_cord, sh_int y_cord, sh_int z_cord)
{

  if (ch == NULL || room < 0 || room > top_of_world)
    log("SYSERR: Illegal value(s) passed to put_char_to_room");
  else {
    ch->next_in_room = world[room].people;
    world[room].people = ch;
    ch->in_room = room;

    if (from_dir > -1) {
      SET_FACING(ch, rev_dir[(int) from_dir]);

      /* The doors may not be at the same height, so let's equalize */
      if (VIRTUAL_EXIT(room, from_dir))
	GET_Z_CORD(ch) = GET_DOOR_Z_CORD(room, from_dir);
    }
    /* a door coord of -1 means we won't alter that stat. It's probably
     * a non-door wilderness move
     */
    if (world[room].x_size >= 3 && world[room].y_size >= 3) {
      if (x_cord > world[room].x_size - 2)
	GET_X_CORD(ch) = world[room].x_size - 2;
      else
	GET_X_CORD(ch) = x_cord;
      if (y_cord > world[room].y_size - 2)
	GET_Y_CORD(ch) = world[room].y_size - 2;
      else
	GET_Y_CORD(ch) = y_cord;
    } else {
      GET_X_CORD(ch) = 1;
      GET_Y_CORD(ch) = 1;
    }

    /* NEED TO CHECK FOR FLYING */
    GET_Z_CORD(ch) = z_cord;
    if (GET_Z_CORD(ch) >= GET_Z_SIZE(room) && !WILDERNESS(room))
      GET_Z_CORD(ch) = 0;

    /* Just in case they are now suspended */
    FallingChar(ch, TRUE);

    autoquest_trigger_check(ch, 0, 0, AQ_ROOM);
    autoquest_trigger_check(ch, 0, 0, AQ_MOB_FIND);

    if (GET_EQ(ch, WEAR_HOLD))
      if (GET_OBJ_TYPE(GET_EQ(ch, WEAR_HOLD)) == ITEM_LIGHT)
	if (GET_OBJ_VAL(GET_EQ(ch, WEAR_HOLD), 2))	/* Light ON */
	  world[room].light += 5;
    if (GET_EQ(ch, WEAR_HOLD2))
      if (GET_OBJ_TYPE(GET_EQ(ch, WEAR_HOLD2)) == ITEM_LIGHT)
	if (GET_OBJ_VAL(GET_EQ(ch, WEAR_HOLD2), 2))	/* Light ON */
	  world[room].light += 5;
  }
}


/* place a character in a room */
/* from_dir is -2 dont change xyz (at), -1 random xyz (goto), or 0-5 */
/* be sure to also change put_char_to_room */
void char_to_room(struct char_data *ch, room_rnum room, sh_int from_dir)
{
  room_rnum previous = 0;
  int dir = -1, modifier = 0, mod2 = 0;

  if (ch == NULL || room < 0 || room > top_of_world)
    log("SYSERR: Illegal value(s) passed to char_to_room");
  else {
    ch->next_in_room = world[room].people;
    world[room].people = ch;
    ch->in_room = room;

    if (from_dir > -1) {
      dir = rev_dir[(int) from_dir];
      SET_FACING(ch, dir);
      previous = world[room].dir_option[from_dir] ?
	  world[room].dir_option[from_dir]->to_room : 0;
      if (previous) {
	if (from_dir == NORTH || from_dir == SOUTH) {
	  modifier = VIRTUAL_EXIT(previous, dir) ? GET_DOOR_Y_CORD(previous, dir) : 0;
	  mod2 = VIRTUAL_EXIT(room, from_dir) ? GET_DOOR_Y_CORD(room, from_dir) : 0;
	} else if (from_dir == EAST || from_dir == WEST) {
	  modifier = VIRTUAL_EXIT(previous, dir) ? GET_DOOR_X_CORD(previous, dir) : 0;
	  mod2 = VIRTUAL_EXIT(room, from_dir) ? GET_DOOR_X_CORD(room, from_dir) : 0;
	}
      }
      if (from_dir == UP || from_dir == DOWN) {
	mod2 = VIRTUAL_EXIT(room, from_dir) ? GET_DOOR_X_CORD(room, from_dir) : 0;
	modifier = VIRTUAL_EXIT(room, from_dir) ? GET_DOOR_Y_CORD(room, from_dir) : 0;
      }
    }
    /* a door coord of -1 means we won't alter that stat. It's probably
     * a non-door wilderness move
     */
    if (from_dir > -1) {
      switch (from_dir) {
      case NORTH:
	GET_X_CORD(ch) = 1;
	GET_Y_CORD(ch) = mod2 + (GET_Y_CORD(ch) - modifier);
	break;
      case SOUTH:
	GET_X_CORD(ch) = GET_X_SIZE(room) - 2;
	GET_Y_CORD(ch) = mod2 + (GET_Y_CORD(ch) - modifier);
	break;
      case EAST:
	GET_X_CORD(ch) = mod2 + (GET_X_CORD(ch) - modifier);
	GET_Y_CORD(ch) = GET_Y_SIZE(room) - 2;
	break;
      case WEST:
	GET_X_CORD(ch) = mod2 + (GET_X_CORD(ch) - modifier);
	GET_Y_CORD(ch) = 1;
	break;
      case UP:
      case DOWN:
	GET_Y_CORD(ch) = modifier;
	GET_X_CORD(ch) = mod2;
	break;
      default:
	/* some kind of error, just randomize */
	GET_X_CORD(ch) = number(1, GET_X_SIZE(room) - 2);
	GET_Y_CORD(ch) = number(1, GET_Y_SIZE(room) - 2);
	break;
      }
      /* The doors may not be at the same height, so let's equalize */
      /* check that they aren't roof walking first though */
      if (GET_Z_CORD(ch) < GET_Z_SIZE(room) && VIRTUAL_EXIT(room, from_dir))
	GET_Z_CORD(ch) = GET_DOOR_Z_CORD(room, from_dir);

      /* if it was a real door, ensure they remain in door square */
      if ((IS_SET(ROOM_FLAGS(room), ROOM_INDOORS)) &&
	  (GET_X_CORD(ch) == 0 || GET_X_CORD(ch) == GET_X_SIZE(room) - 1
      || GET_Y_CORD(ch) == 0 || GET_Y_CORD(ch) == GET_Y_SIZE(room) - 1)) {
	if (from_dir == NORTH || from_dir == SOUTH)
	  GET_Y_CORD(ch) = GET_DOOR_Y_CORD(room, from_dir);
	else if (from_dir == WEST || from_dir == EAST)
	  GET_X_CORD(ch) = GET_DOOR_X_CORD(room, from_dir);
      }
    } else if (from_dir == -1) {
      GET_X_CORD(ch) = number(1, GET_X_SIZE(room) - 2);
      GET_Y_CORD(ch) = number(1, GET_Y_SIZE(room) - 2);
    }
    /* Just in case they are now suspended */
    FallingChar(ch, TRUE);

    autoquest_trigger_check(ch, 0, 0, AQ_ROOM);
    autoquest_trigger_check(ch, 0, 0, AQ_MOB_FIND);

    if (GET_EQ(ch, WEAR_HOLD))
      if (GET_OBJ_TYPE(GET_EQ(ch, WEAR_HOLD)) == ITEM_LIGHT)
	if (GET_OBJ_VAL(GET_EQ(ch, WEAR_HOLD), 2))	/* Light ON */
	  world[room].light += 5;
    if (GET_EQ(ch, WEAR_HOLD2))
      if (GET_OBJ_TYPE(GET_EQ(ch, WEAR_HOLD2)) == ITEM_LIGHT)
	if (GET_OBJ_VAL(GET_EQ(ch, WEAR_HOLD2), 2))	/* Light ON */
	  world[room].light += 5;
  }
}


/* give an object to a char   */
void obj_to_char(struct obj_data *object, struct char_data *ch)
{
  if (object && ch) {
    object->next_content = ch->carrying;
    ch->carrying = object;
    object->carried_by = ch;
    object->in_room = NOWHERE;
    IS_CARRYING_W(ch) += GET_OBJ_WEIGHT(object);
    IS_CARRYING_N(ch)++;

    autoquest_trigger_check(ch, NULL, object, AQ_OBJECT);

    /* set flag for crash-save system */
    SET_BIT(PLR_FLAGS(ch), PLR_CRASH);

    /* testing this one */
    convert_prototype_to_random_item(ch, object, 0);

  }				/* else  not concerned with this error right now
				   log("SYSERR: NULL obj or char passed to obj_to_char"); */
}


/* take an object from a char */
void obj_from_char(struct obj_data *object)
{
  struct obj_data *temp;

  if (object == NULL || !object->carried_by) {
    log("SYSERR: NULL object passed to obj_from_char");
    return;
  }
  REMOVE_FROM_LIST(object, object->carried_by->carrying, next_content);

  /* set flag for crash-save system */
  SET_BIT(PLR_FLAGS(object->carried_by), PLR_CRASH);

  IS_CARRYING_W(object->carried_by) -= GET_OBJ_WEIGHT(object);
  IS_CARRYING_N(object->carried_by)--;
  object->carried_by = NULL;
  object->next_content = NULL;
}



/* Return the effect of a piece of armor in position eq_pos */
int apply_ac(struct char_data *ch, int eq_pos)
{
  int factor;

  assert(GET_EQ(ch, eq_pos));

  if (!(GET_OBJ_TYPE(GET_EQ(ch, eq_pos)) == ITEM_ARMOR))
    return 0;

  return (GET_OBJ_VAL(GET_EQ(ch, eq_pos), 0));

  switch (eq_pos) {

  case WEAR_BODY:
    factor = 3;
    break;			/* 30% */
  case WEAR_HEAD:
    factor = 2;
    break;			/* 20% */
  case WEAR_LEGS:
    factor = 2;
    break;			/* 20% */
  default:
    factor = 1;
    break;			/* all others 10% */
  }

  return (factor * GET_OBJ_VAL(GET_EQ(ch, eq_pos), 0));
}



void equip_char(struct char_data *ch, struct obj_data *obj, int pos)
{
  int j;
  int invalid_class(struct char_data *ch, struct obj_data *obj);

  assert(pos >= 0 && pos < NUM_WEARS);

  if (GET_EQ(ch, pos)) {
    sprintf(buf, "SYSERR: Char is already equipped: %s, %s", GET_NAME(ch),
	    obj->short_description);
    log(buf);
    return;
  }
  if (obj->carried_by) {
    log("SYSERR: EQUIP: Obj is carried_by when equip.");
    return;
  }
  if (obj->in_room != NOWHERE) {
    log("SYSERR: EQUIP: Obj is in_room when equip.");
    return;
  }
/*  if ((IS_OBJ_STAT(obj, ITEM_ANTI_EVIL) && IS_EVIL(ch)) ||
   (IS_OBJ_STAT(obj, ITEM_ANTI_GOOD) && IS_GOOD(ch)) ||
   (IS_OBJ_STAT(obj, ITEM_ANTI_NEUTRAL) && IS_NEUTRAL(ch)) ||
   invalid_class(ch, obj)) {
   act("You are zapped by $p and instantly let go of it.", FALSE, ch, obj, 0, TO_CHAR);
   act("$n is zapped by $p and instantly lets go of it.", FALSE, ch, obj, 0, PROXIMITY);
   obj_to_char(obj, ch);        
   return;
   } */
  GET_EQ(ch, pos) = obj;
  obj->worn_by = ch;
  obj->worn_on = pos;

/*  if (GET_OBJ_TYPE(obj) == ITEM_ARMOR)
   GET_AC(ch) -= apply_ac(ch, pos); */

  if (ch->in_room != NOWHERE) {
    if (pos == WEAR_HOLD && GET_OBJ_TYPE(obj) == ITEM_LIGHT)
      if (GET_OBJ_VAL(obj, 2))	/* if light is ON */
	world[ch->in_room].light += 5;
    if (pos == WEAR_HOLD2 && GET_OBJ_TYPE(obj) == ITEM_LIGHT)
      if (GET_OBJ_VAL(obj, 2))	/* if light is ON */
	world[ch->in_room].light += 5;
  } else if (!IS_NPC(ch))
    log("SYSERR: ch->in_room = NOWHERE when equipping char.");

  for (j = 0; j < MAX_OBJ_AFFECT; j++) {
    affect_modify(ch, obj->affected[j].location,
		  obj->affected[j].modifier,
		  obj->obj_flags.bitvector, obj->obj_flags.bitvector2,
		  obj->obj_flags.bitvector3,
		  obj->obj_flags.bitvector4,
		  obj->obj_flags.protection,
		  obj->obj_flags.detecting,
		  obj->obj_flags.controlling, TRUE);
  }

  affect_total(ch);
  if (IS_AFFECTED4(ch, AFF4_SLNT_MOVE))
    roll_slnt_move(ch);
}



struct obj_data *
 unequip_char(struct char_data *ch, int pos)
{
  int j;
  struct obj_data *obj, *obj2;

  assert(pos >= 0 && pos < NUM_WEARS);
  assert(GET_EQ(ch, pos));

  if (!GET_EQ(ch, pos)) {
    vlogf("%s tried to unequip something they didnt have equipped.", GET_NAME(ch));
    return NULL;
  }
  obj = GET_EQ(ch, pos);
  obj->worn_by = NULL;
  obj->worn_on = -1;

/*  if (GET_OBJ_TYPE(obj) == ITEM_ARMOR)
   GET_AC(ch) += apply_ac(ch, pos); */

  if (ch->in_room != NOWHERE) {
    if (pos == WEAR_HOLD && GET_OBJ_TYPE(obj) == ITEM_LIGHT)
      if (GET_OBJ_VAL(obj, 2))	/* if light is ON */
	world[ch->in_room].light -= 5;
    if (pos == WEAR_HOLD2 && GET_OBJ_TYPE(obj) == ITEM_LIGHT)
      if (GET_OBJ_VAL(obj, 2))	/* if light is ON */
	world[ch->in_room].light -= 5;
  } else
    log("SYSERR: ch->in_room = NOWHERE when equipping char.");

  GET_EQ(ch, pos) = NULL;

  for (j = 0; j < MAX_OBJ_AFFECT; j++) {
    affect_modify(ch, obj->affected[j].location,
		  obj->affected[j].modifier,
		  obj->obj_flags.bitvector, obj->obj_flags.bitvector2,
		  obj->obj_flags.bitvector3, obj->obj_flags.bitvector4,
		  obj->obj_flags.protection,
		  obj->obj_flags.detecting,
		  obj->obj_flags.controlling,
		  FALSE);
  }

  affect_total(ch);


  if (IS_AFFECTED4(ch, AFF4_SLNT_MOVE))
    roll_slnt_move(ch);

  if (IS_SET((obj)->obj_flags.extra2_flags, ITEM2_REMOVE_2HAND)) {
    obj2 = read_object(GET_OBJ_VNUM(obj) - 1, VIRTUAL);
    extract_obj(obj);
    return (obj2);
  }
  return (obj);
}


int get_number(char **name)
{
  int i;
  char *ppos;
  char number[MAX_INPUT_LENGTH];

  *number = '\0';

  if ((ppos = strchr(*name, '.'))) {
    *ppos++ = '\0';
    strcpy(number, *name);
    strcpy(*name, ppos);

    for (i = 0; *(number + i); i++)
      if (!isdigit(*(number + i)))
	return 0;

    return (atoi(number));
  }
  return 1;
}


/* return room with UID n */

/* Search a given list for an object number, and return a ptr to that obj */
struct obj_data *
 get_obj_in_list_num(int num, struct obj_data *list)
{
  struct obj_data *i;

  for (i = list; i; i = i->next_content)
    if (GET_OBJ_RNUM(i) == num)
      return i;

  return NULL;
}



/* search the entire world for an object number, and return a pointer  */
struct obj_data *
 get_obj_num(obj_rnum nr)
{
  struct obj_data *i;

  for (i = object_list; i; i = i->next)
    if (GET_OBJ_RNUM(i) == nr)
      return i;

  return NULL;
}



/* search a room for a char, and return a pointer if found..  */
struct char_data *
 get_char_room(char *name, room_rnum room)
{
  struct char_data *i;
  int j = 0, number;
  char tmpname[MAX_INPUT_LENGTH];
  char *tmp = tmpname;
  if (*name == UID_CHAR) {
    i = find_char(atoi(name + 1));

    if (i && (IN_ROOM(i) == room) && !GET_INVIS_LEV(i))
      return i;
  } else {

    strcpy(tmp, name);
    if (!(number = get_number(&tmp)))
      return NULL;

    for (i = world[room].people; i && (j <= number); i = i->next_in_room)
      if (isname(tmp, i->player.name))
	if (++j == number)
	  return i;
  }
  return NULL;
}



/* search all over the world for a char num, and return a pointer if found */
struct char_data *
 get_char_num(mob_rnum nr)
{
  struct char_data *i;

  for (i = character_list; i; i = i->next)
    if (GET_MOB_RNUM(i) == nr)
      return i;
  return NULL;
}

/* put an object in a room with XYZ cord */
/* be sure to also change obj_to_room */
void put_obj_to_room(struct obj_data *object, room_rnum room, sh_int x_cord, sh_int y_cord, sh_int z_cord)
{
  if (!object || room < 0 || room > top_of_world)
    log("SYSERR: Illegal value(s) passed to put_obj_to_room");
  else {
    object->next_content = world[room].contents;
    world[room].contents = object;
    object->in_room = room;
    if (world[room].x_size >= 3 && world[room].y_size >= 3) {
      if (x_cord > world[room].x_size - 1)
	GET_OBJ_X_CORD(object) = world[room].x_size - 1;
      else
	GET_OBJ_X_CORD(object) = x_cord;
      if (y_cord > world[room].y_size - 1)
	GET_OBJ_Y_CORD(object) = world[room].y_size - 1;
      else
	GET_OBJ_Y_CORD(object) = y_cord;
    } else {
      GET_OBJ_X_CORD(object) = 1;
      GET_OBJ_Y_CORD(object) = 1;
    }
    GET_OBJ_Z_CORD(object) = z_cord;
    object->carried_by = NULL;
    object->worn_by = NULL;
    if (ROOM_FLAGGED(room, ROOM_HOUSE))
      SET_BIT(ROOM_FLAGS(room), ROOM_HOUSE_CRASH);

    /* Just in case its now above the ground level */
    FallingObj(object);

    /* Treasure types, etc */
    convert_prototype_to_random_item(0, object, room);
  }
}

/* put an object in a room */
/* be sure to also change put_obj_to_room */
void obj_to_room(struct obj_data *object, room_rnum room)
{
  if (!object || room < 0 || room > top_of_world)
    log("SYSERR: Illegal value(s) passed to obj_to_room");
  else {
    object->next_content = world[room].contents;
    world[room].contents = object;
    object->in_room = room;
    object->carried_by = NULL;
    object->worn_by = NULL;
    if (ROOM_FLAGGED(room, ROOM_HOUSE))
      SET_BIT(ROOM_FLAGS(room), ROOM_HOUSE_CRASH);

    GET_OBJ_X_CORD(object) = number(1, world[room].x_size - 1);
    GET_OBJ_Y_CORD(object) = number(1, world[room].y_size - 1);

    /* Just in case its now above the ground level */
    FallingObj(object);

    /* Treasure types, etc */
    convert_prototype_to_random_item(0, object, room);
  }
}


/* Take an object from a room */
void obj_from_room(struct obj_data *object)
{
  struct obj_data *temp;

  if (!object || object->in_room == NOWHERE) {
    log("SYSERR: NULL object or obj not in a room passed to obj_from_room");
    return;
  }
  REMOVE_FROM_LIST(object, world[object->in_room].contents, next_content);

  if (ROOM_FLAGGED(object->in_room, ROOM_HOUSE))
    SET_BIT(ROOM_FLAGS(object->in_room), ROOM_HOUSE_CRASH);
  object->in_room = NOWHERE;
  object->next_content = NULL;
}


/* put an object in an object (quaint)  */
void obj_to_obj(struct obj_data *obj, struct obj_data *obj_to)
{
  struct obj_data *tmp_obj;

  if (!obj || !obj_to || obj == obj_to) {
    log("SYSERR: NULL object or same source and target obj passed to obj_to_obj");
    return;
  }
  obj->next_content = obj_to->contains;
  obj_to->contains = obj;
  obj->in_obj = obj_to;

  for (tmp_obj = obj->in_obj; tmp_obj->in_obj; tmp_obj = tmp_obj->in_obj)
    GET_OBJ_WEIGHT(tmp_obj) += GET_OBJ_WEIGHT(obj);

  /* top level object.  Subtract weight from inventory if necessary. */
  GET_OBJ_WEIGHT(tmp_obj) += GET_OBJ_WEIGHT(obj);
  if (tmp_obj->carried_by)
    IS_CARRYING_W(tmp_obj->carried_by) += GET_OBJ_WEIGHT(obj);
}


/* remove an object from an object */
void obj_from_obj(struct obj_data *obj)
{
  struct obj_data *temp, *obj_from;

  if (obj->in_obj == NULL) {
    log("error (handler.c): trying to illegally extract obj from obj");
    return;
  }
  obj_from = obj->in_obj;
  REMOVE_FROM_LIST(obj, obj_from->contains, next_content);

  /* Subtract weight from containers container */
  for (temp = obj->in_obj; temp->in_obj; temp = temp->in_obj)
    GET_OBJ_WEIGHT(temp) -= GET_OBJ_WEIGHT(obj);

  /* Subtract weight from char that carries the object */
  GET_OBJ_WEIGHT(temp) -= GET_OBJ_WEIGHT(obj);
  if (temp->carried_by)
    IS_CARRYING_W(temp->carried_by) -= GET_OBJ_WEIGHT(obj);

  obj->in_obj = NULL;
  obj->next_content = NULL;
}


/* Set all carried_by to point to new owner */
void object_list_new_owner(struct obj_data *list, struct char_data *ch)
{
  if (list) {
    object_list_new_owner(list->contains, ch);
    object_list_new_owner(list->next_content, ch);
    list->carried_by = ch;
  }
}


/* Extract an object from the world */
void extract_obj(struct obj_data *obj)
{
  struct obj_data *temp;

  if (obj->worn_by != NULL)
    if (unequip_char(obj->worn_by, obj->worn_on) != obj)
      log("SYSERR: Inconsistent worn_by and worn_on pointers!!");
  if (obj->in_room != NOWHERE)
    obj_from_room(obj);
  else if (obj->carried_by)
    obj_from_char(obj);
  else if (obj->in_obj)
    obj_from_obj(obj);

  /* Get the people from the object */
  while (obj->people)
    char_from_object(obj->people, obj);

  /* Get rid of the contents of the object, as well. */
  while (obj->contains)
    extract_obj(obj->contains);

  REMOVE_FROM_LIST(obj, object_list, next);

  if (GET_OBJ_RNUM(obj) >= 0)
    (obj_index[GET_OBJ_RNUM(obj)].number)--;
  free_obj(obj);
}



void update_object(struct obj_data *obj, int use)
{
  /* dont update objects with a timer trigger */
  if (!SCRIPT_CHECK(obj, OTRIG_TIMER) && (GET_OBJ_TIMER(obj) > 0))
    GET_OBJ_TIMER(obj) -= use;
  if (obj->contains)
    update_object(obj->contains, use);
  if (obj->next_content)
    update_object(obj->next_content, use);
}


void update_char_objects(struct char_data *ch)
{
  int i;

  if (GET_EQ(ch, WEAR_HOLD) != NULL)
    if (GET_OBJ_TYPE(GET_EQ(ch, WEAR_HOLD)) == ITEM_LIGHT)
      if (GET_OBJ_VAL(GET_EQ(ch, WEAR_HOLD), 2) > 0) {
	i = --GET_OBJ_VAL(GET_EQ(ch, WEAR_HOLD), 2);
	if (i == 1) {
	  act("Your light begins to flicker and fade.", FALSE, ch, 0, 0, TO_CHAR);
	  act("$n's light begins to flicker and fade.", FALSE, ch, 0, 0, PROXIMITY);
	} else if (i == 0) {
	  act("Your light sputters out and dies.", FALSE, ch, 0, 0, TO_CHAR);
	  act("$n's light sputters out and dies.", FALSE, ch, 0, 0, PROXIMITY);
	  world[ch->in_room].light -= 5;
	}
      }
  if (GET_EQ(ch, WEAR_HOLD2) != NULL)
    if (GET_OBJ_TYPE(GET_EQ(ch, WEAR_HOLD2)) == ITEM_LIGHT)
      if (GET_OBJ_VAL(GET_EQ(ch, WEAR_HOLD2), 2) > 0) {
	i = --GET_OBJ_VAL(GET_EQ(ch, WEAR_HOLD2), 2);
	if (i == 1) {
	  act("Your light begins to flicker and fade.", FALSE, ch, 0, 0, TO_CHAR);
	  act("$n's light begins to flicker and fade.", FALSE, ch, 0, 0, PROXIMITY);
	} else if (i == 0) {
	  act("Your light sputters out and dies.", FALSE, ch, 0, 0, TO_CHAR);
	  act("$n's light sputters out and dies.", FALSE, ch, 0, 0, PROXIMITY);
	  world[ch->in_room].light -= 5;
	}
      }
  for (i = 0; i < NUM_WEARS; i++)
    if (GET_EQ(ch, i))
      update_object(GET_EQ(ch, i), 2);

  if (ch->carrying)
    update_object(ch->carrying, 1);
}



/* Extract a ch completely from the world, and leave his stuff behind */
void extract_char(struct char_data *ch)
{
  struct char_data *temp;
  struct descriptor_data *t_desc;
  struct obj_data *obj;
  int i, freed = 0;

  ACMD(do_return);

  void die_follower(struct char_data *ch);
  void die_group_member(struct char_data *ch);

  if (!IS_NPC(ch) && !ch->desc) {
    for (t_desc = descriptor_list; t_desc; t_desc = t_desc->next)
      if (t_desc->original == ch)
	do_return(t_desc->character, "", 0, 0);
  }
  if (ch->in_room == NOWHERE) {
    log("SYSERR: NOWHERE extracting char. (handler.c, extract_char)");
    raise(SIGSEGV);
  }
  if (MORPH_MOB(ch)) {		/* bad... */
    un_morph(ch);
    return;
  }
  if (ch->followers || ch->master)
    die_follower(ch);
  if (ch->watchers || ch->watching)
    die_watcher(ch);
  if (IS_AFFECTED(ch, AFF_GROUP)) {
    die_group_member(ch);
  }
  if (RIDING(ch) || RIDDEN_BY(ch))
    dismount_char(ch);
  if (FURNITURE(ch))
    char_from_object(ch, FURNITURE(ch));
  /* Forget snooping, if applicable */
  if (ch->desc) {
    if (ch->desc->snooping) {
      ch->desc->snooping->snoop_by = NULL;
      ch->desc->snooping = NULL;
    }
    if (ch->desc->snoop_by) {
      SEND_TO_Q("Your victim is no longer among us.\r\n",
		ch->desc->snoop_by);
      ch->desc->snoop_by->snooping = NULL;
      ch->desc->snoop_by = NULL;
    }
  }
  /* transfer objects to room, if any */
  while (ch->carrying) {
    obj = ch->carrying;
    obj_from_char(obj);
    put_obj_to_room(obj, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
  }

  /* transfer equipment to room, if any */
  for (i = 0; i < NUM_WEARS; i++)
    if (GET_EQ(ch, i))
      put_obj_to_room(unequip_char(ch, i), ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));

  /* stop the tank right now, others will get stopped upon the event call */
  if (FIGHTING(ch) && FIGHTING(FIGHTING(ch)))
    stop_fighting(FIGHTING(ch));

  if (FIGHTING(ch))
    stop_fighting(ch);

  /* cancel any pending events */
  if (GET_ACTION(ch)) {
    event_cancel(GET_ACTION(ch));
    GET_ACTION(ch) = NULL;
  }
  /* clear memorization */
  ch->char_specials.spell_mem_list = NULL;

  /* cancel point updates */
  for (i = 0; i < 3; i++)
    if (GET_POINTS_EVENT(ch, i)) {
      event_cancel(GET_POINTS_EVENT(ch, i));
      GET_POINTS_EVENT(ch, i) = NULL;
    }
  char_from_room(ch);

  /* pull the char from the list */
  REMOVE_FROM_LIST(ch, character_list, next);

  if (ch->desc && ch->desc->original)
    do_return(ch, "", 0, 0);

  if (!IS_NPC(ch)) {
    save_char(ch, NOWHERE);
    Crash_delete_crashfile(ch);
  } else {
    if (GET_MOB_RNUM(ch) > -1)	/* if mobile */
      mob_index[GET_MOB_RNUM(ch)].number--;
    clearMemory(ch);		/* Only NPC's can have memory */
    if (SCRIPT(ch))
      extract_script(SCRIPT(ch));
    if (SCRIPT_MEM(ch))
      extract_script_mem(SCRIPT_MEM(ch));
    free_char(ch);
    freed = 1;
  }

  if (!freed && ch->desc != NULL) {
    STATE(ch->desc) = CON_MENU;
    SEND_TO_Q(MENU, ch->desc);
  } else {			/* if a player gets purged from within the game */
    if (!freed)
      free_char(ch);
  }
}



/* ***********************************************************************
   * Here follows high-level versions of some earlier routines, ie functions*
   * which incorporate the actual player-data                               *.
   *********************************************************************** */


struct char_data *
 get_player_vis(struct char_data *ch, char *name, int inroom)
{
  struct char_data *i;

  for (i = character_list; i; i = i->next)
    if (!IS_NPC(i) && (!inroom || i->in_room == ch->in_room) &&
	!str_cmp(i->player.name, name) && CAN_SEE(ch, i))
      return i;

  return NULL;
}


struct char_data *
 get_char_room_vis(struct char_data *ch, char *name)
{
  struct char_data *i;
  int j = 0, number;
  char tmpname[MAX_INPUT_LENGTH];
  char *tmp = tmpname;

  if (*name == UID_CHAR) {
    i = find_char(atoi(name + 1));

    if (i && (IN_ROOM(ch) == IN_ROOM(i)) && CAN_SEE(ch, i))
      return i;
  } else {
    if (!str_cmp(name, "self") || !str_cmp(name, "me"))
      return ch;

    /* 0.<name> means PC with name */
    strcpy(tmp, name);
    if (!(number = get_number(&tmp)))
      return get_player_vis(ch, tmp, 1);

    for (i = world[ch->in_room].people; i && j <= number; i = i->next_in_room)
      if (isname(tmp, i->player.name))
	if (CAN_SEE(ch, i))
	  if (++j == number)
	    return i;
  }
  return NULL;
}

struct char_data *
 get_char_room_cord(struct char_data *ch, char *name)
{
  struct char_data *i;
  char tmpname[MAX_INPUT_LENGTH];
  char *tmp = tmpname;

  strcpy(tmp, name);
  for (i = world[ch->in_room].people; i; i = i->next_in_room)
    if (isname(tmp, i->player.name))
      if (CAN_SEE(ch, i) &&
	  GET_X_CORD(ch) == GET_X_CORD(i) &&
	  GET_Y_CORD(ch) == GET_Y_CORD(i) &&
	  GET_Z_CORD(ch) == GET_Z_CORD(i))
	return i;
  return NULL;
}
struct char_data *
 get_char_vis(struct char_data *ch, char *name)
{
  struct char_data *i;
  int j = 0, number;
  char tmpname[MAX_INPUT_LENGTH];
  char *tmp = tmpname;

  /* check the room first */
  if ((i = get_char_room_vis(ch, name)) != NULL)
    return i;

  strcpy(tmp, name);
  if (!(number = get_number(&tmp)))
    return get_player_vis(ch, tmp, 0);

  if (*name == UID_CHAR) {
    i = find_char(atoi(name + 1));
    if (i && !GET_INVIS_LEV(i))
      return i;
  } else {
    for (i = character_list; i && (j <= number); i = i->next)
      if (isname(tmp, i->player.name) && CAN_SEE(ch, i))
	if (++j == number)
	  return i;
  }
  return NULL;
}



struct obj_data *
 get_obj_in_list_vis(struct char_data *ch, char *name,
		     struct obj_data *list)
{
  struct obj_data *i;
  int j = 0, number;
  char tmpname[MAX_INPUT_LENGTH];
  char *tmp = tmpname;
  long id;

  if (*name == UID_CHAR) {
    id = atoi(name + 1);

    for (i = list; i; i = i->next_content)
      if (id == GET_ID(i) && CAN_SEE_OBJ(ch, i))
	return i;
  } else {
    strcpy(tmp, name);
    if (!(number = get_number(&tmp)))
      return NULL;

    for (i = list; i && (j <= number); i = i->next_content)
      if (isname(tmp, i->name))
	if (CAN_SEE_OBJ(ch, i))
	  if (++j == number)
	    return i;
  }
  return NULL;
}

/* this uses can_see, as well as coordinates. Instead of returning the first
 * in the list, we will return the first in the list that is in the same square
 * of the player. Mostly useful for room contents.
 */
struct obj_data *
 get_obj_in_list_cord(struct char_data *ch, char *name,
		      struct obj_data *list)
{
  struct obj_data *i;
  int j = 0, number;
  char tmpname[MAX_INPUT_LENGTH];
  char *tmp = tmpname;

  strcpy(tmp, name);
  if (!(number = get_number(&tmp)))
    return NULL;

  for (i = list; i && (j <= number); i = i->next_content)
    if (isname(tmp, i->name))
      if (CAN_SEE_OBJ(ch, i) &&
	  GET_X_CORD(ch) == GET_OBJ_X_CORD(i) &&
	  GET_Y_CORD(ch) == GET_OBJ_Y_CORD(i) &&
	  GET_Z_CORD(ch) == GET_OBJ_Z_CORD(i))
	if (++j == number)
	  return i;
  return NULL;
}

/* search the entire world for an object, and return a pointer  */
struct obj_data *
 get_obj_vis(struct char_data *ch, char *name)
{
  struct obj_data *i;
  int j = 0, number;
  char tmpname[MAX_INPUT_LENGTH];
  char *tmp = tmpname;
  long id;

  /* scan items carried */
  if ((i = get_obj_in_list_vis(ch, name, ch->carrying)))
    return i;

  /* scan room */
  if ((i = get_obj_in_list_vis(ch, name, world[ch->in_room].contents)))
    return i;

  if (*name == UID_CHAR) {
    id = atoi(name + 1);

    /* ok.. no luck yet. scan the entire obj list   */
    for (i = object_list; i; i = i->next)
      if ((id == GET_ID(i)) && CAN_SEE_OBJ(ch, i))
	return i;
  } else {
    strcpy(tmp, name);
    if (!(number = get_number(&tmp)))
      return NULL;

    /* ok.. no luck yet. scan the entire obj list   */

    if (*name == UID_CHAR) {
      id = atoi(name + 1);

      for (i = object_list; i; i = i->next)
	if (id == GET_ID(i))
	  return i;
    } else {
      for (i = object_list; i && (j <= number); i = i->next)
	if (isname(tmp, i->name))
	  if (CAN_SEE_OBJ(ch, i) && (!i->carried_by || CAN_SEE(ch, i->carried_by)))
	    if (++j == number)
	      return i;
    }
  }
  return NULL;
}

struct obj_data *
 get_object_in_equip_vis(struct char_data *ch,
			 char *arg, struct obj_data *equipment[], int *j)
{
  char tmpname[MAX_STRING_LENGTH];
  char *tmp = tmpname;
  int number, i = 0;

  strcpy(tmp, arg);

  if (!(number = get_number(&tmp)))
    return NULL;

  for ((*j) = 0; (i <= number) && ((*j) < NUM_WEARS); (*j)++)
    if (equipment[(*j)])
      if (CAN_SEE_OBJ(ch, equipment[(*j)]))
	if (isname(tmp, equipment[(*j)]->name))
	  if (++i == number)
	    return (equipment[(*j)]);

  return NULL;
}

struct obj_data *
 get_held_object_in_equip_vis(struct char_data *ch,
			 char *arg, struct obj_data *equipment[], int *j)
{
  char tmpname[MAX_STRING_LENGTH];
  char *tmp = tmpname;
  int number, i = 0;

  strcpy(tmp, arg);

  if (!(number = get_number(&tmp))) {
    return NULL;
  }
  for ((*j) = WEAR_WIELD; (i <= number) && ((*j) <= WEAR_HOLD2); (*j)++)
    if (equipment[(*j)])
      if (CAN_SEE_OBJ(ch, equipment[(*j)]))
	if (isname(tmp, equipment[(*j)]->name)) {
	  if (++i == number)
	    return (equipment[(*j)]);
	}
  return NULL;
}

char *
 money_desc(int amount, int type, int king)
{
  static char buf[128];

  if (king > 0)
    sprintf(buf2, "%s's ", kingdom_list[king]);
  else
    strcpy(buf2, "");

  if (amount <= 0) {
/*    log("SYSERR: Try to create negative or 0 money."); */
    return NULL;
  }
  if (amount == 1)
    sprintf(buf, "a %s%s coin", buf2, coin_names[type]);
  else if (amount <= 10)
    sprintf(buf, "a tiny pile of %s%s coins", buf2, coin_names[type]);
  else if (amount <= 20)
    sprintf(buf, "a handful of %s%s coins", buf2, coin_names[type]);
  else if (amount <= 75)
    sprintf(buf, "a little pile of %s%s coins", buf2, coin_names[type]);
  else if (amount <= 200)
    sprintf(buf, "a small pile of %s%s coins", buf2, coin_names[type]);
  else if (amount <= 1000)
    sprintf(buf, "a pile of %s%s coins", buf2, coin_names[type]);
  else if (amount <= 5000)
    sprintf(buf, "a big pile of %s%s coins", buf2, coin_names[type]);
  else if (amount <= 10000)
    sprintf(buf, "a large heap of %s%s coins", buf2, coin_names[type]);
  else if (amount <= 20000)
    sprintf(buf, "a huge mound of %s%s coins", buf2, coin_names[type]);
  else if (amount <= 75000)
    sprintf(buf, "an enormous mound of %s%s coins", buf2, coin_names[type]);
  else if (amount <= 150000)
    sprintf(buf, "a small mountain of %s%s coins", buf2, coin_names[type]);
  else if (amount <= 250000)
    sprintf(buf, "a mountain of %s%s coins", buf2, coin_names[type]);
  else if (amount <= 500000)
    sprintf(buf, "a huge mountain of %s%s coins", buf2, coin_names[type]);
  else if (amount <= 1000000)
    sprintf(buf, "an enormous mountain of %s%s coins", buf2, coin_names[type]);
  else
    sprintf(buf, "an absolutely colossal mountain of %s%s coins", buf2, coin_names[type]);

  return buf;
}


struct obj_data *
 create_money(int amount, int type, int king)
{
  struct obj_data *obj;
  struct extra_descr_data *new_descr;
  char buf[200];

  if (amount <= 0) {
    /*   log("SYSERR: Try to create negative or 0 money."); */
    return NULL;
  }
  obj = read_object( /*START_MONEY */ 50 + type, VIRTUAL);
  CREATE(new_descr, struct extra_descr_data, 1);

  switch (type) {
  case 0:
    obj->name = str_dup("coin coins copper");
    new_descr->keyword = str_dup("coin coins copper");
    break;
  case 1:
    obj->name = str_dup("coin coins silver");
    new_descr->keyword = str_dup("coin coins silver");
    break;
  case 2:
    obj->name = str_dup("coin coins electrum");
    new_descr->keyword = str_dup("coin coins electrum");
    break;
  case 3:
    obj->name = str_dup("coin coins gold");
    new_descr->keyword = str_dup("coin coins gold");
    break;
  case 4:
    obj->name = str_dup("coin coins platinum");
    new_descr->keyword = str_dup("coin coins platinum");
    break;
  default:
    obj->name = str_dup("coin coins coins_error");
    new_descr->keyword = str_dup("coin coins coins_error");
    break;
  }

  if (king > 0)
    sprintf(buf2, "%s's ", kingdom_list[king]);
  else
    strcpy(buf2, "");

  obj->short_description = str_dup(money_desc(amount, type, king));
  sprintf(buf, "%s is lying here.", money_desc(amount, type, king));
  obj->description = str_dup(CAP(buf));
  sprintf(buf, "There are %d %s%s coins.", amount, buf2, coin_names[type]);
  new_descr->description = str_dup(buf);

  new_descr->next = NULL;
  obj->ex_description = new_descr;

  GET_OBJ_TYPE(obj) = ITEM_MONEY;
  GET_OBJ_WEAR(obj) = ITEM_WEAR_TAKE;
  GET_OBJ_VAL(obj, 0) = amount;
  GET_OBJ_VAL(obj, 1) = type;
  GET_OBJ_VAL(obj, 2) = king;
  GET_OBJ_COST(obj) = amount;
  GET_OBJ_WEIGHT(obj) = (amount * 0.02);
/*  obj->item_number = NOTHING; */

  return obj;
}


/* Generic Find, designed to find any object/character                    */
/* Calling :                                                              */
/*  *arg     is the sting containing the string to be searched for.       */
/*           This string doesn't have to be a single word, the routine    */
/*           extracts the next word itself.                               */
/*  bitv..   All those bits that you want to "search through".            */
/*           Bit found will be result of the function                     */
/*  *ch      This is the person that is trying to "find"                  */
/*  **tar_ch Will be NULL if no character was found, otherwise points     */
/* **tar_obj Will be NULL if no object was found, otherwise points        */
/*                                                                        */
/* The routine returns a pointer to the next word in *arg (just like the  */
/* one_argument routine).                                                 */

int generic_find(char *arg, int bitvector, struct char_data *ch,
		 struct char_data **tar_ch, struct obj_data **tar_obj)
{
  int i, found;
  char name[256];

  one_argument(arg, name);

  if (!*name)
    return (0);

  *tar_ch = NULL;
  *tar_obj = NULL;

  if (IS_SET(bitvector, FIND_CHAR_ROOM)) {	/* Find person in room */
    if ((*tar_ch = get_char_room_vis(ch, name))) {
      return (FIND_CHAR_ROOM);
    }
  }
  if (IS_SET(bitvector, FIND_CHAR_WORLD)) {
    if ((*tar_ch = get_char_vis(ch, name))) {
      return (FIND_CHAR_WORLD);
    }
  }
  if (IS_SET(bitvector, FIND_OBJ_EQUIP)) {
#if 0
    for (found = FALSE, i = 0; i < NUM_WEARS && !found; i++)
      if (GET_EQ(ch, i) && CAN_SEE_OBJ(ch, GET_EQ(ch, i)) &&
	  isname(name, GET_EQ(ch, i)->name)) {
	*tar_obj = GET_EQ(ch, i);
	found = TRUE;
      }
    if (found) {
      return (FIND_OBJ_EQUIP);
    }
#endif
    if ((*tar_obj = get_object_in_equip_vis(ch, name, ch->equipment, &i))) {
      return (FIND_OBJ_EQUIP);
    }
  }
  if (IS_SET(bitvector, FIND_OBJ_INV)) {
    if ((*tar_obj = get_obj_in_list_vis(ch, name, ch->carrying))) {
      return (FIND_OBJ_INV);
    }
  }
  if (IS_SET(bitvector, FIND_OBJ_ROOM)) {
    if ((*tar_obj = get_obj_in_list_vis(ch, name, world[ch->in_room].contents))) {
      return (FIND_OBJ_ROOM);
    }
  }
  if (IS_SET(bitvector, FIND_OBJ_WORLD)) {
    if ((*tar_obj = get_obj_vis(ch, name))) {
      return (FIND_OBJ_WORLD);
    }
  }
  return (0);
}


/* a function to scan for "all" or "all.x" */
int find_all_dots(char *arg)
{
  if (!strcmp(arg, "all"))
    return FIND_ALL;
  else if (!strncmp(arg, "all.", 4)) {
    strcpy(arg, arg + 4);
    return FIND_ALLDOT;
  } else
    return FIND_INDIV;
}


// dismount_char() / fr: Daniel Koepke (dkoepke@california.com)
//   If a character is mounted on something, we dismount them.  If
//   someone is mounting our character, then we dismount that someone.
//   This is used for cleaning up after a mount is cancelled by
//   something (either intentionally or by death, etc.)
void dismount_char(struct char_data *ch)
{
  if (RIDING(ch)) {
    RIDDEN_BY(RIDING(ch)) = NULL;
    RIDING(ch) = NULL;
  }
  if (RIDDEN_BY(ch)) {
    RIDING(RIDDEN_BY(ch)) = NULL;
    RIDDEN_BY(ch) = NULL;
  }
}


// mount_char() / fr: Daniel Koepke (dkoepke@california.com)
//   Sets _ch_ to mounting _mount_.  This does not make any checks
//   what-so-ever to see if the _mount_ is mountable, etc.  That is
//   left up to the calling function.  This does not present any
//   messages, either.
void mount_char(struct char_data *ch, struct char_data *mount)
{
  RIDING(ch) = mount;
  RIDDEN_BY(mount) = ch;
}

/* place a character in an object */
void char_to_object(struct char_data *ch, struct obj_data *obj)
{
  if (!ch || !obj)
    log("SYSERR: Illegal value(s) passed to char_to_object");
  else {
    ch->next_in_furniture = obj->people;
    obj->people = ch;

    if (GET_OBJ_TYPE(obj) == ITEM_FURNITURE)
      FURNITURE(ch) = obj;

    GET_X_CORD(ch) = GET_OBJ_X_CORD(obj);
    GET_Y_CORD(ch) = GET_OBJ_Y_CORD(obj);
    GET_Z_CORD(ch) = GET_OBJ_Z_CORD(obj);
  }
}

/* move a player out of an object */
void char_from_object(struct char_data *ch, struct obj_data *obj)
{
  struct char_data *temp;
  void dismount_char(struct char_data *ch);

  if (ch == NULL || obj == NULL || FURNITURE(ch) == NULL) {
    log("SYSERR: NULL in handler.c, char_from_object");
    exit(1);
  }
  /* Sort out what type of object it is */
  if (GET_OBJ_TYPE(obj) == ITEM_FURNITURE) {
    REMOVE_FROM_LIST(ch, obj->people, next_in_furniture);
    FURNITURE(ch) = NULL;
  }
}


EVENTFUNC(water_current_event)
{
  struct water_current_event_obj *wed = (struct water_current_event_obj *) event_obj;
  struct char_data *ch;
  struct obj_data *obj;
  int boat = FALSE, dir;
  char *speed;

  ch = wed->ch;
  obj = wed->obj;

  if (!ch)
    return 0;

  if (!GET_ACTION(ch))
    return 0;

  GET_ACTION(ch) = NULL;

  /* check and make sure there is still a current */
  if (world[ch->in_room].current_speed <= 0 ||
      world[ch->in_room].current_direction < 0)
    return 0;
  if (GET_Z_CORD(ch) > 0)
    return 0;

  /* Which dir will they move? */
  dir = world[ch->in_room].current_direction;

  /* Is this is drowning player or a boat? */
  boat = FALSE;

  if (!IS_WATER_CORD(ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch)))
    return 0;


  /* about to hit the shore, lets keep changing dir until we find a
     good one, or we run out of plausible choices */
  if (!HeadingForWaterCoord(ch, dir)) {
    dir = SOUTH;
    if (!HeadingForWaterCoord(ch, dir)) {
      dir = SOUTHEAST;
      if (!HeadingForWaterCoord(ch, dir)) {
	dir = SOUTHWEST;
	if (!HeadingForWaterCoord(ch, dir)) {
	  dir = EAST;
	  if (!HeadingForWaterCoord(ch, dir)) {
	    /* we are probably at the edge of a map. revert to
	       original direction. */
	    dir = world[ch->in_room].current_direction;
	  }
	}
      }
    }
  }
  /* Check if we can actually enter that cord */
  if (!boat && CantEnterCoord(ch, dir))
    return 0;

  /* set up some messaging if needed */
  speed = str_dup("");
  if (world[ch->in_room].current_speed > 90)
    speed = str_dup("raging ");
  else if (world[ch->in_room].current_speed > 70)
    speed = str_dup("swift ");
  else if (world[ch->in_room].current_speed > 40)
    speed = str_dup("");
  else if (world[ch->in_room].current_speed > 20)
    speed = str_dup("liesurely ");
  else if (world[ch->in_room].current_speed > 0)
    speed = str_dup("slow ");

  /* Determine where to move them */
  if (!boat) {
    sprintf(buf, "The %scurrent pulls you along.\r\n", speed);
    send_to_char(buf, ch);
    perform_move(ch, dir, TRUE);
  }
  FREE(event_obj);

  return 0;
}
