/* structs and defines for kingdoms, guilds, troops, land aquisition */

#define MAX_KINGDOMS    20

#define MAX_GUESTS	2
#define MAX_HOUSE_ROOMS 10 
#define MAX_CONTROLLED_LAND 100
#define START_HOUSE_VNUM 48000
#define END_HOUSE_VNUM   48999

#define HMODE_ERROR     0
#define HMODE_PRIVATE   1
#define HMODE_OPEN      2

#define HCONTROL_ERROR  0
#define HCONTROL_HOUSE  1
#define HCONTROL_GUILD  2
#define HCONTROL_CASTLE 3

/* kingdom types */
#define KINGDOM_TYPE_TOWN	1
#define KINGDOM_TYPE_PC		2
#define KINGDOM_TYPE_NPC	3

/* kingdom score adjust types */
#define KINGDOM_LAND_GAIN	1
#define KINGDOM_LAND_LOSS	2

/* objects in the castle_room zones we may want to load */
#define HOUSE_OUTER_DOOR	11007
#define HOUSE_INNER_DOOR	11008
#define HOUSE_FLAG	    	11012

struct house_control_rec {
   int vnum;			/* vnum of atrium        	*/
   int exit_num;		/* direction of house's exit	*/
   time_t built_on;		/* date this house was built	*/
   sh_int mode;			/* mode of ownership		*/
   sh_int type;			/* castle, guild, house		*/
   byte construction;           /* check if its in construction */
   char *owner;	 		/* name of house's owner	*/
   int owner_guild;             /* owners guild # (if any)      */
   int num_of_guests;		/* how many guests for house	*/
   char *guests[MAX_GUESTS];	/* names of house's guests	*/
   int num_of_rooms;            /* # of rooms in the house      */
   int room_vnums[MAX_HOUSE_ROOMS];   /* a list of each vnum          */
   time_t last_payment;		/* date of last house payment   */
   ush_int size; 		/* Size of castle, 1-9		*/
   long upgrades;		/* Extras purchased		*/
   int controlled_land[MAX_CONTROLLED_LAND];  /* map rooms owned */
   struct house_control_rec *next;

   /* for guildhalls, which dont' have normal entrance, just an "enter blah" */
   char *entrance_keyword;

   /* now we list the possible mobs, items, and rooms a guild could have */
   /* (so we can completely automate the loading of the guild hall at boot)*/
   sh_int wizard_golems;
   sh_int warrior_golems;
   sh_int cleric_golems;
   int inn_vnum;
   int fountain_vnum;
   int teleporter1_room;
   int teleporter1_dest;
   int teleporter2_room;
   int teleporter2_dest;
   int shop_vnum;
   int heal_vnum; 
   int board_vnum;
   int mouth_vnum;
   int holy_fount_vnum;
   int unholy_fount_vnum;
   int secret_entrance;

   struct house_sack_rec *sack_list;
};

struct house_sack_rec {
  
  char *name;  /* who's sacking */
  long time;  /* when they started */
  struct house_sack_rec *next;  /* next in line */ 
};

struct house_upgrade_rec {
   int vnum;        /* vnum of house being upgraded */
   long time;       /* time (real time) it was started */
   int type;        /* for mob/room/obj upgrade */
   int location;    /* where the mob/obj/room is going to be */
   int guild;       /* for guildhalls, who's it is */
   
   /* for room upgrades */
   int exit_dir;
   int door;
   char *door_keyword;


   struct house_upgrade_rec *next;  /* next in line */
};

#define HCONTROL_ADD_MAGE_GOLEM 1
#define HCONTROL_ADD_CLERIC_GOLEM 2
#define HCONTROL_ADD_WARRIOR_GOLEM 3
#define HCONTROL_ADD_CHEST 4
#define HCONTROL_ADD_TELEPORTER 5
#define HCONTROL_ADD_FOUNTAIN 6
#define HCONTROL_ADD_INN 7
#define HCONTROL_ADD_SHOP 8
#define HCONTROL_ADD_BOARD 9
#define HCONTROL_ADD_ROOM 10
#define HCONTROL_DESC_ROOM 11
#define HCONTROL_NAME_ROOM 12
#define HCONTROL_ADD_MOUTH 13
#define HCONTROL_ADD_HEAL 14
#define HCONTROL_ADD_HOLY 15
#define HCONTROL_ADD_UNHOLY 16
#define HCONTROL_INITIAL_HOUSE 17
#define HCONTROL_INITIAL_GUILD 18
#define HCONTROL_ADD_CABINET 19
#define HCONTROL_ADD_SECRET 20

/* Troop Stuff */
/* Item Troop has following values:
   0: troop kingdom
   1: troop count 
   2: troop type
   3:
   4:
   5:
   6:
   7:
*/

#define NUM_OF_TROOPS 10

/* different kind of troosp you can build */
/* old stuff
#define TROOP_NOBODY  0
#define TROOP_MILITIA 1
#define TROOP_INFANTRY 2
#define TROOP_CAVALRY  3
#define TROOP_SEIGE_ENGINES 4
#define TROOP_SAPPERS 5
#define TROOP_MONSTERS 6
#define TROOP_WINGED_CAVALRY 7
#define TROOP_U1 8
#define TROOP_U2 9
#define TROOP_U3 10
/*

/* Land resources */

#define FLAT_RESOURCE_NONE		0
#define FLAT_RESOURCE_FOOD		1
#define FLAT_RESOURCE_ORE		2

#define TROOP_FROM_BARRACKS	1
#define TROOP_FROM_TRANSIT	2
#define TROOP_FROM_ROOM		3

#define TROOP_TO_BARRACKS	1
#define TROOP_TO_TRANSIT	2
#define TROOP_TO_DEATH		3

#define TASK_ACQUIRE_LAND	1

#define NUM_RESOURCES		3

/* Kingdom score, stores all info about a kingdom */
struct kingdom_score_type{
   int land[NUM_SECT_TYPES];
   int food;
   int ore;
   int last_payment;
   int virtual_troops[NUM_TROOP_TYPES][NUM_TROOP_LEVELS][NUM_TROOP_OFFENSE][NUM_TROOP_DEFENSE];
   int troops_in_barracks[NUM_TROOP_TYPES][NUM_TROOP_LEVELS][NUM_TROOP_OFFENSE][NUM_TROOP_DEFENSE];   
   int troops_in_transit[NUM_TROOP_TYPES][NUM_TROOP_LEVELS][NUM_TROOP_OFFENSE][NUM_TROOP_DEFENSE];      
};

/*
   This is the global. kingdom_score should be 
   allocated on booting kingdoms, or upon kingdom
   creation.
*/
struct kingdom_global{
   struct kingdom_score_type *score;
};

#define TROOP_LOOP for(w=0;w<NUM_TROOP_TYPES;w++) \
	 	    for(x=0;x<NUM_TROOP_LEVELS;x++) \
                      for(y=0;y<NUM_TROOP_OFFENSE;y++) \
                        for(z=0;z<NUM_TROOP_DEFENSE;z++)

#define TROOP_TYPE_LOOP for(x=0;x<NUM_TROOP_LEVELS;x++) \
                          for(y=0;y<NUM_TROOP_OFFENSE;y++) \
                            for(z=0;z<NUM_TROOP_DEFENSE;z++)

#define TROOP_TYPE_LEVEL_LOOP for(y=0;y<NUM_TROOP_OFFENSE;y++) \
                                for(z=0;z<NUM_TROOP_DEFENSE;z++)

struct troop_values {
   int troops[NUM_TROOP_TYPES][NUM_TROOP_LEVELS][NUM_TROOP_OFFENSE][NUM_TROOP_DEFENSE];
};
