
/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "utils.h"
#include "db.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "spells.h"
#include "olc.h"
#include "events.h"

/* external structs */
extern struct char_data *character_list;
extern struct index_data *mob_index;
extern struct index_data *obj_index;
extern struct room_data *world;
extern struct str_app_type str_app[];
extern struct char_data *mob_proto;
extern struct time_info_data time_info;
extern int is_empty(int zone_nr);
extern int find_first_step(int src, int target);
extern char *dirs[];
extern char *race_types[];
extern char *spells[];
extern struct spell_info_type spell_info[TOP_SPELL_DEFINE + 1];

/* required external functions */
int CurrentSpellsMemmed(struct char_data *ch, int circle, char pray);
int GetSpellCircle(struct char_data *ch, int spellnum);
int MaxSpellsPerCircle(struct char_data *ch, int circle, byte sphere);
byte GetMaxCircle(struct char_data *ch, byte pray);
EVENTFUNC(cast_event);

/* Assign casting classes spells. Casting monsters should already be
 * set in the .mob file. As well, builders may pre-assign a few or all
 * of a classed mobs spells, creating uniqueness. 
 * Various notes are listed in sphere 1 cleric, and apply to all the rest.
 */
void GiveSpellsToMob(struct char_data *ch)
{
  int circle, maxcircle, num_needed = 0, spl, counter = 0;

  /* clear them all */
  for (spl = 0; spl < NUM_SPELLS; spl++)
    GET_SPELL(ch, spl) = 0;

  maxcircle = GetMaxCircle(ch, IS_PRIEST(ch));

  for (circle = 1; circle <= maxcircle; circle++) {
    /* for each circle, see how many more we need to add,
       taking into account any the builder has already
       given (if any) */
    num_needed = (MaxSpellsPerCircle(ch, circle, IS_PRIEST(ch)) -
		  CurrentSpellsMemmed(ch, circle, IS_PRIEST(ch)));
    num_needed = BOUNDED(0, num_needed, 10);	/* a healthy amount, imho */
    switch (circle) {
    case 1:
      switch (GET_CLASS(ch)) {
      case CLASS_CLERIC:
      case CLASS_PALADIN:
      case CLASS_MULTI_C_R:
      case CLASS_MULTI_C_F:
      case CLASS_MULTI_C_T:
      case CLASS_MULTI_C_I:
	for (counter = 0; counter < num_needed; counter++) {
	  /* Start spells out 1 below ready, so the +1 aspect ensures
	   * at least one is ready, and additionals are properly
	   * added as well
	   */
	  GET_SPELL(ch, SPELL_CURE_LIGHT) =
	      MAX(GET_SPELL(ch, SPELL_CURE_LIGHT), MEMINFO_READY - 1);
	  /* this block gives them up to 5 cure light spells. Once
	   * they reach that, they will fall through to the next 
	   * spell choice.
	   */
	  if (GET_SPELL(ch, SPELL_CURE_LIGHT) < MEMINFO_READY + 5)
	    GET_SPELL(ch, SPELL_CURE_LIGHT) += 1;
	}
	break;
      case CLASS_DRUID:
      case CLASS_RANGER:
	for (counter = 0; counter < num_needed; counter++) {
	  GET_SPELL(ch, SPELL_ENTANGLE) =
	      MAX(GET_SPELL(ch, SPELL_ENTANGLE), MEMINFO_READY - 1);
	  GET_SPELL(ch, SPELL_ENTANGLE) += 1;
	}
	break;
      case CLASS_MULTI_F_M:
      case CLASS_MULTI_M_T:
      case CLASS_MULTI_F_M_T:
      case CLASS_MAGIC_USER:
	for (counter = 0; counter < num_needed; counter++) {
	  /* up to 5 missles, then we'll pick other spells */
	  if (GET_SPELL(ch, SPELL_MAGIC_MISSILE) < MEMINFO_READY + 5) {
	    GET_SPELL(ch, SPELL_MAGIC_MISSILE) =
		MAX(GET_SPELL(ch, SPELL_MAGIC_MISSILE), MEMINFO_READY - 1);
	    GET_SPELL(ch, SPELL_MAGIC_MISSILE) += 1;
	  }
	  if (GET_SPELL(ch, SPELL_CHARM_PERSON) < MEMINFO_READY)
	    GET_SPELL(ch, SPELL_CHARM_PERSON) = MEMINFO_READY;
	  if (GET_SPELL(ch, SPELL_SLEEP) < MEMINFO_READY)
	    GET_SPELL(ch, SPELL_SLEEP) = MEMINFO_READY;
	  if (GET_SPELL(ch, SPELL_ENLARGE) < MEMINFO_READY)
	    GET_SPELL(ch, SPELL_ENLARGE) = MEMINFO_READY;
	}
	break;
      case CLASS_ILLUSIONIST:
      case CLASS_MULTI_F_I:
      case CLASS_MULTI_I_T:
	for (counter = 0; counter < num_needed; counter++) {
	  if (GET_SPELL(ch, SPELL_COLOR_SPRAY) < MEMINFO_READY + 2) {
	    GET_SPELL(ch, SPELL_COLOR_SPRAY) =
		MAX(GET_SPELL(ch, SPELL_COLOR_SPRAY), MEMINFO_READY - 1);
	    GET_SPELL(ch, SPELL_COLOR_SPRAY) += 1;
	  }
	  if (GET_SPELL(ch, SPELL_DARKNESS) < MEMINFO_READY)
	    GET_SPELL(ch, SPELL_DARKNESS) = MEMINFO_READY;
	}
	break;
      case CLASS_BARD:
	/* needs songs! */
	break;
      case CLASS_MULTI_C_M:
      case CLASS_MULTI_C_F_M:
	for (counter = 0; counter < num_needed; counter++) {
	  /* multi chooses 1 missle, then fills rest with cures */
	  if (GET_SPELL(ch, SPELL_MAGIC_MISSILE) < MEMINFO_READY)
	    GET_SPELL(ch, SPELL_MAGIC_MISSILE) = MEMINFO_READY;

	  GET_SPELL(ch, SPELL_CURE_LIGHT) =
	      MAX(GET_SPELL(ch, SPELL_CURE_LIGHT), MEMINFO_READY - 1);
	  GET_SPELL(ch, SPELL_CURE_LIGHT) += 1;
	}
	break;
      default:
	break;
      }
      break;
    case 2:
      switch (GET_CLASS(ch)) {
      case CLASS_CLERIC:
      case CLASS_MULTI_C_T:
      case CLASS_MULTI_C_R:
      case CLASS_MULTI_C_F:
      case CLASS_PALADIN:
	for (counter = 0; counter < num_needed; counter++) {
	  if (GET_SPELL(ch, SPELL_RESIST_FIRE) < MEMINFO_READY)
	    GET_SPELL(ch, SPELL_RESIST_FIRE) = MEMINFO_READY;
	  if (GET_SPELL(ch, SPELL_SILENCE_15) < MEMINFO_READY)
	    GET_SPELL(ch, SPELL_SILENCE_15) = MEMINFO_READY;
	  GET_SPELL(ch, SPELL_HOLD_PERSON) =
	      MAX(GET_SPELL(ch, SPELL_HOLD_PERSON), MEMINFO_READY - 1);
	  GET_SPELL(ch, SPELL_HOLD_PERSON) += 1;
	}
	break;
      case CLASS_DRUID:
      case CLASS_RANGER:
	for (counter = 0; counter < num_needed; counter++) {
	  if (GET_SPELL(ch, SPELL_HEAT_METAL) < MEMINFO_READY)
	    GET_SPELL(ch, SPELL_HEAT_METAL) = MEMINFO_READY;
	  GET_SPELL(ch, SPELL_CURE_LIGHT) =
	      MAX(GET_SPELL(ch, SPELL_CURE_LIGHT), MEMINFO_READY - 1);
	  GET_SPELL(ch, SPELL_CURE_LIGHT) += 1;
	}
	break;
      case CLASS_MAGIC_USER:
      case CLASS_MULTI_M_T:
      case CLASS_MULTI_F_M:
      case CLASS_MULTI_F_M_T:
	for (counter = 0; counter < num_needed; counter++) {
	  if (GET_SPELL(ch, SPELL_MIRROR_IMAGE) < MEMINFO_READY)
	    GET_SPELL(ch, SPELL_MIRROR_IMAGE) = MEMINFO_READY;
	  if (GET_SPELL(ch, SPELL_INVISIBILITY) < MEMINFO_READY)
	    GET_SPELL(ch, SPELL_INVISIBILITY) = MEMINFO_READY;
	  GET_SPELL(ch, SPELL_STINKING_CLOUD) =
	      MAX(GET_SPELL(ch, SPELL_STINKING_CLOUD), MEMINFO_READY - 1);
	  GET_SPELL(ch, SPELL_STINKING_CLOUD) += 1;
	}
	break;
      case CLASS_ILLUSIONIST:
      case CLASS_MULTI_F_I:
      case CLASS_MULTI_I_T:
	for (counter = 0; counter < num_needed; counter++) {
	  if (GET_SPELL(ch, SPELL_MIRROR_IMAGE) < MEMINFO_READY)
	    GET_SPELL(ch, SPELL_MIRROR_IMAGE) = MEMINFO_READY;
	  if (GET_SPELL(ch, SPELL_INVISIBILITY) < MEMINFO_READY)
	    GET_SPELL(ch, SPELL_INVISIBILITY) = MEMINFO_READY;
	  GET_SPELL(ch, SPELL_FOG_CLOUD) =
	      MAX(GET_SPELL(ch, SPELL_FOG_CLOUD), MEMINFO_READY - 1);
	  GET_SPELL(ch, SPELL_FOG_CLOUD) += 1;
	}
	break;
      case CLASS_BARD:
	break;
      case CLASS_MULTI_C_M:
      case CLASS_MULTI_C_F_M:
	break;
      case CLASS_MULTI_C_I:
	break;
      default:
	break;
      }
    case 3:
      switch (GET_CLASS(ch)) {
      case CLASS_CLERIC:
      case CLASS_MULTI_C_T:
      case CLASS_MULTI_C_R:
      case CLASS_MULTI_C_F:
      case CLASS_PALADIN:
	break;
      case CLASS_DRUID:
      case CLASS_RANGER:
	break;
      case CLASS_MAGIC_USER:
      case CLASS_MULTI_M_T:
      case CLASS_MULTI_F_M:
      case CLASS_MULTI_F_M_T:
	for (counter = 0; counter < num_needed; counter++) {
	  if (GET_SPELL(ch, SPELL_FIREBALL) < MEMINFO_READY + 3) {
	    GET_SPELL(ch, SPELL_FIREBALL) =
		MAX(GET_SPELL(ch, SPELL_FIREBALL), MEMINFO_READY - 1);
	    GET_SPELL(ch, SPELL_FIREBALL) += 1;
	  }
	}
	break;
      case CLASS_ILLUSIONIST:
      case CLASS_MULTI_F_I:
      case CLASS_MULTI_I_T:
	break;
      case CLASS_BARD:
	break;
      case CLASS_MULTI_C_M:
      case CLASS_MULTI_C_F_M:
	break;
      case CLASS_MULTI_C_I:
	break;
      default:
	break;
      }
    case 4:
      switch (GET_CLASS(ch)) {
      case CLASS_CLERIC:
      case CLASS_MULTI_C_T:
      case CLASS_MULTI_C_R:
      case CLASS_MULTI_C_F:
      case CLASS_PALADIN:
	break;
      case CLASS_DRUID:
      case CLASS_RANGER:
	break;
      case CLASS_MAGIC_USER:
      case CLASS_MULTI_M_T:
      case CLASS_MULTI_F_M:
      case CLASS_MULTI_F_M_T:
	for (counter = 0; counter < num_needed; counter++) {
	  if (GET_SPELL(ch, SPELL_CONFUSION) < MEMINFO_READY + 3) {
	    GET_SPELL(ch, SPELL_CONFUSION) =
		MAX(GET_SPELL(ch, SPELL_CONFUSION), MEMINFO_READY - 1);
	    GET_SPELL(ch, SPELL_CONFUSION) += 1;
	  }
	}
	break;
      case CLASS_ILLUSIONIST:
      case CLASS_MULTI_F_I:
      case CLASS_MULTI_I_T:
	break;
      case CLASS_BARD:
	break;
      case CLASS_MULTI_C_M:
      case CLASS_MULTI_C_F_M:
	break;
      case CLASS_MULTI_C_I:
	break;
      default:
	break;
      }
    case 5:
      switch (GET_CLASS(ch)) {
      case CLASS_CLERIC:
      case CLASS_MULTI_C_T:
      case CLASS_MULTI_C_R:
      case CLASS_MULTI_C_F:
      case CLASS_PALADIN:
	break;
      case CLASS_DRUID:
      case CLASS_RANGER:
	break;
      case CLASS_MAGIC_USER:
      case CLASS_MULTI_M_T:
      case CLASS_MULTI_F_M:
      case CLASS_MULTI_F_M_T:
	for (counter = 0; counter < num_needed; counter++) {
	  if (GET_SPELL(ch, SPELL_CLOUDKILL) < MEMINFO_READY + 3) {
	    GET_SPELL(ch, SPELL_CLOUDKILL) =
		MAX(GET_SPELL(ch, SPELL_CLOUDKILL), MEMINFO_READY - 1);
	    GET_SPELL(ch, SPELL_CLOUDKILL) += 1;
	  }
	}
	break;
      case CLASS_ILLUSIONIST:
      case CLASS_MULTI_F_I:
      case CLASS_MULTI_I_T:
	break;
      case CLASS_BARD:
	break;
      case CLASS_MULTI_C_M:
      case CLASS_MULTI_C_F_M:
	break;
      case CLASS_MULTI_C_I:
	break;
      default:
	break;
      }
    case 6:
      switch (GET_CLASS(ch)) {
      case CLASS_CLERIC:
      case CLASS_MULTI_C_T:
      case CLASS_MULTI_C_R:
      case CLASS_MULTI_C_F:
      case CLASS_PALADIN:
	break;
      case CLASS_DRUID:
      case CLASS_RANGER:
	break;
      case CLASS_MAGIC_USER:
      case CLASS_MULTI_M_T:
      case CLASS_MULTI_F_M:
      case CLASS_MULTI_F_M_T:
	break;
      case CLASS_ILLUSIONIST:
      case CLASS_MULTI_F_I:
      case CLASS_MULTI_I_T:
	break;
      case CLASS_BARD:
	break;
      case CLASS_MULTI_C_M:
      case CLASS_MULTI_C_F_M:
	break;
      case CLASS_MULTI_C_I:
	break;
      default:
	break;
      }
    case 7:
      switch (GET_CLASS(ch)) {
      case CLASS_CLERIC:
      case CLASS_MULTI_C_T:
      case CLASS_MULTI_C_R:
      case CLASS_MULTI_C_F:
      case CLASS_PALADIN:
	break;
      case CLASS_DRUID:
      case CLASS_RANGER:
	break;
      case CLASS_MAGIC_USER:
      case CLASS_MULTI_M_T:
      case CLASS_MULTI_F_M:
      case CLASS_MULTI_F_M_T:
	break;
      case CLASS_ILLUSIONIST:
      case CLASS_MULTI_F_I:
      case CLASS_MULTI_I_T:
	break;
      case CLASS_BARD:
	break;
      case CLASS_MULTI_C_M:
      case CLASS_MULTI_C_F_M:
	break;
      case CLASS_MULTI_C_I:
	break;
      default:
	break;
      }
    case 8:
      switch (GET_CLASS(ch)) {
      case CLASS_CLERIC:
      case CLASS_MULTI_C_T:
      case CLASS_MULTI_C_R:
      case CLASS_MULTI_C_F:
      case CLASS_PALADIN:
	break;
      case CLASS_DRUID:
      case CLASS_RANGER:
	break;
      case CLASS_MAGIC_USER:
      case CLASS_MULTI_M_T:
      case CLASS_MULTI_F_M:
      case CLASS_MULTI_F_M_T:
	break;
      case CLASS_ILLUSIONIST:
      case CLASS_MULTI_F_I:
      case CLASS_MULTI_I_T:
	break;
      case CLASS_BARD:
	break;
      case CLASS_MULTI_C_M:
      case CLASS_MULTI_C_F_M:
	break;
      case CLASS_MULTI_C_I:
	break;
      default:
	break;
      }
    case 9:
      switch (GET_CLASS(ch)) {
      case CLASS_CLERIC:
      case CLASS_MULTI_C_T:
      case CLASS_MULTI_C_R:
      case CLASS_MULTI_C_F:
      case CLASS_PALADIN:
	break;
      case CLASS_DRUID:
      case CLASS_RANGER:
	break;
      case CLASS_MAGIC_USER:
      case CLASS_MULTI_M_T:
      case CLASS_MULTI_F_M:
      case CLASS_MULTI_F_M_T:
	break;
      case CLASS_ILLUSIONIST:
      case CLASS_MULTI_F_I:
      case CLASS_MULTI_I_T:
	break;
      case CLASS_BARD:
	break;
      case CLASS_MULTI_C_M:
      case CLASS_MULTI_C_F_M:
	break;
      case CLASS_MULTI_C_I:
	break;
      default:
	break;
      }

    }				// switch circle

  }				// loop circles

}


int mob_cast_spell(struct char_data *ch, struct char_data *tch, struct obj_data *tobj, int spellnum, int dir, int distance)
{
  struct location_data *rtar = NULL;
  struct spell_event *se;
  int range_location = 0, real_range = 0, rge = 0;
  char range[80];

  /* dummy checks */
  if (!cast_spell(ch, tch, tobj, spellnum))
    return FALSE;

  /* somewhere in here, we need to check the dir and range args,
   * creating the location_data struct as needed. For now, all
   * mob spells are simple targets.
   */

  /* below is an almost exact copy of the latter half
   * of ACMD(do_cast)
   */
#if 1
  real_range = SINFO.range + SINFO.range_by_level * GET_LEVEL(ch);
  if (!real_range)
    real_range = 1;

  if (dir && distance)
    sprintf(range, "%d %d", distance, dir);
  else if (dir)
    sprintf(range, "%d", dir);

  if (IS_SET(SINFO.targets, TAR_ROOM_RANGE | TAR_ROOM_WORLD)) {
    CREATE(rtar, struct location_data, 1);
    range_location = find_spell_location(ch, range, rtar, spellnum);
    if (!range_location)
      return 0;			// We've already given an error message in find_spell_location

  }
  if (range_location && !WithinRange(ch, rtar, real_range, &rge)
      && !IS_SET(SINFO.targets, TAR_ROOM_WORLD)) {
    send_to_char("That location is out of range!\r\n", ch);
    FREE(rtar);
    return 0;
  }
#endif
  /* create and initialize the cast event */
  CREATE(se, struct spell_event, 1);
  se->ch = ch;
  se->in_loc = FALSE;

  if (tch) {
    sprintf(buf, "Targets name is %s and has an ID of %ld. I am casting %s.", GET_NAME(tch), GET_ID(tch), spells[spellnum]);
    send_to_char(buf, ch);
    se->target = GET_ID(tch);
  } else if (tobj) {
    se->target = GET_ID(tobj);
  } else if (range_location) {
    se->in_loc = TRUE;
    se->target = rtar->real_room;
    se->location.real_room = rtar->real_room;
    se->location.x_cord = rtar->x_cord;
    se->location.y_cord = rtar->y_cord;
    se->location.z_cord = rtar->z_cord;
    se->location.angle = rtar->angle;
    FREE(rtar);
  } else
    se->target = -1;

  //FREE(range);

  strcpy(se->arg, arg);
  se->spellnum = spellnum;

  GET_ACTION(ch) = event_create(cast_event, se, (SINFO.delay * PASSES_PER_SEC * SECS_PER_SEGMENT));

  SET_BIT(ch->char_specials.saved.affected_by2, AFF2_CASTING);

  /* it would be nice to have a different message for each class here */
  if (IS_PRIEST(ch) && IS_PRIEST_SPELL(spellnum)) {
    act("$n raises $s holy symbol and begins chanting.", FALSE, ch, NULL, NULL, PROXIMITY);
    act("You raise your holy symbol and begin chanting.", FALSE, ch, NULL, NULL, TO_CHAR);
  } else {
    act("$n begins to cast a spell.", FALSE, ch, NULL, NULL, PROXIMITY);
    act("You begin to cast your spell.", FALSE, ch, NULL, NULL, TO_CHAR);
  }
  return TRUE;
}



bool
CastClericSpell(struct char_data * ch)
{
  int spellnum = 0, dir = -1, distance = -1;
  struct char_data *target_ch = NULL;
  struct obj_data *target_obj = NULL;

  /* Attempt a healing first off */
  if ((GET_MAX_HIT(ch) / GET_HIT(ch)) > 2) {
    target_ch = ch;
//    send_to_char("DEBUG: looking for a spell to cast (heal)\r\n", ch);
    if (!spellnum && GET_SPELL(ch, SPELL_CURE_CRITIC) >= MEMINFO_READY)
      spellnum = SPELL_CURE_CRITIC;
    if (!spellnum && GET_SPELL(ch, SPELL_CURE_SERIOUS) >= MEMINFO_READY)
      spellnum = SPELL_CURE_SERIOUS;
    if (!spellnum && GET_SPELL(ch, SPELL_CURE_LIGHT) >= MEMINFO_READY)
      spellnum = SPELL_CURE_LIGHT;
  } else {
//    send_to_char("DEBUG: looking for a spell to cast (cleric)\r\n", ch);
  }
  /* cure other things here, blind deaf etc */
  /* next try some offense, higher to lower */
  /* circle 1 */
  if (!spellnum && GET_SPELL(ch, SPELL_ENTANGLE) >= MEMINFO_READY)
    spellnum = SPELL_ENTANGLE;

  if (IS_SET(SINFO.targets, TAR_ROOM_RANGE)) {
    /* mobs for now will simply target the spell a decent distance away */
    dir = 1 /*GET_FACING(ch) */ ;
    distance = 3;
  }
  if (spellnum) {
    if (mob_cast_spell(ch, target_ch, target_obj, spellnum, dir, distance))
      return TRUE;
  }
  return FALSE;
}

bool
CastMageSpell(struct char_data * ch)
{
  int spellnum = 0, dir = -1, distance = -1;
  struct char_data *target_ch = NULL;
  struct obj_data *target_obj = NULL;

  /* simplistic, it should work for now */
  if (FIGHTING(ch))
    target_ch = FIGHTING(ch);
  else {
    send_to_char("Hmm, we aren't fighting anyone, so we don't know how to target this.\r\n", ch);
    return 0;
  }
//  send_to_char("DEBUG: looking for a spell to cast (mage)\r\n", ch);
  /* higher to lower will work the best */
  /* circle 1 */
  if (!spellnum && GET_SPELL(ch, SPELL_COLOR_SPRAY) >= MEMINFO_READY)
    spellnum = SPELL_COLOR_SPRAY;
  if (!spellnum && GET_SPELL(ch, SPELL_SLEEP) >= MEMINFO_READY)
    spellnum = SPELL_SLEEP;
  if (!spellnum && GET_SPELL(ch, SPELL_CHARM_PERSON) >= MEMINFO_READY)
    spellnum = SPELL_CHARM_PERSON;
  if (!spellnum && GET_SPELL(ch, SPELL_DARKNESS) >= MEMINFO_READY)
    spellnum = SPELL_DARKNESS;
  /* defensive, needs to be moved around. Its here just so I dont
     forget about it later 
     if (!spellnum && GET_SPELL(ch, SPELL_ENLARGE) >= MEMINFO_READY)
     spellnum = SPELL_ENLARGE; */

  /* Always default to magic missle, its nice at any level */
  if (!spellnum && GET_SPELL(ch, SPELL_MAGIC_MISSILE) >= MEMINFO_READY)
    spellnum = SPELL_MAGIC_MISSILE;

  if (IS_SET(SINFO.targets, TAR_ROOM_RANGE)) {
    /* mobs for now will simply target the spell a decent distance away */
    dir = 1 /*GET_FACING(ch) */ ;
    distance = 3;
  }
  if (spellnum) {
//    send_to_char("DEBUG: found a spell to cast, attempting it now.\r\n", ch);
    if (mob_cast_spell(ch, target_ch, target_obj, spellnum, dir, distance))
      return TRUE;
  }
  return FALSE;
}
