

/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"
#include "quest.h"
#include "olc.h"
#include "boards.h"
#include "dg_scripts.h"
#include "rand_items.h"

struct index_data *obj_index;
struct obj_data *create_random_metal_armor(byte magic);
struct obj_data *create_random_leather_armor(byte magic);
struct obj_data *create_random_weapon(int level, byte magic);
extern int top_of_objt;
void create_random_potion(struct obj_data *obj);
struct obj_data *create_random_magic(void);
struct obj_data *create_random_scroll(struct obj_data *obj);
void random_gems(struct obj_data *gem, int base_value);
void random_jewelry(struct obj_data *gem, int base_value);
void CheckEqWorthUsing(struct char_data *ch, struct obj_data *obj);
struct obj_data *create_money(int amount, int type, int king);
void load_treasure_by_header(struct obj_data *obj, int type);
extern struct obj_data *obj_proto;
extern struct spell_info_type spell_info[TOP_SPELL_DEFINE + 1];
int AddSpellToSpellBook(struct char_data *ch, struct obj_data *obj, int spl);
extern bool IsMemmingClass(struct char_data *ch);
extern char *kingdom_list[];
extern int GetSpellCircle(struct char_data *ch, int spellnum);

const struct adj_data adjective_table[MAX_ADJ] =
{
  {"age", 100, 0},
  {"ageless", 100, 0},
  {"ancient", 300, ADJ_JEWELRY | ADJ_STAFF},
  {"antique", 200, ADJ_JEWELRY | ADJ_STAFF},
  {"arched", 100, ADJ_STAFF | ADJ_SHAPE},
  {"barbed", 100, ADJ_STAFF | ADJ_SHAPE},
  {"battered", 50,
   ADJ_NEGATIVE | ADJ_SHAPE | ADJ_STAFF | ADJ_JEWELRY},
  {"beautiful", 150,
   ADJ_POSITIVE | ADJ_JEWELRY | ADJ_STAFF},
  {"bent", 80, ADJ_SHAPE | ADJ_STAFF},
  {"black", 100, ADJ_COLOR | ADJ_STAFF | ADJ_JEWELRY},
  {"blazing", 200, ADJ_COLOR | ADJ_JEWELRY},
  {"bleak", 100, ADJ_STAFF | ADJ_JEWELRY},
  {"blistered", 100, ADJ_STAFF | ADJ_SHAPE},
  {"bloody", 100, 0},
  {"blunt", 100, ADJ_STAFF | ADJ_SHAPE},
  {"blurry", 80, ADJ_JEWELRY},
  {"bowed", 100, ADJ_STAFF | ADJ_SHAPE},
  {"bright", 120,
   ADJ_JEWELRY | ADJ_POSITIVE | ADJ_QUALITY},
  {"brilliant", 150,
   ADJ_JEWELRY | ADJ_POSITIVE | ADJ_QUALITY},
  {"brittle", 100, 0},
  {"bumpy", 100, ADJ_STAFF | ADJ_SHAPE},
  {"burnished", 130,
   ADJ_STAFF | ADJ_JEWELRY | ADJ_POSITIVE | ADJ_QUALITY},
  {"burnt", 50, ADJ_STAFF},
  {"carved", 130,
   ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"chromatic", 100, ADJ_JEWELRY},
  {"cloudy", 70, ADJ_JEWELRY | ADJ_NEGATIVE | ADJ_QUALITY},
  {"coiled", 100, ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"colorful", 110, ADJ_JEWELRY | ADJ_STAFF},
  {"colorless", 90, ADJ_JEWELRY | ADJ_STAFF},
  {"common", 70, ADJ_JEWELRY},
  {"course", 100, ADJ_STAFF | ADJ_SHAPE},
  {"cracked", 50,
   ADJ_SHAPE | ADJ_STAFF | ADJ_JEWELRY},
  {"crooked", 100, ADJ_STAFF | ADJ_SHAPE},
  {"crude", 50,
   ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"crushed", 30, 0},
  {"crystalline", 100, ADJ_STAFF},
  {"curious", 100, 0},
  {"curved", 100, ADJ_STAFF | ADJ_SHAPE},
  {"delicate", 200, ADJ_JEWELRY},
  {"dark", 100, ADJ_STAFF | ADJ_JEWELRY},
  {"dim", 80, ADJ_JEWELRY},
  {"dirty", 100, 0},
  {"distinctive", 130, ADJ_JEWELRY},
  {"drab", 70, ADJ_JEWELRY},
  {"dull", 80, ADJ_JEWELRY | ADJ_STAFF},
  {"eccentric", 100, 0},
  {"elaborate", 150, ADJ_JEWELRY | ADJ_STAFF},
  {"engraved", 150,
   ADJ_JEWELRY | ADJ_STAFF | ADJ_SHAPE},
  {"faint", 80,
   ADJ_JEWELRY},
  {"fiery", 500,
   ADJ_JEWELRY},
  {"filthy", 100, 0},
  {"fine", 150,
   ADJ_JEWELRY},
  {"flaming", 150, ADJ_JEWELRY},
  {"forked", 100, ADJ_STAFF | ADJ_SHAPE},
  {"fractal", 350, ADJ_JEWELRY},
  {"fragmented", 100, 0},
  {"frosty", 200, ADJ_JEWELRY},
  {"glazed", 180, ADJ_JEWELRY},
  {"glinting", 200, ADJ_JEWELRY},
  {"glittering", 300, ADJ_JEWELRY},
  {"glorious", 140, ADJ_JEWELRY},
  {"glossy", 140, ADJ_JEWELRY},
  {"glowing", 500, ADJ_JEWELRY},
  {"gnarled", 100, ADJ_STAFF | ADJ_SHAPE},
  {"grimy", 100, 0},
  {"hazy", 70, ADJ_JEWELRY},
  {"holy", 100, 0},
  {"horned", 100, ADJ_STAFF | ADJ_SHAPE},
  {"illuminated", 300, ADJ_JEWELRY},
  {"impressive", 300, ADJ_JEWELRY},
  {"inscribed", 100, ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"intricate", 300,
   ADJ_JEWELRY | ADJ_STAFF | ADJ_SHAPE},
  {"iridescent", 200, ADJ_JEWELRY},
  {"jaded", 100, 0},
  {"jagged", 100, ADJ_STAFF | ADJ_SHAPE},
  {"jewelled", 500, ADJ_STAFF},
  {"knotted", 100, ADJ_STAFF | ADJ_SHAPE},
  {"knurled", 100, ADJ_STAFF | ADJ_SHAPE},
  {"long", 100, ADJ_STAFF | ADJ_SHAPE},
  {"luminous", 200, ADJ_JEWELRY},
  {"magnificent", 150, ADJ_JEWELRY},
  {"majestic", 150, ADJ_JEWELRY},
  {"massive", 100, ADJ_STAFF | ADJ_SHAPE},
  {"mighty", 100, 0},
  {"muddy", 50, 0},
  {"murky", 50, 0},
  {"mysterious", 100, 0},
  {"mystical", 100, 0},
  {"neon", 300, ADJ_JEWELRY},
  {"noble", 200, 0},
  {"notched", 100, ADJ_STAFF | ADJ_SHAPE},
  {"obscure", 100, 0},
  {"odd", 100, 0},
  {"old", 120, 0},
  {"ornate", 200, ADJ_STAFF | ADJ_JEWELRY},
  {"oval", 100, ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"pale", 70, ADJ_JEWELRY | ADJ_STAFF},
  {"peculiar", 100, 0},
  {"plain", 100, ADJ_JEWELRY},
  {"pointy", 100, ADJ_STAFF | ADJ_SHAPE},
  {"polished", 150, ADJ_JEWELRY},
  {"prickly", 100, ADJ_STAFF | ADJ_SHAPE},
  {"prismatic", 150, ADJ_JEWELRY},
  {"pronged", 100, ADJ_STAFF | ADJ_SHAPE},
  {"radiant", 300, ADJ_JEWELRY},
  {"ragged", 100, ADJ_STAFF | ADJ_SHAPE},
  {"rainbow", 250, ADJ_JEWELRY},
  {"ravishing", 200, ADJ_JEWELRY},
  {"rough", 50,
   ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"round", 100, ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"rugged", 100, ADJ_STAFF | ADJ_SHAPE},
  {"runed", 500, ADJ_STAFF | ADJ_JEWELRY},
  {"rusty", 50, ADJ_STAFF | ADJ_JEWELRY},
  {"sacred", 100, 0},
  {"scaled", 100, ADJ_STAFF | ADJ_SHAPE},
  {"scarred", 60,
   ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"scored", 50,
   ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"serpentine", 100, ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"serrated", 100, ADJ_STAFF | ADJ_SHAPE},
  {"shadowy", 80, ADJ_JEWELRY},
  {"shattered", 50, 0},
  {"shimmering", 300, ADJ_JEWELRY},
  {"shiny", 150, ADJ_JEWELRY},
  {"shoddy", 50, 0},
  {"short", 100, ADJ_STAFF | ADJ_SHAPE},
  {"shriveled", 100, ADJ_STAFF | ADJ_SHAPE},
  {"shrouded", 100, 0},
  {"simple", 100, 0},
  {"sleek", 100, ADJ_STAFF | ADJ_SHAPE},
  {"slender", 100, ADJ_STAFF | ADJ_SHAPE},
  {"smoky", 130, ADJ_JEWELRY},
  {"smooth", 100, ADJ_STAFF | ADJ_SHAPE},
  {"sparkling", 250, ADJ_JEWELRY},
  {"spectral", 150, ADJ_JEWELRY},
  {"spiked", 100, ADJ_STAFF | ADJ_SHAPE},
  {"spiny", 100, ADJ_STAFF | ADJ_SHAPE},
  {"spiraled", 100, ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"splendid", 100, 0},
  {"splintered", 100, ADJ_STAFF | ADJ_SHAPE},
  {"square", 100, ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"stalwart", 100, ADJ_STAFF | ADJ_SHAPE},
  {"stout", 100, ADJ_STAFF | ADJ_SHAPE},
  {"striking", 120, ADJ_JEWELRY},
  {"stunning", 150, ADJ_JEWELRY},
  {"sturdy", 100, ADJ_STAFF | ADJ_SHAPE},
  {"superb", 200, ADJ_JEWELRY},
  {"tangled", 100, ADJ_STAFF | ADJ_SHAPE},
  {"tarnished", 80, ADJ_JEWELRY},
  {"tattered", 100, 0},
  {"thick", 100, ADJ_STAFF | ADJ_SHAPE},
  {"thin", 100, ADJ_STAFF | ADJ_SHAPE},
  {"tined", 100, ADJ_STAFF | ADJ_SHAPE},
  {"twisted", 100, ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"unadorned", 100, ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"unembellished", 100, ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"unformed", 100, ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"unscathed", 100, ADJ_STAFF | ADJ_JEWELRY | ADJ_SHAPE},
  {"varnished", 100, ADJ_STAFF | ADJ_JEWELRY},
  {"veneered", 100, ADJ_STAFF | ADJ_JEWELRY},
  {"vibrant", 300, ADJ_JEWELRY},
  {"vivid", 200, ADJ_JEWELRY},
  {"white", 100, ADJ_STAFF | ADJ_JEWELRY},
  {"winding", 100, ADJ_STAFF | ADJ_SHAPE},
  {"winged", 100, ADJ_STAFF | ADJ_SHAPE},
  {"wiry", 100, ADJ_STAFF | ADJ_SHAPE},
  {"withered", 100, ADJ_STAFF | ADJ_SHAPE},
  {"lastadj", 0, 0}
/* EDIT MAX_ADJECTIVE ABOVE IF ADD MORE */
};

const struct matl_data material_table[MAX_MATL] =
{
  {"wood", 1, 1, MATERIAL_SOFTWOOD, COLOR_BROWN},
  {"adamantite", 1000, 1000, MATERIAL_ADAMANTIUM, COLOR_SILVER},
  {"agate", 50, 200, MATERIAL_GEM, 0},
  {"alexandrite", 10, 200, MATERIAL_ELEMENT, 0},
  {"aluminum", 30, 150, MATERIAL_METAL, COLOR_LIGHT | COLOR_SILVER},
  {"amber", 90, 200, MATERIAL_GEM, 0},
  {"amethyst", 150, 200, MATERIAL_GEM, 0},
  {"antler", 20, 50, MATERIAL_BONE, COLOR_LIGHT | COLOR_BROWN},
  {"aquamarine", 120, 200, MATERIAL_GEM, 0},
  {"arsenic", 50, 1, MATERIAL_ELEMENT, 0},
  {"ash", 15, 30, MATERIAL_ORGANIC, COLOR_BROWN},
  {"azurite", 10, 200, MATERIAL_GEM, 0},
  {"balsa", 30, 10, MATERIAL_SOFTWOOD, COLOR_LIGHT | COLOR_BROWN},
  {"bamboo", 30, 40, MATERIAL_BAMBOO, COLOR_LIGHT | COLOR_BROWN | COLOR_GREEN},
  {"bark", 5, 5, MATERIAL_BARK, COLOR_BROWN},
  {"birch", 20, 30, MATERIAL_SOFTWOOD, COLOR_LIGHT | COLOR_BROWN | COLOR_WHITE},
  {"bloodstone", 40, 200, MATERIAL_GEM, 0},
  {"blue quartz", 20, 200, MATERIAL_GEM, COLOR_BLUE | COLOR_TRANSLUCENT},
  {"bone", 5, 60, MATERIAL_BONE, COLOR_WHITE},
  {"brass", 40, 130, MATERIAL_BRASS, COLOR_LIGHT | COLOR_GOLD | COLOR_ORANGE | COLOR_YELLOW},
  {"bronze", 60, 400, MATERIAL_BRONZE, COLOR_DARK | COLOR_GOLD},
  {"calcium", 40, 1, MATERIAL_ELEMENT, COLOR_WHITE},
  {"carbon", 70, 1, MATERIAL_ORGANIC, COLOR_BLACK},
  {"carnelian", 20, 200, MATERIAL_GEM, 0},
  {"chalcedony", 30, 200, MATERIAL_GEM, 0},
  {"cherrywood", 80, 20, MATERIAL_HARDWOOD, COLOR_DARK | COLOR_RED},
  {"chromium", 100, 100, MATERIAL_METAL, 0},
  {"chrysoberyl", 30, 200, MATERIAL_GEM, 0},
  {"chrysoprase", 30, 200, MATERIAL_GEM, 0},
  {"citrine", 30, 200, MATERIAL_ELEMENT, 0},
  {"clay", 8, 2, MATERIAL_ELEMENT, COLOR_RED | COLOR_BROWN},
  {"cobalt", 20, 1, MATERIAL_ELEMENT, COLOR_DARK | COLOR_BLUE},
  {"copper", 50, 100, MATERIAL_COPPER, COLOR_DARK | COLOR_GOLD | COLOR_ORANGE | COLOR_GREEN},
  {"coral", 40, 250, MATERIAL_BONE, 0},
{"diamond", 800, 1000, MATERIAL_DIAMOND, COLOR_TRANSLUCENT | COLOR_WHITE},
  {"dragon tooth", 500, 700, MATERIAL_BONE, 0},
  {"ebony", 30, 150, MATERIAL_ELEMENT, COLOR_BLACK},
  {"eggshell", 1, 2, MATERIAL_EGGSHELL, COLOR_WHITE | COLOR_LIGHT | COLOR_BROWN},
  {"electrum", 100, 200, MATERIAL_ELECTRUM, 0},
  {"elm", 10, 30, MATERIAL_HARDWOOD, COLOR_LIGHT | COLOR_BROWN},
  {"emerald", 300, 200, MATERIAL_GEM, COLOR_TRANSLUCENT | COLOR_GREEN},
  {"eucalyptus", 30, 20, MATERIAL_SOFTWOOD, COLOR_LIGHT | COLOR_BROWN},
  {"fire opal", 500, 200, MATERIAL_GEM, 0},
  {"flint", 50, 250, MATERIAL_ELEMENT, 0},
  {"fruit apple", 3, 7, MATERIAL_GENERICFOOD, 0},
  {"fruit grapes", 5, 2, MATERIAL_GENERICFOOD, 0},
  {"fur", 10, 10, MATERIAL_HIDE, 0},
  {"garnet", 100, 200, MATERIAL_GEM, 0},
  {"gold", 300, 50, MATERIAL_GOLD, COLOR_GOLD | COLOR_YELLOW},
  {"glass", 30, 70, MATERIAL_CRYSTAL, COLOR_TRANSPARENT},
  {"granite", 10, 150, MATERIAL_GRANITE, COLOR_BLACK | COLOR_WHITE},
  {"graphite", 10, 50, MATERIAL_ELEMENT, COLOR_BLACK},
  {"hematite", 20, 200, MATERIAL_GEM, 0},
  {"iodine", 50, 1, MATERIAL_LIQUID, COLOR_BLUE},
  {"iron", 40, 300, MATERIAL_IRON, COLOR_SILVER | COLOR_WHITE},
  {"ivory", 150, 100, MATERIAL_IVORY, COLOR_WHITE},
  {"jacinth", 30, 200, MATERIAL_GEM, 0},
  {"jade", 200, 200, MATERIAL_ELEMENT, COLOR_GREEN | COLOR_LIGHT},
  {"jasper", 30, 200, MATERIAL_GEM, 0},
  {"lapis lazuli", 20, 200, MATERIAL_GEM, 0},
  {"lead", 20, 20, MATERIAL_METAL, COLOR_DARK | COLOR_SILVER},
  {"leather", 40, 1, MATERIAL_LEATHER, COLOR_LIGHT | COLOR_DARK | COLOR_BROWN | COLOR_RED},
  {"magnesium", 250, 120, MATERIAL_METAL, COLOR_SILVER},
  {"malachite", 20, 200, MATERIAL_ELEMENT, 0},
  {"maple", 30, 30, MATERIAL_HARDWOOD, COLOR_DARK | COLOR_BROWN},
  {"marble", 100, 200, MATERIAL_MARBLE, COLOR_WHITE | COLOR_BLACK},
  {"meat raw", 1, 1, MATERIAL_GENERICFOOD, COLOR_RED},
  {"mercury", 300, 1, MATERIAL_ELEMENT, COLOR_SILVER},
  {"mithril", 1200, 1100, MATERIAL_MITHRIL, COLOR_LIGHT | COLOR_SILVER},
  {"moonstone", 30, 200, MATERIAL_GEM, 0},
  {"nickel", 20, 100, MATERIAL_METAL, COLOR_SILVER},
  {"oak", 30, 40, MATERIAL_HARDWOOD, COLOR_BROWN},
  {"obsidian", 40, 20, MATERIAL_ELEMENT, COLOR_BLACK},
  {"onyx", 200, 200, MATERIAL_ELEMENT, COLOR_BLACK},
  {"opal", 400, 200, MATERIAL_GEM, 0},
  {"palladium", 100, 200, MATERIAL_METAL, 0},
  {"pearl", 200, 30, MATERIAL_PEARL, COLOR_WHITE},
  {"peridot", 20, 200, MATERIAL_GEM, 0},
  {"pewter", 30, 15, MATERIAL_METAL, 0},
  {"phosphorous", 100, 1, MATERIAL_ELEMENT, 0},
  {"pine", 40, 30, MATERIAL_SOFTWOOD, COLOR_BROWN},
  {"platinum", 500, 35, MATERIAL_PLATINUM, COLOR_SILVER},
  {"potassium", 100, 1, MATERIAL_ELEMENT, 0},
  {"pyrite", 50, 30, MATERIAL_ELEMENT, COLOR_GOLD},
  {"quartz", 10, 200, MATERIAL_ELEMENT, COLOR_TRANSLUCENT | COLOR_WHITE},
  {"redwood", 60, 35, MATERIAL_HARDWOOD, COLOR_DARK | COLOR_BROWN | COLOR_RED},
  {"reed", 7, 3, MATERIAL_BAMBOO, 0},
  {"rhodochrosite", 20, 200, MATERIAL_GEM, 0},
  {"rock crystal", 30, 200, MATERIAL_GEM, 0},
  {"ruby", 600, 700, MATERIAL_GEM, COLOR_TRANSLUCENT | COLOR_RED},
  {"sapphire", 500, 600, MATERIAL_GEM, COLOR_TRANSLUCENT | COLOR_BLUE},
  {"sardonyx", 20, 200, MATERIAL_GEM, 0},
  {"shell", 30, 350, MATERIAL_EGGSHELL, COLOR_WHITE},
  {"silicon", 30, 40, MATERIAL_ELEMENT, COLOR_WHITE},
  {"silver", 200, 150, MATERIAL_SILVER, COLOR_SILVER},
  {"sodium", 50, 1, MATERIAL_ELEMENT, 0},
  {"spinel", 30, 200, MATERIAL_GEM, 0},
  {"star sapphire", 1000, 200, MATERIAL_GEM, COLOR_TRANSLUCENT | COLOR_LIGHT | COLOR_BLUE},
  {"star rose quartz", 700, 200, MATERIAL_GEM, COLOR_TRANSLUCENT | COLOR_LIGHT | COLOR_RED},
  {"star ruby", 1200, 200, MATERIAL_GEM, COLOR_TRANSLUCENT | COLOR_LIGHT | COLOR_RED},
  {"steel", 160, 900, MATERIAL_STEEL, COLOR_SILVER},
  {"stone", 1, 250, MATERIAL_STONE, 0},
  {"sulfur", 100, 1, MATERIAL_ELEMENT, 0},
  {"teak", 90, 30, MATERIAL_SOFTWOOD, COLOR_BROWN},
  {"tiger eye", 30, 200, MATERIAL_GEM, 0},
  {"tin", 20, 80, MATERIAL_METAL, COLOR_SILVER},
  {"titanium", 800, 700, MATERIAL_METAL, COLOR_SILVER | COLOR_LIGHT | COLOR_BLUE},
  {"topaz", 200, 200, MATERIAL_GEM, 0},
  {"tourmaline", 30, 200, MATERIAL_GEM, 0},
  {"turquoise", 250, 200, MATERIAL_GEM, 0},
  {"wax", 20, 1, MATERIAL_WAX, 0},
  {"wicker", 10, 5, MATERIAL_BAMBOO, 0},
  {"zinc", 100, 100, MATERIAL_METAL, COLOR_SILVER},
  {"zircon", 20, 200, MATERIAL_GEM, 0},
  {"lastmatl", 1, 1, 0, 0}
	/* EDIT MAX_MATERIAL ABOVE IF ADD MORE */
};

/* maybe these should be structs, also hold material type */
static char *const color_insignia_leather[2] =
{
  "&+L",
  "&+y"
};

static char *const color_insignia_metal[6] =
{
  "&+L",
  "&+Y",
  "&+W",
  "&+C",
  "",
  ""
};

static char *const scroll_table[] =
{
  "blackened", "stretched", "printed", "mass produced", "tattered",
  "tied", "encoded", "radiating", "dark", "darkened", "translucent",
/* 10 */
  "soft bark", "heavily inked", "hard", "flaked", "gilded", "modly",
  "onion-skin", "rolled", "pliable", "parchment", "tattered", "rolled",
/* 22 */
  "bound", "old", "strange", "runed", "encrypted", "bloodridden",
  "gold inlaid", "engraved", "crinkled", "folded", "painted",
/* 33 */
  "stone", "papyrus", "speckled", "ivory", "brittle", "decomposing",
  "white", "yellow", "papyrus", "flayed skin", "pressed leather",
/* 44 */
  "dirty", "dusty", "thin", "scratched", "silver inlaid", "wax",
  "chalked", "faded", "crimped", "ordinary", "fried", "dragon",
/* 56 */
  "wet", "bronzed", "long", "torn", "heavy", "cotton", "sealed",
  "burnt", "half eaten", "bonded", "messy", "inked", "clay",
/* 69 */
  "wax stained", "crumpled", "washed out", "lamb skin", "odd",
  "fringed", "moss covered", "dry", "brazen", "fine printed",
/* 79 */
  "stained", "soiled"
/* 81 */
};


const char *potion_color[MAX_POTION_COLORS + 1] =
{
  "&+Wwhite&n",
  "&+Lblack&n",
  "&+Mpurple&n",
  "&+Bblue&n",
  "&+bblue&n",
  "&+Rred&n",
  "&+rred&n",
  "&+ggreen&n",
  "&+Yyellow&n",
  "&+Lchalky&n",
  "&+Cwatery&n",
  "&+Br&+ya&+Gi&+gn&+Rb&+Mo&+bw&n",
  "&+Wcloudy&n",
  "lastcolor"
};

const struct weapon_mod_data weapon_mod_table[MAX_WEAPON_MOD] =
{
  {"adamantite", 1, 2, -2},
  {"barbed", 0, 1, 0},
  {"beautiful", 0, 0, 0},
  {"bone", 0, 0, 0},
  {"black", 0, 0, 0},
  {"blunt", 0, -3, 0},
  {"cracked", 0, -3, 0},
  {"crude", -2, -2, 0},
  {"curved", -1, 0, 0},
  {"dull", 0, -2, 0},
  {"engraved", 0, 0, 0},
  {"etched", 0, 0, 0},
  {"fine", 0, 1, 0},
  {"gold", -1, -1, 3},
  {"granite", -4, 4, 10},
  {"heavy", -1, 1, 2},
  {"iron", 0, 1, 1},
  {"jagged", 0, 1, 0},
  {"jeweled", 0, 0, 0},
  {"mithril", 1, 1, -2},
  {"obsidian", 0, 0, 0},
  {"ornate", 0, 0, 0},
  {"polished", 0, 0, 0},
  {"ragged", -1, -1, 0},
  {"runed", 2, 2, 0},
  {"serrated", 0, 1, 0},
  {"shiny", 0, 0, 0},
  {"silver", 0, -1, 2},
  {"slender", 0, 0, -2},
  {"steel", 0, 1, 0},
  {"sturdy", 0, 1, 1},
  {"tarnished", 0, 0, 0},
  {"titanium", 0, 1, -1},
  {"warped", -1, -2, 0},
  {"well balanced", 1, 0, -1},
  {"well crafted", 0, 1, -1}
};

/************ add 2 more to this struct */
const struct weapon_data weapon_table[MAX_WEAPON] =
{
  {"a", "dagger", 1, 4, 1, 4, 2, ITEM_WEAPON, 1, 0, 0, 0},
  {"a", "knife", 1, 4, 1, 4, 3, ITEM_WEAPON, 1, 0, 0, 0},
  {"a", "hand axe", 1, 6, 1, 6, 5, ITEM_WEAPON, 3, 0, 0, 0},
  {"a", "club", 1, 6, 1, 6, 8, ITEM_WEAPON, 2, 0, 0, 0},
  {"a", "sling", 1, 6, 1, 6, 0, ITEM_FIREWEAPON, 1, 0, 0, 0},
  {"a", "flail", 1, 7, 1, 7, 4, ITEM_WEAPON, 3, 0, 0, 0},
  {"a", "warhammer", 1, 7, 1, 7, 7, ITEM_WEAPON, 3, 0, 0, 0},
  {"a", "short sword", 2, 3, 2, 3, 11, ITEM_WEAPON, 4, 0, 0, 0},
  {"a", "stiletto", 1, 7, 1, 7, 2, ITEM_WEAPON, 1, 0, 0, 0},
  {"a", "long sword", 1, 8, 1, 8, 3, ITEM_WEAPON, 4, 0, 0, 0},
  {"a", "throwing hammer", 1, 8, 1, 8, 7, ITEM_WEAPON, 3, 0, 0, 0},
  {"a", "sickle", 1, 8, 1, 8, 1, ITEM_WEAPON, 3, 0, 0, 0},
  {"a", "javelin", 1, 8, 1, 8, 12, ITEM_WEAPON, 2, 0, 0, 0},
  {"a", "spear", 1, 8, 1, 8, 12, ITEM_WEAPON, 5, 0, 0, 0},
  {"a", "mace", 1, 8, 1, 8, 8, ITEM_WEAPON, 6, 0, 0, 0},
  {"a", "morning star", 2, 4, 2, 4, 8, ITEM_WEAPON, 12, 0, 0, 0},
  {"a", "cutlass", 2, 4, 2, 4, 3, ITEM_WEAPON, 5, 0, 0, 0},
  {"a", "khopesh sword", 2, 4, 2, 4, 3, ITEM_WEAPON, 5, 0, 0, 0},
  {"a", "falchion", 2, 4, 2, 4, 3, ITEM_WEAPON, 5, 0, 0, 0},
  {"a", "short bow", 2, 4, 2, 4, 1, ITEM_FIREWEAPON, 2, 0, 0, 0},
  {"a", "sabre", 2, 4, 2, 4, 3, ITEM_WEAPON, 5, 0, 0, 0},
  {"a", "halfsword", 2, 4, 2, 4, 3, ITEM_WEAPON, 5, 0, 0, 0},
  {"an", "awl pike", 1, 10, 1, 10, 12, ITEM_WEAPON, 12, 0, 0, 0},
  {"a", "glaive", 1, 10, 1, 10, 3, ITEM_WEAPON, 8, 0, 0, 0},
  {"a", "battle axe", 1, 10, 1, 10, 10, ITEM_WEAPON, 7, 0, 0, 0},
  {"a", "quarterstaff", 1, 10, 1, 10, 7, ITEM_WEAPON, 4, 0, 0, 0},
  {"a", "chain whip", 1, 10, 1, 10, 4, ITEM_WEAPON, 1, 0, 0, 0},
  {"a", "rapier", 1, 10, 1, 10, 11, ITEM_WEAPON, 2, 0, 0, 0},
  {"a", "large club", 2, 5, 2, 5, 8, ITEM_WEAPON, 7, 0, 0, 0},
  {"a", "bastard sword", 2, 5, 2, 5, 3, ITEM_WEAPON, 10, 0, 0, 0},
  {"a", "spiked mace", 2, 5, 2, 5, 8, ITEM_WEAPON, 6, 0, 0, 0},
  {"a", "katana", 2, 5, 2, 5, 3, ITEM_WEAPON, 4, 0, 0, 0},
  {"a", "broadsword", 2, 5, 2, 5, 3, ITEM_WEAPON, 4, 0, 0, 0},
  {"a", "scimitar", 2, 5, 2, 5, 3, ITEM_WEAPON, 4, 0, 0, 0},
  {"a", "staff", 2, 5, 2, 5, 7, ITEM_WEAPON, 3, 0, 0, 0},
  {"a", "shiv", 2, 5, 2, 5, 2, ITEM_WEAPON, 1, 0, 0, 0},
  {"a", "long bow", 2, 5, 2, 5, 1, ITEM_FIREWEAPON, 3, 0, 0, 0},
  {"a", "trident", 3, 3, 3, 3, 11, ITEM_WEAPON, 5, 0, 0, 0},
  {"a", "military fork", 1, 12, 1, 12, 12, ITEM_WEAPON, 7, 0, 0, 0},
{"a", "long handled warhammer", 1, 12, 1, 12, 8, ITEM_WEAPON, 8, 0, 0, 0},
  {"a", "lucern hammer", 2, 6, 2, 6, 7, ITEM_WEAPON, 15, 0, 0, 0},
  {"a", "two handed sword", 2, 6, 2, 6, 3, ITEM_WEAPON, 15, 0, 0, 0},
  {"a", "composite bow", 2, 7, 2, 7, 1, ITEM_FIREWEAPON, 3, 0, 0, 0},
  {"a", "double bladed battle axe", 2, 6, 2, 6, 10, ITEM_WEAPON, 15, 0, 0, 0},
  {"a", "scythe", 3, 4, 3, 4, 1, ITEM_WEAPON, 12, 0, 0, 0},
  {"a", "halberd", 2, 7, 2, 7, 10, ITEM_WEAPON, 15, 0, 0, 0},
  {"a", "great sword", 2, 7, 2, 7, 3, ITEM_WEAPON, 12, 0, 0, 0},
  {"a", "long bladed dirk", 3, 4, 3, 4, 2, ITEM_WEAPON, 1, 0, 0, 0},
  {"a", "claymore", 2, 8, 2, 8, 10, ITEM_WEAPON, 17, 0, 0, 0},
  {"a", "crossbow", 2, 8, 2, 8, 2, ITEM_FIREWEAPON, 8, 0, 0, 0}
};


const struct metal_armor_adj_data metal_armor_adj_table[MAX_METAL_ARMOR_ADJ] =
{
  {"pewter", -2, 50, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"&+Lobsidian", -2, 0, ITEM_ALLOW_ALL_BUT_THIEF},
  {"battered", -1, 0, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"&+ycopper", -1, 20, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"&+Ygold", -1, 40, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"&+Mj&ne&+Bw&ne&+gl&ne&+Rd&n", -1, 0, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"tarnished", -1, 30, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"silver", -1, 30, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"elven", 0, -50, 0},
  {"&+Lblack", 0, 0, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"&+rblood-stained", 0, 0, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"&+Ybronze", 0, 0, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"&+Lburnt", 0, 0, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"carved", 0, 0, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"dirty", 0, 0, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"engraved", 0, 0, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"etched", 0, 0, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"heavy", 0, 40, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"&+Cpainted", 0, 0, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"&+Wpolished", 0, 0, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"scarred", 0, 0, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"&+Wshiny", 0, 0, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"&+Liron", 1, 20, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"sturdy", 1, 20, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"&+Wtitanium", 1, -20, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"&+Wsteel", 2, 10, ITEM_ALLOW_ALL_BUT_MAGE_THIEF},
  {"&+Cmithril", 2, 0, ITEM_ALLOW_ALL_BUT_THIEF},
  {"&+cadamantite", 3, 0, ITEM_ALLOW_ALL_BUT_THIEF}
};


const struct metal_armor_type_data metal_armor_type_table[MAX_METAL_ARMOR_TYPE] =
{
  {"%s cap&n", -2, 5, ITEM_WEAR_HEAD},
  {"%s helmet&n", 0, 10, ITEM_WEAR_HEAD},
  {"%s helm&n", 1, 12, ITEM_WEAR_HEAD},
  {"%s great helm&n", 2, 15, ITEM_WEAR_HEAD},
  {"%s gloves&n", -2, 2, ITEM_WEAR_HANDS},
  {"%s gauntlets&n", 0, 5, ITEM_WEAR_HANDS},
  {"%s skirt&n", -1, 20, ITEM_WEAR_LEGS},
  {"%s leggings&n", 0, 25, ITEM_WEAR_LEGS},
  {"%s greaves&n", 1, 28, ITEM_WEAR_LEGS},
  {"%s bracers&n", -2, 5, ITEM_WEAR_ARMS},
  {"%s sleeves&n", 0, 15, ITEM_WEAR_ARMS},
  {"%s vambraces&n", 1, 19, ITEM_WEAR_ARMS},
  {"%s belt&n", -2, 8, ITEM_WEAR_WAIST},
  {"%s girth&n", 0, 12, ITEM_WEAR_WAIST},
  {"%s shoes&n", -2, 6, ITEM_WEAR_FEET},
  {"%s boots&n", 0, 10, ITEM_WEAR_FEET},
  {"%s vest&n", -1, 30, ITEM_WEAR_BODY},
  {"%s shirt&n", 0, 35, ITEM_WEAR_BODY},
  {"%s hauberk&n", 0, 35, ITEM_WEAR_BODY},
  {"a suit of %s&n", 1, 40, ITEM_WEAR_BODY}
};

const struct metal_armor_data metal_armor_table[MAX_METAL_ARMOR] =
{
  {"ring mail", 4},
  {"brigandine", 4},
  {"chain mail", 5},
  {"banded mail", 6},
  {"splint mail", 6},
  {"plate", 6},
  {"full plate", 7},
  {"field plate", 7}
};


const struct leather_armor_adj_data leather_armor_adj_table[MAX_LEATHER_ARMOR_ADJ] =
{
  {"&+yragged", -1},
  {"&+ytorn", -1},
  {"&+Lblack", 0},
  {"bleached", 0},
  {"&+ybrown", 0},
  {"&+ydark", 0},
  {"&+yde&+Bc&+yor&+Ma&+yte&+Bd&+y", 0},
  {"&+cemb&+yroi&+cder&+yed", 0},
  {"&+yfine", 0},
  {"&+ggreen", 0},
  {"&+mindigo", 0},
  {"&+ylight brown", 0},
  {"&+golive", 0},
  {"&+ystitched", 0},
  {"&+ytan", 0},
  {"&+Mviolet", 0},
  {"&+Wwhite", 0},
  {"&+ywoven", 0},
  {"&+ythick", 1},
  {"&+ytough", 1},
};


const struct leather_armor_type_data leather_armor_type_table[MAX_LEATHER_ARMOR_TYPE] =
{
  {"%s cap&n", 2, ITEM_WEAR_HEAD},
  {"%s skullcap&n", 2, ITEM_WEAR_HEAD},
  {"%s gloves&n", 1, ITEM_WEAR_HANDS},
  {"%s leggings&n", 8, ITEM_WEAR_LEGS},
  {"%s pants&n", 8, ITEM_WEAR_LEGS},
  {"%s sleeves&n", 6, ITEM_WEAR_ARMS},
  {"%s sleeves&n", 6, ITEM_WEAR_ARMS},
  {"%s belt&n", 8, ITEM_WEAR_WAIST},
  {"%s shoes&n", 4, ITEM_WEAR_FEET},
  {"%s boots&n", 5, ITEM_WEAR_FEET},
  {"%s cuirass&n", 10, ITEM_WEAR_BODY},
  {"%s jacket&n", 10, ITEM_WEAR_BODY},
  {"%s jerkin&n", 10, ITEM_WEAR_BODY}
};

const struct leather_armor_data leather_armor_table[MAX_LEATHER_ARMOR] =
{
  {"silk", 0, 0},
  {"canvas", 1, 0},
  {"cotton", 1, 0},
  {"leather", 2, 0},
  {"soft leather", 2, 0},
  {"bearskin", 3, ITEM_ALLOW_ALL_BUT_MAGE},
  {"cuirbouilli", 3, ITEM_ALLOW_ALL_BUT_MAGE},
  {"hard leather", 3, 0},
  {"studded leather", 3, ITEM_ALLOW_ALL_BUT_MAGE},
  {"wolf hide", 3, ITEM_ALLOW_ALL_BUT_MAGE},
  {"spiked leather", 4, ITEM_ALLOW_ALL_BUT_MAGE},
};


/* Give these next two a clean object, a basic value of 1-14, and out
   pops a nice little item! */

void random_gems(struct obj_data *gem, int base_value)
{
  float bv;
  int bv_die, orig_base_value_place, current_base_value_place;
  int ignore_results_below_2 = 0, inc_or_dec_die, prop;
  char buf[MAX_STRING_LENGTH];
  static int bv_table[13][2] =
  {
    {25, 10.0},
    {50, 50.0},
    {70, 100.0},
    {90, 500.0},
    {99, 1000.0},
    {100, 5000.0},
    {0, 10000.0},
    {0, 25000.0},
    {0, 50000.0},
    {0, 100000.0},
    {0, 250000.0},
    {0, 500000.0},
    {0, 1000000.0}};
  static float decreased_bv_table[6] =
  {5.0, 1.0, 0.5, 0.25, 0.05};
  const char *properties[6][14] =
  {
    {				/* Ornamental Stones, base value 1 GP */
      "&+bazurite&n",
      "&+yba&n&+Bn&n&+yded a&N&+Rg&N&+yate&n",
      "&+Bblue quartz&n",
      "&+Weye a&n&+Lg&n&+Wate&n",
      "&+Lhematite&n",
      "&+bla&+Bp&n&+bis la&n&+Bz&n&+buli&n",
      "&+gm&+Ga&n&+gl&+Ga&n&+gc&+Gh&n&+gi&+Gt&N&+ge&n",
      "&+Wmo&n&+gs&+Ws ag&n&+ga&+Wte&n",
      "&+Lobsidian&n",
      "&+Rrhodochrosite&n",
      "&+ytiger &+Yeye&n",
      "&+Bturquiose&n",
      "",
      ""},

    {				/* Semi-precious Stones, base value 5 GP */
      "&+Yamber&n",
      "&+Rcarnelian&n",
      "&+Wchalcedony&n",
      "&+gchrysoprase&n",
      "&+ycitrine&n",
      "&+bjasper&n",
      "&+Wmoon&n&+Bstone&n",
      "&+Lonyx&n",
      "rock crystal",
      "&+Rs&n&+War&n&+Rdo&n&+Wny&n&+Rx&n",
      "smoky quartz",
      "&+Rstar &n&+Wrose&n&+R quartz&n",
      "&+Bzircon&n",
      ""},

    {				/* Fancy Stones, base value 10 GP */
      "&+yamber&n",
      "&+Yalexandrite&n",
      "&+mamethyst&n",
      "",
      "&+gchrysoberyl&n",
      "&+Rcoral&n",
      "&+rgarnet&n",
      "&+gjade&n",
      "&+Ljet&n",
      "&+Wpearl&n",
      "",
      "&+gspinel&n",
      "",
      "&+Btourmaline&n"},

    {				/* Fancy Stones, base value 50 GP */
      "",
      "",
      "",
      "&+gaquamarine&n",
      "",
      "",
      "&+rgarnet&n",
      "",
      "",
      "&+Wpearl&n",
      "&+gperidot&n",
      "&+Rspinel&n",
      "&+Ytopaz&n",
      ""},

    {				/* Gem Stones, base value 100 GP */
      "&+Lblack opal&n",
      "",
      "",
      "&+gemerald&n",
      "&+Rfire opal",
      "",
      "&+Wopal&n",
      "&+Moriental amethyst&n",
      "",
      "&+Yoriental topaz&n",
      "",
      "&+Bsapphire&n",
      "&+Rstar &n&+Wr&n&+Ruby&n",
      "&+Bstar &n&+Ws&n&+Bapphire&n"},

    {				/* Gem Stones, base value 500 GP */
      "&+Lblack opal&n",
      "&+Lblack sapphire&n",
      "&+Wdiamond&n",
      "&+gemerald&n",
      "&+rfire opal&n",
      "&+Rjacinth&n",
      "&+Wopal&n",
      "&+Boriental amethyst&n",
      "&+goriental emerald&n",
      "&+Yoriental topaz&n",
      "&+Rruby&n",
      "&+Bsapphire&n",
      "&+Rstar &n&+Wr&n&+Ruby&n",
      "&+Bstar &n&+Ws&n&+Bapphire&n"}
  };

  if (!gem)
    return;

  buf[0] = '\0';

  if (base_value) {
    bv = base_value;
    /* Calculate orig_base_value_place for later, since no gem may *
     * increase more than 7 places beyond its original base value  */
    orig_base_value_place = 0;
    while ((orig_base_value_place < 12)
	   && (base_value > bv_table[orig_base_value_place][1]))
      ++orig_base_value_place;
    if (orig_base_value_place >= 6)
      orig_base_value_place = 5;
    /* randomize base value */
  } else {
    bv_die = number(1, 100);
    orig_base_value_place = 0;
    while ((orig_base_value_place < 5)
	   && (bv_die >= bv_table[orig_base_value_place][0]))
      orig_base_value_place++;

    bv = bv_table[orig_base_value_place][1];
    /* Remember orig_base_value_place for later, since no gem may increase *
     * more than 7 places beyond its original base value              */
  }
  current_base_value_place = orig_base_value_place;

  ignore_results_below_2 = 0;	/* Later, if we roll a 10, we ignore 1's */

inc_or_dec:
  inc_or_dec_die = number(1, 10);

  switch (inc_or_dec_die) {
  case 1:
    if ((current_base_value_place <= 12)
	&& (current_base_value_place <= orig_base_value_place + 6)
	&& !ignore_results_below_2)
      bv = bv_table[++current_base_value_place][1];
    goto inc_or_dec;

  case 2:
    bv = 2.0 * bv;
    break;

  case 3:
    bv = bv * (1.0 + (((float) number(0, 6)) / 10.0));
    break;
  case 4:
  case 5:
  case 6:
  case 7:
  case 8:
    break;
  case 9:
    bv = bv * (1.0 - (((float) number(0, 4)) / 10.0));
    break;
  case 10:
    ignore_results_below_2 = 1;
    if (current_base_value_place >= orig_base_value_place - 4) {
      if (current_base_value_place > 0)
	bv = bv_table[--current_base_value_place][1];
      else
	bv = decreased_bv_table[-1 - (--current_base_value_place)];
      goto inc_or_dec;
    }
  } /* end switch */ ;

try_again:
  prop = number(0, 14);
  if (strlen(properties[orig_base_value_place][prop]) == 0)
    goto try_again;

  /* description and name */
  sprintf(buf, "a %s", properties[orig_base_value_place][prop]);
  gem->short_description = str_dup(buf);
  sprintf(buf, "A %s", properties[orig_base_value_place][prop]);
  gem->description = str_dup(buf);
  gem->name = str_dup("gem RANDOM");

  /* randomize some more */
  if (!number(0, 5)) {
    SET_BIT(GET_OBJ_EXTRA(gem), ITEM_EVIL);
    SET_BIT(GET_OBJ_EXTRA(gem), ITEM_NOSELL);
  }
  GET_OBJ_COST(gem) = bv * 100;
}

/* base value 1-6 */
void random_jewelry(struct obj_data *obj, int base_value)
{
  long bv;
  int bv_die, base_value_place, increase_from_gems, temp;
  char *nice_gems, *nice_work, buf[MAX_STRING_LENGTH];
  static int bv_table[7][5] =
  {
  /*  %     n   d     m  g  Jewelry value is n dice of size d, multiplied by m *
   *---    --  --  ----  -  If g = 1, this piece has gems and may increase     */
    {10, 10, 10, 1, 0},		/* base value place 0 */
    {20, 2, 6, 100, 0},		/* base value place 1 */
    {40, 3, 6, 100, 0},		/* base value place 2 */
    {50, 5, 6, 100, 0},		/* base value place 3 */
    {70, 1, 6, 1000, 1},	/* base value place 4 */
    {90, 2, 4, 1000, 1},	/* base value place 5 */
    {100, 2, 6, 1000, 1}};	/* base value place 6 */
  const char *descriptions[7] =
  {
    "wrought silver",
    "&+Ywrought&n silver and &+Ygold",
    "&+Ywrought gold",
    "&+Wwrought platinum",
    "silver and &+Rg&n&+Ye&n&+gm&n encrusted",
    "&+Ygold&n and &+Rg&n&+Ye&n&+gm&n encrusted&+Y",
    "&+Wplatinum and &+Rg&n&+Ye&n&+gm&n encrusted&+W"};
  const char *kind[51] =
  {
    "anklet",
    "arm band",
    "belt",
    "box(small)",
    "bracelet",
    "brooch",
    "buckle",
    "chain",
    "chalice",
    "choker",
    "clasp",
    "collar",
    "comb",
    "coronet",
    "crown",
    "decanter",
    "diadem",
    "earring",
    "fob",
    "goblet",
    "headband(fillet)",
    "idol",
    "locket",
    "medal",
    "medallion",
    "necklace",
    "pendant",
    "pin",
    "orb",
    "ring",
    "sceptre",
    "seal",
    "statuette",
    "tiara",
    "bracers",
    "cup",
    "dagger",
    "flask",
    "helm",
    "horn",
    "mirror",
    "periapt",
    "phylactery",
    "rod",
    "scarab",
    "skull",
    "shield",
    "staff",
    "sword",
    "talisman",
    "trident"};

  if (base_value) {
    bv = base_value;
    /* Calculate base_value_place for later, since no jewelry may *
     * increase more than 7 places beyond its original base value  */
    base_value_place = 0;
    while ((base_value_place < 12)
	   && (base_value > bv_table[base_value_place][1]))
      ++base_value_place;
    if (base_value_place >= 6)
      base_value_place = 5;

  } else {
    bv_die = number(0, 100);
    base_value_place = 0;
    while ((base_value_place < 6)
	   && (bv_die >= bv_table[base_value_place][0]))
      base_value_place++;
  }

  nice_work = str_dup("");

roll_bv_dice:
  if (base_value)
    bv = base_value;
  else {
    /* Next, roll the dice and determine bv */
    bv = 0;
    for (bv_die = 1; bv_die <= bv_table[base_value_place][1]; bv_die++)
      bv += number(0, bv_table[base_value_place][2])
	  * bv_table[base_value_place][3];
  }


check_for_exceptional:
  /* Check for exceptional workmanship */
  if (number(0, 10) == 1) {
    nice_work = str_dup(" (exquisite workmanship)");
    if (bv == bv_table[base_value_place][1] * bv_table[base_value_place][2]
	* bv_table[base_value_place][3])
      if (base_value_place < 6) {
	++base_value_place;
	goto roll_bv_dice;
      } else;			/* If already at max, do nothing */
    else {
      /* Set bv to maximum for this base_value_place */
      bv = bv_table[base_value_place][1] * bv_table[base_value_place][2]
	  * bv_table[base_value_place][3];
      goto check_for_exceptional;
    }
  }
  nice_gems = str_dup("");
  increase_from_gems = 0;
  if ((bv_table[base_value_place][4]) && (number(0, 6) == 1)) {
    increase_from_gems = 5000;
    nice_gems = str_dup(" (quality gems)");
    while ((number(0, 6) == 1) && (increase_from_gems <= 200000))
      increase_from_gems = increase_from_gems * 2;
  }
  bv += increase_from_gems;

  temp = number(0, 50);
  sprintf(buf, "%s jewelry RANDOM", strip_color(kind[temp]));
  obj->name = str_dup(buf);
  sprintf(buf, "a %s %s &n%s%s", descriptions[base_value_place],
	  kind[temp], nice_gems, nice_work);
  obj->short_description = str_dup(buf);
  sprintf(buf, "A %s %s&n", descriptions[base_value_place],
	  kind[temp]);
  obj->description = str_dup(buf);
  /* randomize some more */
  if (!number(0, 5)) {
    SET_BIT(GET_OBJ_EXTRA(obj), ITEM_EVIL);
    SET_BIT(GET_OBJ_EXTRA(obj), ITEM_NOSELL);
  }
  GET_OBJ_COST(obj) = bv * 100;
}

struct obj_data *
 create_random_metal_armor(byte magic)
{
  struct obj_data *obj = NULL;
  int armor, num, type, adj, color_insignia;;

  armor = dice(1, MAX_METAL_ARMOR) - 1;
  type = dice(1, MAX_METAL_ARMOR_TYPE) - 1;
  color_insignia = number(0, 5);

  if ((obj = read_object(RANDOM_MAGIC_ARMOR, VIRTUAL)) == NULL)
    return NULL;

  GET_OBJ_VAL(obj, 0) = metal_armor_table[armor].armor
      + metal_armor_type_table[type].modifier;
  GET_OBJ_WEIGHT(obj) = metal_armor_type_table[type].weight;
  GET_OBJ_WEAR(obj) = metal_armor_type_table[type].wear | ITEM_TAKE;

  if (number(0, 1)) {
    sprintf(buf, metal_armor_type_table[type].pattern,
	    metal_armor_table[armor].name);
    sprintf(buf1, "%s RANDOM", buf);
    obj->name = str_dup(buf1);
    sprintf(buf1, "%s%s&n", buf, color_insignia_metal[color_insignia]);
    obj->short_description = str_dup(buf1);
    CAP(buf);
    sprintf(buf1, "%s%s&n have been left here.", buf, color_insignia_metal[color_insignia]);
    obj->description = str_dup(buf1);
    GET_OBJ_ANTI(obj) = ITEM_ALLOW_ALL_BUT_MAGE_THIEF;
  } else {
    adj = dice(1, MAX_METAL_ARMOR_ADJ) - 1;

    GET_OBJ_VAL(obj, 0) += metal_armor_adj_table[adj].modifier;
    GET_OBJ_WEIGHT(obj) = (GET_OBJ_WEIGHT(obj) *
			(metal_armor_adj_table[adj].weight + 100)) / 100;
    GET_OBJ_ANTI(obj) = metal_armor_adj_table[adj].extra;

    sprintf(buf, "%s %s",
	    metal_armor_adj_table[adj].name,
	    metal_armor_table[armor].name);
    sprintf(buf, metal_armor_type_table[type].pattern, buf);
    sprintf(buf1, "%s RANDOM", buf);
    obj->name = str_dup(buf1);
    sprintf(buf1, "%s%s&n", buf, color_insignia_metal[color_insignia]);
    obj->short_description = str_dup(buf1);
    CAP(buf);
    sprintf(buf1, "%s%s&n have been left here.", buf, color_insignia_metal[color_insignia]);
    obj->description = str_dup(buf1);
  }
  if (magic && IS_SET(GET_OBJ_WEAR(obj), ITEM_WEAR_BODY)) {
    num = (number(-1, -3));
    obj->affected[0].location = APPLY_AC;
    obj->affected[0].modifier = num;
    SET_BIT(GET_OBJ_EXTRA(obj), ITEM_MAGIC);
  }
  return obj;
}

struct obj_data *
 create_random_leather_armor(byte magic)
{
  struct obj_data *obj = NULL;
  int armor, num, type, adj, color_insignia;

  armor = dice(1, MAX_LEATHER_ARMOR) - 1;
  type = dice(1, MAX_LEATHER_ARMOR_TYPE) - 1;
  color_insignia = number(0, 5);

  if ((obj = read_object(RANDOM_MAGIC_LEATHER_ARMOR, VIRTUAL)) == NULL)
    return NULL;

  GET_OBJ_VAL(obj, 0) = leather_armor_table[armor].armor;
  GET_OBJ_WEIGHT(obj) = leather_armor_type_table[type].weight;
  GET_OBJ_WEAR(obj) = leather_armor_type_table[type].wear | ITEM_TAKE;
  GET_OBJ_ANTI(obj) = leather_armor_table[armor].extra;

  if (number(0, 1)) {
    sprintf(buf, leather_armor_type_table[type].pattern,
	    leather_armor_table[armor].name);
    sprintf(buf1, "%s RANDOM", buf);
    obj->name = str_dup(buf1);
    sprintf(buf1, "%s%s&n", buf, color_insignia_leather[color_insignia]);
    obj->short_description = str_dup(buf1);
    CAP(buf);
    sprintf(buf1, "%s%s&n have been left here.", buf, color_insignia_leather[color_insignia]);
    obj->description = str_dup(buf1);
  } else {
    adj = dice(1, MAX_LEATHER_ARMOR_ADJ) - 1;
    GET_OBJ_VAL(obj, 0) =
	MAX(0, GET_OBJ_VAL(obj, 0) + leather_armor_adj_table[adj].modifier);

    sprintf(buf, "%s %s",
	    leather_armor_adj_table[adj].name,
	    leather_armor_table[armor].name);
    sprintf(buf, leather_armor_type_table[type].pattern, buf);
    sprintf(buf1, "%s RANDOM", buf);
    obj->name = str_dup(buf1);
    sprintf(buf1, "%s%s&n", buf, color_insignia_leather[color_insignia]);
    obj->short_description = str_dup(buf1);
    CAP(buf);
    sprintf(buf1, "%s%s&n have been left here.", buf, color_insignia_leather[color_insignia]);
    obj->description = str_dup(buf1);
  }
  if (magic && IS_SET(GET_OBJ_WEAR(obj), ITEM_WEAR_BODY)) {
    num = (number(-1, -3));
    obj->affected[0].location = APPLY_AC;
    obj->affected[0].modifier = num;
    SET_BIT(GET_OBJ_EXTRA(obj), ITEM_MAGIC);
  }
  return obj;
}

struct obj_data *
 create_random_weapon(int level, byte magic)
{
  struct obj_data *obj = NULL;
  int roll, num, type, adj;

  if (magic) {
    if ((obj = read_object(RANDOM_MAGIC_WEAPON, VIRTUAL)) == NULL)
      return NULL;
  } else {
    if ((obj = read_object(RANDOM_MUNDANE_WEAPON, VIRTUAL)) == NULL)
      return NULL;
  }
  if (level < 1) {
    roll = MAX_WEAPON;
  } else {
    roll = MIN(MAX_WEAPON, 3 * level);
  }
  type = dice(1, roll) - 1;

  GET_OBJ_TYPE(obj) = weapon_table[type].obj_type;
  GET_OBJ_EXTRA(obj) = weapon_table[type].flags;
  GET_OBJ_VAL(obj, 1) = weapon_table[type].no_dice;
  GET_OBJ_VAL(obj, 2) = weapon_table[type].size_dice;
  GET_OBJ_VAL(obj, 3) = weapon_table[type].dam_type;
  GET_OBJ_VAL(obj, 4) = weapon_table[type].lno_dice;
  GET_OBJ_VAL(obj, 5) = weapon_table[type].lsize_dice;
  GET_OBJ_VAL(obj, 6) = weapon_table[type].speed;
  GET_OBJ_VAL(obj, 7) = weapon_table[type].weapon_type;
  GET_OBJ_WEIGHT(obj) = weapon_table[type].weight;
  GET_OBJ_COST(obj) = weapon_table[type].cost;

  if (weapon_table[type].obj_type == ITEM_FIREWEAPON)
    GET_OBJ_WEAR(obj) = 1 | ITEM_WEAR_HOLD;

  if (obj->name)
    FREE(obj->name);
  if (obj->description)
    FREE(obj->description);
  if (obj->short_description)
    FREE(obj->short_description);

  if (number(0, 1)) {
    obj->name = str_dup(weapon_table[type].name);
    sprintf(buf, "%s %s",
	    weapon_table[type].a_an,
	    weapon_table[type].name);
    obj->short_description = str_dup(buf);
    sprintf(buf, "%s", obj->short_description);
    obj->description = str_dup(buf);
  } else {
    adj = dice(1, MAX_WEAPON_MOD) - 1;
    sprintf(buf, "%s %s",
	    weapon_table[type].name,
	    weapon_mod_table[adj].adjective);
    obj->name = str_dup(buf);
    sprintf(buf, "%s %s %s",
	    weapon_table[type].a_an,
	    weapon_mod_table[adj].adjective,
	    weapon_table[type].name);
    obj->short_description = str_dup(buf);
    sprintf(buf, "%s",
	    obj->short_description);
    obj->description = str_dup(buf);

    if (weapon_mod_table[adj].weight < 0) {
      if (GET_OBJ_WEIGHT(obj) != 0)
	GET_OBJ_WEIGHT(obj) =
	    MAX(1, GET_OBJ_WEIGHT(obj) + weapon_mod_table[adj].weight);
    } else
      GET_OBJ_WEIGHT(obj) += weapon_mod_table[adj].weight;

    /* non magical modifiers, mostly material related */
    if (weapon_mod_table[adj].hitroll != 0) {
      obj->affected[0].location = APPLY_HITROLL;
      obj->affected[0].modifier = weapon_mod_table[adj].hitroll;
    }
    if (weapon_mod_table[adj].damroll != 0) {
      obj->affected[1].location = APPLY_DAMROLL;
      obj->affected[1].modifier = weapon_mod_table[adj].hitroll;
    }
  }

  if (magic) {
    num = (number(1, 3));
    obj->affected[0].location = APPLY_HITROLL;
    obj->affected[0].modifier = num;
    obj->affected[1].location = APPLY_DAMROLL;
    obj->affected[1].modifier = num;
    SET_BIT(GET_OBJ_EXTRA(obj), ITEM_MAGIC);
  }
  return obj;
}



void create_random_potion(struct obj_data *obj)
{
  int color1, color2, color3;

  color1 = number(0, MAX_POTION_COLORS - 1);
  while (1) {
    color2 = number(0, MAX_POTION_COLORS - 1);
    if (color2 != color1)
      break;
  }
  while (1) {
    color3 = number(0, MAX_POTION_COLORS - 1);
    if (color3 != color1 && color3 != color2)
      break;
  }
  sprintf(buf, "%s vial RANDOM", obj->name);
  obj->name = str_dup(buf);

  if (number(0, 2)) {
    sprintf(buf, "[? id : %s \\ ? !id : a vial containing a %s liquid]", obj->short_description,
	    potion_color[color1]);
    obj->short_description = str_dup(buf);
    sprintf(buf, "[? id : %s \\ ? !id : A vial containing a %s liquid.]", obj->description,
	    potion_color[color1]);
    obj->description = str_dup(buf);
  } else if (number(0, 1)) {
    sprintf(buf, "[? id : %s \\ ? !id : a vial containing a %s liquid with %s swirls]",
	    obj->short_description,
	    potion_color[color1],
	    potion_color[color2]);
    obj->short_description = str_dup(buf);
    sprintf(buf, "[? id : %s \\ ? !id : A vial containing a %s liquid with %s swirls.]",
	    obj->description,
	    potion_color[color1],
	    potion_color[color2]);
    obj->description = str_dup(buf);
  } else {
    sprintf(buf, "[? id : %s \\ ? !id : a vial containing a %s liquid with %s swirls and %s spots]",
	    obj->short_description,
	    potion_color[color1],
	    potion_color[color2],
	    potion_color[color3]);
    obj->short_description = str_dup(buf);
    sprintf(buf, "[? id : %s \\ ? !id : A vial containing a %s liquid with %s swirls and %s spots.]",
	    obj->description,
	    potion_color[color1],
	    potion_color[color2],
	    potion_color[color3]);
    obj->description = str_dup(buf);
  }
  return;
}


/* The heart of the true randomizer, this will change an items strings
   to something else, depending on its type */
/**** do be sure to add an entry to load_treasure_by_header() if you add here */
void rename_object_random(struct obj_data *obj, int table)
{
  char *nbuf, *abuf;
  int adj = -1, matl = -1, armor, num, type, color_insignia;

  switch (table) {
  case TABLE_MAGIC_METAL_ARMOR:
    armor = dice(1, MAX_METAL_ARMOR) - 1;
    type = dice(1, MAX_METAL_ARMOR_TYPE) - 1;
    adj = dice(1, MAX_METAL_ARMOR_ADJ) - 1;
    color_insignia = number(0, 5);

    GET_OBJ_VAL(obj, 0) = metal_armor_table[armor].armor
	+ metal_armor_type_table[type].modifier;
    GET_OBJ_VAL(obj, 0) += metal_armor_adj_table[adj].modifier;
    GET_OBJ_WEIGHT(obj) = metal_armor_type_table[type].weight;
    GET_OBJ_WEIGHT(obj) = (GET_OBJ_WEIGHT(obj) *
			(metal_armor_adj_table[adj].weight + 100)) / 100;
    GET_OBJ_WEAR(obj) = metal_armor_type_table[type].wear | ITEM_TAKE;
    GET_OBJ_ANTI(obj) = metal_armor_adj_table[adj].extra;

    sprintf(buf1, "%s %s",
	    metal_armor_adj_table[adj].name,
	    metal_armor_table[armor].name);
    sprintf(buf, metal_armor_type_table[type].pattern, buf1);
    sprintf(buf1, "%s RANDOM", buf);
    obj->name = str_dup(buf1);
    sprintf(buf1, "%s%s&n", buf, color_insignia_metal[color_insignia]);
    obj->short_description = str_dup(buf1);
    CAP(buf);
    sprintf(buf1, "%s%s&n have been left here.", buf, color_insignia_metal[color_insignia]);
    obj->description = str_dup(buf1);

    if (IS_SET(GET_OBJ_WEAR(obj), ITEM_WEAR_BODY)) {
      num = (number(-1, -3));
      obj->affected[0].location = APPLY_AC;
      obj->affected[0].modifier = num;
      SET_BIT(GET_OBJ_EXTRA(obj), ITEM_MAGIC);
    }
    return;
  case TABLE_MAGIC_LEATHER_ARMOR:
    armor = dice(1, MAX_LEATHER_ARMOR) - 1;
    type = dice(1, MAX_LEATHER_ARMOR_TYPE) - 1;
    adj = dice(1, MAX_LEATHER_ARMOR_ADJ) - 1;
    color_insignia = number(0, 1);

    GET_OBJ_WEIGHT(obj) = leather_armor_type_table[type].weight;
    GET_OBJ_WEAR(obj) = leather_armor_type_table[type].wear | ITEM_TAKE;
    GET_OBJ_ANTI(obj) = leather_armor_table[armor].extra;
    GET_OBJ_VAL(obj, 0) = leather_armor_table[armor].armor;
    GET_OBJ_VAL(obj, 0) =
	MAX(0, GET_OBJ_VAL(obj, 0) + leather_armor_adj_table[adj].modifier);

    sprintf(buf1, "%s %s",
	    leather_armor_adj_table[adj].name,
	    leather_armor_table[armor].name);
    sprintf(buf, leather_armor_type_table[type].pattern, buf1);
    sprintf(buf1, "%s RANDOM", buf);
    obj->name = str_dup(buf1);
    sprintf(buf1, "%s%s&n", buf, color_insignia_leather[color_insignia]);
    obj->short_description = str_dup(buf1);
    CAP(buf);
    sprintf(buf1, "%s%s&n have been left here.", buf, color_insignia_leather[color_insignia]);
    obj->description = str_dup(buf1);

    if (IS_SET(GET_OBJ_WEAR(obj), ITEM_WEAR_BODY)) {
      num = (number(-1, -3));
      obj->affected[0].location = APPLY_AC;
      obj->affected[0].modifier = num;
      SET_BIT(GET_OBJ_EXTRA(obj), ITEM_MAGIC);
    }
    return;
  case TABLE_METAL_ARMOR:
    armor = dice(1, MAX_METAL_ARMOR) - 1;
    type = dice(1, MAX_METAL_ARMOR_TYPE) - 1;
    adj = dice(1, MAX_METAL_ARMOR_ADJ) - 1;
    color_insignia = number(0, 5);

    GET_OBJ_VAL(obj, 0) = metal_armor_table[armor].armor
	+ metal_armor_type_table[type].modifier;
    GET_OBJ_VAL(obj, 0) += metal_armor_adj_table[adj].modifier;
    GET_OBJ_WEIGHT(obj) = metal_armor_type_table[type].weight;
    GET_OBJ_WEIGHT(obj) = (GET_OBJ_WEIGHT(obj) *
			(metal_armor_adj_table[adj].weight + 100)) / 100;
    GET_OBJ_WEAR(obj) = metal_armor_type_table[type].wear | ITEM_TAKE;
    GET_OBJ_ANTI(obj) = metal_armor_adj_table[adj].extra;

    sprintf(buf1, "%s %s",
	    metal_armor_adj_table[adj].name,
	    metal_armor_table[armor].name);
    sprintf(buf, metal_armor_type_table[type].pattern, buf1);
    sprintf(buf1, "%s RANDOM", buf);
    obj->name = str_dup(buf1);
    sprintf(buf1, "%s%s&n", buf, color_insignia_metal[color_insignia]);
    obj->short_description = str_dup(buf1);
    CAP(buf);
    sprintf(buf1, "%s%s&n have been left here.", buf, color_insignia_metal[color_insignia]);
    obj->description = str_dup(buf1);
    return;
  case TABLE_LEATHER_ARMOR:
    armor = dice(1, MAX_LEATHER_ARMOR) - 1;
    type = dice(1, MAX_LEATHER_ARMOR_TYPE) - 1;
    adj = dice(1, MAX_LEATHER_ARMOR_ADJ) - 1;
    color_insignia = number(0, 1);

    GET_OBJ_WEIGHT(obj) = leather_armor_type_table[type].weight;
    GET_OBJ_WEAR(obj) = leather_armor_type_table[type].wear | ITEM_TAKE;
    GET_OBJ_ANTI(obj) = leather_armor_table[armor].extra;
    GET_OBJ_VAL(obj, 0) = leather_armor_table[armor].armor;
    GET_OBJ_VAL(obj, 0) =
	MAX(0, GET_OBJ_VAL(obj, 0) + leather_armor_adj_table[adj].modifier);

    sprintf(buf1, "%s %s",
	    leather_armor_adj_table[adj].name,
	    leather_armor_table[armor].name);
    sprintf(buf, leather_armor_type_table[type].pattern, buf1);
    sprintf(buf1, "%s RANDOM", buf);
    obj->name = str_dup(buf1);
    sprintf(buf1, "%s%s&n", buf, color_insignia_leather[color_insignia]);
    obj->short_description = str_dup(buf1);
    CAP(buf);
    sprintf(buf1, "%s%s&n have been left here.", buf, color_insignia_leather[color_insignia]);
    obj->description = str_dup(buf1);
    return;
  case TABLE_POTION:
    create_random_potion(obj);
    return;
  case TABLE_STAFF:
    while (1) {
      matl = number(0, MAX_MATL - 1);
      if (IS_SET(material_table[matl].flags, MATERIAL_METAL)
	  || IS_SET(material_table[matl].flags, MATERIAL_HARDWOOD)
	  || IS_SET(material_table[matl].flags, MATERIAL_SOFTWOOD)
	  || IS_SET(material_table[matl].flags, MATERIAL_ELEMENT))
	break;
    }
    nbuf = material_table[matl].name;
    while (1) {
      adj = number(0, MAX_ADJ - 1);
      if (IS_SET(adjective_table[adj].flags, ADJ_STAFF))
	break;
    }
    sprintf(buf, "%s RANDOM", obj->name);
    obj->name = str_dup(buf);
    abuf = adjective_table[adj].adjective;
    sprintf(buf1, "%s", obj->short_description);
    sprintf(buf, "[? id :%s \\ ? !id :%s %s %s staff]",
	    buf1, AN(abuf), abuf, nbuf);
    obj->short_description = str_dup(buf);
    sprintf(buf, "[? id :%s \\ ? !id :%s %s %s staff.]",
	    buf1, CAP_AN(abuf), abuf, nbuf);
    obj->description = str_dup(buf);
    return;
  case TABLE_WAND:
    while (1) {
      matl = number(0, MAX_MATL - 1);
      if (IS_SET(material_table[matl].flags, MATERIAL_METAL)
	  || IS_SET(material_table[matl].flags, MATERIAL_HARDWOOD)
	  || IS_SET(material_table[matl].flags, MATERIAL_SOFTWOOD)
	  || IS_SET(material_table[matl].flags, MATERIAL_ELEMENT))
	break;
    }
    nbuf = material_table[matl].name;
    while (1) {
      adj = number(0, MAX_ADJ - 1);
      if (IS_SET(adjective_table[adj].flags, ADJ_STAFF))
	break;
    }
    abuf = adjective_table[adj].adjective;
    sprintf(buf, "%s RANDOM", obj->name);
    obj->name = str_dup(buf);
    sprintf(buf1, "%s", obj->short_description);
    sprintf(buf, "[? id :%s \\ ? !id :%s %s %s staff]",
	    buf1, AN(abuf), abuf, nbuf);
    obj->short_description = str_dup(buf);
    sprintf(buf, "[? id :%s \\ ? !id :%s %s %s staff.]",
	    buf1, CAP_AN(abuf), abuf, nbuf);
    obj->description = str_dup(buf);
    return;
  case TABLE_SCROLL:
    nbuf = scroll_table[number(0, MAX_SCROLL)];
    sprintf(buf, "%s RANDOM", obj->name);
    obj->name = str_dup(buf);
    sprintf(buf1, "%s", obj->short_description);
    sprintf(buf, "[? id :%s \\ ? !id :%s %s scroll]",
	    buf1, AN(nbuf), nbuf);
    obj->short_description = str_dup(buf);
    sprintf(buf, "[? id :%s \\ ? !id :%s %s scroll.]",
	    buf1, CAP_AN(nbuf), nbuf);
    obj->description = str_dup(buf);
    return;
  }

  log("SYSERR: rename_object_random dropped past all returns.");
  return;
}

/* pick a random spell from a given class and circle, up to 20 each */
int choose_a_spell(int class, int circle)
{
  int spell, i;
  int spell_list[20];

  for (i = 0; i < 20; i++)
    spell_list[i] = -1;
  i = 0;
  for (spell = 1; spell < NUM_SPELLS; spell++) {
    if (class == CLASS_MAGIC_USER && spell_info[spell].school) {
      if (circle == spell_info[spell].min_level[CLASS_MAGIC_USER]) {
	if (i >= 20)
	  break;
	spell_list[i++] = spell;
      } else if (class == CLASS_CLERIC && spell_info[spell].sphere)
	if (circle == spell_info[spell].min_level[CLASS_CLERIC]) {
	  if (i >= 20)
	    break;
	  spell_list[i++] = spell;
	}
    }
  }
  spell = (spell_list[number(0, i)]);
  if (spell > -1)
    return spell;
  else if (class == CLASS_CLERIC)
    return SPELL_CURE_LIGHT;
  else
    return SPELL_DETECT_MAGIC;
}

struct obj_data *
 create_random_scroll(struct obj_data *obj)
{
  int count = 0, i, load, cost = 0;

  /* if we weren't passed a scroll already, then make one */
  if (!obj) {
    /* first we loop the range, and count how many we have */
    for (i = 1; i < 100; i++)	/* start at 1, since 0 is the header item */
      if (real_object(i + RANDOM_SCROLL) > 0)
	count++;
      else
	break;

    /* now, randomly load one */
    load = number(1, count);
    if (real_object(load + RANDOM_SCROLL) > 0)	/* call me paranoid */
      obj = read_object(load + RANDOM_SCROLL, VIRTUAL);
  }
  /* now we have the scroll item, let's see if it needs random spells */

  i = 1;
  if (GET_OBJ_ORIG_VNUM(obj) == 2117)
    GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_MAGIC_USER, number(1, 4));
  else if (GET_OBJ_ORIG_VNUM(obj) == 2118)
    GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_MAGIC_USER, number(1, 6));
  else if (GET_OBJ_ORIG_VNUM(obj) == 2119) {
    for (count = 0; count < 2; count++)
      GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_MAGIC_USER, number(1, 4));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2120) {
    for (count = 0; count < 2; count++)
      GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_MAGIC_USER, number(1, 8));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2121) {
    for (count = 0; count < 3; count++)
      GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_MAGIC_USER, number(1, 4));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2122) {
    for (count = 0; count < 3; count++)
      GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_MAGIC_USER, number(2, 9));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2122) {
    for (count = 0; count < 3; count++)
      GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_MAGIC_USER, number(2, 9));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2123) {
    for (count = 0; count < 4; count++)
      GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_MAGIC_USER, number(1, 6));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2124) {
    for (count = 0; count < 4; count++)
      GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_MAGIC_USER, number(1, 8));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2125) {
    for (count = 0; count < 5; count++)
      GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_MAGIC_USER, number(1, 6));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2126) {
    for (count = 0; count < 5; count++)
      GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_MAGIC_USER, number(1, 8));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2127) {
    for (count = 0; count < 6; count++)
      GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_MAGIC_USER, number(1, 6));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2128) {
    for (count = 0; count < 6; count++)
      GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_MAGIC_USER, number(3, 8));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2129) {
    for (count = 0; count < 7; count++)
      GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_MAGIC_USER, number(1, 8));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2130) {
    for (count = 0; count < 7; count++)
      GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_MAGIC_USER, number(2, 9));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2131) {
    for (count = 0; count < 7; count++)
      GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_MAGIC_USER, number(4, 9));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2132) {
    GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_CLERIC, number(1, 4));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2133) {
    GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_CLERIC, number(1, 6));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2134) {
    for (count = 0; count < 2; count++)
      GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_CLERIC, number(1, 4));
  } else if (GET_OBJ_ORIG_VNUM(obj) == 2135) {
    for (count = 0; count < 2; count++)
      GET_OBJ_VAL(obj, i++) = choose_a_spell(CLASS_CLERIC, number(1, 7));
  }
  /* name the thing! */
  rename_object_random(obj, TABLE_SCROLL);

  SET_BIT(GET_OBJ_EXTRA(obj), ITEM_MAGIC);

  if (!GET_OBJ_COST(obj))
    for (i = 1; i <= 7 && GET_OBJ_VAL(obj, i); i++)
      cost += 1000 * GetSpellCircle(NULL, GET_OBJ_VAL(obj, i));

  GET_OBJ_COST(obj) += cost;
  return obj;
}

/* randomly chooses which type to load */
struct obj_data *
 create_random_magic(void)
{
  struct obj_data *obj = NULL;
  int type;

  type = number(20, 34) * 100;
  obj = read_object(type, VIRTUAL);
  if (obj) {
    load_treasure_by_header(obj, type);
    return obj;
  } else
    return NULL;

  return obj;
}


/* loads items once we know where */
/* either to room or mob */
void load_treasure_by_type(struct char_data *mob, int type, int room)
{
  struct obj_data *obj = NULL;
  int i = 0, count = 0, tempval = 0;
  int c_perc, c_num, s_perc, s_num, g_perc, g_num, e_perc, e_num;
  int p_perc, p_num, gem_perc, gem_num, art_perc, art_num;

  c_perc = c_num = s_perc = s_num = g_perc = g_num = e_perc = e_num = 0;
  p_perc = p_num = gem_perc = gem_num = art_perc = art_num = 0;

  /* Lair Treasures */
  switch (type) {
  case 0:			/* A */
    c_perc = 25;
    c_num = 1500;
    s_perc = 30;
    s_num = 1000;
    g_perc = 40;
    g_num = 3000;
    e_perc = 35;
    e_num = 900;
    p_perc = 30;
    p_num = 450;
    gem_perc = 60;
    gem_num = 20;
    art_perc = 50;
    art_num = 6;
    for (i = 0; i < 3; i++) {
      if (number(1, 100) <= 30)
	switch (number(0, 3)) {
	case 0:
	  obj = create_random_magic();
	  break;
	case 1:
	  obj = read_object(RANDOM_POTION, VIRTUAL);
	  break;
	case 2:
	  if (number(0, 1))
	    obj = create_random_metal_armor(1);
	  else
	    obj = create_random_leather_armor(1);
	  break;
	case 3:
	  obj = create_random_weapon(-1, 1);
	  break;
	}
      if (obj && mob)
	obj_to_char(obj, mob);
      else if (obj && room > 1)
	obj_to_room(obj, room);
    }
    break;
  case 1:			/* B */
    c_perc = 50;
    c_num = 3000;
    s_perc = 25;
    s_num = 1500;
    g_perc = 25;
    g_num = 1000;
    e_perc = 25;
    e_num = 900;
    p_perc = 20;
    p_num = 450;
    gem_perc = 30;
    gem_num = 4;
    art_perc = 20;
    art_num = 2;
    for (i = 0; i < 2; i++) {
      if (number(1, 100) <= 10)
	switch (i) {
	case 0:
	  if (number(0, 1))
	    obj = create_random_metal_armor(1);
	  else
	    obj = create_random_leather_armor(1);
	  break;
	case 1:
	  obj = create_random_weapon(-1, 1);
	  break;
	}
      if (obj && mob)
	obj_to_char(obj, mob);
      else if (obj && room > 1)
	obj_to_room(obj, room);
    }
    break;
  case 2:			/* C */
    c_perc = 20;
    c_num = 5000;
    s_perc = 30;
    s_num = 3000;
    g_perc = 0;
    g_num = 0;
    e_perc = 10;
    e_num = 300;
    p_perc = 0;
    p_num = 0;
    gem_perc = 25;
    gem_num = 3;
    art_perc = 20;
    art_num = 2;
    for (i = 0; i < 2; i++) {
      if (number(1, 100) <= 10)
	switch (number(0, 3)) {
	case 0:
	  obj = create_random_magic();
	  break;
	case 1:
	  obj = read_object(RANDOM_POTION, VIRTUAL);
	  break;
	case 2:
	  if (number(0, 1))
	    obj = create_random_metal_armor(1);
	  else
	    obj = create_random_leather_armor(1);
	  break;
	case 3:
	  obj = create_random_weapon(-1, 1);
	  break;
	}
      if (obj && mob)
	obj_to_char(obj, mob);
      else if (obj && room > 1)
	obj_to_room(obj, room);
    }
    break;
  case 3:			/* D */
    c_perc = 10;
    c_num = 3000;
    s_perc = 15;
    s_num = 5000;
    g_perc = 50;
    g_num = 1500;
    e_perc = 25;
    e_num = 750;
    p_perc = 15;
    p_num = 300;
    gem_perc = 30;
    gem_num = 5;
    art_perc = 25;
    art_num = 3;
    for (i = 0; i < 3; i++) {
      if ((number(1, 100)) <= 15) {
	if (i == 3) {
	  obj = read_object(RANDOM_POTION, VIRTUAL);
	} else
	  switch (number(0, 3)) {
	  case 0:
	    obj = create_random_magic();
	    break;
	  case 1:
	    obj = read_object(RANDOM_POTION, VIRTUAL);
	    break;
	  case 2:
	    if (number(0, 1))
	      obj = create_random_metal_armor(1);
	    else
	      obj = create_random_leather_armor(1);
	    break;
	  case 3:
	    obj = create_random_weapon(-1, 1);
	    break;
	  }
      }
      if (obj && mob)
	obj_to_char(obj, mob);
      else if (obj && room > 1)
	obj_to_room(obj, room);
    }
    break;
  case 4:			/* E */
    c_perc = 5;
    c_num = 3000;
    s_perc = 25;
    s_num = 5000;
    g_perc = 25;
    g_num = 900;
    e_perc = 25;
    e_num = 2000;
    p_perc = 25;
    p_num = 450;
    gem_perc = 15;
    gem_num = 6;
    art_perc = 10;
    art_num = 3;
    for (i = 0; i < 3; i++) {
      if (number(1, 100) <= 25)
	switch (number(0, 3)) {
	case 0:
	  obj = create_random_magic();
	  break;
	case 1:
	  obj = read_object(RANDOM_POTION, VIRTUAL);
	  break;
	case 2:
	  if (number(0, 1))
	    obj = create_random_metal_armor(1);
	  else
	    obj = create_random_leather_armor(1);
	  break;
	case 3:
	  obj = create_random_weapon(-1, 1);
	  break;
	}
      if (obj && mob)
	obj_to_char(obj, mob);
      else if (obj && room > 1)
	obj_to_room(obj, room);
    }
    break;
  case 5:			/* F */
    c_perc = 0;
    c_num = 0;
    s_perc = 10;
    s_num = 9000;
    g_perc = 40;
    g_num = 3000;
    e_perc = 15;
    e_num = 2000;
    p_perc = 10;
    p_num = 1000;
    gem_perc = 20;
    gem_num = 10;
    art_perc = 10;
    art_num = 4;
    for (i = 0; i < 5; i++) {
      if (number(1, 100) <= 30)
	switch (number(0, 2)) {
	case 0:
	  obj = create_random_magic();
	  tempval = 0;
	  break;
	case 1:
	  obj = read_object(RANDOM_POTION, VIRTUAL);
	  tempval = 1;
	  break;
	case 2:
	  if (number(0, 1)) {
	    obj = create_random_metal_armor(1);
	    tempval = 2;
	  } else {
	    obj = create_random_leather_armor(1);
	    tempval = 3;
	  }
	  break;
	}
      if (obj && !obj->name)
	vlogf("RANDOM, object was missing info! %d", tempval);
      if (obj && mob)
	obj_to_char(obj, mob);
      else if (obj && room > 1)
	obj_to_room(obj, room);
    }
    break;
  case 6:			/* G */
    c_perc = 0;
    c_num = 0;
    s_perc = 0;
    s_num = 0;
    g_perc = 50;
    g_num = 10000;
    e_perc = 0;
    e_num = 0;
    p_perc = 50;
    p_num = 5000;
    gem_perc = 30;
    gem_num = 9;
    art_perc = 25;
    art_num = 3;
    for (i = 0; i < 5; i++) {
      if (number(1, 100) <= 35)
	switch (number(0, 3)) {
	case 0:
	  obj = create_random_magic();
	  break;
	case 1:
	  obj = read_object(RANDOM_POTION, VIRTUAL);
	  break;
	case 2:
	  if (number(0, 1))
	    obj = create_random_metal_armor(1);
	  else
	    obj = create_random_leather_armor(1);
	  break;
	case 3:
	  obj = create_random_weapon(-1, 1);
	  break;
	}
      if (obj && mob)
	obj_to_char(obj, mob);
      else if (obj && room > 1)
	obj_to_room(obj, room);
    }
    break;
  case 7:			/* H */
    c_perc = 25;
    c_num = 9000;
    s_perc = 40;
    s_num = 10000;
    g_perc = 45;
    g_num = 9000;
    e_perc = 55;
    e_num = 10000;
    p_perc = 40;
    p_num = 4000;
    gem_perc = 50;
    gem_num = 15;
    art_perc = 50;
    art_num = 10;
    for (i = 0; i < 6; i++) {
      if (number(1, 100) <= 15)
	switch (number(0, 3)) {
	case 0:
	  obj = create_random_magic();
	  break;
	case 1:
	  obj = read_object(RANDOM_POTION, VIRTUAL);
	  break;
	case 2:
	  if (number(0, 1))
	    obj = create_random_metal_armor(1);
	  else
	    obj = create_random_leather_armor(1);
	  break;
	case 3:
	  obj = create_random_weapon(-1, 1);
	  break;
	}
      if (obj && mob)
	obj_to_char(obj, mob);
      else if (obj && room > 1)
	obj_to_room(obj, room);
    }
    break;
  case 8:			/* I */
    c_perc = 0;
    c_num = 0;
    s_perc = 0;
    s_num = 0;
    g_perc = 0;
    g_num = 0;
    e_perc = 0;
    e_num = 0;
    p_perc = 30;
    p_num = 300;
    gem_perc = 55;
    gem_num = 6;
    art_perc = 50;
    art_num = 4;
    if (number(1, 100) <= 15)
      switch (number(0, 3)) {
      case 0:
	obj = create_random_magic();
	break;
      case 1:
	obj = read_object(RANDOM_POTION, VIRTUAL);
	break;
      case 2:
	if (number(0, 1))
	  obj = create_random_metal_armor(1);
	else
	  obj = create_random_leather_armor(1);
	break;
      case 3:
	obj = create_random_weapon(-1, 1);
	break;
      }
    if (obj && mob)
      obj_to_char(obj, mob);
    else if (obj && room > 1)
      obj_to_room(obj, room);
    break;

    /* Individual Treasures */
  case 9:			/* J */
    c_perc = 101;
    c_num = 24;
    s_perc = 0;
    s_num = 0;
    g_perc = 0;
    g_num = 0;
    e_perc = 0;
    e_num = 0;
    p_perc = 0;
    p_num = 0;
    gem_perc = 0;
    gem_num = 0;
    art_perc = 0;
    art_num = 0;
    break;
  case 10:			/* K */
    c_perc = 0;
    c_num = 0;
    s_perc = 101;
    s_num = 18;
    g_perc = 0;
    g_num = 0;
    e_perc = 0;
    e_num = 0;
    p_perc = 0;
    p_num = 0;
    gem_perc = 0;
    gem_num = 0;
    art_perc = 0;
    art_num = 0;
    break;
  case 11:			/* L */
    c_perc = 0;
    c_num = 0;
    s_perc = 0;
    s_num = 0;
    g_perc = 0;
    g_num = 0;
    e_perc = 101;
    e_num = 12;
    p_perc = 0;
    p_num = 0;
    gem_perc = 0;
    gem_num = 0;
    art_perc = 0;
    art_num = 0;
    break;
  case 12:			/* M */
    c_perc = 0;
    c_num = 0;
    s_perc = 0;
    s_num = 0;
    g_perc = 101;
    g_num = 8;
    e_perc = 0;
    e_num = 0;
    p_perc = 0;
    p_num = 0;
    gem_perc = 0;
    gem_num = 0;
    art_perc = 0;
    art_num = 0;
    break;
  case 13:			/* N */
    c_perc = 0;
    c_num = 0;
    s_perc = 0;
    s_num = 0;
    g_perc = 0;
    g_num = 0;
    e_perc = 0;
    e_num = 0;
    p_perc = 101;
    p_num = 6;
    gem_perc = 0;
    gem_num = 0;
    art_perc = 0;
    art_num = 0;
    break;
  case 14:			/* O */
    c_perc = 101;
    c_num = 40;
    s_perc = 101;
    s_num = 30;
    g_perc = 0;
    g_num = 0;
    e_perc = 0;
    e_num = 0;
    p_perc = 0;
    p_num = 0;
    gem_perc = 0;
    gem_num = 0;
    art_perc = 0;
    art_num = 0;
    break;
  case 15:			/* P */
    c_perc = 0;
    c_num = 0;
    s_perc = 101;
    s_num = 60;
    g_perc = 0;
    g_num = 0;
    e_perc = 101;
    e_num = 20;
    p_perc = 0;
    p_num = 0;
    gem_perc = 0;
    gem_num = 0;
    art_perc = 0;
    art_num = 0;
    break;
  case 16:			/* Q */
    c_perc = 0;
    c_num = 0;
    s_perc = 0;
    s_num = 0;
    g_perc = 0;
    g_num = 0;
    e_perc = 0;
    e_num = 0;
    p_perc = 0;
    p_num = 0;
    gem_perc = 101;
    gem_num = 4;
    art_perc = 0;
    art_num = 0;
    break;
  case 17:			/* R */
    c_perc = 0;
    c_num = 0;
    s_perc = 0;
    s_num = 0;
    g_perc = 101;
    g_num = 20;
    e_perc = 0;
    e_num = 0;
    p_perc = 101;
    p_num = 60;
    gem_perc = 101;
    gem_num = 8;
    art_perc = 101;
    art_num = 3;
    break;
  case 18:			/* S */
    c_perc = 0;
    c_num = 0;
    s_perc = 0;
    s_num = 0;
    g_perc = 0;
    g_num = 0;
    e_perc = 0;
    e_num = 0;
    p_perc = 0;
    p_num = 0;
    gem_perc = 0;
    gem_num = 0;
    art_perc = 0;
    art_num = 0;
    count = number(1, 8);
    for (i = 0; i < count; i++) {
      obj = read_object(RANDOM_POTION, VIRTUAL);
      if (obj && mob)
	obj_to_char(obj, mob);
      else if (obj && room > 1)
	obj_to_room(obj, room);
    }
    break;
  case 19:			/* T */
    c_perc = 0;
    c_num = 0;
    s_perc = 0;
    s_num = 0;
    g_perc = 0;
    g_num = 0;
    e_perc = 0;
    e_num = 0;
    p_perc = 0;
    p_num = 0;
    gem_perc = 0;
    gem_num = 0;
    art_perc = 0;
    art_num = 0;
    count = number(1, 4);
    for (i = 0; i < count; i++) {
      obj = create_random_scroll(0);
      if (obj && mob)
	obj_to_char(obj, mob);
      else if (obj && room > 1)
	obj_to_room(obj, room);
    }
    break;
  case 20:			/* U */
    c_perc = 0;
    c_num = 0;
    s_perc = 0;
    s_num = 0;
    g_perc = 0;
    g_num = 0;
    e_perc = 0;
    e_num = 0;
    p_perc = 0;
    p_num = 0;
    gem_perc = 90;
    gem_num = 16;
    art_perc = 80;
    art_num = 6;
    if (number(1, 100) <= 70)
      switch (number(0, 3)) {
      case 0:
	obj = create_random_magic();
	break;
      case 1:
	obj = read_object(RANDOM_POTION, VIRTUAL);
	break;
      case 2:
	if (number(0, 1))
	  obj = create_random_metal_armor(1);
	else
	  obj = create_random_leather_armor(1);
	break;
      case 3:
	obj = create_random_weapon(-1, 1);
	break;
      }
    if (obj && mob)
      obj_to_char(obj, mob);
    else if (obj && room > 1)
      obj_to_room(obj, room);

    break;
  case 21:			/* V */
    c_perc = 0;
    c_num = 0;
    s_perc = 0;
    s_num = 0;
    g_perc = 0;
    g_num = 0;
    e_perc = 0;
    e_num = 0;
    p_perc = 0;
    p_num = 0;
    gem_perc = 0;
    gem_num = 0;
    art_perc = 0;
    art_num = 0;
    for (i = 0; i < 2; i++) {
      if (number(1, 100) <= 15)
	switch (number(0, 2)) {
	case 0:
	  obj = create_random_magic();
	  break;
	case 1:
	  obj = read_object(RANDOM_POTION, VIRTUAL);
	  break;
	case 2:
	  obj = create_random_scroll(0);
	  break;
	}
      if (obj && mob)
	obj_to_char(obj, mob);
      else if (obj && room > 1)
	obj_to_room(obj, room);
    }

    break;
  case 22:			/* W */
    c_perc = 0;
    c_num = 0;
    s_perc = 0;
    s_num = 0;
    g_perc = 0;
    g_num = 0;
    e_perc = 0;
    e_num = 0;
    p_perc = 0;
    p_num = 0;
    gem_perc = 0;
    gem_num = 0;
    art_perc = 0;
    art_num = 0;
    for (i = 0; i < 3; i++) {
      if (number(1, 100) <= 60)
	switch (number(0, 4)) {
	case 0:
	  obj = create_random_magic();
	  break;
	case 1:
	  obj = read_object(RANDOM_POTION, VIRTUAL);
	  break;
	case 2:
	  if (number(0, 1))
	    obj = create_random_metal_armor(1);
	  else
	    obj = create_random_leather_armor(1);
	  break;
	case 3:
	  obj = create_random_weapon(-1, 1);
	  break;
	case 4:
	  obj = create_random_scroll(0);
	  break;
	}
      if (obj && mob)
	obj_to_char(obj, mob);
      else if (obj && room > 1)
	obj_to_room(obj, room);
    }

    break;
  case 23:			/* X */
    c_perc = 0;
    c_num = 0;
    s_perc = 0;
    s_num = 0;
    g_perc = 0;
    g_num = 0;
    e_perc = 0;
    e_num = 0;
    p_perc = 0;
    p_num = 0;
    gem_perc = 0;
    gem_num = 0;
    art_perc = 0;
    art_num = 0;
    for (i = 0; i < 2; i++) {
      obj = read_object(RANDOM_POTION, VIRTUAL);
      if (obj && mob)
	obj_to_char(obj, mob);
      else if (obj && room > 1)
	obj_to_room(obj, room);
    }

    break;
  case 24:			/* Y */
    c_perc = 0;
    c_num = 0;
    s_perc = 0;
    s_num = 0;
    g_perc = 0;
    g_num = 0;
    e_perc = 100;
    e_num = 600;
    p_perc = 0;
    p_num = 0;
    gem_perc = 0;
    gem_num = 0;
    art_perc = 0;
    art_num = 0;
    break;
  case 25:			/* Z */
    c_perc = 100;
    c_num = 150;
    s_perc = 100;
    s_num = 200;
    g_perc = 100;
    g_num = 200;
    e_perc = 100;
    e_num = 200;
    p_perc = 100;
    p_num = 100;
    gem_perc = 55;
    gem_num = 3;
    art_perc = 50;
    art_num = 6;
    for (i = 0; i < 3; i++) {
      if (number(1, 100) <= 50)
	switch (number(0, 3)) {
	case 0:
	  obj = create_random_magic();
	  break;
	case 1:
	  obj = read_object(RANDOM_POTION, VIRTUAL);
	  break;
	case 2:
	  if (number(0, 1))
	    obj = create_random_metal_armor(1);
	  else
	    obj = create_random_leather_armor(1);
	  break;
	case 3:
	  obj = create_random_weapon(-1, 1);
	  break;
	}
      if (obj && mob)
	obj_to_char(obj, mob);
      else if (obj && room > 1)
	obj_to_room(obj, room);
    }
    break;
  default:
    send_to_room("DEBUG: Error in loading treasure type\r\n", room);
    break;
  }
  if (c_perc && number(1, 100) <= c_perc) {
    if (c_num)
      obj = create_money(number(1, c_num), MONEY_COPPER, 0);
    if (obj && mob)
      obj_to_char(obj, mob);
    else if (obj && room > 1)
      obj_to_room(obj, room);
  }
  if (s_perc && number(1, 100) <= s_perc) {
    if (s_num)
      obj = create_money(number(1, s_num), MONEY_SILVER, 0);
    if (obj && mob)
      obj_to_char(obj, mob);
    else if (obj && room > 1)
      obj_to_room(obj, room);
  }
  if (e_perc && number(1, 100) <= e_perc) {
    if (e_num)
      obj = create_money(number(1, s_num), MONEY_ELECTRUM, 0);
    if (obj && mob)
      obj_to_char(obj, mob);
    else if (obj && room > 1)
      obj_to_room(obj, room);
  }
  if (g_perc && number(1, 100) <= g_perc) {
    if (g_num)
      obj = create_money(number(1, g_num), MONEY_GOLD, 0);
    if (obj && mob)
      obj_to_char(obj, mob);
    else if (obj && room > 1)
      obj_to_room(obj, room);
  }
  if (p_perc && number(1, 100) <= p_perc) {
    if (p_num)
      obj = create_money(number(1, g_num), MONEY_PLATINUM, 0);
    if (obj && mob)
      obj_to_char(obj, mob);
    else if (obj && room > 1)
      obj_to_room(obj, room);
  }
  if (gem_perc && number(1, 100) <= gem_perc) {
    if (gem_num)
      for (i = 0; i <= gem_num; i++) {
	obj = read_object(RANDOM_GEM, VIRTUAL);
	if (obj && mob)
	  obj_to_char(obj, mob);
	else if (obj && room > 1)
	  obj_to_room(obj, room);
      }
  }
  if (art_perc && number(1, 100) <= art_perc) {
    if (art_num)
      for (i = 0; i <= art_num; i++) {
	obj = read_object(RANDOM_ART, VIRTUAL);
	if (obj && mob)
	  obj_to_char(obj, mob);
	else if (obj && room > 1)
	  obj_to_room(obj, room);
      }
  }
}

void load_treasure_by_header(struct obj_data *obj, int type)
{
  int count = 0, i, load;
  struct obj_data *new_obj = NULL;

  if (isname("RANDOM", obj->name))
    return;

  /* first we loop the range, and count how many we have */
  for (i = 1; i < 100; i++)	/* start at 1, since 0 is the header item */
    if (real_object(i + type) > 0)
      count++;
    else
      break;

  /* now, randomly load one */
  load = number(1, count);
  if (real_object(load + type) > 0)	/* call me paranoid */
    new_obj = read_object(load + type, VIRTUAL);
  if (new_obj) {
    if (type == RANDOM_POTION)
      rename_object_random(new_obj, TABLE_POTION);
    else if (type == RANDOM_SCROLL)
      create_random_scroll(new_obj);
    else if (type == RANDOM_ROD)
      rename_object_random(new_obj, TABLE_STAFF);
    else if (type == RANDOM_WAND)
      rename_object_random(new_obj, TABLE_WAND);
    else if (type == RANDOM_MAGIC_METAL_ARMOR)
      rename_object_random(new_obj, TABLE_MAGIC_METAL_ARMOR);
    else if (type == RANDOM_MAGIC_LEATHER_ARMOR)
      rename_object_random(new_obj, TABLE_MAGIC_LEATHER_ARMOR);
    else if (type == RANDOM_MUNDANE_ARMOR)
      if (number(0, 1))
	rename_object_random(new_obj, TABLE_LEATHER_ARMOR);
      else
	rename_object_random(new_obj, TABLE_METAL_ARMOR);
    else if (type == RANDOM_MAGIC_ARMOR) {
      if (number(0, 1))
	rename_object_random(new_obj, TABLE_MAGIC_LEATHER_ARMOR);
      else
	rename_object_random(new_obj, TABLE_MAGIC_METAL_ARMOR);
    }
    if (new_obj && obj->in_room > 1) {
      put_obj_to_room(new_obj, obj->in_room, GET_OBJ_X_CORD(obj), GET_OBJ_Y_CORD(obj), GET_OBJ_Z_CORD(obj));
      obj_from_room(obj);
      extract_obj(obj);
    } else if (new_obj && obj->carried_by) {
      obj_to_char(new_obj, obj->carried_by);
      obj_from_char(obj);
      extract_obj(obj);
    } else if (new_obj)
      log("RANDOM: obj wasn't in room, but another case may be true");
    else
      log("RANDOM: new_obj didnt exist");

  } else
    vlogf("RANDOM: new_obj didnt exist!");
}

/* this is called when an item is first placed into the game. Some items
   require a room (loading of multiple items at once), so we call this
   code from two spots */
void convert_prototype_to_random_item(struct char_data *ch, struct obj_data *obj, int room)
{
  /* These are treasure types A-Z */
  if (!obj) {
    log("RANDOM: Bad obj passed to convert_prototype_to_random_item()#1");
    return;
  } else if (obj->item_number > top_of_objt) {
    log("RANDOM: Bad obj passed to convert_prototype_to_random_item()#2");
    return;
  }
  if (GET_OBJ_VNUM(obj) >= 10 && GET_OBJ_VNUM(obj) <= 35) {
    load_treasure_by_type(ch, GET_OBJ_VNUM(obj) - 10, room);	/* 10 = A */
    if (obj->in_room)
      obj_from_room(obj);
    else if (obj->carried_by)
      obj_from_char(obj);
    extract_obj(obj);
  } else if (GET_OBJ_VNUM(obj) == RANDOM_POTION)
    load_treasure_by_header(obj, RANDOM_POTION);
  else if (GET_OBJ_VNUM(obj) == RANDOM_MUNDANE_WEAPON)
    load_treasure_by_header(obj, RANDOM_MUNDANE_WEAPON);
  else if (GET_OBJ_VNUM(obj) == RANDOM_MAGIC_WEAPON)
    load_treasure_by_header(obj, RANDOM_MAGIC_WEAPON);
  else if (GET_OBJ_VNUM(obj) == RANDOM_SCROLL)
    load_treasure_by_header(obj, RANDOM_SCROLL);
  else if (GET_OBJ_ORIG_VNUM(obj) > RANDOM_SCROLL && GET_OBJ_ORIG_VNUM(obj) < RANDOM_RING)
    create_random_scroll(obj);
  else if (GET_OBJ_VNUM(obj) == RANDOM_RING)
    load_treasure_by_header(obj, RANDOM_RING);
  else if (GET_OBJ_VNUM(obj) == RANDOM_WAND)
    load_treasure_by_header(obj, RANDOM_WAND);
  else if (GET_OBJ_VNUM(obj) == RANDOM_ROD)
    load_treasure_by_header(obj, RANDOM_ROD);
  else if (GET_OBJ_VNUM(obj) == RANDOM_BOOK)
    load_treasure_by_header(obj, RANDOM_BOOK);
  else if (GET_OBJ_VNUM(obj) == RANDOM_JEWELRY)
    load_treasure_by_header(obj, RANDOM_JEWELRY);
  else if (GET_OBJ_VNUM(obj) == RANDOM_CLOAK)
    load_treasure_by_header(obj, RANDOM_CLOAK);
  else if (GET_OBJ_VNUM(obj) == RANDOM_BOOT)
    load_treasure_by_header(obj, RANDOM_BOOT);
  else if (GET_OBJ_VNUM(obj) == RANDOM_BELT)
    load_treasure_by_header(obj, RANDOM_BELT);
  else if (GET_OBJ_VNUM(obj) == RANDOM_BAG)
    load_treasure_by_header(obj, RANDOM_BAG);
  else if (GET_OBJ_VNUM(obj) == RANDOM_ODD)
    load_treasure_by_header(obj, RANDOM_ODD);
  else if (GET_OBJ_VNUM(obj) == RANDOM_TOOL)
    load_treasure_by_header(obj, RANDOM_TOOL);
  else if (GET_OBJ_VNUM(obj) == RANDOM_INSTRUMENT)
    load_treasure_by_header(obj, RANDOM_INSTRUMENT);
  else if (GET_OBJ_VNUM(obj) == RANDOM_MAGIC_ARMOR)
    load_treasure_by_header(obj, RANDOM_MAGIC_ARMOR);
  else if (GET_OBJ_VNUM(obj) == RANDOM_MAGIC_METAL_ARMOR)
    rename_object_random(obj, TABLE_MAGIC_METAL_ARMOR);
  else if (GET_OBJ_VNUM(obj) == RANDOM_MAGIC_LEATHER_ARMOR)
    rename_object_random(obj, TABLE_MAGIC_LEATHER_ARMOR);
  else if (GET_OBJ_VNUM(obj) == RANDOM_MUNDANE_ARMOR)
    load_treasure_by_header(obj, RANDOM_MUNDANE_ARMOR);
}


void SetBookRandomSpells(struct obj_data *book, int circle, int num)
{
  int spell, i;

  if (num) {
    AddSpellToSpellBook(0, book, SPELL_MAGIC_MISSILE);
  }
  for (i = 1; i < num; i++) {
    spell = choose_a_spell(CLASS_MAGIC_USER, circle);
    AddSpellToSpellBook(0, book, spell);
  }
}

void CursedScroll(struct obj_data *obj, struct char_data *ch)
{
  int roll;

  roll = number(1, 100);

  if (roll >= 1 && roll <= 25) {
  } else if (roll >= 26 && roll <= 30) {
  } else if (roll >= 31 && roll <= 40) {
  } else if (roll >= 41 && roll <= 50) {
  } else if (roll >= 51 && roll <= 75) {
  } else if (roll >= 76 && roll <= 90) {
  } else if (roll >= 91 && roll <= 99) {
  } else if (roll == 100) {
  }
  send_to_char("Default cursed scroll message.\r\n", ch);
}


  /* outfit the random mobs with clothing, and basic eq */
void outfit_random_mobs(struct char_data *mob)
{
  struct obj_data *obj = NULL;
  int temp = 0, temp2 = 0, temp3 = 0, counter = 0, loaded_num = 1, poor = 0;

  const char *lights[] =
  {
    "&+ytorch", "&+Llantern", "&+Wstrange fungus", "&+Wglowing crystal"
  };
  int nlights = 3;
  const char *bags[] =
  {
    "&+yleather", "&+ybrown", "&+Lblack", "&+gsnakeskin", "large",
    "&+Wruffled", "&+mvelvet", "torn", "&+Rdragonscale", "&+yfurred",
    "silverish"
  };
  int nbags = 10;
  const char *drinks[] =
  {
    "&+ybottle", "flagon", "jar", "&+ygoatskin", "&+Lwineskin",
    "&+Wdrinking horn", "&+ymug"
  };
  int ndrinks = 6;
  const char *foods[] =
  {
    "pastry", "&+yloaf of bread", "&+Ltravel ration",
    "&+Rleg of beef", "&+Ljuicy steak", "&+gpear", "&+Rcarrot", "&+ysuspicious-looking mushroom",
    "&+Rapple", "&+Lwyvern jerky", "&+Ldragon-gut", "&+ypiece of bread"
  };
  int nfoods = 11;
  const char *earrings[] =
  {
    "&+Wdiamond", "&+gemerald", "&+ytopaz", "&+ywooden", "&+gjade",
    "&+Wwhite gold", "&+Lonyx", "tin", "glass", "&+Wmarble", "&+Lblack", "&+Lgranite"
  };
  int nearrings = 11;
  const char *rings[] =
  {
    "tarnished", "jeweled", "silver", "&+Ygolden",
    "&+Wshiny", "dull", "sparkling", "attractive", "&+Lblack", "glass"
  };
  int nrings = 9;
  const char *generics[] =
  {
  /* These MUST have blank at end */
    "", "", "", "", "",		/* not all needs describing :) */
    "chain ", "ancient ", "dusty ", "scratched ", "antique ", "old ",
    "leather ", "sooty ", "plain ", "ornate ", "dull ", "woolen ",
    "odd-looking ", "tarnished ", "unusual ", "spiffy "
  };
  int ngenerics = 20;
  const char *necks[] =
  {
    "necklace", "collar", "cape", "amulet", "pendant"
  };
  int nnecks = 4;
  const char *bodys[] =
  {
    "shirt", "coat", "jacket", "chain shirt", "blouse"
  };
  int nbodys = 4;
  const char *heads[] =
  {
    "hat", "headdress", "cap", "helm", "coif", "derby"
  };
  int nheads = 5;
  const char *feets[] =
  {
    "sandals", "boots", "shoes", "riding boots", "leathers"
  };
  int nfeets = 4;
  const char *shields[] =
  {
    "dented", "round", "oval", "center-grip", "heater",
    "door", "shiny", "embossed", "coffin-shaped", "curved"
  };
  int nshields = 9;
  const char *weps[] =
  {
    "axe", "sword", "mace", "club", "bardiche", "pike",
    "whip", "maul", "bone", "staff", "dagger", "spear"
  };
  int nweps = 11;
  const char *color[] =		/* better colors are duped often */
  {
    "&+L", "&+g", "&+y", "&+m", "&+b", "&+c", "&+w", "&+r",
    "&+G", "&+Y", "&+M", "&+B", "&+C", "&+W", "&+R", "&+L",
    "&+L", "&+L", "&+y", "&+y", "&+y", "&+y", "&+W", "&+c"
  };
  int ncolors = 23;

  buf[0] = '\0';

  if (!RANDOM_MOB(mob) && IS_PC(mob))
    return;

  if (isname("beggar", mob->player.name))
    poor = TRUE;

  for (;;) {
    obj = NULL;
    temp2 = number(0, ngenerics);
    temp3 = number(0, ncolors);
    switch (counter) {
    case 0:			/* light */
      if (CAN_SEE(mob, mob))
	break;			/* only if its dark, do they get a light */
      obj = read_object(1, VIRTUAL);
      if (!obj)
	return;
      temp = number(0, nlights);
      sprintf(buf, "%s %s&n", AN(lights[temp]), lights[temp]);
      obj->short_description = str_dup(buf);
      sprintf(buf, "%s %s&n.", CAP_AN(lights[temp]), lights[temp]);
      obj->description = str_dup(buf);
      sprintf(buf, "%s RANDOM", strip_color(lights[temp]));
      obj->name = str_dup(buf);
      GET_OBJ_TYPE(obj) = ITEM_LIGHT;
      GET_OBJ_VAL(obj, 2) = number(3, 10);	/* burn time */
      GET_OBJ_SIZE(obj) = ITEM_SIZE_SMALL;
      break;
    case 1:			/* container */
      obj = read_object(1, VIRTUAL);
      if (!obj)
	return;
      temp = number(0, nbags);
      sprintf(buf, "%s %s bag&n", AN(bags[temp]), bags[temp]);
      obj->short_description = str_dup(buf);
      sprintf(buf, "%s %s bag&n.", CAP_AN(bags[temp]), bags[temp]);
      obj->description = str_dup(buf);
      sprintf(buf, "bag %s RANDOM", strip_color(bags[temp]));
      obj->name = str_dup(buf);
      GET_OBJ_TYPE(obj) = ITEM_CONTAINER;
      GET_OBJ_VAL(obj, 1) = 1;
      GET_OBJ_VAL(obj, 2) = -1;
      GET_OBJ_VAL(obj, 0) = number(5, 20);
      GET_OBJ_SIZE(obj) = ITEM_SIZE_SMALL;
      GET_OBJ_WEAR(obj) = ITEM_ATTACH_BELT;
      break;
    case 2:			/* drinkcon */
      if (number(0, 2))
	break;
      obj = read_object(1, VIRTUAL);
      if (!obj)
	return;
      temp = number(0, ndrinks);
      sprintf(buf, "%s %s&n", AN(drinks[temp]), drinks[temp]);
      obj->short_description = str_dup(buf);
      sprintf(buf, "%s %s&n.", CAP_AN(drinks[temp]), drinks[temp]);
      obj->description = str_dup(buf);
      sprintf(buf, "%s RANDOM", strip_color(drinks[temp]));
      obj->name = str_dup(buf);
      GET_OBJ_TYPE(obj) = ITEM_DRINKCON;
      GET_OBJ_VAL(obj, 0) = 2;
      GET_OBJ_VAL(obj, 1) = 2;
      GET_OBJ_VAL(obj, 2) = number(1, 20);
      GET_OBJ_SIZE(obj) = ITEM_SIZE_SMALL;
      GET_OBJ_WEAR(obj) = ITEM_ATTACH_BELT;
      break;
    case 3:			/* food */
      if (!number(0, 2) || poor)
	break;
      obj = read_object(1, VIRTUAL);
      if (!obj)
	return;
      temp = number(0, nfoods);
      sprintf(buf, "%s %s&n", AN(foods[temp]), foods[temp]);
      obj->short_description = str_dup(buf);
      sprintf(buf, "%s %s&n.", CAP_AN(foods[temp]), foods[temp]);
      obj->description = str_dup(buf);
      sprintf(buf, "%s RANDOM", strip_color(foods[temp]));
      obj->name = str_dup(buf);
      GET_OBJ_TYPE(obj) = ITEM_FOOD;
      GET_OBJ_VAL(obj, 0) = number(1, 5);
      GET_OBJ_SIZE(obj) = ITEM_SIZE_SMALL;
      break;
    case 4:			/* rings */
      if (!poor && (GET_SEX(mob) == SEX_FEMALE || number(0, 1))) {
	obj = read_object(1, VIRTUAL);
	if (!obj)
	  return;
	temp = number(0, nrings);
	sprintf(buf, "%s %s ring&n", AN(rings[temp]), rings[temp]);
	obj->short_description = str_dup(buf);
	sprintf(buf, "%s %s ring&n.", CAP_AN(rings[temp]), rings[temp]);
	obj->description = str_dup(buf);
	obj->name = str_dup("ring RANDOM");
	GET_OBJ_TYPE(obj) = ITEM_WORN;
	GET_OBJ_WEAR(obj) = ITEM_WEAR_FINGER;
	GET_OBJ_SIZE(obj) = ITEM_SIZE_TINY;
      }
      break;
    case 5:			/* necklaces */
      if (number(0, 3) || poor)
	break;
      obj = read_object(1, VIRTUAL);
      if (!obj)
	return;
      temp = number(0, nnecks);
      if (*generics[temp2])
	sprintf(buf1, "%s", generics[temp2]);
      else
	sprintf(buf1, "%s", necks[temp]);
      sprintf(buf, "%s %s%s%s&n", AN(buf1), color[temp3], generics[temp2], necks[temp]);
      obj->short_description = str_dup(buf);
      sprintf(buf, "%s %s%s%s&n.", CAP_AN(buf1), color[temp3], generics[temp2], necks[temp]);
      obj->description = str_dup(buf);
      sprintf(buf, "%s RANDOM", strip_color(necks[temp]));
      obj->name = str_dup(buf);
      GET_OBJ_TYPE(obj) = ITEM_WORN;
      GET_OBJ_WEAR(obj) = ITEM_WEAR_NECK;
      GET_OBJ_SIZE(obj) = ITEM_SIZE_SMALL;
      break;
    case 6:			/* shirt/blouse/armor */
      if (isname("soldier", mob->player.name) ||
	  isname("guard", mob->player.name) ||
	  isname("sailor", mob->player.name)) {
	obj = read_object(1, VIRTUAL);
	if (!obj)
	  return;
	temp = dice(1, MAX_LEATHER_ARMOR) - 1;
	temp2 = number(10, 12);	/* wear_body armors */
	temp3 = number(1, NUM_KINGDOMS - 1);
	GET_HOMETOWN(mob) = temp3;
	GET_OBJ_VAL(obj, 0) = leather_armor_table[temp].armor;
	GET_OBJ_WEIGHT(obj) = leather_armor_type_table[temp2].weight;
	GET_OBJ_TYPE(obj) = ITEM_ARMOR;
	GET_OBJ_ANTI(obj) = leather_armor_table[temp].extra;
	sprintf(buf, "%s %s", AN(leather_armor_table[temp].name), leather_armor_table[temp].name);
	sprintf(buf, leather_armor_type_table[temp2].pattern, buf);
	sprintf(buf1, "%s RANDOM", buf);
	obj->name = str_dup(buf1);
	sprintf(buf1, "%s from the %s&n", buf, kingdom_list[temp3]);
	obj->short_description = str_dup(buf1);
	CAP(buf);
	sprintf(buf1, "%s&n have been left here.", buf);
	obj->description = str_dup(buf1);
      } else {
	obj = read_object(1, VIRTUAL);
	if (!obj)
	  return;
	temp = number(0, nbodys);
	if (GET_SEX(mob) == SEX_MALE && temp == nbodys)
	  temp = 0;		/* men don't wear blouses */
	if (*generics[temp2])
	  sprintf(buf1, "%s", generics[temp2]);
	else
	  sprintf(buf1, "%s", bodys[temp]);
	sprintf(buf, "%s %s%s%s&n", AN(buf1), color[temp3], generics[temp2], bodys[temp]);
	obj->short_description = str_dup(buf);
	sprintf(buf, "%s %s%s%s&n.", CAP_AN(buf1), color[temp3], generics[temp2], bodys[temp]);
	obj->description = str_dup(buf);
	sprintf(buf, "%s RANDOM", strip_color(bodys[temp]));
	obj->name = str_dup(buf);
	GET_OBJ_TYPE(obj) = ITEM_WORN;
      }
      GET_OBJ_WEAR(obj) = ITEM_WEAR_BODY;
      GET_OBJ_SIZE(obj) = GET_SIZE(mob);
      break;
    case 7:			/* hats/helmets */
      if (number(0, 2) || poor)
	break;
      obj = read_object(1, VIRTUAL);
      if (!obj)
	return;
      temp = number(0, nheads);
      if (*generics[temp2])
	sprintf(buf1, "%s", generics[temp2]);
      else
	sprintf(buf1, "%s", heads[temp]);
      sprintf(buf, "%s %s%s%s&n", AN(buf1), color[temp3], generics[temp2], heads[temp]);
      obj->short_description = str_dup(buf);
      sprintf(buf, "%s %s%s%s&n.", CAP_AN(buf1), color[temp3], generics[temp2], heads[temp]);
      obj->description = str_dup(buf);
      sprintf(buf, "%s RANDOM", strip_color(heads[temp]));
      obj->name = str_dup(buf);
      GET_OBJ_TYPE(obj) = ITEM_WORN;
      GET_OBJ_WEAR(obj) = ITEM_WEAR_HEAD;
      GET_OBJ_SIZE(obj) = GET_SIZE(mob);
      break;
    case 8:			/* pants/stockings/leggings */
      obj = read_object(1, VIRTUAL);
      if (!obj)
	return;
      temp = number(0, ngenerics);
      if (GET_SEX(mob) == SEX_MALE)
	sprintf(buf1, "%s", number(0, 1) ? "pants" : "trousers");
      else if (GET_SEX(mob) == SEX_FEMALE)
	sprintf(buf1, "%s", number(0, 1) ? "knickers" : "stockings");
      sprintf(buf, "some %s%s%s&n", color[temp3], generics[temp], buf1);
      obj->short_description = str_dup(buf);
      sprintf(buf, "Some %s%s%s&n.", color[temp3], generics[temp], buf1);
      obj->description = str_dup(buf);
      obj->name = str_dup("leggings RANDOM");
      GET_OBJ_TYPE(obj) = ITEM_WORN;
      GET_OBJ_WEAR(obj) = ITEM_WEAR_LEGS;
      GET_OBJ_SIZE(obj) = GET_SIZE(mob);
      break;
    case 9:			/* footwear */
      obj = read_object(1, VIRTUAL);
      if (!obj)
	return;
      temp = number(0, nfeets);
      sprintf(buf, "a pair of %s%s%s&n", color[temp3], generics[temp2], feets[temp]);
      obj->short_description = str_dup(buf);
      sprintf(buf, "A pair of %s%s%s &nlie", color[temp3], generics[temp2], feets[temp]);
      obj->description = str_dup(buf);
      sprintf(buf, "%s RANDOM", strip_color(feets[temp]));
      obj->name = str_dup(buf);
      GET_OBJ_TYPE(obj) = ITEM_WORN;
      GET_OBJ_WEAR(obj) = ITEM_WEAR_FEET;
      GET_OBJ_SIZE(obj) = GET_SIZE(mob);
      break;
    case 10:			/* shield */
      if (!IS_WARRIOR(mob) || !number(0, 1))
	break;
      obj = read_object(1, VIRTUAL);
      if (!obj)
	return;
      temp = number(0, nshields);
      sprintf(buf, "%s %s shield&n", AN(shields[temp]), shields[temp]);
      obj->short_description = str_dup(buf);
      sprintf(buf, "%s %s shield&n.", AN(shields[temp]), shields[temp]);
      obj->description = str_dup(buf);
      obj->name = str_dup("shield RANDOM");
      GET_OBJ_TYPE(obj) = ITEM_SHIELD;
      GET_OBJ_WEAR(obj) = ITEM_WEAR_SHIELD;
      GET_OBJ_SIZE(obj) = GET_SIZE(mob);
      break;
    case 11:			/* cloaks/dresses/coats/robes */
      if ((GET_SEX(mob) == SEX_FEMALE && !number(0, 3)) ||
	  (GET_SEX(mob) == SEX_MALE && number(0, 3)))
	break;
      obj = read_object(1, VIRTUAL);
      if (!obj)
	return;
      temp = number(0, ngenerics);
      if (*generics[temp2])
	sprintf(buf1, "%s", generics[temp2]);
      else
	sprintf(buf1, "a");
      if (GET_SEX(mob) == SEX_FEMALE && number(0, 2))
	sprintf(buf2, "dress");
      else if (IsMemmingClass(mob))
	sprintf(buf2, "robe");
      else
	sprintf(buf2, "cloak");
      sprintf(buf, "%s %s%s%s&n", AN(buf1), color[temp3], generics[temp], buf2);
      obj->short_description = str_dup(buf);
      sprintf(buf, "%s %s%s%s&n.", CAP_AN(buf1), color[temp3], generics[temp], buf2);
      obj->description = str_dup(buf);
      obj->name = str_dup("cloak dress robe RANDOM");
      GET_OBJ_TYPE(obj) = ITEM_WORN;
      GET_OBJ_WEAR(obj) = ITEM_WEAR_ABOUT;
      GET_OBJ_SIZE(obj) = GET_SIZE(mob);
      break;
    case 12:			/* belt */
      obj = read_object(1, VIRTUAL);
      if (!obj)
	return;
      temp = number(0, ngenerics);
      if (*generics[temp2])
	sprintf(buf1, "%s", generics[temp2]);
      else
	sprintf(buf1, "a");
      sprintf(buf, "%s %s%sbelt&n", AN(buf1), color[temp3], generics[temp]);
      obj->short_description = str_dup(buf);
      sprintf(buf, "%s %s%sbelt&n.", CAP_AN(buf1), color[temp3], generics[temp]);
      obj->description = str_dup(buf);
      obj->name = str_dup("belt RANDOM");
      GET_OBJ_TYPE(obj) = ITEM_WORN;
      GET_OBJ_WEAR(obj) = ITEM_WEAR_WAIST;
      GET_OBJ_SIZE(obj) = GET_SIZE(mob);
      break;
    case 13:			/* bracelet */
      if (!number(0, 2) || poor)
	break;
      obj = read_object(1, VIRTUAL);
      if (!obj)
	return;
      temp = number(0, ngenerics);
      if (*generics[temp2])
	sprintf(buf1, "%s", generics[temp2]);
      else
	sprintf(buf1, "a");
      sprintf(buf, "%s %s%sbracelet&n", AN(buf1), color[temp3], generics[temp]);
      obj->short_description = str_dup(buf);
      sprintf(buf, "%s %s%sbracelet&n.", CAP_AN(buf1), color[temp3], generics[temp]);
      obj->description = str_dup(buf);
      obj->name = str_dup("bracelet RANDOM");
      GET_OBJ_TYPE(obj) = ITEM_WORN;
      GET_OBJ_WEAR(obj) = ITEM_WEAR_WRIST;
      GET_OBJ_SIZE(obj) = GET_SIZE(mob);
      break;
    case 14:			/* gloves/gauntlets */
      if (IS_WARRIOR(mob) || !number(0, 2)) {
	obj = read_object(1, VIRTUAL);
	if (!obj)
	  return;
	temp = number(0, ngenerics);
	sprintf(buf, "some %s%sgloves&n", color[temp3], generics[temp]);
	obj->short_description = str_dup(buf);
	sprintf(buf, "Some %s%sgloves &nlie here.", color[temp3], generics[temp]);
	obj->description = str_dup(buf);
	obj->name = str_dup("gloves RANDOM");
	GET_OBJ_TYPE(obj) = ITEM_WORN;
	GET_OBJ_WEAR(obj) = ITEM_WEAR_HANDS;
	GET_OBJ_SIZE(obj) = GET_SIZE(mob);
      }
      break;
    case 15:			/* weapons */
      if (IS_WARRIOR(mob) || !number(0, 2)) {
	obj = read_object(1, VIRTUAL);
	if (!obj)
	  return;
	temp = number(0, nweps);
	sprintf(buf, "a %s%s&n", generics[temp2], weps[temp]);
	obj->short_description = str_dup(buf);
	sprintf(buf, "A %s%s&n.", generics[temp2], weps[temp]);
	obj->description = str_dup(buf);
	sprintf(buf, "%s RANDOM", strip_color(weps[temp]));
	obj->name = str_dup(buf);
      }
      break;
    case 16:			/* earrings */
      if (!poor && ((GET_SEX(mob) == SEX_FEMALE && number(0, 3)) ||
		    (loaded_num == 1 && number(0, 2)))) {
	obj = read_object(1, VIRTUAL);
	if (!obj)
	  return;
	temp = number(0, nearrings);
	sprintf(buf, "%s %s earring&n", AN(earrings[temp]), earrings[temp]);
	obj->short_description = str_dup(buf);
	sprintf(buf, "%s %s earring&n.", CAP_AN(earrings[temp]), earrings[temp]);
	obj->description = str_dup(buf);
	obj->name = str_dup("earring RANDOM");
	GET_OBJ_TYPE(obj) = ITEM_WORN;
	GET_OBJ_WEAR(obj) = ITEM_WEAR_EARRING;
	GET_OBJ_SIZE(obj) = ITEM_SIZE_TINY;
      }
      break;
    case 17:
    default:
      return;			/* escape the loop */
    }
    if (obj && mob) {
      GET_OBJ_WEAR(obj) = GET_OBJ_WEAR(obj) | ITEM_WEAR_TAKE | ITEM_WEAR_HOLD;
      GET_OBJ_COST(obj) = 50;	/* temp */
      obj_to_char(obj, mob);
/*      CheckEqWorthUsing(mob, obj);  */
    }
    /* this will load some items twice */
    if ((counter == 3 || counter == 4 || counter == 13
	 || counter == 15 || counter == 16) && loaded_num == 1)
      loaded_num = 2;
    else {
      loaded_num = 1;
      counter++;
    }
  }
  return;
}
