/********************************************************
 * ships.c
 *
 * Main ship code is here
 *******************************************************/

#include "conf.h"
#include "sysdep.h"
#include <math.h>

#include "structs.h"
#include "utils.h"
#include "shipvar.h"
#include "interpreter.h"
#include "comm.h"
#include "handler.h"
#include "db.h"
#include "map.h"
#include "olc.h"

extern struct room_data *world;
struct ship_data ships[MAXSHIPS];
struct contact_data contacts[30];
struct ship_map tactical[151][151];
char status[20];
char position[20];
char weapon[100];
char contact[256];
char arc[3];
char debug[256];
char arg1[80];
char arg2[80];
void getstatus(int slot, int rnum);
void getposition(int slot, int rnum);
void dispweapon(int slot, int rnum);
int loadship(int template, int to_room, short int x_cord, short int y_cord, short int z_cord);
void nameship(char *name, int shipnum);
bool setsail(int class, int shipnum);
int weaprange(int shipnum, int slot, char range);
int bearing(float x1, float y1, float x2, float y2);
float range(float x1, float y1, float z1, float x2, float y2, float z2);
void dispcontact(int i);
int getcontacts(int shipnum);
void setcontact(int i, struct obj_data *obj, int shipnum, int xoffset, int yoffset);
int getarc(int ship1, int ship2);
void getmap(int shipnum);
void setsymbol(int x, int y, int symbol);

SPECIAL(ship_commands)
{
  int i, j, k, p;
  if (!ch || !cmd)
    return FALSE;

  if (CMD_IS("tactical")) {
    int x, y;
    float shiprange;

    two_arguments(argument, arg1, arg2);

    if (!*arg1) {
      x = (int) SHIPX(ch->in_room);
      y = (int) SHIPY(ch->in_room);
    } else {
      if (!*arg2) {
	send_to_char("&+WSyntax: Tactical <x> <y> or Tactical&N\r\n", ch);
	return TRUE;
      }
      if (is_number(arg1) && is_number(arg2)) {
	shiprange = range(SHIPX(ch->in_room),
			  SHIPY(ch->in_room),
			  0,
			  atoi(arg1),
			  atoi(arg2),
			  0);
	if ((int) (shiprange + .5) <= 35) {
	  x = atoi(arg1);
	  y = atoi(arg2);
	} else {
	  sprintf(buf, "This coord is out of range.\r\nMust be within 35 units.\r\nCurrent range: %3.1f\r\n", shiprange);
	  send_to_char(buf, ch);
	  return TRUE;
	}
      } else {
	send_to_char("&+WSyntax: Tactical <x> <y> or Tacitcal&N\r\n", ch);
	return TRUE;
      }
    }
    getmap(SHIPNUM(ch->in_room));
    sprintf(buf, "&+W     %-3d   %-3d   %-3d   %-3d   %-3d  %-3d   %-3d   %-3d   %-3d   %-3d   %-3d   %-3d&N\r\n",
	    x - 11, x - 9, x - 7, x - 5, x - 3, x - 1, x + 1, x + 3, x + 5, x + 7, x + 9, x + 11);
    send_to_char(buf, ch);
    sprintf(buf, "     __ &+W%-3d&N__ &+W%-3d&N__ &+W%-3d&N__ &+W%-3d&N__ &+W%-3d&N__ &+W%-3d&N__ &+W%-3d&N__ &+W%-3d&N__ &+W%-3d&N__ &+W%-3d&N__ &+W%-3d&N__&N\r\n",
	    x - 10, x - 8, x - 6, x - 4, x - 2, x, x + 2, x + 4, x + 6, x + 8, x + 10);
    send_to_char(buf, ch);
    y = 150 - y;
    for (i = y - 7; i < y + 8; i++) {
      sprintf(buf, "&+W%-3d&N /%s\\__/%s\\__/%s\\__/%s\\__/%s\\__/%s\\__/%s\\__/%s\\__/%s\\__/%s\\__/%s\\__/%s\\\r\n",
	      150 - i,
	      tactical[x - 11][i].map,
	      tactical[x - 9][i].map,
	      tactical[x - 7][i].map,
	      tactical[x - 5][i].map,
	      tactical[x - 3][i].map,
	      tactical[x - 1][i].map,
	      tactical[x + 1][i].map,
	      tactical[x + 3][i].map,
	      tactical[x + 5][i].map,
	      tactical[x + 7][i].map,
	      tactical[x + 9][i].map,
	      tactical[x + 11][i].map);
      send_to_char(buf, ch);
      sprintf(buf, "    \\__/%s\\__/%s\\__/%s\\__/%s\\__/%s\\__/%s\\__/%s\\__/%s\\__/%s\\__/%s\\__/%s\\__/\r\n",
	      tactical[x - 10][i].map,
	      tactical[x - 8][i].map,
	      tactical[x - 6][i].map,
	      tactical[x - 4][i].map,
	      tactical[x - 2][i].map,
	      tactical[x][i].map,
	      tactical[x + 2][i].map,
	      tactical[x + 4][i].map,
	      tactical[x + 6][i].map,
	      tactical[x + 8][i].map,
	      tactical[x + 10][i].map);
      send_to_char(buf, ch);
    }
    send_to_char("       \\__/  \\__/  \\__/  \\__/  \\__/  \\__/  \\__/  \\__/  \\__/  \\__/  \\__/\r\n", ch);
    return TRUE;
  }
  if (CMD_IS("contacts")) {
    k = getcontacts(SHIPNUM(ch->in_room));
    send_to_char("&+WContact listing\r\n===============================================&N\r\n", ch);
    if (k > 0) {
      for (p = 0; p < k; p++) {
	dispcontact(p);
	send_to_char(contact, ch);
      }
    }
    return TRUE;
  }
  if (CMD_IS("weaponspec")) {
    if (world[ch->in_room].ship != NULL) {
      send_to_char("&+rWeapon Specifications&N\r\n", ch);
      send_to_char("&+r=================================================&N\r\n", ch);
      send_to_char("Num  Name                 Min    Med    Long   Dam  Status\r\n", ch);
      j = 0;
      for (i = 0; i < MAXSLOTS; i++) {
	if (SHIPSLOT(ch->in_room)[i].type == 1) {
	  getstatus(i, ch->in_room);
	  sprintf(buf, "&+W[%d]  %-20s &N%2d-%-2d  %2d-%-2d  %2d-%-2d  %-3d  %s&N\r\n",
		  j, SHIPSLOT(ch->in_room)[i].desc,
		  SHIPSLOT(ch->in_room)[i].val1,
		  weaprange(SHIPNUM(ch->in_room), i, SHRTRANGE),
		  weaprange(SHIPNUM(ch->in_room), i, SHRTRANGE) + 1,
		  weaprange(SHIPNUM(ch->in_room), i, MEDRANGE),
		  weaprange(SHIPNUM(ch->in_room), i, MEDRANGE) + 1,
		  weaprange(SHIPNUM(ch->in_room), i, LNGRANGE),
		  SHIPSLOT(ch->in_room)[i].val2,
		  status);
	  send_to_char(buf, ch);
	  j++;
	}
      }
      return TRUE;
    }
    return FALSE;
  }
  if (CMD_IS("status")) {
    if (world[ch->in_room].ship != NULL) {
      sprintf(buf, "%s\r\n", SHIPNAME(ch->in_room));
      send_to_char(buf, ch);
      send_to_char("&+L-========================================================================-&N\r\n", ch);
      sprintf(buf, "&+LCaptain: &+W%-58s&+LID[&+Y%s&+L]\r\n\r\n", SHIPOWNER(ch->in_room), SHIPID(ch->in_room));
      send_to_char(buf, ch);

      sprintf(buf, "        &+G%3d&N/&+G%-3d  &+LSpeed Range: &+W0-%-3d&+L       Sail Crew: &+W%-20s&N\r\n",
	      SHIPFARMOR(ch->in_room), SHIPMAXFARMOR(ch->in_room), SHIPMAXSPEED(ch->in_room), SHIPSAILNAME(ch->in_room));
      send_to_char(buf, ch);

      sprintf(buf, "                      &+LWeight: &+W%3d,000  &+LGunnery Crew: &+W%-20s&N\r\n",
	      SHIPHULLWEIGHT(ch->in_room), SHIPGUNNAME(ch->in_room));
      send_to_char(buf, ch);
      send_to_char("           &+y||&N\r\n", ch);
      send_to_char("          &+y/..\\            &N&+rWeapons:&N\r\n", ch);

      sprintf(buf, "         &+y/.&+g%2d&+y.\\        &+r-=================================================-&N\r\n",
	      SHIPFINTERNAL(ch->in_room));
      send_to_char(buf, ch);
      send_to_char("        &+y/..&N--&+y..\\        &+LNum  Name                 Status  Positon   Ammo&N\r\n", ch);

      dispweapon(0, ch->in_room);
      sprintf(buf, "        &+y|..&+g%2d&+y..|        &+W[0]  %s&N\r\n",
	      SHIPMAXFINTERNAL(ch->in_room), weapon);
      send_to_char(buf, ch);

      dispweapon(1, ch->in_room);
      sprintf(buf, "        &+y|......| &+g%2d     &+W[1]  %s&N\r\n",
	      SHIPMAINSAIL(ch->in_room), weapon);
      send_to_char(buf, ch);

      dispweapon(2, ch->in_room);
      sprintf(buf, "        &+y\\__..__/ &N--     &+W[2]  %s&N\r\n",
	      weapon);
      send_to_char(buf, ch);

      dispweapon(3, ch->in_room);
      sprintf(buf, "        &+y|..||..|&+L/&N&+g%2d     &+W[3]  %s&N\r\n",
	      SHIPMAXMAINSAIL(ch->in_room), weapon);
      send_to_char(buf, ch);

      dispweapon(4, ch->in_room);
      sprintf(buf, "        &+y|......&+L/        &+W[4]  %s&N\r\n",
	      weapon);
      send_to_char(buf, ch);

      dispweapon(5, ch->in_room);
      sprintf(buf, "        &+y|.....&+L/&N&+y|        &+W[5]  %s&N\r\n",
	      weapon);
      send_to_char(buf, ch);

      dispweapon(6, ch->in_room);
      sprintf(buf, "        &+y|....&+L/&N&+y.|        &+W[6]  %s&N\r\n",
	      weapon);
      send_to_char(buf, ch);

      dispweapon(7, ch->in_room);
      sprintf(buf, "    &+G%3d &N&+y|&+g%2d&+y.&+L/&N&+g%2d&+y| &+G%3d    &+W[7]  %s&N\r\n",
	      SHIPPARMOR(ch->in_room), SHIPPINTERNAL(ch->in_room), SHIPSINTERNAL(ch->in_room), SHIPSARMOR(ch->in_room), weapon);
      send_to_char(buf, ch);

      dispweapon(8, ch->in_room);
      sprintf(buf, "    &N--- &+y|&N--&+Y/\\&N--&+y| &N---    &+W[8] %s&N\r\n",
	      weapon);
      send_to_char(buf, ch);

      dispweapon(9, ch->in_room);
      sprintf(buf, "    &+G%3d &N&+y|&+g%2d&+Y\\/&N&+g%2d&+y| &+G%3d    &+W[9]  %s&N\r\n",
	      SHIPMAXPARMOR(ch->in_room), SHIPMAXPINTERNAL(ch->in_room), SHIPMAXSINTERNAL(ch->in_room), SHIPMAXSARMOR(ch->in_room), weapon);
      send_to_char(buf, ch);
      send_to_char("        &+y|......|&N\r\n", ch);
      send_to_char("        &+y|__||__|&N\r\n", ch);
      send_to_char("        &+y/......\\&N\r\n", ch);
      send_to_char("        &+y|......|&N\r\n", ch);

      sprintf(buf, "        &+y|..&+g%2d&+y..|      &NX:&+W%3.1f  &NY:&+W%3.1f  &NZ:&+W%3.1f&N\r\n",
	      SHIPRINTERNAL(ch->in_room), SHIPX(ch->in_room), SHIPY(ch->in_room), SHIPZ(ch->in_room));
      send_to_char(buf, ch);
      send_to_char("        &+y|..&N--&+y..|&N\r\n", ch);

      sprintf(buf, "        &+y|..&+g%2d&+y..|    &NSet Heading: &+W%-3d   &NSet Speed: &+W%-4d&N\r\n",
	      SHIPMAXRINTERNAL(ch->in_room), SHIPSETHEADING(ch->in_room), SHIPSETSPEED(ch->in_room));
      send_to_char(buf, ch);

      sprintf(buf, "        &+y\\______/        &NHeading: &+W%-3d       &NSpeed: &+W%-4d&N\r\n\r\n",
	      SHIPHEADING(ch->in_room), SHIPSPEED(ch->in_room));
      send_to_char(buf, ch);

      sprintf(buf, "        &+G%3d&N/&+G%-3d&N\r\n",
	      SHIPRARMOR(ch->in_room), SHIPMAXRARMOR(ch->in_room));
      send_to_char(buf, ch);
    } else {
      send_to_char("&+WThis ship has no Data.&N\r\n", ch);
    }
    return TRUE;
  }
  if (CMD_IS("speed")) {
    char speed;

    one_argument(argument, arg);

    if (!*arg) {
      sprintf(buf, "Current speed: &+W%d&N\r\nSet speed: &+W%d&N\r\n",
	      SHIPSPEED(ch->in_room), SHIPSETSPEED(ch->in_room));
      send_to_char(buf, ch);
    } else {
      if (is_number(arg)) {
	speed = atoi(arg);
	if ((SHIPMINSPEED(ch->in_room) <= speed) && (speed <= SHIPMAXSPEED(ch->in_room))) {
	  SHIPSETSPEED(ch->in_room) = speed;
	  sprintf(buf, "Speed set to &+W%d&N.\r\n", speed);
	  send_to_room(buf, ch->in_room);
	} else {
	  sprintf(buf, "This ship can only go from &+W%d&N to &+W%d&N.\r\n",
		  SHIPMINSPEED(ch->in_room), SHIPMAXSPEED(ch->in_room));
	  send_to_char(buf, ch);
	}
      } else {
	sprintf(buf, "Please enter a number value between %3d-%-d.\r\n",
		SHIPMINSPEED(ch->in_room), SHIPMAXSPEED(ch->in_room));
	send_to_char(buf, ch);
      }
    }
    return TRUE;
  }
  if (CMD_IS("heading")) {
    int heading;
    one_argument(argument, arg);

    if (!*arg || !is_number(arg)) {
      sprintf(buf, "Current heading: &+W%d&N\r\nSet heading: &+W%d&N\r\n",
	      SHIPHEADING(ch->in_room), SHIPSETHEADING(ch->in_room));
      send_to_char(buf, ch);
      return TRUE;
    }
    heading = atoi(arg);
    if ((0 <= heading) && (heading <= 360)) {
      SHIPSETHEADING(ch->in_room) = heading;
      sprintf(buf, "Heading set to &+W%d&N.\r\n", heading);
      send_to_room(buf, ch->in_room);
      return TRUE;
    }
    send_to_char("Please enter a heading from 0-360.\r\n", ch);
    return TRUE;
  }
  if (CMD_IS("disembark")) {
    int to_room, in_room;
    short int x, y, z;

    y = SHIPX(ch->in_room) - 50;
    x = 100 - SHIPY(ch->in_room);
    z = SHIPZ(ch->in_room);
    to_room = real_room(SHIPLOCATION(ch->in_room));
    in_room = ch->in_room;
    if (to_room == NOWHERE) {
      send_to_char("This ship is in Limbo.\r\n", ch);
      return TRUE;
    }
    sprintf(buf, "You disembark %s.\r\n", SHIPNAME(ch->in_room));
    send_to_char(buf, ch);
    sprintf(buf, "%s disembarks %s.\r\n", GET_NAME(ch), SHIPNAME(ch->in_room));
    char_from_room(ch);
    send_to_room(buf, to_room);
    send_to_room(buf, in_room);
    put_char_to_room(ch, to_room, -1, x, y, z);
    return TRUE;
  }
  if (CMD_IS("debug")) {
    sprintf(buf, "SHIPLOCATION: %d\r\n", SHIPLOCATION(ch->in_room));
    send_to_char(buf, ch);
    return TRUE;
  }
  return FALSE;
}

SPECIAL(ship_object)
{
  struct obj_data *obj;
  int to_room;

  obj = me;

  if (!ch || !cmd)
    return FALSE;

  if (GET_OBJ_TYPE(obj) != ITEM_SHIP)
    return FALSE;

  if (CMD_IS("enter")) {
    one_argument(argument, arg);

    if (!*arg) {
      return FALSE;
    } else {
      if (isname(arg, GET_OBJ_NAME(obj))) {
	to_room = real_room(GET_OBJ_VAL(obj, 0));
	if (to_room == NOWHERE) {
	  send_to_char("ERROR, tell a god NOW!\r\n", ch);
	  return TRUE;
	}
	sprintf(buf, "You board %s.\r\n", obj->description);
	send_to_char(buf, ch);
	sprintf(buf, "%s boards %s.\r\n", GET_NAME(ch), obj->description);
	char_from_room(ch);
	send_to_room(buf, obj->in_room);
	send_to_room(buf, to_room);
	char_to_room(ch, to_room, -1);
	return TRUE;
      }
    }
  }
  return FALSE;
}

SPECIAL(ship_loader)
{
  if (!ch || !cmd)
    return FALSE;

  if (CMD_IS("showmap")) {
    int x, y;
    two_arguments(argument, arg1, arg2);
    if (!*arg1 || !*arg2) {
      send_to_char("need X Y\r\n", ch);
      return TRUE;
    }
    if (is_number(arg1) && is_number(arg2)) {
      x = atoi(arg1);
      y = atoi(arg2);
      sprintf(buf, "map symbol for this coord: %s\r\n", tactical[x][y].map);
      send_to_char(buf, ch);
      sprintf(buf, "mapgrid symbol: %d\r\n", world[ch->in_room].map->map_grid[GET_X_CORD(ch)][GET_Y_CORD(ch)]);
      send_to_char(buf, ch);
      return TRUE;
    }
    send_to_char("X Y must be numbers\r\n", ch);
    return TRUE;
  }
  if (CMD_IS("mapload")) {
    int x, y;
    for (x = 0; x < 150; x++) {
      for (y = 0; y < 150; y++) {
	strcpy(tactical[x][y].map, "&+y^^&N");
      }
    }
    send_to_char("Loaded test map\r\n", ch);
    return TRUE;
  }
  if (CMD_IS("shipload")) {
    int template, shipnum, shiproom;

    one_argument(argument, arg);

    if (!*arg) {
      send_to_char("Please specify template number.\r\n", ch);
      return TRUE;
    }
    if (!is_number(arg)) {
      send_to_char("Invalid template!\r\n", ch);
      return TRUE;
    }
    template = atoi(arg);
    send_to_char("Loading ship data\r\n", ch);
    shipnum = loadship(template, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
    if (!setsail(150, shipnum))
      send_to_char("Ship cannot support this sail.\r\n", ch);
    shiproom = real_room(ships[shipnum].shiproom);
    sprintf(buf, "&+LThe 'Chtorran'&N");
    nameship(buf, SHIPNUM(shiproom));
    strcpy(SHIPOWNER(shiproom), GET_NAME(ch));
    strcpy(SHIPID(shiproom), "AB");
    strcpy(SHIPSAILNAME(shiproom), "Magical Automotons");
    strcpy(SHIPGUNNAME(shiproom), "Magical Automotons");
    strcpy(SHIPSLOT(shiproom)[0].desc, "Catapult of Slaying");
    strcpy(SHIPSLOT(shiproom)[1].desc, "Large Ballistae");
    strcpy(SHIPSLOT(shiproom)[2].desc, "Large Ballistae");
    SHIPSLOT(shiproom)[0].type = 1;
    SHIPSLOT(shiproom)[0].timer = 0;
    SHIPSLOT(shiproom)[0].position = FORE;
    SHIPSLOT(shiproom)[0].val0 = 25;
    SHIPSLOT(shiproom)[0].val1 = 10;
    SHIPSLOT(shiproom)[0].val2 = 20;
    SHIPSLOT(shiproom)[0].val3 = 100;
    SHIPSLOT(shiproom)[1].type = 1;
    SHIPSLOT(shiproom)[1].timer = 0;
    SHIPSLOT(shiproom)[1].position = PORT;
    SHIPSLOT(shiproom)[1].val0 = 15;
    SHIPSLOT(shiproom)[1].val1 = 0;
    SHIPSLOT(shiproom)[1].val2 = 7;
    SHIPSLOT(shiproom)[1].val3 = 100;
    SHIPSLOT(shiproom)[2].type = 1;
    SHIPSLOT(shiproom)[2].timer = 23;
    SHIPSLOT(shiproom)[2].position = STARBOARD;
    SHIPSLOT(shiproom)[2].val0 = 15;
    SHIPSLOT(shiproom)[2].val1 = 0;
    SHIPSLOT(shiproom)[2].val2 = 7;
    SHIPSLOT(shiproom)[2].val3 = 100;
    send_to_char("Done.\r\n", ch);
    return TRUE;
  }
  if (CMD_IS("setsail")) {
    if (world[ch->in_room].ship == NULL) {
      send_to_char("&+WThis room is not a ship or has no data.&N\r\n", ch);
      return TRUE;
    }
    one_argument(argument, arg);

    if (!*arg || !is_number(arg)) {
      send_to_char("Please select a Sail Class: 0-150\r\n", ch);
      return TRUE;
    }
    if (!setsail(atoi(arg), SHIPNUM(ch->in_room))) {
      send_to_char("This ship cannot support this class of sail.\r\n", ch);
      return TRUE;
    }
    send_to_char("done.\r\n", ch);
    return TRUE;
  }
  return FALSE;
}
void getstatus(int slot, int rnum)
{
  if (world[rnum].ship->slot[slot].timer > 0)
    sprintf(status, "&+R%-6d", world[rnum].ship->slot[slot].timer);
  if (world[rnum].ship->slot[slot].timer == 0)
    strcpy(status, "Ready");
  if (world[rnum].ship->slot[slot].timer < 0)
    strcpy(status, "&+L***   ");
  if (world[rnum].ship->slot[slot].desc == NULL)
    strcpy(status, NULL);
}

void getposition(int slot, int rnum)
{
  switch (world[rnum].ship->slot[slot].position) {
  case FORE:
    strcpy(position, "Forward");
    break;
  case REAR:
    strcpy(position, "Rear");
    break;
  case PORT:
    strcpy(position, "Port");
    break;
  case STARBOARD:
    strcpy(position, "Starboard");
    break;
  default:
    strcpy(position, "ERROR");
    break;
  }
  if (world[rnum].ship->slot[slot].desc == NULL)
    strcpy(position, NULL);
}

void dispweapon(int slot, int rnum)
{
  if (world[rnum].ship->slot[slot].type != 1)
    strcpy(weapon, " ");
  else {
    getstatus(slot, rnum);
    getposition(slot, rnum);
    sprintf(weapon, "%-20s &N%-6s  &+W%-9s  %d",
	    world[rnum].ship->slot[slot].desc, status, position, world[rnum].ship->slot[slot].val3);
  }
}
int loadship(int template, int to_room, short int x_cord, short int y_cord, short int z_cord)
{
  extern void olc_add_to_save_list(int zone, byte type);
  extern redit_save_to_disk(int zone);
  struct obj_data *shiptemplateobj = NULL, *obj = NULL;
  int i = 45000, j = 0, x, y, c, rroom;
  unsigned char armor, internal;

  shiptemplateobj = read_object(template, VIRTUAL);
  obj = read_object(4500, VIRTUAL);
  (unsigned char) armor = GET_OBJ_VAL(shiptemplateobj, 2) / 2;
  (unsigned char) internal = armor / 4;

  if (armor == 0)
    armor = 1;
  if (internal == 0)
    internal = 1;

  while (world[real_room(i)].ship != NULL) {
    i++;
  }
  while (ships[j].shiproom != 0) {
    j++;
  }
  world[real_room(i)].ship = &ships[j];
  ships[j].shipnum = j;
  ships[j].shiproom = i;
  ships[j].farmor = armor;
  ships[j].maxfarmor = armor;
  ships[j].rarmor = armor;
  ships[j].maxrarmor = armor;
  ships[j].parmor = armor;
  ships[j].maxparmor = armor;
  ships[j].sarmor = armor;
  ships[j].maxsarmor = armor;
  ships[j].finternal = internal;
  ships[j].maxfinternal = internal;
  ships[j].rinternal = internal;
  ships[j].maxrinternal = internal;
  ships[j].pinternal = internal;
  ships[j].maxpinternal = internal;
  ships[j].sinternal = internal;
  ships[j].maxsinternal = internal;
  ships[j].hullweight = GET_OBJ_VAL(shiptemplateobj, 2);
  (float) ships[j].x = y_cord + 50;
  (float) ships[j].y = (50 - x_cord) + 50;
  ships[j].z = z_cord;
  ships[j].location = world[to_room].number;
  ships[j].shipobj = obj;
  GET_OBJ_VAL(ships[j].shipobj, 0) = i;
  GET_OBJ_VAL(ships[j].shipobj, 1) = j;
  rroom = real_room(i);
  world[rroom].room_flags = world[real_room(template)].room_flags;
  world[rroom].x_size = world[real_room(template)].x_size;
  world[rroom].y_size = world[real_room(template)].y_size;
  world[rroom].z_size = world[real_room(template)].z_size;

  if (world[rroom].map != NULL) {
#ifdef NEWMAP
    for (c = 0; c < world[rroom].x_size + 1; c++)
      FREE(world[rroom].map->map_grid[c]);
    FREE(world[rroom].map->map_grid);
#endif
    FREE(world[rroom].map);
  }
  if (world[real_room(template)].map) {
    if (world[rroom].map == NULL) {
      CREATE(world[rroom].map, struct map_data, 1);
#ifdef NEWMAP
      CREATE(world[rroom].map->map_grid, sh_int *,
	     world[real_room(template)].x_size + 1);
      for (c = 0; c < world[real_room(template)].x_size + 1; c++)
	CREATE(world[rroom].map->map_grid[c], sh_int,
	       world[rroom].y_size + 1);
#endif
    }
    for (x = 0; x < world[rroom].x_size; x++) {
      for (y = 0; y < world[rroom].y_size; y++) {
	world[rroom].map->map_grid[x][y] = world[real_room(template)].map->map_grid[x][y];
      }
    }
  }
//  olc_add_to_save_list((i / 100), OLC_SAVE_ROOM);
  //  redit_save_to_disk(i/100);
  put_obj_to_room(ships[j].shipobj, to_room, x_cord, y_cord, z_cord);
  return j;
}
void nameship(char *name, int shipnum)
{
  sprintf(ships[shipnum].name, name);
  ships[shipnum].shipobj->description = NULL;
  ships[shipnum].shipobj->short_description = NULL;
  ships[shipnum].shipobj->description = str_dup(name);
  ships[shipnum].shipobj->short_description = str_dup(name);
}
bool setsail(int class, int shipnum)
{
  int weight, i;
  if (ships[shipnum].hullweight < class - 25)
    return FALSE;

  weight = 0;
  for (i = 0; i < 10; i++) {
    weight += ships[shipnum].slot[i].weight;
  }

  if ((weight + class / 3) > (ships[shipnum].hullweight / 2))
    return FALSE;
  ships[shipnum].minspeed = -1;
  (int) ships[shipnum].maxspeed = class * (float) (66.0 / ships[shipnum].hullweight) + 0.5;
  if (ships[shipnum].maxspeed < 0)
    ships[shipnum].maxspeed = 0;
  (unsigned char) ships[shipnum].mainsail = class / 2;
  (unsigned char) ships[shipnum].maxmainsail = class / 2;
  return TRUE;
}

int weaprange(int shipnum, int slot, char range)
{
  if (ships[shipnum].slot[slot].type != 1)
    return 0;
  switch (range) {
  case SHRTRANGE:
    return (int) ((float) (ships[shipnum].slot[slot].val0
			   - ships[shipnum].slot[slot].val1) / 3
		  + ships[shipnum].slot[slot].val1);
    break;
  case MEDRANGE:
    return (int) ((float) ((ships[shipnum].slot[slot].val0
			    - ships[shipnum].slot[slot].val1) / 3) * 2
		  + ships[shipnum].slot[slot].val1);
    break;
  case LNGRANGE:
    return (ships[shipnum].slot[slot].val0);
    break;
  default:
    return 0;
    break;
  }
  return 0;
}

int bearing(float x1, float y1, float x2, float y2)
{
  int val;
  if (y1 == y2) {
    if (x1 > x2)
      return 270;
    return 90;
  }
  if (x1 == x2) {
    if (y1 > y2)
      return 180;
    else
      return 0;
  }
  val = atan((x2 - x1) / (y2 - y1)) * 180 / PI;
  if (y1 < y2) {
    if (val >= 0)
      return val;
    return (val + 360);
  } else {
    return val + 180;
  }
  return 0;
}

float range(float x1, float y1, float z1, float x2, float y2, float z2)
{
  float dx, dy, dz, range;

  dx = x2 - x1;
  dy = y2 - y1;
  dz = z2 - z1;

  range = sqrt((dx * dx) + (dy * dy) + (dz * dz));
  return range;
}

void dispcontact(int i)
{
  int x, y, z, bearing, j;
  float range;

  x = contacts[i].x;
  y = contacts[i].y;
  z = contacts[i].z;
  range = contacts[i].range;
  bearing = contacts[i].bearing;
  j = contacts[i].shipnum;

  sprintf(contact, "[%s] %-30s X:%-3d Y:%-3d Z:%-3d R:%-5.1f B:%-3d H:%-3d S:%-3d|%s\r\n",
   ships[j].id, ships[j].name, x, y, z, range, bearing, ships[j].heading,
	  ships[j].speed, contacts[i].arc);
}

int getcontacts(int shipnum)
{
  int rroom, i, j, to_room;
  struct obj_data *obj, *obj_next;

  rroom = real_room(ships[shipnum].location);

  i = 0;

  for (obj = world[rroom].contents; obj; obj = obj_next) {
    obj_next = obj->next_content;
    if (GET_OBJ_TYPE(obj) == ITEM_SHIP) {
      if (GET_OBJ_VAL(obj, 1) != shipnum) {
	if (range(ships[shipnum].x,
		  ships[shipnum].y,
		  ships[shipnum].z,
		  ships[GET_OBJ_VAL(obj, 1)].x,
		  ships[GET_OBJ_VAL(obj, 1)].y,
		  ships[GET_OBJ_VAL(obj, 1)].z) <= 35) {
	  setcontact(i, obj, shipnum, 0, 0);
	  i++;
	}
      }
    }
  }
  for (j = 0; j < NUM_OF_DIRS; j++) {
    if (world[rroom].dir_option[j]) {
      if (world[rroom].dir_option[j]->to_room != NOWHERE) {
	to_room = world[rroom].dir_option[j]->to_room;
	for (obj = world[to_room].contents; obj; obj = obj_next) {
	  obj_next = obj->next_content;
	  if (GET_OBJ_TYPE(obj) == ITEM_SHIP) {
	    if (GET_OBJ_VAL(obj, 1) != shipnum) {
	      switch (j) {
	      case NORTH:
		if (range(ships[shipnum].x,
			  ships[shipnum].y,
			  ships[shipnum].z,
			  ships[GET_OBJ_VAL(obj, 1)].x,
			  ships[GET_OBJ_VAL(obj, 1)].y + 50,
			  ships[GET_OBJ_VAL(obj, 1)].z) <= 35) {
		  setcontact(i, obj, shipnum, 0, 50);
		  i++;
		}
		break;
	      case NORTHEAST:
		if (range(ships[shipnum].x,
			  ships[shipnum].y,
			  ships[shipnum].z,
			  ships[GET_OBJ_VAL(obj, 1)].x + 50,
			  ships[GET_OBJ_VAL(obj, 1)].y + 50,
			  ships[GET_OBJ_VAL(obj, 1)].z) <= 35) {
		  setcontact(i, obj, shipnum, 50, 50);
		  i++;
		}
		break;
	      case EAST:
		if (range(ships[shipnum].x,
			  ships[shipnum].y,
			  ships[shipnum].z,
			  ships[GET_OBJ_VAL(obj, 1)].x + 50,
			  ships[GET_OBJ_VAL(obj, 1)].y,
			  ships[GET_OBJ_VAL(obj, 1)].z) <= 35) {
		  setcontact(i, obj, shipnum, 50, 0);
		  i++;
		}
		break;
	      case SOUTHEAST:
		if (range(ships[shipnum].x,
			  ships[shipnum].y,
			  ships[shipnum].z,
			  ships[GET_OBJ_VAL(obj, 1)].x + 50,
			  ships[GET_OBJ_VAL(obj, 1)].y - 50,
			  ships[GET_OBJ_VAL(obj, 1)].z) <= 35) {
		  setcontact(i, obj, shipnum, 50, -50);
		  i++;
		}
		break;
	      case SOUTH:
		if (range(ships[shipnum].x,
			  ships[shipnum].y,
			  ships[shipnum].z,
			  ships[GET_OBJ_VAL(obj, 1)].x,
			  ships[GET_OBJ_VAL(obj, 1)].y - 50,
			  ships[GET_OBJ_VAL(obj, 1)].z) <= 35) {
		  setcontact(i, obj, shipnum, 0, -50);
		  i++;
		}
		break;
	      case SOUTHWEST:
		if (range(ships[shipnum].x,
			  ships[shipnum].y,
			  ships[shipnum].z,
			  ships[GET_OBJ_VAL(obj, 1)].x - 50,
			  ships[GET_OBJ_VAL(obj, 1)].y - 50,
			  ships[GET_OBJ_VAL(obj, 1)].z) <= 35) {
		  setcontact(i, obj, shipnum, -50, -50);
		  i++;
		}
		break;
	      case WEST:
		if (range(ships[shipnum].x,
			  ships[shipnum].y,
			  ships[shipnum].z,
			  ships[GET_OBJ_VAL(obj, 1)].x - 50,
			  ships[GET_OBJ_VAL(obj, 1)].y,
			  ships[GET_OBJ_VAL(obj, 1)].z) <= 35) {
		  setcontact(i, obj, shipnum, -50, 0);
		  i++;
		}
		break;
	      case NORTHWEST:
		if (range(ships[shipnum].x,
			  ships[shipnum].y,
			  ships[shipnum].z,
			  ships[GET_OBJ_VAL(obj, 1)].x - 50,
			  ships[GET_OBJ_VAL(obj, 1)].y + 50,
			  ships[GET_OBJ_VAL(obj, 1)].z) <= 35) {
		  setcontact(i, obj, shipnum, -50, 50);
		  i++;
		}
		break;
	      case UP:
		break;
	      case DOWN:
		break;
	      default:
		break;
	      }
	    }
	  }
	}
      }
    }
  }
  return i;
}

void setcontact(int i, struct obj_data *obj, int shipnum, int xoffset, int yoffset)
{
  contacts[i].bearing = bearing(ships[shipnum].x,
				ships[shipnum].y,
			(ships[GET_OBJ_VAL(obj, 1)].x + (float) xoffset),
		       (ships[GET_OBJ_VAL(obj, 1)].y + (float) yoffset));

  contacts[i].range = range(ships[shipnum].x,
			    ships[shipnum].y,
			    ships[shipnum].z,
			    ships[GET_OBJ_VAL(obj, 1)].x + xoffset,
			    ships[GET_OBJ_VAL(obj, 1)].y + yoffset,
			    ships[GET_OBJ_VAL(obj, 1)].z);

  contacts[i].x = (int) ships[GET_OBJ_VAL(obj, 1)].x + xoffset;
  contacts[i].y = (int) ships[GET_OBJ_VAL(obj, 1)].y + yoffset;
  contacts[i].z = (int) ships[GET_OBJ_VAL(obj, 1)].z;
  getarc(shipnum, GET_OBJ_VAL(obj, 1));
  strcpy(contacts[i].arc, arc);
}

int getarc(int ship1, int ship2)
{
  float x1, y1, x2, y2;
  int shipbearing, shipheading, rroom, to_room, i;

  rroom = real_room(ships[ship1].location);
  to_room = real_room(ships[ship2].location);

  x1 = ships[ship1].x;
  y1 = ships[ship1].y;
  shipheading = ships[ship1].heading;

  x2 = ships[ship2].x;
  y2 = ships[ship2].y;
  for (i = 0; i < NUM_OF_DIRS; i++) {
    if (world[rroom].dir_option[i]) {
      if (world[rroom].dir_option[i]->to_room == to_room) {
	switch (i) {
	case NORTH:
	  y2 += 50;
	  break;
	case NORTHEAST:
	  x2 += 50;
	  y2 += 50;
	  break;
	case EAST:
	  x2 += 50;
	  break;
	case SOUTHEAST:
	  x2 += 50;
	  y2 -= 50;
	  break;
	case SOUTH:
	  y2 -= 50;
	  break;
	case SOUTHWEST:
	  x2 -= 50;
	  y2 -= 50;
	  break;
	case WEST:
	  x2 -= 50;
	  break;
	case NORTHWEST:
	  x2 -= 50;
	  y2 += 50;
	  break;
	default:
	  return FALSE;
	  break;
	}
      }
    }
  }
  shipbearing = bearing(x1, y1, x2, y2);
  if (shipheading < shipbearing)
    shipheading += 360;
  sprintf(arc, "*");
  if ((shipbearing > (shipheading - 140)) && (shipbearing < (shipheading - 40))) {
    sprintf(arc, "P");
    return PORT;
  }
  if ((shipbearing > (shipheading - 220)) && (shipbearing < (shipheading - 40))) {
    sprintf(arc, "R");
    return REAR;
  }
  if ((shipbearing > (shipheading - 320)) && (shipbearing < (shipheading - 40))) {
    sprintf(arc, "S");
    return STARBOARD;
  }
  if (((shipbearing > (shipheading - 40)) && (shipbearing < (shipheading + 40))) || (shipbearing < (shipheading - 320))
      || (shipbearing == shipheading)) {
    sprintf(arc, "F");
    return FORE;
  }
  return FORE;
}

void getmap(int shipnum)
{
  extern reload_room_maps(int real_nr);
  extern unload_room_maps(int zone);
  int x, y, i, rroom, to_room = 0, k;
  for (x = 0; x < 150; x++) {
    for (y = 0; y < 150; y++) {
      strcpy(tactical[x][y].map, "  ");
    }
  }
  rroom = real_room(ships[shipnum].location);
  if (world[rroom].map == NULL)
    reload_room_maps(rroom);
  for (y = 50; y < 100; y++) {
    for (x = 50; x < 100; x++) {
      if (world[rroom].map) {
	setsymbol(x, y, world[rroom].map->map_grid[y - 50][x - 50]);
      }
    }
  }
  for (i = 0; i < NUM_OF_DIRS; i++) {
    if (world[rroom].dir_option[i]) {
      if (world[rroom].dir_option[i]->to_room != NOWHERE) {
	switch (i) {
	case NORTH:
	  to_room = world[rroom].dir_option[i]->to_room;
	  if (world[to_room].map == NULL)
	    reload_room_maps(to_room);
	  for (y = 0; y < 50; y++) {
	    for (x = 50; x < 100; x++) {
	      if (world[to_room].map) {
		setsymbol(x, y, world[to_room].map->map_grid[y][x - 50]);
	      }
	    }
	  }
	  break;
	case NORTHEAST:
	  to_room = world[rroom].dir_option[i]->to_room;
	  if (world[to_room].map == NULL)
	    reload_room_maps(to_room);
	  for (y = 0; y < 50; y++) {
	    for (x = 1000; x < 150; x++) {
	      if (world[to_room].map)
		setsymbol(x, y, world[to_room].map->map_grid[y][x - 100]);
	    }
	  }
	  break;
	case EAST:
	  to_room = world[rroom].dir_option[i]->to_room;
	  if (world[to_room].map == NULL)
	    reload_room_maps(to_room);
	  for (y = 50; y < 100; y++) {
	    for (x = 100; x < 150; x++) {
	      if (world[to_room].map)
		setsymbol(x, y, world[to_room].map->map_grid[y - 50][x - 100]);
	    }
	  }
	  break;
	case SOUTHEAST:
	  to_room = world[rroom].dir_option[i]->to_room;
	  if (world[to_room].map == NULL)
	    reload_room_maps(to_room);
	  for (y = 100; y < 150; y++) {
	    for (x = 100; x < 150; x++) {
	      if (world[to_room].map)
		setsymbol(x, y, world[to_room].map->map_grid[y - 100][x - 100]);
	    }
	  }
	  break;
	case SOUTH:
	  to_room = world[rroom].dir_option[i]->to_room;
	  if (world[to_room].map == NULL)
	    reload_room_maps(to_room);
	  for (y = 100; y < 150; y++) {
	    for (x = 50; x < 100; x++) {
	      if (world[to_room].map)
		setsymbol(x, y, world[to_room].map->map_grid[y - 100][x - 50]);
	    }
	  }
	  break;
	case SOUTHWEST:
	  to_room = world[rroom].dir_option[i]->to_room;
	  if (world[to_room].map == NULL)
	    reload_room_maps(to_room);
	  for (y = 100; y < 150; y++) {
	    for (x = 0; x < 50; x++) {
	      if (world[to_room].map)
		setsymbol(x, y, world[to_room].map->map_grid[y - 100][x]);
	    }
	  }
	  break;
	case WEST:
	  to_room = world[rroom].dir_option[i]->to_room;
	  if (world[to_room].map == NULL)
	    reload_room_maps(to_room);
	  for (y = 50; y < 100; y++) {
	    for (x = 0; x < 50; x++) {
	      if (world[to_room].map)
		setsymbol(x, y, world[to_room].map->map_grid[y - 50][x]);
	    }
	  }
	  break;
	case NORTHWEST:
	  to_room = world[rroom].dir_option[i]->to_room;
	  if (world[to_room].map == NULL)
	    reload_room_maps(to_room);
	  for (y = 0; y < 50; y++) {
	    for (x = 0; x < 50; x++) {
	      if (world[to_room].map)
		setsymbol(x, y, world[to_room].map->map_grid[y][x]);
	    }
	  }
	  break;
	default:
	  break;
	}
      }
    }
  }
  k = getcontacts(shipnum);
  for (i = 0; i < k; i++) {
    sprintf(tactical[contacts[i].x][150 - contacts[i].y].map, "&+W%s&N",
	    ships[contacts[i].shipnum].id);
  }
  sprintf(tactical[(int) ships[shipnum].x][150 - (int) ships[shipnum].y].map, "&+W**&N");
}

void setsymbol(int x, int y, int symbol)
{
  switch (symbol) {
  case 11:
    strcpy(tactical[x][y].map, "&+g\"\"&N");
    break;
  case 12:
    strcpy(tactical[x][y].map, "&+g**&N");
    break;
  case 13:
    strcpy(tactical[x][y].map, "&+y^^&N");
    break;
  case 15:
    strcpy(tactical[x][y].map, "&+L++&N");
    break;
  case 17:
    strcpy(tactical[x][y].map, "&+b~~&N");
    break;
  case 18:
    strcpy(tactical[x][y].map, "&+y^^&N");
    break;
  case 19:
    strcpy(tactical[x][y].map, "&+L%%&N");
    break;
  case 20:
    strcpy(tactical[x][y].map, "&+g**&N");
    break;
  case 21:
    strcpy(tactical[x][y].map, "&+g..&N");
    break;
  case 24:
    strcpy(tactical[x][y].map, "&+b~~&N");
    break;
  case 35:
    strcpy(tactical[x][y].map, "&+Y..&N");
    break;
  case 36:
    strcpy(tactical[x][y].map, "&+Y**&N");
    break;
  case 37:
    strcpy(tactical[x][y].map, "&+W..&N");
    break;
  case 38:
    strcpy(tactical[x][y].map, "&+W^^&N");
    break;
  case 50:
    strcpy(tactical[x][y].map, "&+R%%&N");
    break;
  default:
    strcpy(tactical[x][y].map, "&+y..&N");
    break;
  }
}
