
/*
 **************************************************************************
 *  File: sound.c                                          Part of Greyhawk
 *  Usage: sound fonction for MSP
 *  Copyright  1998 - Greyhawk Adventures.
 **************************************************************************
 */

#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "olc.h"
#include "sound.h"

/* extern variables */
extern struct room_data *world;
extern struct descriptor_data *descriptor_list;
extern struct char_data *character_list;


void sound_to_char(char *messg, struct char_data *ch)
{
  if (ch && ch->desc && messg && (ch->desc->term_type == TERM_MSP || ch->desc->term_type == TERM_GREY)) {
    send_to_char(messg, ch);
  }
}

void sound_to_outdoor(char *messg)
{
  struct descriptor_data *i;

  if (!messg || !*messg)
    return;

  for (i = descriptor_list; i; i = i->next)
    if (!i->connected && i->character && AWAKE(i->character) &&
	(i->term_type == TERM_MSP || i->term_type == TERM_GREY) &&
      OUTSIDE(i->character) && !PLR_FLAGGED((i->character), PLR_WRITING))
      SEND_TO_Q(messg, i);
}

void sound_to_all(const char *messg)
{
  struct descriptor_data *i;

  if (messg)
    for (i = descriptor_list; i; i = i->next)
      if (!i->connected && (i->term_type == TERM_MSP || i->term_type == TERM_GREY))
	SEND_TO_Q(messg, i);
}

void sound_to_room(const char *messg, int room, int xcord, int ycord, int zcord)
{
  struct char_data *i;

  if (messg)
    for (i = world[room].people; i; i = i->next_in_room)
      if (i->desc && (i->desc->term_type == TERM_MSP || i->desc->term_type == TERM_GREY))
	if (GET_Z_CORD(i) == zcord)
	  SEND_TO_Q(messg, i->desc);
}

void sound_to_zone(const char *messg, int zone)
{
  struct descriptor_data *i;

  if (messg)
    for (i = descriptor_list; i; i = i->next)
      if (!i->connected && (i->term_type == TERM_MSP || i->term_type == TERM_GREY))
	if (world[i->character->in_room].zone == zone)
	  SEND_TO_Q(messg, i);
}

void sound_to_zone_outdoor(int zone_rnum, char *messg)
{
  struct descriptor_data *i;

  if (!messg || !*messg)
    return;

  for (i = descriptor_list; i; i = i->next)
    if (!i->connected && i->character && AWAKE(i->character) &&
	(IN_ROOM(i->character) != NOWHERE) &&
	!IS_SET(ROOM_FLAGS(i->character->in_room), ROOM_INDOORS) &&
	(i->term_type == TERM_MSP || i->term_type == TERM_GREY) &&
	(world[IN_ROOM(i->character)].zone == zone_rnum))
      SEND_TO_Q(messg, i);
}

void sound_to_zone_indoor(int zone_rnum, char *messg)
{
  struct descriptor_data *i;

  if (!messg || !*messg)
    return;

  for (i = descriptor_list; i; i = i->next)
    if (!i->connected && i->character && AWAKE(i->character) &&
	(IN_ROOM(i->character) != NOWHERE) &&
	IS_SET(ROOM_FLAGS(i->character->in_room), ROOM_INDOORS) &&
	(i->term_type == TERM_MSP || i->term_type == TERM_GREY) &&
	(world[IN_ROOM(i->character)].zone == zone_rnum))
      SEND_TO_Q(messg, i);
}

void play_sound(const char *mess, struct char_data *ch, int where, int xcord, int ycord, int zcord, int type)
{

  switch (type) {
  case TO_CHAR:
    if (ch)
      sound_to_char((char *) mess, ch);
    break;
  case TO_ROOM:
    if (where > 0)
      sound_to_room(mess, where, xcord, ycord, zcord);
    break;
  case TO_ZONE:
    if (where > 0)
      sound_to_zone(mess, where);
    break;
  case TO_WORLD:
    sound_to_all(mess);
    break;
  }
}
