/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"

#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"


/*   external vars  */
extern struct room_data *world;
extern struct char_data *character_list;
extern struct descriptor_data *descriptor_list;
extern struct index_data *mob_index;
extern struct index_data *obj_index;
extern struct time_info_data time_info;
extern char *dirs[];
extern int rev_dir[];
extern char *activating_commands[];

/* extern functions */
void add_follower(struct char_data *ch, struct char_data *leader);
extern void smallmap_look(struct char_data *ch, int size, int dif_room);

struct social_type {
  char *cmd;
  int next_line;
};


/* ********************************************************************
   *  Special procedures for mobiles                                     *
   ******************************************************************** */

int spell_sort_info[MAX_SPELLS + 1];

extern char *spells[];

void sort_spells(void)
{
  int a, b, tmp;

  /* initialize array */
  for (a = 1; a < MAX_SPELLS; a++)
    spell_sort_info[a] = a;

  /* Sort.  'a' starts at 1, not 0, to remove 'RESERVED' */
  for (a = 1; a < MAX_SPELLS - 1; a++)
    for (b = a + 1; b < MAX_SPELLS; b++)
      if (strcmp(spells[spell_sort_info[a]], spells[spell_sort_info[b]]) > 0) {
	tmp = spell_sort_info[a];
	spell_sort_info[a] = spell_sort_info[b];
	spell_sort_info[b] = tmp;
      }
}


char *how_good(int percent)
{
  static char buf[256];

  if (percent == 0)
    strcpy(buf, " (not learned)");
  else if (percent <= 10)
    strcpy(buf, " (awful)");
  else if (percent <= 20)
    strcpy(buf, " (bad)");
  else if (percent <= 40)
    strcpy(buf, " (poor)");
  else if (percent <= 55)
    strcpy(buf, " (average)");
  else if (percent <= 70)
    strcpy(buf, " (fair)");
  else if (percent <= 80)
    strcpy(buf, " (good)");
  else if (percent <= 85)
    strcpy(buf, " (very good)");
  else
    strcpy(buf, " (superb)");

  return (buf);
}

SPECIAL(dump)
{
  struct obj_data *k;
  int value = 0;

  ACMD(do_drop);
  char *fname(char *namelist);

  for (k = world[ch->in_room].contents; k; k = world[ch->in_room].contents) {
    act("$p vanishes in a puff of smoke!", FALSE, 0, k, 0, PROXIMITY);
    extract_obj(k);
  }

  if (!CMD_IS("drop"))
    return 0;

  do_drop(ch, argument, cmd, 0);

  for (k = world[ch->in_room].contents; k; k = world[ch->in_room].contents) {
    act("$p vanishes in a puff of smoke!", FALSE, 0, k, 0, PROXIMITY);
    value += MAX(1, MIN(50, GET_OBJ_COST(k) / 10));
    extract_obj(k);
  }

  if (value) {
    act("You are awarded for outstanding performance.", FALSE, ch, 0, 0, TO_CHAR);
    act("$n has been awarded for being a good citizen.", TRUE, ch, 0, 0, PROXIMITY);

    if (GET_LEVEL(ch) < 3)
      gain_exp(ch, value);
    else
      ADD_MONEY(ch, value);
  }
  return 1;
}


SPECIAL(mayor)
{
  ACMD(do_gen_door);

  static char open_path[] =
  "W3a3003b33000c111d0d111Oe333333Oe22c222112212111a1S.";

  static char close_path[] =
  "W3a3003b33000c111d0d111CE333333CE22c222112212111a1S.";

  static char *path;
  static int index;
  static bool move = FALSE;

  if (!move) {
    if (time_info.hours == 6) {
      move = TRUE;
      path = open_path;
      index = 0;
    } else if (time_info.hours == 20) {
      move = TRUE;
      path = close_path;
      index = 0;
    }
  }
  if (cmd || !move || (GET_STAT(ch) < STAT_SLEEPING) ||
      IS_FIGHTING(ch))
    return FALSE;

  switch (path[index]) {
  case '0':
  case '1':
  case '2':
  case '3':
    perform_move(ch, path[index] - '0', 1);
    break;

  case 'W':
    SET_POS(ch, POS_STANDING + STAT_NORMAL);
    act("$n awakens and groans loudly.", FALSE, ch, 0, 0, PROXIMITY);
    break;

  case 'S':
    SET_POS(ch, POS_PRONE + STAT_SLEEPING);
    act("$n lies down and instantly falls asleep.", FALSE, ch, 0, 0, PROXIMITY);
    break;

  case 'a':
    act("$n says 'Hello Honey!'", FALSE, ch, 0, 0, PROXIMITY);
    act("$n smirks.", FALSE, ch, 0, 0, PROXIMITY);
    break;

  case 'b':
    act("$n says 'What a view!  I must get something done about that dump!'",
	FALSE, ch, 0, 0, PROXIMITY);
    break;

  case 'c':
    act("$n says 'Vandals!  Youngsters nowadays have no respect for anything!'",
	FALSE, ch, 0, 0, PROXIMITY);
    break;

  case 'd':
    act("$n says 'Good day, citizens!'", FALSE, ch, 0, 0, PROXIMITY);
    break;

  case 'e':
    act("$n says 'I hereby declare the bazaar open!'", FALSE, ch, 0, 0, PROXIMITY);
    break;

  case 'E':
    act("$n says 'I hereby declare Midgaard closed!'", FALSE, ch, 0, 0, PROXIMITY);
    break;

  case 'O':
    do_gen_door(ch, "gate", 0, SCMD_UNLOCK);
    do_gen_door(ch, "gate", 0, SCMD_OPEN);
    break;

  case 'C':
    do_gen_door(ch, "gate", 0, SCMD_CLOSE);
    do_gen_door(ch, "gate", 0, SCMD_LOCK);
    break;

  case '.':
    move = FALSE;
    break;

  }

  index++;
  return FALSE;
}


/* ********************************************************************
   *  General special procedures for mobiles                             *
   ******************************************************************** */


void npc_steal(struct char_data *ch, struct char_data *victim)
{
  int gold;

  if (IS_NPC(victim))
    return;
  if (GET_LEVEL(victim) >= LVL_IMMORT)
    return;

  if (AWAKE(victim) && (number(0, GET_LEVEL(ch)) == 0)) {
    act("You discover that $n has $s hands in your wallet.", FALSE, ch, 0, victim, TO_VICT);
    act("$n tries to steal gold from $N.", TRUE, ch, 0, victim, TO_NOTVICT);
  } else {
    /* Steal some gold coins */
    gold = (int) ((GET_MONEY(victim) * number(1, 10)) / 100);
    if (gold > 0) {
      ADD_MONEY(ch, gold);
      SUB_MONEY(victim, gold);
    }
  }
}




/* ********************************************************************
   *  Special procedures for mobiles                                      *
   ******************************************************************** */



SPECIAL(puff)
{
  ACMD(do_say);

  if (cmd)
    return (0);

  switch (number(0, 60)) {
  case 0:
    do_say(ch, "My god!  It's full of stars!", 0, 0);
    return (1);
  case 1:
    do_say(ch, "How'd all those fish get up here?", 0, 0);
    return (1);
  case 2:
    do_say(ch, "I'm a very female dragon.", 0, 0);
    return (1);
  case 3:
    do_say(ch, "I've got a peaceful, easy feeling.", 0, 0);
    return (1);
  default:
    return (0);
  }
}



SPECIAL(fido)
{

  struct obj_data *i, *temp, *next_obj;

  if (cmd || !AWAKE(ch))
    return (FALSE);

  for (i = world[ch->in_room].contents; i; i = i->next_content) {
    if (GET_OBJ_TYPE(i) == ITEM_CONTAINER && GET_OBJ_VAL(i, 3)) {
      act("$n savagely devours a corpse.", FALSE, ch, 0, 0, PROXIMITY);
      for (temp = i->contains; temp; temp = next_obj) {
	next_obj = temp->next_content;
	obj_from_obj(temp);
	put_obj_to_room(temp, ch->in_room, GET_X_CORD(ch), GET_Y_CORD(ch), GET_Z_CORD(ch));
      }
      extract_obj(i);
      return (TRUE);
    }
  }
  return (FALSE);
}



SPECIAL(janitor)
{
  struct obj_data *i;

  if (cmd || !AWAKE(ch))
    return (FALSE);

  for (i = world[ch->in_room].contents; i; i = i->next_content) {
    if (!CAN_WEAR(i, ITEM_WEAR_TAKE))
      continue;
    if (GET_OBJ_TYPE(i) != ITEM_DRINKCON && GET_OBJ_COST(i) >= 15)
      continue;
    act("$n picks up some trash.", FALSE, ch, 0, 0, PROXIMITY);
    obj_from_room(i);
    obj_to_char(i, ch);
    return TRUE;
  }

  return FALSE;
}


SPECIAL(cityguard)
{
  struct char_data *tch, *evil;

  if (cmd || !AWAKE(ch) || FIGHTING(ch))
    return FALSE;

  evil = 0;

  for (tch = world[ch->in_room].people; tch; tch = tch->next_in_room) {
    if (CAN_SEE(ch, tch) && FIGHTING(tch)) {
      if (IS_EVIL(tch) &&
	  (IS_NPC(tch) || IS_NPC(FIGHTING(tch)))) {
	evil = tch;
      }
    }
  }

  if (evil && (!IS_EVIL(FIGHTING(evil)))) {
    act("$n screams 'PROTECT THE INNOCENT!  BANZAI!  CHARGE!  ARARARAGGGHH!'", FALSE, ch, 0, 0, PROXIMITY);
    hit(ch, evil, TYPE_UNDEFINED);
    return (TRUE);
  }
  return (FALSE);
}


#define PET_PRICE(pet) (GET_LEVEL(pet) * 300)

SPECIAL(pet_shops)
{
  char buf[MAX_STRING_LENGTH], pet_name[256];
  int pet_room;
  struct char_data *pet;

  pet_room = ch->in_room + 1;

  if (CMD_IS("list")) {
    send_to_char("Available pets are:\r\n", ch);
    for (pet = world[pet_room].people; pet; pet = pet->next_in_room) {
      sprintf(buf, "%8d - %s\r\n", PET_PRICE(pet), GET_NAME(pet));
      send_to_char(buf, ch);
    }
    return (TRUE);
  } else if (CMD_IS("buy")) {

    argument = one_argument(argument, buf);
    argument = one_argument(argument, pet_name);

    if (!(pet = get_char_room(buf, pet_room))) {
      send_to_char("There is no such pet!\r\n", ch);
      return (TRUE);
    }
    if (GET_MONEY(ch) < PET_PRICE(pet)) {
      send_to_char("You don't have enough gold!\r\n", ch);
      return (TRUE);
    }
    SUB_MONEY(ch, PET_PRICE(pet));

    pet = read_mobile(GET_MOB_RNUM(pet), REAL);
    GET_EXP(pet) = 0;
    SET_BIT(AFF_FLAGS(pet), AFF_CHARM);

    if (*pet_name) {
      sprintf(buf, "%s %s", pet->player.name, pet_name);
      /* FREE(pet->player.name); don't free the prototype! */
      pet->player.name = str_dup(buf);

      sprintf(buf, "%sA small sign on a chain around the neck says 'My name is %s'\r\n",
	      pet->player.description, pet_name);
      /* FREE(pet->player.description); don't free the prototype! */
      pet->player.description = str_dup(buf);
    }
    char_to_room(pet, ch->in_room, -1);
    add_follower(pet, ch);

    /* Be certain that pets can't get/carry/use/wield/wear items */
    IS_CARRYING_W(pet) = 1000;
    IS_CARRYING_N(pet) = 100;

    send_to_char("May you enjoy your pet.\r\n", ch);
    act("$n buys $N as a pet.", FALSE, ch, 0, pet, PROXIMITY);

    return 1;
  }
  /* All commands except list and buy */
  return 0;
}



/* ********************************************************************
   *  Special procedures for objects                                     *
   ******************************************************************** */


SPECIAL(changelog)
{
  FILE *f = NULL;
  char o_buf[MAX_STRING_LENGTH];
  char *ret;

  if (!ch || !cmd)
    return FALSE;

  if (!CMD_IS("read"))
    return FALSE;

  skip_spaces(&argument);

  if (!str_cmp(argument, "book")) {
    send_to_char("The book of files contain: log, bug, idea, typo, misc.\r\n", ch);
    return TRUE;
  }
  o_buf[0] = '\0';

  if (!str_cmp(argument, "log"))
    f = fopen("../src/ChangeLog", "r");
  else if (!str_cmp(argument, "misc"))
    f = fopen("text/misc.txt", "r");
  if (!f) {
    return FALSE;
  }
  do {
    ret = fgets(buf, MAX_STRING_LENGTH, f);
    if (ret)
      if ((strlen(o_buf) + strlen(buf)) >= (MAX_STRING_LENGTH - 10)) {
	ret = NULL;
      } else {
	if (isalpha(buf[0]))
	  strcat(o_buf, "&+B");
	else
	  strcat(o_buf, "&+c");
	strcat(o_buf, buf);
	strcat(o_buf, "\r&n");
      }
  } while (ret);
  strcat(o_buf, "&n\r\n");

  fclose(f);
  page_string(ch->desc, o_buf, 1);
  return TRUE;
}



/*
   use object with ITEM_SWITCH type:
   value[0] = command to trigger
   value[1] = room containing the blocked exit
   value[2] = direction of the exit in the room
   value[3] = 0:wall moves, 1:switch item moves
 */

SPECIAL(item_switch)
{
  int door, in_room, bits, back, cmd_given;
  struct char_data *dummy;
  struct obj_data *object, *obj;

  return FALSE;			/* turned off */

  if (!ch || !argument || !*argument || !cmd)
    return FALSE;

  obj = me;

  skip_spaces(&argument);

  bits = generic_find(argument, FIND_OBJ_INV | FIND_OBJ_EQUIP | FIND_OBJ_ROOM, ch, &dummy, &object);

  if (obj != object) {
    send_to_char("&+CDEBUG:&n obj != object in item_switch.\r\n", ch);
    return FALSE;
  }
  if (GET_OBJ_TYPE(obj) != ITEM_SWITCH) {
    send_to_char("&+CDEBUG:&n not a switch\r\n", ch);
    return FALSE;
  }
  if ((cmd_given = search_block(command, activating_commands, TRUE)) < 0)
    return FALSE;

  if (GET_OBJ_VAL(object, 0) != cmd_given++) {
    sprintf(buf, "&+CDEBUG:&n wrong command %d %d\r\n", GET_OBJ_VAL(obj, 0), cmd_given);
    send_to_char(buf, ch);
    return FALSE;
  }
  in_room = real_room(GET_OBJ_VAL(object, 1));
  if (in_room < 0) {
    send_to_char("This item is broken.  Talk to a god! #1\r\n", ch);
    return TRUE;
  }
  door = GET_OBJ_VAL(object, 2);
  if ((door < 0) || (door > 5)) {
    send_to_char("This item is broken.  Talk to a god! #2\r\n", ch);
    return TRUE;
  }
  if (!world[in_room].dir_option[door]) {
    send_to_char("This item is broken.  Talk to a god! #3\r\n", ch);
    return TRUE;
  }
  if (!IS_SET(world[in_room].dir_option[door]->exit_info, EX_BLOCKED)) {
    if (object->in_room != in_room) {
      send_to_char("You hear a rumbling sound in the distance.\r\n", ch);
      act("You hear a rumbling sound in the distance.", FALSE, ch, 0, 0, PROXIMITY);
    }
    SET_BIT(world[in_room].dir_option[door]->exit_info, EX_BLOCKED);
    SET_BIT(world[in_room].dir_option[door]->exit_info, EX_CLOSED);
    back = world[in_room].dir_option[door]->to_room;

    if (IS_SET(world[in_room].dir_option[door]->exit_info, EX_SECRET)) {
      if (GET_OBJ_VAL(object, 3) == 1) {
	sprintf(buf, "%s moves a bit, covering the wall again.\r\n",
		obj->short_description);
	CAP(buf);
	send_to_room(buf, in_room);
      } else {
	sprintf(buf, "The %s wall seems to be moving.\r\n", dirs[door]);
	send_to_room(buf, in_room);
      }
    } else {
      if (back == in_room)
	REMOVE_BIT(world[back].dir_option[(int) rev_dir[door]]->exit_info, EX_BLOCKED);

      if (GET_OBJ_VAL(object, 3) == 1) {
	sprintf(buf, "%s moves to close off the passageway.\r\n",
		obj->short_description);
	CAP(buf);
	send_to_room(buf, in_room);
      } else {
	sprintf(buf, "The %s wall moves to close off the passageway.\r\n",
		dirs[door]);
	send_to_room(buf, in_room);
      }
    }
    return TRUE;
  }
  if (object->in_room != in_room) {
    send_to_char("You hear a rumbling sound in the distance.\r\n", ch);
    act("You hear a rumbling sound in the distance.", FALSE, ch, 0, 0, PROXIMITY);
  }
  REMOVE_BIT(world[in_room].dir_option[door]->exit_info, EX_BLOCKED);
  REMOVE_BIT(world[in_room].dir_option[door]->exit_info, EX_CLOSED);
  back = world[in_room].dir_option[door]->to_room;

  if (IS_SET(world[in_room].dir_option[door]->exit_info, EX_SECRET)) {
    if (GET_OBJ_VAL(object, 3) == 1) {
      sprintf(buf, "%s moves aside, revealing a wall behind.\r\n",
	      obj->short_description);
      CAP(buf);
      send_to_room(buf, in_room);
    } else {
      sprintf(buf, "The %s wall seems to be moving.\r\n", dirs[door]);
      send_to_room(buf, in_room);
    }
  } else {
    if (back == in_room)
      REMOVE_BIT(world[back].dir_option[(int) rev_dir[door]]->exit_info, EX_BLOCKED);

    if (GET_OBJ_VAL(object, 3) == 1) {
      sprintf(buf, "%s moves aside, revealing a passageway.\r\n",
	      obj->short_description);
      CAP(buf);
      send_to_room(buf, in_room);
    } else {
      sprintf(buf, "!!SOUND(secretdoor* T=general)The %s wall moves aside, revealing a passageway.\r\n",
	      dirs[door]);
      send_to_room(buf, in_room);
    }
  }
  return TRUE;
}

/*
   use object with ITEM_TELEPORT type:
   value[0] = where this door leads to
   value[1] = type (door, portal, wormhole, etc)
   value[2] = charges
   value[3] = 
 */
SPECIAL(portal_object)
{
  int bits;
  int to_room;
  struct char_data *dummy;
  struct obj_data *obj, *object = NULL;

  return FALSE;
  if (!ch || !argument)
    return FALSE;

  obj = me;

  skip_spaces(&argument);

  bits = generic_find(argument, FIND_OBJ_ROOM, ch, &dummy, &object);

  if (obj != object)
    return FALSE;

  if (GET_OBJ_TYPE(obj) != ITEM_TELEPORT)
    return FALSE;
/***/

  if (strstr(command, "look")) {
    while (*argument == ' ')
      argument++;
    if (!*argument)
      return FALSE;
    if (strn_cmp(argument, "in ", 3))
      return FALSE;
    argument += 3;
  }
  to_room = real_room(GET_OBJ_VAL(obj, 0));
  if (to_room == NOWHERE) {
    send_to_char("Bug with portal!!! Tell a god.\r\n", ch);
    return (TRUE);
  }
  if (strstr("look", command)) {
    act("You peer into $p and see...", 0, ch, obj, 0, TO_CHAR);
    if (0)
      send_to_char("It is pitch black over there...\r\n", ch);
    else {
      send_to_char(world[to_room].name, ch);
      send_to_char("\r\n", ch);
    }
    return TRUE;
  }
  /* otherwise cmd == enter */

  if (!obj->action_description) {
    act("&+W$p suddenly glows brightly!", FALSE, ch, obj, 0, PROXIMITY);
    act("$n enters $p and disappears among the mist.", FALSE, ch, obj, 0, PROXIMITY);
  } else
    act(obj->action_description, FALSE, ch, obj, 0, PROXIMITY);

  char_from_room(ch);
  char_to_room(ch, to_room, -1);
  act("$n has arrived.", FALSE, ch, obj, 0, PROXIMITY);

  if (ch->desc != NULL) {
    smallmap_look(ch, view_distance(ch), -1);
    look_at_room(ch, 2);
  }
  if (IS_SET(world[to_room].room_flags, ROOM_DEATH) && !IS_TRUSTED(ch))
    extract_char(ch);

#if 0
  if (obj->value[2] > 0) {	/* gates have no charges */
    obj->value[2]--;
    if (obj->value[2] == 0) {	/* last one */
      Decay(obj);
    }
  }
#endif
  return TRUE;
}

/*
   use object with ITEM_PIT type:
   value[0] = where this pit leads to
   value[1] = type (open, wieght activated)
   value[2] = weight it can hold before opening
   value[3] = 
 */
SPECIAL(pit_object)
{
  int to_room;
  struct obj_data *obj;

  if (!ch)
    return FALSE;

  obj = me;

  if (GET_OBJ_TYPE(obj) != ITEM_PIT)
    return FALSE;

  if (GET_OBJ_X_CORD(obj) != GET_X_CORD(ch) ||
      GET_OBJ_Y_CORD(obj) != GET_Y_CORD(ch) ||
      GET_OBJ_Z_CORD(obj) != GET_Z_CORD(ch))
    return FALSE;

/* add weight check for trapdoor here */

  to_room = real_room(GET_OBJ_VAL(obj, 0));
  if (to_room == NOWHERE)
    return (TRUE);

  if (!obj->action_description) {
    act("$p suddenly opens beneath $n's feet!", FALSE, ch, obj, 0, PROXIMITY);
    act("$n falls into $p with a slight scream.", FALSE, ch, obj, 0, PROXIMITY);
  } else
    act(obj->action_description, FALSE, ch, obj, 0, PROXIMITY);

  char_from_room(ch);
  char_to_room(ch, to_room, -1);

  act("$n falls in from above.", FALSE, ch, obj, 0, PROXIMITY);

  if (ch->desc != NULL) {
    smallmap_look(ch, view_distance(ch), -1);
    look_at_room(ch, 2);
  }
  if (IS_SET(world[to_room].room_flags, ROOM_DEATH) && !IS_TRUSTED(ch))
    extract_char(ch);

  return TRUE;
}

/*
   use object with ITEM_DOOR type:
   value[0] = where this door leads to
   value[1] = x it leads to
   value[2] = y it leads to
   value[3] = z it leads to
 */
SPECIAL(door_object)
{
  int to_room;
  struct obj_data *obj;

  if (!ch)
    return FALSE;

  if (!MOVE_CMD(cmd))
    return FALSE;

  obj = me;

  if (GET_OBJ_TYPE(obj) != ITEM_DOOR)
    return FALSE;

  if (GET_OBJ_X_CORD(obj) != GET_X_CORD(ch) ||
      GET_OBJ_Y_CORD(obj) != GET_Y_CORD(ch) ||
      GET_OBJ_Z_CORD(obj) != GET_Z_CORD(ch))
    return FALSE;

  to_room = real_room(GET_OBJ_VAL(obj, 0));
  if (to_room == NOWHERE)
    return (TRUE);

  if (!obj->action_description) {
    act("$n enters the $p.", FALSE, ch, obj, 0, PROXIMITY);
  } else
    act(obj->action_description, FALSE, ch, obj, 0, PROXIMITY);

  char_from_room(ch);
  put_char_to_room(ch, to_room, -1, GET_OBJ_VAL(obj, 1), GET_OBJ_VAL(obj, 2), GET_OBJ_VAL(obj, 3));

  if (ch->desc != NULL) {
    smallmap_look(ch, view_distance(ch), -1);
    look_at_room(ch, 2);
  }
  if (IS_SET(world[to_room].room_flags, ROOM_DEATH) && !IS_TRUSTED(ch))
    extract_char(ch);

  return TRUE;
}
