/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"
#include <signal.h>

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "handler.h"
#include "db.h"
#include "spells.h"


/*   external vars  */
extern struct room_data *world;
extern struct char_data *character_list;
extern struct descriptor_data *descriptor_list;
extern struct index_data *mob_index;
extern struct index_data *obj_index;
extern struct time_info_data time_info;
extern char *dirs[];
extern int rev_dir[];
extern char *activating_commands[];
extern char *coin_stringv(int amount);
extern void send_to_zone_indoor(int zone_rnum, char *messg);
extern void send_to_zone_outdoor(int zone_rnum, char *messg);
extern void mobaction(struct char_data *ch, char *string, char *action);
extern void add_follower(struct char_data *ch, struct char_data *leader);
extern int apprehender(struct char_data *ch, struct char_data *pl, int cmd, char *arg);

SPECIAL(generic_drow_eq)
{
  struct obj_data *obj = NULL;

  if (!ch)
    return FALSE;
  obj = me;
  if (obj->in_obj)		/* in a bag */
    return FALSE;

  if (DECAYABLE(ch->in_room)) {
    act("Your $p quickly decays in the sunlight!", TRUE, ch, obj, 0,
	TO_CHAR);
    act("$n's $p quickly decays in the sunlight!", TRUE, ch, obj, 0,
	TO_ROOM);
    /* first, remove the item! */
    if (obj->worn_by)
      unequip_char(obj->worn_by, obj->worn_on);
    else if (obj->carried_by)
      obj_from_char(obj);
    extract_obj(obj);
  }
  return FALSE;
}
#undef DECAYABLE


static int noiseact = 1;
static int noisezone = 1;

void greyhawk_city_noises(void)
{
  int tempzone;

  switch (noisezone) {
  case 0:
    tempzone = real_room(5000);
    break;
  case 1:
    tempzone = real_room(5100);
    break;
  case 2:
    tempzone = real_room(5200);
    break;
  case 3:
    tempzone = real_room(5300);
    break;
  case 4:
    tempzone = real_room(5400);
    break;
  case 5:
    tempzone = real_room(5500);
    break;
  case 6:
    tempzone = real_room(5600);
    break;
  case 7:
    tempzone = real_room(5700);
    break;
  case 8:
    tempzone = real_room(5800);
    break;
  case 9:
    tempzone = real_room(5900);
    break;
  default:
    tempzone = 0;
    break;
  }

  noiseact++;
  if (noiseact == 101)
    noiseact = 1;

  if ((tempzone != 0) && (tempzone != NOWHERE))
    tempzone = world[tempzone].zone;

  if (tempzone != 0) {
    switch (noiseact) {
    case 1:
      send_to_zone_outdoor(tempzone, "The sound of a horse's neigh can be heard nearby.\n");
      send_to_zone_indoor(tempzone, "You can hear a rat screeching from somewhere.\n");
      break;
    case 2:
      send_to_zone_outdoor(tempzone, "A child can be heard shouting in the distance.\n");
      break;
    case 3:
      send_to_zone_outdoor(tempzone, "From somewhere close by, you can hear a woman's playful scream.\n");
      break;
    case 4:
      send_to_zone_outdoor(tempzone, "The annoying sound of a dog's incessant barking can be heard nearby.\n");
      break;
    case 5:
      send_to_zone_outdoor(tempzone, "Someone swears loudly a few streets over.\n");
      break;
    case 6:
      send_to_zone_outdoor(tempzone, "Someone screams, 'Ouch! .. God Dammit!' from within a building near here.\n");
      send_to_zone_indoor(tempzone, "You hear a creeking noise.\n");
      break;
    case 7:
      send_to_zone_outdoor(tempzone, "The sounds of carriage wheels can be heard on a nearby street.\n");
      break;
    case 8:
      send_to_zone_outdoor(tempzone, "There is a loud crashing noise from somewhere close by. *BANG*\n");
      break;
    case 9:
      send_to_zone_outdoor(tempzone, "The sounds of laughter can be heard from the next building.\n");
      break;
    case 10:
      send_to_zone_outdoor(tempzone, "A cat dashes past you at a full run, disappearing as quick as it came.\n");
      break;
    case 11:
      send_to_zone_outdoor(tempzone, "The wonderful smell of charcoal and cooking meat wafts by. *YUM*\n");
      break;
    case 12:
      send_to_zone_indoor(tempzone, "You can hear a bird's loud call from outside.\n");
      send_to_zone_outdoor(tempzone, "The awful smell of fresh horse dung wafts by. \n");
      break;
    case 13:
      send_to_zone_outdoor(tempzone, "A cityguard yells, 'Stop thief!'\n");
      break;
    case 14:
      send_to_zone_outdoor(tempzone, "A child wails, 'MOM!' from somewhere close by.\n");
      break;
    case 15:
      send_to_zone_outdoor(tempzone, "You can smell burnt meat coming from one of the nearby buildings.\n");
      break;
    case 16:
      send_to_zone_outdoor(tempzone, "A man yells, 'God, I love this city!'\n");
      break;
    case 17:
      send_to_zone_outdoor(tempzone, "The sound of someone dropping a metal pot rings through the area.\n");
      break;
    case 18:
      send_to_zone_indoor(tempzone, "From outside, you can hear a woman's scream..\n");
      send_to_zone_outdoor(tempzone, "A woman cries out in terror from inside a building near here!\n");
      break;
    case 19:
      send_to_zone_outdoor(tempzone, "A cool breeze blows through, bringing some welcome fresh air.\n");
      break;
    case 20:
      send_to_zone_outdoor(tempzone, "A wailing cry of a dog in pain can be heard in the distance.\n");
      break;
    case 21:
      send_to_zone_outdoor(tempzone, "In the distance, a merchant can be heard loudly soliciting his goods.\n");
      break;
    case 22:
      send_to_zone_outdoor(tempzone, "The sizzling sound of cooking meat can be heard.\n");
      break;
    case 23:
      send_to_zone_outdoor(tempzone, "A strange smell wafts by, similar to hide or worked leather.\n");
      break;
    case 24:
      send_to_zone_indoor(tempzone, "You feel a presence .. almost as if you're not alone.\n");
      send_to_zone_outdoor(tempzone, "An unusually large crow flies overhead, croaking loudly, 'Caw! Caw!'\n");
      break;
    case 25:
      send_to_zone_outdoor(tempzone, "A couple can be heard bartering over some purchased goods nearby.\n");
      break;
    case 26:
      send_to_zone_outdoor(tempzone, "A sparrow dives on a piece of discarded meat, and flies off.\n");
      break;
    case 27:
      send_to_zone_outdoor(tempzone, "From somewhere close, a man shouts, 'Yeah? Well piss off asshole!'\n");
      break;
    case 28:
      send_to_zone_outdoor(tempzone, "The strong and appetizing smells of baking bread waft by. *Mmmm*\n");
      break;
    case 29:
      send_to_zone_outdoor(tempzone, "The strong odor of a dead animal near here offends your nose.\n");
      break;
    case 30:
      send_to_zone_indoor(tempzone, "You can make out the sounds of two people arguing from outside.\n");
      send_to_zone_outdoor(tempzone, "The rising shouts of a lovers' quarrel can be heard near here.\n");
      break;
    case 31:
      send_to_zone_outdoor(tempzone, "A dog can be heard chasing someone down a nearby street.\n");
      break;
    case 32:
      send_to_zone_outdoor(tempzone, "The cloying smell of ladies perfume wafts by, enticing you.\n");
      break;
    case 33:
      send_to_zone_outdoor(tempzone, "Several seagulls fly overhead, calling loudly.\n");
      break;
    case 34:
      send_to_zone_outdoor(tempzone, "Several children can be heard playing and yelling loudly in the distance.\n");
      break;
    case 35:
      send_to_zone_outdoor(tempzone, "A bird dropping lands in front of you, just missing your head.\n");
      break;
    case 36:
      send_to_zone_indoor(tempzone, "A cool draft blows by.\n");
      send_to_zone_outdoor(tempzone, "The banging sounds of a smithy hammer ring through the air.\n");
      break;
    case 37:
      send_to_zone_outdoor(tempzone, "A loud splash can be heard, as if someone dumped a bucket of liquid out a window.\n");
      break;
    case 38:
      send_to_zone_outdoor(tempzone, "From inside a building, someone lets out a tremendous belch!\n");
      break;
    case 39:
      send_to_zone_outdoor(tempzone, "A rare and beautiful hawk circles overhead, drifting on the wind currents.\n");
      break;
    case 40:
      send_to_zone_outdoor(tempzone, "The sound of breaking glass pierces the air, much like that of a fallen bottle.\n");
      break;
    case 41:
      send_to_zone_outdoor(tempzone, "Someone yells, 'Hey, you little runt! Get back here!'\n");
      break;
    case 42:
      send_to_zone_indoor(tempzone, "The smells of cooking food from outside waft by.\n");
      send_to_zone_outdoor(tempzone, "The gentle and musical sound of singing songbirds fills the air.\n");
      break;
    case 43:
      send_to_zone_outdoor(tempzone, "A dark figure dashes in and out of a shadow in your peripheral vision. \n");
      break;
    case 44:
      send_to_zone_outdoor(tempzone, "Shiphands can be heard yelling instructions from somewhere in the harbor.\n");
      break;
    case 45:
      send_to_zone_outdoor(tempzone, "A woman's erotic cry echoes from within a building near here.\n");
      break;
    case 46:
      send_to_zone_outdoor(tempzone, "The 'neigh' of many horses can be heard riding down a nearby street.\n");
      break;
    case 47:
      send_to_zone_outdoor(tempzone, "You hear a salesman peddling his goods to someone close by.\n");
      break;
    case 48:
      send_to_zone_indoor(tempzone, "The is a loud crash from outside!\n");
      send_to_zone_outdoor(tempzone, "Shouts and running feet can be heard, as if guards were pursuing someone.\n");
      break;
    case 49:
      send_to_zone_outdoor(tempzone, "Several guards or soldiers can be heard marching near here.\n");
      break;
    case 50:
      send_to_zone_outdoor(tempzone, "The bangs and thuds of loading cargo can be heard from within the harbor.\n");
      break;
    case 51:
      send_to_zone_outdoor(tempzone, "A man yells, ' Hey! What are you doing in here, kid?! Get outta here!'\n");
      break;
    case 52:
      send_to_zone_outdoor(tempzone, "The ringing of swords can be heard, as if someone were fighting close by!\n");
      break;
    case 53:
      send_to_zone_outdoor(tempzone, "A merchant and customer can be heard bartering over a price.\n");
      break;
    case 54:
      send_to_zone_indoor(tempzone, "A shout can be heard from outside somewhere.\n");
      send_to_zone_outdoor(tempzone, "The thick smell of cigar smoke wafts by.  \n");
      break;
    case 55:
      send_to_zone_outdoor(tempzone, "You can see smoke above the city, as if someone was burning something.\n");
      break;
    case 56:
      send_to_zone_outdoor(tempzone, "A loud argument can be heard taking place near here.\n");
      break;
    case 57:
      send_to_zone_outdoor(tempzone, "Cheers and jolly shouts can be heard from a tavern near here.\n");
      break;
    case 58:
      send_to_zone_outdoor(tempzone, "Someone shouts several instructions over a bullhorn in the harbor.\n");
      break;
    case 59:
      send_to_zone_outdoor(tempzone, "A gust of wind whips through the area, amplified by all the buildings.\n");
      break;
    case 60:
      send_to_zone_indoor(tempzone, "A dog's incessant barking can be heard from outside.\n");
      send_to_zone_outdoor(tempzone, "You hear someone's tortured groaning from inside a building nearby.\n");
      break;
    case 61:
      send_to_zone_outdoor(tempzone, "A grubby boy with apples spilling from his pockets storms by.\n");
      break;
    case 62:
      send_to_zone_outdoor(tempzone, "A commoner yells, 'Help! Help! I'm being repressed!'\n");
      send_to_zone_outdoor(tempzone, "Lord Piergeiron yells, 'Shut up, you bloody peasant!'\n");
      break;
    case 63:
      send_to_zone_outdoor(tempzone, "An old man pushing a two-wheeled cart yells, 'Bring out yer dead!'\n");
      break;
    case 64:
      send_to_zone_outdoor(tempzone, "A *large* gooey bird dropping strikes the ground in front of you! *SPLAT*\n");
      break;
    case 65:
      send_to_zone_outdoor(tempzone, "The long, eerie howl of a timber wolf can be heard echoing in the distance.\n");
      break;
    case 66:
      send_to_zone_outdoor(tempzone, "Someone yells, 'Hey! Would you bring that over here, please?'\n");
      send_to_zone_indoor(tempzone, "You can hear the faint sound of a mouse scrabbling in the walls.\n");
      break;
    case 67:
      send_to_zone_outdoor(tempzone, "Someone lets off a resounding, rip-roaring fart about a block or so.away \n");
      break;
    case 68:
      send_to_zone_outdoor(tempzone, "You hear the sounds of laughter nearby.\n");
      break;
    case 69:
      send_to_zone_outdoor(tempzone, "A guard can be heard shouting instructions to someone.\n");
      break;
    case 70:
      send_to_zone_outdoor(tempzone, "The pungent odor of rotting trash floats into the area.\n");
      break;
    case 71:
      send_to_zone_outdoor(tempzone, "A child screams in pain from somewhere and begins wailing.\n");
      break;
    case 72:
      send_to_zone_outdoor(tempzone, "From within a building, someone says loudly, 'Oops!'\n");
      send_to_zone_indoor(tempzone, "You can hear shouting going on outdoors.\n");
      break;
    case 73:
      send_to_zone_outdoor(tempzone, "A large flock of birds flies over this area of the city.\n");
      send_to_zone_outdoor(tempzone, "A small bird dropping strikes you squarely on the forearm. *SPLOCH*\n");
      break;
    case 74:
      send_to_zone_outdoor(tempzone, "You hear a tortured death cry echoing in the distance!\n");
      break;
    case 75:
      send_to_zone_outdoor(tempzone, "The sound of someone pounding metal with a smithy hammer echoes in the distance.\n");
      break;
    case 76:
      send_to_zone_outdoor(tempzone, "The sounds of a man singing a loud religous hymn echoes throughout the city.\n");
      break;
    case 77:
      send_to_zone_outdoor(tempzone, "You can hear the pleasant sound of someone playing a lyre nearby.\n");
      break;
    case 78:
      send_to_zone_outdoor(tempzone, "Off in the distance, you can faintly hear the blast of a horn.\n");
      send_to_zone_indoor(tempzone, "Something resembling the blast of a horn can faintly be hear outside.\n");
      break;
    case 79:
      send_to_zone_outdoor(tempzone, "A woman yells, 'Tommy! Dinner time! Get yer butt home!'\n");
      break;
    case 80:
      send_to_zone_outdoor(tempzone, "An unusually cold wind blows through the area.\n");
      break;
    case 81:
      send_to_zone_outdoor(tempzone, "Far in the distance, the echo of an explosion can faintly be heard.\n");
      break;
    case 82:
      send_to_zone_outdoor(tempzone, "A few streets down you hear a man yell, 'Keep the noise down, dammit!'\n");
      break;
    case 83:
      send_to_zone_outdoor(tempzone, "The cheers of about a dozen people echo through the streets.\n");
      break;
    case 84:
      send_to_zone_outdoor(tempzone, "You can hear the music of a snake charmer's flute from somewhere nearby.\n");
      break;
    case 85:
      send_to_zone_outdoor(tempzone, "You hear a noise from a nearby rooftop, like that of running footsteps.\n");
      send_to_zone_indoor(tempzone, "You hear someone walking across the rooftop above.\n");
      break;
    case 86:
      send_to_zone_outdoor(tempzone, "You hear a the loud *slosh* of water hitting the ground somewhere.\n");
      break;
    case 87:
      send_to_zone_outdoor(tempzone, "A salesman yells, 'Come one, come all! Fine goods for sale in the bazaar!'\n");
      break;
    case 88:
      send_to_zone_outdoor(tempzone, "A man says, 'Awww, go piss on yourself. That price is too high!' from nearby.\n");
      break;
    case 89:
      send_to_zone_outdoor(tempzone, "A dog comes out of nowhere, running quickly past you and out of sight.\n");
      break;
    case 90:
      send_to_zone_outdoor(tempzone, "You hear the pleasant sounds of a balalaika from somewhere close by.\n");
      send_to_zone_indoor(tempzone, "You hear the pleasant sounds of a balalaika from somewhere outside.\n");
      break;
    case 91:
      send_to_zone_outdoor(tempzone, "You hear a dog barking from somewhere.\n");
      break;
    case 92:
      send_to_zone_outdoor(tempzone, "You hear, 'No! No, don't touch that!' and then a loud explosion *BOOM*\n");
      break;
    case 93:
      send_to_zone_outdoor(tempzone, "You hear the loud frightened neigh of a spooked horse from somewhere nearby.\n");
      break;
    case 94:
      send_to_zone_outdoor(tempzone, "A child and his dog race by you, with the child giggling wildly. \n");
      break;
    case 95:
      send_to_zone_outdoor(tempzone, "You hear the heated tones of an argument taking place nearby.\n");
      send_to_zone_indoor(tempzone, "You hear the heated tones of an argument taking place outside.\n");
      break;
    case 96:
      send_to_zone_outdoor(tempzone, "You hear the gleeful shouts of girls playing nearby.\n");
      break;
    case 97:
      send_to_zone_outdoor(tempzone, "A small cloud of smoke floats into the area from someone's cooking fire.\n");
      break;
    case 98:
      send_to_zone_outdoor(tempzone, "You hear a man yell, 'Be quiet! Or I shall taunt thee a second time!'\n");
      break;
    case 99:
      send_to_zone_outdoor(tempzone, "You hear the sounds of dice hitting a wall nearby, as two men gamble.\n");
      break;
    case 100:
      send_to_zone_outdoor(tempzone, "The stench of urine floats into the area, offending your nose.\n");
      send_to_zone_indoor(tempzone, "The stench of urine floats in from outside, offending your nose.\n");
      break;
    default:
      break;
    }
  }
  if (noisezone == 10)
    noisezone = 0;
  noisezone++;

}

SPECIAL(greyhawk_cityguard)
{
  struct char_data *pl = NULL;

  if (me != ch)
    pl = me;

  if (cmd || !AWAKE(ch))
    return FALSE;

  apprehender(ch, NULL, 0, "");

  if (MIN_POS(ch, POS_STANDING + STAT_NORMAL) && !FIGHTING(ch)) {
    switch (number(0, 30)) {
    case 0:
      act("$n eyes you suspiciously.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 1:
    case 2:
    case 3:
      act("$n keeps his hand on the hilt of $s weapon while near you.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 4:
    case 5:
      act("$n stops suddenly as if having heard something odd.", TRUE, ch, 0, 0, TO_ROOM);
      act("After a few moments, $n continues on $s way.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 7:
      act("$n casts you a wary glance.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 8:
      mobsay(ch, "Get lost, kid, or I might decide to relieve you of your pathetic existence.");
      break;
    case 9:
      act("$n growls at you.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 10:
      act("$n glares icily at you.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 11:
      act("$n gives you a casual glance.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    default:
      return FALSE;
      break;
    }
    return TRUE;
  }
  return FALSE;
}

SPECIAL(city_dog)
{
  struct obj_data *i, *temp, *next_obj;

  if (cmd || !AWAKE(ch))
    return FALSE;

  if (MIN_POS(ch, POS_STANDING + STAT_NORMAL)) {
    switch (number(0, 10)) {
    case 0:
      act("$n whines as if looking for a handout.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 1:
      act("$n hears something in the distance and barks.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 2:
      act("$n wanders around, sniffing the ground.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 3:
      act("$n growls dangerously.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 4:
      act("$n howls.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 5:
      act("$n looks your way and snarls.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 6:
      act("$n whirls, listening intently for some unheard sound.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    default:
      break;
    }
  }
  for (i = world[ch->in_room].contents; i; i = i->next_content) {
    if ((GET_OBJ_TYPE(i) == ITEM_CORPSE) || (GET_OBJ_TYPE(i) == ITEM_FOOD)) {
      if (!(i)) {
	log("SYSERR: city_dog proc");
	raise(SIGSEGV);
      }
      if (GET_OBJ_TYPE(i) == ITEM_CORPSE)
	for (temp = i->contains; temp; temp = next_obj) {
	  next_obj = temp->next_content;
	  obj_from_obj(temp);
	  obj_to_room(temp, ch->in_room);
	}
      GET_X_CORD(ch) = GET_OBJ_X_CORD(i);
      GET_Y_CORD(ch) = GET_OBJ_Y_CORD(i);
      act("$n savagely devours the $o.", FALSE, ch, i, 0, TO_ROOM);
      extract_obj(i);
      return TRUE;
    }
  }
  return FALSE;
}

SPECIAL(rosalia_waitress)
{
  if (cmd || !AWAKE(ch) || cmd)
    return FALSE;

  switch (number(0, 40)) {
  case 0:
    mobsay(ch, "Are'ye ready ta order yet?");
    return TRUE;
  case 1:
    mobaction(ch, 0, "squeal");
    mobsay(ch, "No pinching!");
    sprintf(buf, "patron");
    mobaction(ch, buf, "slap");
    return TRUE;
  case 2:
  case 3:
    mobaction(ch, 0, "hum");
    return TRUE;
  case 4:
    mobaction(ch, 0, "mumble");
    act("$n cleans up a mess on the floor.", TRUE, ch, 0, 0, TO_ROOM);
    return TRUE;
  case 5:
  case 6:
  case 7:
    act("$n wipes another table clean.", TRUE, ch, 0, 0, TO_ROOM);
    return TRUE;
  case 8:
    mobaction(ch, 0, "sigh");
    mobsay(ch, "The day sure is dragging on.");
    mobaction(ch, 0, "yawn");
    return TRUE;
  case 9:
    mobaction(ch, 0, "mutter");
    mobsay(ch, "Have all ye lads forgotten yer manners? Stiffed again!");
    mobsay(ch, "Jes a copper tis okay, but to leave nothing is so rude.");
    mobaction(ch, 0, "sigh");
    return TRUE;
  case 10:
    sprintf(buf, "tits");
    mobaction(ch, buf, "wiggle");
  default:
    return FALSE;
  }
}
SPECIAL(drunk_one)
{
  if (cmd || !AWAKE(ch) || cmd)
    return FALSE;

  switch (number(0, 50)) {
  case 0:
    mobsay(ch, "What the hell wassh that shtupid word again?");
    return TRUE;
  case 1:
    mobsay(ch, "Shnake? No.");
    return TRUE;
  case 2:
    mobsay(ch, "Shlitherer? Nope nope nope.");
    return TRUE;
  case 3:
    mobsay(ch, "Definitely shomething to do with a shea monshter...");
    return TRUE;
  case 4:
    mobsay(ch, "Wench! Bring me another tankard of ale!");
    return TRUE;
  case 5:
    mobsay(ch, "Time to be getting home, maybe.");
    return TRUE;
  case 6:
    act("$n hiccups, and a little bolt of lightning shoots from his fingers, scorching the floor.", TRUE, ch, 0, 0, TO_ROOM);
    mobsay(ch, "'Shcuse me.");
    act("$n grins sheepishly.", TRUE, ch, 0, 0, TO_ROOM);
    return TRUE;
  case 7:
    act("$n teeters for a moment, but regains his balance.",
	TRUE, ch, 0, 0, TO_ROOM);
    return TRUE;
  case 8:
    mobaction(ch, 0, "mutter");
    return TRUE;
  case 9:
    mobsay(ch, "Damnit! If I could jes remember the power-word, I could sell this wand.");
  default:
    return FALSE;
  }
}

SPECIAL(drunk_two)
{
  if (!ch || !AWAKE(ch) || cmd)
    return FALSE;

  switch (number(1, 80)) {
  case 1:{
      mobaction(ch, 0, "burp");
      return TRUE;
    }
  case 2:{
      mobsay(ch, "'Course, Kilten's been sayin'...'scuse me.");
      mobaction(ch, 0, "cough");
      mobsay(ch, "Where was I? Oh.");
      strcpy(buf, "carafe");
      mobaction(ch, buf, "sip");
      mobsay(ch, "His damned ship can make it to GreyHawk in under two days.");
      mobaction(ch, 0, "roll");
      return TRUE;
    }
  case 3:{
      mobaction(ch, 0, "smirk");
      return TRUE;
    }
  case 4:{
      strcpy(buf, "wench");
      mobaction(ch, buf, "pinch");
      return TRUE;
    }
  case 5:{
      mobsay(ch, "I once caught a dragonfish _this_ big!");
      act("$n stretches $s arms wide.", TRUE, ch, 0, 0, TO_ROOM);
      return TRUE;
    }
  case 6:{
      mobsay(ch, "Those damned harpies must have eaten half me crew.");
      mobaction(ch, 0, "shiver");
      return TRUE;
    }
  case 7:{
      mobsay(ch, "I was stranded in the NyrDyv for damn near a month.");
      return TRUE;
    }
  case 8:{
      mobsay(ch, "Yeah, me wife's been nagging at me to get her some pearls.");
      return TRUE;
    }
  case 9:{
      mobaction(ch, 0, "scratch");
      return TRUE;
    }
  case 10:{
      mobsay(ch, "I've got a couple of brats down south, but I hardly ever see 'em.");
      mobaction(ch, 0, "cry");
      return TRUE;
    }
  case 11:{
      mobsay(ch, "So I roll a five, another five, and suddenly I'm ahead forty fire-eyes!");
      return TRUE;
    }
  case 12:{
      mobsay(ch, "Aaaaah, feels good to be on dry land for a stretch.");
      mobaction(ch, 0, "stretch");
      mobaction(ch, 0, "wink");
      return TRUE;
    }
  case 13:{
      mobaction(ch, 0, "hiccup");
      return TRUE;
    }
  case 14:{
      mobsay(ch, "Murkas Magintii?  Yeah, I done heared of him.");
      mobsay(ch, "Wasn't he that guy that got swallowed by a whale?");
      return TRUE;
    }
  case 15:{
      strcpy(buf, "wench");
      mobaction(ch, buf, "ogle");
      return TRUE;
    }
  case 16:{
      mobsay(ch, "Calim harem girls know tricks that'll blow yer jerkin off!");
      mobaction(ch, 0, "wink");
      return TRUE;
    }
  case 17:{
      mobsay(ch, "I heard there be a spell lets ye ken any language.");
      mobaction(ch, 0, "shrug");
      mobaction(ch, 0, "ponder");
      mobsay(ch, "Valkur knows that'd be helpful when I go tradin'!");
      mobaction(ch, 0, "cackle");
      return TRUE;
    }
  case 18:{
      mobsay(ch, "Y'know, me pa was a fisherman too.");
      return TRUE;
    }
  case 19:{
      mobsay(ch, "Ye'll never be as strong fighting wimpy dragons as if ye work the sea.");
      mobaction(ch, 0, "flex");
      return TRUE;
    }
  case 20:{
      mobsay(ch, "Wench, remind me to bring you back some turtle legs from Nhavan Island next trip.");
      strcpy(buf, "wench");
      mobaction(ch, buf, "smile");
      return TRUE;
    }
  case 21:{
      strcpy(buf, "me");
      mobaction(ch, buf, "scratch");
      mobsay(ch, "What're ye after, anyway, ye crazy old coot?");
      strcpy(buf, "magus");
      mobaction(ch, buf, "poke");
      return TRUE;
    }
  default:
    return FALSE;
  }
}

SPECIAL(drunk_three)
{
  if (!ch || !AWAKE(ch) || cmd)
    return FALSE;

  if (MIN_POS(ch, POS_STANDING + STAT_NORMAL)) {
    switch (number(0, 50)) {
    case 1:
      act("$n stumbles and nearly falls, lost in his drunken stupor.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 2:
      mobsay(ch, "Heeeeyyyy, matie, got any whiskey?");
      break;
    case 3:
      act("$n mumbles something incoherent.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 4:
      act("$n turns green and nearly hurls, but amazingly recovers.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    default:
      break;
    }
  }
  return FALSE;
}

SPECIAL(drunk_four)
{
  if (!ch || !AWAKE(ch) || cmd)
    return FALSE;

  if (MIN_POS(ch, POS_STANDING + STAT_NORMAL)) {
    switch (number(0, 50)) {
    case 1:
      act("$n begins singing loudly, though his awful tone makes you cringe.", TRUE, ch, 0, 0, TO_ROOM);
      act("Dogs can be heard howling in the distance.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 2:
      mobsay(ch, "OOoohhh! Loookie what weee have here, a worthless ball offf horse manuure..");
      break;
    case 3:
      act("$n points at you and laughs uncontrollably for several minutes..", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 4:
      act("$n flips you the bird and mumbles something incoherent under his breath.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    default:
      break;
    }
  }
  return FALSE;
}

SPECIAL(drunk_five)
{
  if (!ch || !AWAKE(ch) || cmd)
    return FALSE;

  if (MIN_POS(ch, POS_STANDING + STAT_NORMAL)) {
    switch (number(0, 50)) {
    case 1:
      act("$n shouts annoyingly, 'Where is that damn Rosalia!'", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 2:
      mobsay(ch, "Hey, pssssst, you. Yeah, you.");
      mobsay(ch, "Know of any good places to gamble around here?");
      break;
    case 3:
      act("$n loses his balance and falls to the ground, cursing all the while.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    case 4:
      act("$n stares off into space, seemingly lost in some mindless thought.", TRUE, ch, 0, 0, TO_ROOM);
      break;
    default:
      break;
    }
  }
  return FALSE;
}


#define NN_LOOSE  0
#define NN_FOLLOW 1
#define NN_STOP   2

SPECIAL(nudge_nudge)
{
  struct char_data *vict, *vict_next;
  int done;

  if (cmd || !AWAKE(ch))
    return FALSE;

  if (FIGHTING(ch))
    return FALSE;

  switch (GET_MOB_SPEC_VALUE(ch, 0)) {
  case NN_LOOSE:
    done = FALSE;
    for (vict = world[ch->in_room].people; vict && !done; vict = vict_next) {
      vict_next = vict->next_in_room;
      if (number(0, 4) && CAN_SEE(ch, vict) && IS_PC(vict) && !circle_follow(ch, vict)) {
	if (ch->master)
	  stop_follower(ch);
	add_follower(ch, vict);
	SET_BIT(MOB_FLAGS(ch), MOB_SENTINEL);
	done = TRUE;
      }
    }
    if (!ch->master)
      return FALSE;
    GET_MOB_SPEC_VALUE(ch, 0) = NN_FOLLOW;
    mobsay(ch, "Evenin' squire!");
    act("$n winks at you.", FALSE, ch, 0, 0, TO_CHAR);
    act("$n winks at $N.", FALSE, ch, 0, ch->master, TO_ROOM);
    act("$n nudges you.", FALSE, ch, 0, 0, TO_CHAR);
    act("$n nudges $N.", FALSE, ch, 0, ch->master, TO_ROOM);
    break;
  case NN_FOLLOW:
    switch (number(0, 20)) {
    case 0:
      mobsay(ch, "Is, uh,...Is your wife a goer, eh? Know whatahmean, know whatahmean, nudge nudge, know whatahmean, say no more?");
      act("$n winks at you.", FALSE, ch, 0, 0, TO_CHAR);
      act("$n winks at $N.", FALSE, ch, 0, ch->master, TO_ROOM);
      act("$n nudges you.", FALSE, ch, 0, 0, TO_CHAR);
      act("$n nudges $N.", FALSE, ch, 0, ch->master, TO_ROOM);
      break;
    case 1:
      mobsay(ch, "Say no more!  Say no MORE!");
      act("$n winks at you.", FALSE, ch, 0, 0, TO_CHAR);
      act("$n winks at $N.", FALSE, ch, 0, ch->master, TO_ROOM);
      act("$n nudges you.", FALSE, ch, 0, 0, TO_CHAR);
      act("$n nudges $N.", FALSE, ch, 0, ch->master, TO_ROOM);
      break;
    case 2:
      mobsay(ch, "You been around, eh?");
      mobsay(ch, "...I mean you've ..... done it, eh?");
      act("$n winks at you.", FALSE, ch, 0, 0, TO_CHAR);
      act("$n winks at $N.", FALSE, ch, 0, ch->master, TO_ROOM);
      act("$n nudges you.", FALSE, ch, 0, 0, TO_CHAR);
      act("$n nudges $N.", FALSE, ch, 0, ch->master, TO_ROOM);
      break;
    case 3:
      mobsay(ch, "Follow me. Follow me. That's good, that's good! A nod's as good as a wink to a blind bat!");
      act("$n winks at you.", FALSE, ch, 0, 0, TO_CHAR);
      act("$n winks at $N.", FALSE, ch, 0, ch->master, TO_ROOM);
      act("$n nudges you.", FALSE, ch, 0, 0, TO_CHAR);
      act("$n nudges $N.", FALSE, ch, 0, ch->master, TO_ROOM);
      break;
    case 4:
      mobsay(ch, "Oooh! Ya wicked Ay! Wicked Ay! Oooh hooh! Say No MORE!");
      act("$n winks at you.", FALSE, ch, 0, 0, TO_CHAR);
      act("$n winks at $N.", FALSE, ch, 0, ch->master, TO_ROOM);
      act("$n nudges you.", FALSE, ch, 0, 0, TO_CHAR);
      act("$n nudges $N.", FALSE, ch, 0, ch->master, TO_ROOM);
      break;
    case 5:
    case 6:
    case 7:
      mobsay(ch, "Wink wink, nudge nudge, know what I mean, eh! :P");
      act("$n winks at you.", FALSE, ch, 0, 0, TO_CHAR);
      act("$n winks at $N.", FALSE, ch, 0, ch->master, TO_ROOM);
      act("$n nudges you.", FALSE, ch, 0, 0, TO_CHAR);
      act("$n nudges $N.", FALSE, ch, 0, ch->master, TO_ROOM);
      break;
    case 8:
      GET_MOB_SPEC_VALUE(ch, 0) = NN_STOP;
      break;
    default:
      break;
    }
    break;
  case NN_STOP:
  default:
    stop_follower(ch);
    GET_MOB_SPEC_VALUE(ch, 0) = NN_LOOSE;
    break;
  }
  return TRUE;
}

#if 0
#define ACT_OVER_21 1
#define ACT_SNICKER 2

struct pub_beers {
  int container;
  int contains;
  int howman;
  int actflag;
};

struct pub_beers sold_here[] =
{
  {1179, 3902, 6, 1},
  {1179, 3910, 6, 3},
  {1179, 3912, 6, 3},
  {1178, 0, 0, 1},
  {1178, 0, 0, 0},
  {1178, 0, 0, 0},
  {-1}
};


SPECIAL(bartender)
{
  /* things you MUST change if you install this on another mud:
     THE_PUB, room number of the Pub where he will sell beer.
     sold_here, object numbers of the containers and the
     corresponding beer that they will contain.  If you don't
     have multi-buy code, change the #if 1 to #if 0.
   */
#define THE_PUB 5361
  static int open = 1;		/* 0 closed;  1 open;  2 last call */
  char argm[100];
  struct obj_data *temp1, *temp2;
  struct char_data *temp_char;
  struct char_data *gouldan;
  int num, i, cost;
  struct pub_beers *scan;


  gouldan = 0;

  temp_char = me;

  if (open == 0 && time_info.hours == 11) {
    open = 1;
    do_unlock(gouldan, "door", 0);
    do_open(gouldan, "door", 0);
    act("$n says 'We're open for lunch, come on in.'", FALSE, gouldan, 0, 0, TO_ROOM);
  }
  if (open == 1 && time_info.hours == 1) {
    open = 2;
    act("$n says 'Last call, guys and gals.'", FALSE, gouldan, 0, 0, TO_ROOM);
  }
  if (open == 2 && time_info.hours == 2) {
    open = 0;
    act("$n says 'We're closing for the night.\n  Thanks for coming, all, and come again!'", FALSE, gouldan, 0, 0, TO_ROOM);
    do_close(gouldan, "door", 0);
    do_lock(gouldan, "door", 0);
  }
  switch (cmd) {
  case 25:			/* kill */
  case 70:			/* hit */
  case 157:			/* bash */
  case 159:			/* kick */
    only_argument(arg, argm);

    if (gouldan == ch)
      return TRUE;
    if (gouldan == get_char_room(argm, ch->in_room)) {
      int hitsleft;
      act("$n says 'Get this, $N wants to kill me', and\n falls down laughing.", FALSE, gouldan, 0, ch, TO_ROOM);
      hitsleft = dice(2, 6) + 6;
      if (hitsleft < GET_HIT(ch) && !IS_TRUSTED(ch)) {
	act("$n beats the shit out of $N.", FALSE, gouldan, 0, ch, TO_NOTVICT);
	act("$n beats the shit out of you.  OUCH!", FALSE, gouldan, 0, ch, TO_VICT);
	GET_HIT(ch) = hitsleft;
      } else {
	act("$n grabs $N in a vicious sleeper hold.", FALSE, gouldan, 0, ch, TO_NOTVICT);
	act("$n puts you in a vicious sleeper hold.", FALSE, gouldan, 0, ch, TO_VICT);
      }
      SET_POS(ch, POS_PRONE + STAT_SLEEPING);
    } else {
      mobaction(gouldan, ch->player.name, "slap");
      act("$n says 'Hey guys, I run a quiet pub.  Take it outside.'",
	  FALSE, gouldan, 0, 0, TO_ROOM);
    }
    return TRUE;
    break;

  case 156:			/* steal */
    if (gouldan == ch)
      return TRUE;
    mobaction(gouldan, ch->player.name, "slap");
    act("$n tells you 'Who the hell do you think you are?'",
	FALSE, gouldan, 0, ch, TO_VICT);
    mobaction(gouldan, ch->player.name, "glare");
    return TRUE;
    break;

  case 84:
  case 207:
  case 172:			/* cast, recite, use */
    if (gouldan == ch)
      return TRUE;
    mobaction(gouldan, ch->player.name, "poke");
    act("$n tells you 'Hey, no funny stuff.'.", FALSE, gouldan, 0, ch, TO_VICT);
    return TRUE;
    break;

  case 56:			/* buy */
    if (ch->in_room != THE_PUB) {
      act("$n tells you 'Hey man, I'm on my own time, but stop by the Pub some time.'", FALSE, gouldan, 0, ch, TO_VICT);
      return TRUE;
    }
    if (open == 0) {
      act("$n tells you 'Sorry, we're closed, come back for lunch.'",
	  FALSE, gouldan, 0, ch, TO_VICT);
      return TRUE;
    }
    only_argument(arg, argm);
    if (!(*argm)) {
      act("$n tells you 'Sure, what do you want to buy?'",
	  FALSE, gouldan, 0, ch, TO_VICT);
      return TRUE;
    }
    if (!(temp1 = get_obj_in_list_vis(ch, argm, gouldan->carrying))) {
      act("$n tells you 'Sorry, but I don't sell that.'", FALSE, gouldan, 0, ch, TO_VICT);
      return TRUE;
    }
    for (scan = sold_here; scan->container >= 0; scan++) {
      if (scan->container == obj_index[temp1->item_number].virtual)
	break;
    }
    if (scan->container < 0) {
      act("$n tells you 'Sorry, that's not for sale.'", FALSE, gouldan, 0, ch, TO_VICT);
      return TRUE;
    }
    if (scan->actflag & ACT_OVER_21 && GET_AGE(ch) < 21) {
      if (IS_TRUSTED(ch) || HasClass(ch, CLASS_THIEF)) {
	act("$N manages to slip a fake ID past $n.",
	    FALSE, gouldan, 0, ch, TO_NOTVICT);
      } else if (!IS_NPC(ch)) {
	act("$n tells you 'Sorry, I could lose my license if I served you alcohol.'", FALSE, gouldan, 0, ch, TO_VICT);
	act("$n cards $N and $N is BUSTED.", FALSE, gouldan, 0, ch, TO_NOTVICT);
	return TRUE;
      }
    }
    temp2 = read_object(scan->contains, VIRTUAL);
    cost = (temp2 ? (scan->howman * temp2->obj_flags.cost) : 0)
	+ temp1->obj_flags.cost;
    cost *= 9;
    cost /= 10;
    cost++;
    if (temp2)
      extract_obj(temp2);

    for (; num > 0; num--) {
      if (GET_GOLD(ch) < cost) {
	act("$n tells you 'Sorry, man, no bar tabs.'",
	    FALSE, gouldan, 0, ch, TO_VICT);
	return TRUE;
      }
      temp1 = read_object(temp1->item_number, REAL);
      for (i = 0; i < scan->howman; i++) {
	temp2 = read_object(scan->contains, VIRTUAL);
	obj_to_obj(temp2, temp1);
      }
      obj_to_char(temp1, ch);
      GET_GOLD(ch) -= cost;
      act("$N buys a $p from $n", FALSE, gouldan, temp1, ch, TO_NOTVICT);
      if (scan->actflag & ACT_SNICKER) {
	act("$n snickers softly.", FALSE, gouldan, NULL, ch, TO_ROOM);
      } else {
	act((scan->actflag & ACT_OVER_21) ?
	    "$n tells you 'Drink in good health' and gives you $p" :
	    "$n tells you 'Enjoy' and gives you $p",
	    FALSE, gouldan, temp1, ch, TO_VICT);
      }
    }
    return TRUE;
    break;

  case 59:			/* list */
    act("$n says 'We have", FALSE, gouldan, NULL, ch, TO_VICT);
    for (scan = sold_here; scan->container >= 0; scan++) {
      temp1 = read_object(scan->container, VIRTUAL);
      temp2 = scan->contains ? read_object(scan->contains, VIRTUAL) : NULL;
      cost = (temp2 ? (scan->howman * temp2->obj_flags.cost) : 0)
	  + temp1->obj_flags.cost;
      cost *= 9;
      cost /= 10;
      cost++;
      sprintf(buf, "%s for %d gold coins.\n\r", temp1->short_description, cost);
      send_to_char(buf, ch);
      extract_obj(temp1);
      if (temp2)
	extract_obj(temp2);
    }
    return TRUE;
    break;
  }

  return FALSE;
}
int Magic_Fountain(struct char_data *ch, int cmd, char *arg, struct room_data *rp, int type)
{

  char buf[MAX_INPUT_LENGTH];

  extern int drink_aff[][3];

  extern struct weather_data weather_info;
  void name_to_drinkcon(struct obj_data *obj, int type);
  void name_from_drinkcon(struct obj_data *obj);


  if (cmd == 11) {		/* drink */

    only_argument(arg, buf);

    if (str_cmp(buf, "fountain") && str_cmp(buf, "water")) {
      return FALSE;
    }
    send_to_char("You drink from the fountain\n\r", ch);
    act("$n drinks from the fountain", FALSE, ch, 0, 0, TO_ROOM);


    if (GET_COND(ch, THIRST) > 20) {
      act("You do not feel thirsty.", FALSE, ch, 0, 0, TO_CHAR);
      return TRUE;
    }
    if (GET_COND(ch, FULL) > 20) {
      act("You do are full.", FALSE, ch, 0, 0, TO_CHAR);
      return TRUE;
    }
    GET_COND(ch, THIRST) = 24;
    GET_COND(ch, FULL) += 1;

    switch (number(0, 40)) {

      /* Lets try and make 1-10 Good, 11-26 Bad, 27-40 Nothing */
    case 1:
      cast_refresh(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 2:
      cast_stone_skin(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 3:
      cast_cure_serious(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 4:
      cast_cure_light(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 5:
      cast_armor(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 6:
      cast_bless(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 7:
      cast_invisibility(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 8:
      cast_strength(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 9:
      cast_remove_poison(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 10:
      cast_true_seeing(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;

/* Time for the nasty Spells */

    case 11:
      cast_dispel_magic(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 12:
      cast_teleport(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 13:
      cast_web(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 14:
      cast_curse(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 15:
      cast_blindness(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 16:
      cast_weakness(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 17:
      cast_poison(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 18:
      cast_cause_light(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 19:
      cast_cause_critic(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 20:
      cast_dispel_magic(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 21:
      cast_magic_missile(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 22:
      cast_faerie_fire(Fountain_Level, ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 23:
      cast_flamestrike(GetMaxLevel(ch), ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 24:
      cast_burning_hands(GetMaxLevel(ch), ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 25:
      cast_acid_blast(GetMaxLevel(ch), ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;
    case 26:
      cast_energy_drain(GetMaxLevel(ch), ch, "", SPELL_TYPE_SPELL, ch, 0);
      break;

      /* And of course nothing */

    default:
      send_to_char("The fluid tastes like dry sand in your mouth.\n\r", ch);
      break;
    }
    return TRUE;
  }
  /* All commands except fill and drink */
  return FALSE;
}

#if 0
/*
   shanty town kids
 */
int shanty_town_kids(struct char_data *ch, int cmd, char *arg, struct char_data *mob, int type)
{

  if (!AWAKE(ch))
    return (FALSE);


  /*
     harrass low levellers.
   */
  if (cmd >= 1 && cmd <= 6) {
    if (GetMaxLevel(ch) <= 5 && number(0, 2) == 0) {
      act("A street kid sticks out a foot and trips you as you try to leave",
	  FALSE, ch, 0, 0, TO_CHAR);
      act("A street kid sticks out a foot and trips $n",
	  FALSE, ch, 0, 0, TO_ROOM);
      GET_POS(ch) = POSITION_SITTING;
      act("The street kid laughs at you", FALSE, ch, 0, 0, TO_CHAR);
      act("The street kid laughs at $n", FALSE, ch, 0, 0, TO_ROOM);
    }
  }
  /*
     steal from mid-levellers
   */


  /*
     backstab high levellers
   */


  if (cmd) {


  }
  if (ch->specials.fighting) {
    act("$N runs between $n's legs", FALSE, ch->specials.fighting, 0, ch, TO_ROOM);
    act("$N runs between your legs", FALSE, ch->specials.fighting, 0, ch, TO_CHAR);
    vict = ch->specials.fighting;
    stop_fighting(ch);
    stop_fighting(vict);
  }
}

#endif
#endif


char *fake_char_names[100];
int fake_char_info[100][2];
int num_fake_chars = 0;

void boot_fake_chars(void)
{
  FILE *f;
  int count, num_we_want;
  char Gbuf[80], line[MAX_STRING_LENGTH];

  /* appearance first */
  if (!(f = fopen("descs/fake_chars", "r"))) {
    log("FAKE: Can't open fake_chars!");
    return;
  }
  count = 0;
  Gbuf[0] = '\0';
  num_we_want = number(5, 25);
  while (num_we_want > count) {
    if (!get_line(f, line)) {
      log("FAKE: 1 Error in reading desc/fake_chars");
      raise(SIGSEGV);
    }
    if (line[0] == '$' || line[1] == '$')
      break;
    if ((sscanf(line, " %s", Gbuf)) != 1) {
      log("FAKE: 2 Error in reading desc/fake_chars");
      raise(SIGSEGV);
    }
    Gbuf[strlen(Gbuf)] = '\0';
    fake_char_names[count] = str_dup(Gbuf);
    fake_char_info[count][0] = number(1, NUM_PC_RACES);
    fake_char_info[count][1] = number(0, 20);
    fake_char_info[count][2] = number(1, 5);
    if (fake_char_info[count][1] == CLASS_PALADIN)
      fake_char_info[count][0] = RACE_HUMAN;
    if (fake_char_info[count][1] >= 9)	/* multi */
      fake_char_info[count][2] = 1;
    count++;
  }
  num_fake_chars = count;
  fclose(f);
  vlogf("FAKE: %d chars loaded", num_fake_chars);
}
