/*****************************************************************/
/*                      World of GreyHawk                        */
/*      Begun in 1998, originally based on Circle3.0 and others  */
/* File:                                                         */
/* Purpose:                                                      */
/*****************************************************************/

#include "conf.h"
#include "sysdep.h"
#include <stdarg.h>
#include <math.h>
#include <signal.h>

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "spells.h"
#include "handler.h"
#include "interpreter.h"
#include "map.h"
#include "events.h"
#include "skills.h"
#include "db.h"

extern const char *dirs[];
extern int rev_dir[];
extern const char *spells[];
extern char *int_ratings[];
extern struct str_app_type str_app[];
extern struct time_info_data time_info;
extern struct room_data *world;
extern const char *race_types[];
extern int top_of_world;
extern int sunlight;
extern struct zone_data *zone_table;
void remove_and_destroy_money(struct char_data *ch, int amount, int type);
void create_and_give_money(struct char_data *ch, int amount, int type);
int CoinAmount(struct char_data *ch, int type);

/* creates a random number in interval [from;to] */
int number(int from, int to)
{
  /* error checking in case people call number() incorrectly */
  if (from > to) {
    int tmp = from;
    from = to;
    to = tmp;
  }
  return ((circle_random() % (to - from + 1)) + from);
}


/* simulates dice roll */
int dice(int numdice, int size)
{
  int sum = 0;

  if (size <= 0 || numdice <= 0)
    return 0;

  while (numdice-- > 0)
    sum += ((circle_random() % size) + 1);

  return sum;
}

int BOUNDED(int a, int b, int c)
{
  return (MIN(MAX(a, b), c));
}

int IS_FENCING(struct obj_data *obj)
{
  int obj_type = GET_OBJ_VAL(obj, 7);
  if ((obj_type == WEAPON_SABER) || (obj_type == WEAPON_RAPIER))
    return 1;
  return 0;
}

int GET_PD(struct char_data *ch, int location)
{
  int pd = MIN(6, ac_to_pd(GET_AC(ch)));
  struct obj_data *obj = GET_EQ(ch, location);
  struct obj_data *shield = GET_EQ(ch, WEAR_SHIELD);
  int modifier = 0;

  extern struct armor_apply_type limb_app[];
  extern struct armor_apply_type torso_app[];
  extern struct armor_apply_type extremity_app[];
  extern struct armor_apply_type helmet_app[];
  extern struct body_armor_type armor_app[];
  extern int shield_pr[];

  if (shield) {
    pd += shield_pr[GET_OBJ_VAL(shield, 0)];
  }
  if (!obj)
    obj = GET_EQ(ch, WEAR_BODY);

  if (!obj)
    return pd;

  if (!(GET_OBJ_TYPE(obj) == ITEM_ARMOR))
    return pd;

  modifier = GET_OBJ_CONDITION(obj) / 10;

  switch (location) {
  case WEAR_ARMS:
    if (IS_SET(GET_OBJ_EXTRA(obj), ITEM_WHOLE_BODY))
      pd = armor_app[GET_OBJ_VAL(obj, 0)].arms.pd * modifier / 10;
    else
      pd = limb_app[GET_OBJ_VAL(obj, 0)].pd * modifier / 10;
    break;
  case WEAR_LEGS:
    if (IS_SET(GET_OBJ_EXTRA(obj), ITEM_WHOLE_BODY))
      pd = armor_app[GET_OBJ_VAL(obj, 0)].legs.pd * modifier / 10;
    else
      pd = limb_app[GET_OBJ_VAL(obj, 0)].pd * modifier / 10;
    break;
  case WEAR_BODY:
    if (IS_SET(GET_OBJ_EXTRA(obj), ITEM_WHOLE_BODY))
      pd = armor_app[GET_OBJ_VAL(obj, 0)].torso.pd * modifier / 10;
    else
      pd = torso_app[GET_OBJ_VAL(obj, 0)].pd * modifier / 10;
    break;
  case WEAR_HANDS:
  case WEAR_FEET:
    pd = extremity_app[GET_OBJ_VAL(obj, 0)].pd * modifier / 10;
    break;
  case WEAR_HEAD:
  case WEAR_FACE:
    pd = helmet_app[GET_OBJ_VAL(obj, 0)].pd * modifier / 10;
    break;
  default:
  }

  return MIN(6, pd);
}

int GET_DR(struct char_data *ch, int location)
{
  int dr = ac_to_dr(GET_AC(ch)), whole_body = 0;
  struct obj_data *obj = GET_EQ(ch, location), *obj2 = NULL;
  int modifier = 0;

  extern struct armor_apply_type limb_app[];
  extern struct armor_apply_type torso_app[];
  extern struct armor_apply_type extremity_app[];
  extern struct armor_apply_type helmet_app[];
  extern struct body_armor_type armor_app[];

  whole_body = (((obj2 = GET_EQ(ch, WEAR_BODY)) != NULL) &&
		IS_SET(GET_OBJ_EXTRA(obj2), ITEM_WHOLE_BODY)) ? 1 : 0;

  if (!obj && !(((location == WEAR_ARMS) || (location == WEAR_LEGS)) &&
		whole_body)) {
    if (affected_by_spell(ch, SPELL_ARMOR)) {
      dr += ac_to_dr(6);
    }
    return dr;
  }
  if (!obj)
    obj = GET_EQ(ch, WEAR_BODY);

  if (!obj)
    return dr;

  if (!(GET_OBJ_TYPE(obj) == ITEM_ARMOR))
    return dr;

  modifier = GET_OBJ_CONDITION(obj) / 10;

  switch (location) {
  case WEAR_ARMS:
    if (IS_SET(GET_OBJ_EXTRA(obj), ITEM_WHOLE_BODY))
      dr += armor_app[GET_OBJ_VAL(obj, 0)].arms.dr * modifier / 10;
    else
      dr += limb_app[GET_OBJ_VAL(obj, 0)].dr * modifier / 10;
    break;
  case WEAR_LEGS:
    if (IS_SET(GET_OBJ_EXTRA(obj), ITEM_WHOLE_BODY))
      dr += armor_app[GET_OBJ_VAL(obj, 0)].legs.dr * modifier / 10;
    else
      dr += limb_app[GET_OBJ_VAL(obj, 0)].dr * modifier / 10;
    break;
  case WEAR_BODY:
    if (IS_SET(GET_OBJ_EXTRA(obj), ITEM_WHOLE_BODY))
      dr += armor_app[GET_OBJ_VAL(obj, 0)].torso.dr * modifier / 10;
    else
      dr += torso_app[GET_OBJ_VAL(obj, 0)].dr * modifier / 10;
    break;
  case WEAR_HANDS:
  case WEAR_FEET:
    dr += extremity_app[GET_OBJ_VAL(obj, 0)].dr * modifier / 10;
    break;
  case WEAR_HEAD:
  case WEAR_FACE:
    dr += helmet_app[GET_OBJ_VAL(obj, 0)].dr * modifier / 10;
    break;
  default:
  }

  return dr;
}

int IN_RANGE(int a, int b, int c)
{
  if (b >= a && b <= c)
    return 1;
  else
    return 0;
}

int MIN(int a, int b)
{
  return a < b ? a : b;
}


int MAX(int a, int b)
{
  return a > b ? a : b;
}

/* string to lower case */
void strToLower(char *s1)
{
  int i, len;
  len = strlen(s1);
  for (i = 0; i < len; i++)
    s1[i] = toupper(s1[i]);
}
#if 0
/* Create a duplicate of a string */
char *str_dup(const char *source)
{
  char *new;

  CREATE(new, char, strlen(source) + 1);
  return (strcpy(new, source));
}
#endif

char *my_str_dup(const char *dup, const char *file, int line)
{
  if (!dup) {
    vlogf("SYSERR: NULL str_dup from %s:%d", file, line);
    return NULL;
  }
  return strdup(dup);
}


/* str_cmp: a case-insensitive version of strcmp */
/* returns: 0 if equal, 1 if arg1 > arg2, -1 if arg1 < arg2  */
/* scan 'till found different or end of both                 */
int str_cmp(char *arg1, char *arg2)
{
  int chk, i;

  for (i = 0; *(arg1 + i) || *(arg2 + i); i++)
    if ((chk = LOWER(*(arg1 + i)) - LOWER(*(arg2 + i)))) {
      if (chk < 0)
	return (-1);
      else
	return (1);
    }
  return (0);
}


/* strn_cmp: a case-insensitive version of strncmp */
/* returns: 0 if equal, 1 if arg1 > arg2, -1 if arg1 < arg2  */
/* scan 'till found different, end of both, or n reached     */
int strn_cmp(char *arg1, char *arg2, int n)
{
  int chk, i;

  for (i = 0; (*(arg1 + i) || *(arg2 + i)) && (n > 0); i++, n--)
    if ((chk = LOWER(*(arg1 + i)) - LOWER(*(arg2 + i)))) {
      if (chk < 0)
	return (-1);
      else
	return (1);
    }
  return (0);
}


/* log a death trap hit */
void log_death_trap(struct char_data *ch)
{
  extern struct room_data *world;
  char buf[MAX_STRING_LENGTH];

  sprintf(buf, "%s hit death trap #%d (%s)", GET_NAME(ch),
	  world[ch->in_room].number, world[ch->in_room].name);
  mudlog(buf, BRF, LVL_IMMORT, TRUE);
}


/* writes a string to the log */
void log(char *str)
{
  time_t ct;
  char *tmstr;

  ct = time(0);
  tmstr = asctime(localtime(&ct));
  *(tmstr + strlen(tmstr) - 1) = '\0';
  fprintf(stderr, "%-19.19s :: %s\n", tmstr, strip_color(str));
}


/* the "touch" command, essentially. */
int touch(char *path)
{
  FILE *fl;

  if (!(fl = fopen(path, "a"))) {
    perror(path);
    return -1;
  } else {
    fclose(fl);
    return 0;
  }
}


/*
 * mudlog -- log mud messages to a file & to online imm's syslogs
 */
void mudlog(char *str, char type, int level, byte file)
{
  extern struct descriptor_data *descriptor_list;
  struct descriptor_data *i;
  char buf[MAX_STRING_LENGTH];
  char *tmp, tp;
  time_t ct;

  ct = time(0);
  tmp = asctime(localtime(&ct));

  if (file)
    fprintf(stderr, "%-19.19s :: %s\n", tmp, str);
  if (level < 0)
    return;

  sprintf(buf, "[ %s ]\r\n", str);

  for (i = descriptor_list; i; i = i->next)
    if (!i->connected && !PLR_FLAGGED(i->character, PLR_WRITING)) {
      tp = ((PRF_FLAGGED(i->character, PRF_LOG1) ? 1 : 0) +
	    (PRF_FLAGGED(i->character, PRF_LOG2) ? 2 : 0));

      if ((GET_LEVEL(i->character) >= level) && (tp >= type)) {
	send_to_char(buf, i->character);
      }
    }
}



void sprintbit(ulong bitvector, char *names[], char *result)
{
  ulong nr;

  *result = '\0';

  if (bitvector < 0) {
    strcpy(result, "<INVALID BITVECTOR>");
    return;
  }
  for (nr = 0; bitvector; bitvector >>= 1) {
    if (IS_SET(bitvector, 1)) {
      if (*names[nr] != '\n') {
	strcat(result, names[nr]);
	strcat(result, " ");
      } else
	strcat(result, "UNDEFINED ");
    }
    if (*names[nr] != '\n')
      nr++;
  }

  if (!*result)
    strcpy(result, "NOBITS ");
}



void sprinttype(int type, char *names[], char *result)
{
  int nr = 0;

  while (type && *names[nr] != '\n') {
    type--;
    nr++;
  }

  if (*names[nr] != '\n')
    strcpy(result, names[nr]);
  else
    strcpy(result, "UNDEFINED");
}


/* Calculate the REAL time passed over the last t2-t1 centuries (secs) */
struct time_info_data real_time_passed(time_t t2, time_t t1)
{
  long secs;
  struct time_info_data now;

  secs = (long) (t2 - t1);

  now.hours = (secs / SECS_PER_REAL_HOUR) % 24;		/* 0..23 hours */
  secs -= SECS_PER_REAL_HOUR * now.hours;

  now.day = (secs / SECS_PER_REAL_DAY);		/* 0..27 days  */
  secs -= SECS_PER_REAL_DAY * now.day;

  now.month = -1;
  now.year = -1;

  return now;
}



/* Calculate the MUD time passed over the last t2-t1 centuries (secs) */
struct time_info_data mud_time_passed(time_t t2, time_t t1)
{
  long secs;
  struct time_info_data now;

  secs = (long) (t2 - t1);

  now.hours = (secs / SECS_PER_MUD_HOUR) % 24;	/* 0..23 hours */
  secs -= SECS_PER_MUD_HOUR * now.hours;

  now.day = (secs / SECS_PER_MUD_DAY) % 28;	/* 0..27 days  */
  secs -= SECS_PER_MUD_DAY * now.day;

  now.month = (secs / SECS_PER_MUD_MONTH) % 12;		/* 0..11 months */
  secs -= SECS_PER_MUD_MONTH * now.month;

  now.year = (secs / SECS_PER_MUD_YEAR);	/* 0..XX? years */
  now.minutes = 0;

  return now;
}



struct time_info_data age(struct char_data *ch)
{
  struct time_info_data player_age;

  player_age = mud_time_passed(time(0), ch->player.time.birth);

  switch (GET_RACE(ch)) {
  case RACE_HUMAN:
    player_age.year += 17;
    break;
  case RACE_DWARF:
    player_age.year += 55;
    break;
  case RACE_ELF:
    player_age.year += 115;
    break;
  case RACE_GNOME:
    player_age.year += 78;
    break;
  case RACE_HALFORC:
    player_age.year += 17;
    break;
  case RACE_HALFLING:
    player_age.year += 24;
    break;
  case RACE_HALFELF:
    player_age.year += 18;
    break;
  default:
    player_age.year += 5;
  }
  return player_age;
}


/* Check if making CH follow VICTIM will create an illegal */
/* Follow "Loop/circle"                                    */
bool circle_follow(struct char_data * ch, struct char_data * victim)
{
  struct char_data *k;

  for (k = victim; k; k = k->master) {
    if (k == ch)
      return TRUE;
  }

  return FALSE;
}



/* Called when stop following persons, or stopping charm */
/* This will NOT do if a character quits/dies!!          */
void stop_follower(struct char_data *ch)
{
  struct follow_type *j, *k;

  assert(ch->master);

  if (IS_AFFECTED(ch, AFF_CHARM)) {
    act("You realize that $N is a jerk!", FALSE, ch, 0, ch->master, TO_CHAR);
    act("$n realizes that $N is a jerk!", FALSE, ch, 0, ch->master, TO_NOTVICT);
    act("$n hates your guts!", FALSE, ch, 0, ch->master, TO_VICT);
/*    if (affected_by_spell(ch, SPELL_CHARM))
   affect_from_char(ch, SPELL_CHARM); */
  } else {
    act("You stop following $N.", FALSE, ch, 0, ch->master, TO_CHAR);
    act("$n stops following $N.", TRUE, ch, 0, ch->master, TO_NOTVICT);
    act("$n stops following you.", TRUE, ch, 0, ch->master, TO_VICT);
  }

  if (ch->master->followers->follower == ch) {	/* Head of follower-list? */
    k = ch->master->followers;
    ch->master->followers = k->next;
    FREE(k);
  } else {			/* locate follower who is not head of list */
    for (k = ch->master->followers; k->next->follower != ch; k = k->next);

    j = k->next;
    k->next = j->next;
    FREE(j);
  }

  ch->master = NULL;
  REMOVE_BIT(AFF_FLAGS(ch), AFF_CHARM);
}

/* Called when a character that is part of a group dies. */
void die_group_member(struct char_data *ch)
{
  struct char_data *vict;
  struct group_type *g, *g_next;
//  struct group_type *j, *k;
  void remove_from_group(struct char_data *ch, struct char_data *fol);

  if (ch->leader) {
    act("$n is no longer a member of your group.", FALSE, ch, 0, ch->leader, TO_VICT);
    act("You leave $N's group!", FALSE, ch, 0, ch->leader, TO_CHAR);
    act("$N leaves $n's group!", FALSE, ch, 0, ch->leader, TO_OTHERS);
    remove_from_group(ch->leader, ch);
  } else {
    act("You leave the group!", FALSE, ch, 0, ch->leader, TO_CHAR);
    remove_from_group(ch, ch);
    if (!ch->group)
      return;
    vict = ch->group->follower;
    vict->leader = NULL;
    for (g = ch->group->next; g; g = g_next) {
      g_next = g->next;

      g->next = vict->group;
      vict->group = g;
      g->follower->leader = vict;
//        act("$N leaves the group!", FALSE, g->follower, 0, ch, TO_CHAR);
    }
    ch->group = NULL;
  }
}

/* Called when a character that follows/is followed dies */
void die_follower(struct char_data *ch)
{
  struct follow_type *j, *k;

  if (ch->master)
    stop_follower(ch);

  for (k = ch->followers; k; k = j) {
    j = k->next;
    stop_follower(k->follower);
  }
}


/* Do NOT call this before having checked if a circle of followers */
/* will arise. CH will follow leader                               */
void add_follower(struct char_data *ch, struct char_data *leader)
{
  struct follow_type *k;

  assert(!ch->master);

  ch->master = leader;

  CREATE(k, struct follow_type, 1);

  k->follower = ch;
  k->next = leader->followers;
  leader->followers = k;

  act("You now follow $N.", FALSE, ch, 0, leader, TO_CHAR);
  if (CAN_SEE(leader, ch))
    act("$n starts following you.", TRUE, ch, 0, leader, TO_VICT);
  act("$n starts to follow $N.", TRUE, ch, 0, leader, TO_NOTVICT);
}


/*
 * get_line reads the next non-blank line off of the input stream.
 * The newline character is removed from the input.  Lines which begin
 * with '*' are considered to be comments.
 *
 * Returns the number of lines advanced in the file.
 */
int get_line(FILE * fl, char *buf)
{
  char temp[256];
  int lines = 0;

  do {
    lines++;
    fgets(temp, 256, fl);
    if (*temp)
      temp[strlen(temp) - 1] = '\0';
  } while (!feof(fl) && (*temp == '*' || !*temp));

  if (feof(fl))
    return 0;
  else {
    strcpy(buf, temp);
    return lines;
  }
}


int get_filename(char *orig_name, char *filename, int mode)
{
  char *prefix, *middle, *suffix, *ptr, name[64];

  switch (mode) {
  case CRASH_FILE:
    prefix = "plrobjs";
    suffix = "objs";
    break;
  case ETEXT_FILE:
    prefix = "plrtext";
    suffix = "text";
    break;
  default:
    return 0;
    break;
  }

  if (!*orig_name)
    return 0;

  strcpy(name, orig_name);
  for (ptr = name; *ptr; ptr++)
    *ptr = LOWER(*ptr);

  switch (LOWER(*name)) {
  case 'a':
  case 'b':
  case 'c':
  case 'd':
  case 'e':
    middle = "A-E";
    break;
  case 'f':
  case 'g':
  case 'h':
  case 'i':
  case 'j':
    middle = "F-J";
    break;
  case 'k':
  case 'l':
  case 'm':
  case 'n':
  case 'o':
    middle = "K-O";
    break;
  case 'p':
  case 'q':
  case 'r':
  case 's':
  case 't':
    middle = "P-T";
    break;
  case 'u':
  case 'v':
  case 'w':
  case 'x':
  case 'y':
  case 'z':
    middle = "U-Z";
    break;
  default:
    middle = "ZZZ";
    break;
  }

  sprintf(filename, "%s/%s/%s.%s", prefix, middle, name, suffix);
  return 1;
}


int num_pc_in_room(struct room_data *room)
{
  int i = 0;
  struct char_data *ch;

  for (ch = room->people; ch != NULL; ch = ch->next_in_room)
    if (!IS_NPC(ch))
      i++;

  return i;
}


/* substitute appearances of 'pattern' with 'replacement' in string */
/* and return the # of replacements */
int replace_str(char **string, char *pattern, char *replacement, int rep_all,
		int max_size)
{
  char *replace_buffer = NULL;
  char *flow, *jetsam, temp;
  int len, i;

  if ((strlen(*string) - strlen(pattern)) + strlen(replacement) > max_size)
    return -1;

  CREATE(replace_buffer, char, max_size);
  i = 0;
  jetsam = *string;
  flow = *string;
  *replace_buffer = '\0';
  if (rep_all) {
    while ((flow = (char *) strstr(flow, pattern)) != NULL) {
      i++;
      temp = *flow;
      *flow = '\0';
      if ((strlen(replace_buffer) + strlen(jetsam) + strlen(replacement)) > max_size) {
	i = -1;
	break;
      }
      strcat(replace_buffer, jetsam);
      strcat(replace_buffer, replacement);
      *flow = temp;
      flow += strlen(pattern);
      jetsam = flow;
    }
    strcat(replace_buffer, jetsam);
  } else {
    if ((flow = (char *) strstr(*string, pattern)) != NULL) {
      i++;
      flow += strlen(pattern);
      len = ((char *) flow - (char *) *string) - strlen(pattern);

      strncpy(replace_buffer, *string, len);
      strcat(replace_buffer, replacement);
      strcat(replace_buffer, flow);
    }
  }
  if (i == 0)
    return 0;
  if (i > 0) {
    RECREATE(*string, char, strlen(replace_buffer) + 3);
    strcpy(*string, replace_buffer);
  }
  FREE(replace_buffer);
  return i;
}


/* re-formats message type formatted char * */
/* (for strings edited with d->str) (mostly olc and mail)     */
void format_text(char **ptr_string, int mode, struct descriptor_data *d, int maxlen)
{
  int total_chars, cap_next = TRUE, cap_next_next = FALSE;
  char *flow, *start = NULL, temp;
  /* warning: do not edit messages with max_str's of over this value */
  char formated[MAX_STRING_LENGTH];

  flow = *ptr_string;
  if (!flow)
    return;

  if (IS_SET(mode, FORMAT_INDENT)) {
    strcpy(formated, "   ");
    total_chars = 3;
  } else {
    *formated = '\0';
    total_chars = 0;
  }

  while (*flow != '\0') {
    while ((*flow == '\n') ||
	   (*flow == '\r') ||
	   (*flow == '\f') ||
	   (*flow == '\t') ||
	   (*flow == '\v') ||
	   (*flow == ' '))
      flow++;

    if (*flow != '\0') {

      start = flow++;
      while ((*flow != '\0') &&
	     (*flow != '\n') &&
	     (*flow != '\r') &&
	     (*flow != '\f') &&
	     (*flow != '\t') &&
	     (*flow != '\v') &&
	     (*flow != ' ') &&
	     (*flow != '.') &&
	     (*flow != '?') &&
	     (*flow != '&') &&
	     (*flow != '!'))
	flow++;

      if (cap_next_next) {
	cap_next_next = FALSE;
	cap_next = TRUE;
      }
      /* this is so that if we stopped on a sentance .. we move off the sentance delim. */
      while ((*flow == '.') || (*flow == '!') || (*flow == '?')) {
	cap_next_next = TRUE;
	flow++;
      }

      temp = *flow;
      *flow = '\0';

      if ((total_chars + strlen(start) + 1) > 79) {
	strcat(formated, "\r\n");
	total_chars = 0;
      }
      if (!cap_next) {
	if (total_chars > 0) {
	  strcat(formated, " ");
	  total_chars++;
	}
      } else {
	cap_next = FALSE;
	*start = UPPER(*start);
      }

      total_chars += strlen(start);
      strcat(formated, start);

      *flow = temp;
    }
    if (cap_next_next) {
      if ((total_chars + 3) > 79) {
	strcat(formated, "\r\n");
	total_chars = 0;
      } else {
	strcat(formated, "  ");
	total_chars += 2;
      }
    }
  }
  strcat(formated, "\r\n");

  if (strlen(formated) > maxlen)
    formated[maxlen] = '\0';
  RECREATE(*ptr_string, char, MIN(maxlen, strlen(formated) + 3));
  strcpy(*ptr_string, formated);
}


int is_introd(struct char_data *ch, struct char_data *vict)
{
  /* binary search of introd array */
  int hi, lo, mid;

  if (ch == vict)
    return TRUE;

  if (IS_MORPH(ch, NULL))
    ch = MORPH_MOB(ch);		/* get back to orig char for morphs */

  if (IS_NPC(ch) || IS_NPC(vict))
    return TRUE;
  if (IS_TRUSTED(ch) || IS_TRUSTED(vict))
    return TRUE;

  hi = MAX_INTRO - 1;
  lo = 0;
  mid = (hi + lo) >> 1;

  while ((ch->player_specials->saved.introd_list[mid] != vict->nr) &&
	 (hi >= lo)) {
    if (ch->player_specials->saved.introd_list[mid] < vict->nr) {
      lo = mid + 1;
    } else {
      hi = mid - 1;
    }

    mid = (hi + lo) >> 1;
  }

  if (ch->player_specials->saved.introd_list[mid] == vict->nr) {
    ch->player_specials->saved.introd_times[mid] = time(0);
    return TRUE;
  }
  return FALSE;
}

/* add vict to ch's list */
void add_intro(struct char_data *vict, struct char_data *ch)
{
  int oldest;
  int x;
  int found;

  if (ch->player_specials->saved.introd_list[0] != 0) {		/* no free slot */
    oldest = 0;
    for (x = 0; x < MAX_INTRO; x++)
      if (ch->player_specials->saved.introd_times[x] < ch->player_specials->saved.introd_times[oldest])
	oldest = x;
    /* got oldest, put 0, bubble 0 to top */
    for (x = oldest; x > 0; x--) {
      ch->player_specials->saved.introd_times[x] = ch->player_specials->saved.introd_times[x - 1];
      ch->player_specials->saved.introd_list[x] = ch->player_specials->saved.introd_list[x - 1];
    }
    ch->player_specials->saved.introd_times[0] = 0;
    ch->player_specials->saved.introd_list[0] = 0;
  }
  /* Okay, now we got 0 at top of array */
  /* step down, bumping everything up till we don't go down anymore */
  x = 1;
  found = 0;
  while ((x < MAX_INTRO) && (found == FALSE)) {
    if (vict->nr < ch->player_specials->saved.introd_list[x + 1]) {
      found = TRUE;
      ch->player_specials->saved.introd_list[x - 1] = ch->player_specials->saved.introd_list[x];
      ch->player_specials->saved.introd_times[x - 1] = ch->player_specials->saved.introd_times[x];
      ch->player_specials->saved.introd_list[x] = vict->nr;
      ch->player_specials->saved.introd_times[x] = time(0);
    } else {
      ch->player_specials->saved.introd_list[x - 1] = ch->player_specials->saved.introd_list[x];
      ch->player_specials->saved.introd_times[x - 1] = ch->player_specials->saved.introd_times[x];
    }
    x++;
  }
  if (!found) {			/* tack to end */
    x--;
    ch->player_specials->saved.introd_list[x] = vict->nr;
    ch->player_specials->saved.introd_times[x] = time(0);
  }
}

void purge_old_intros(struct char_data *ch)
{
  int swaps;
  int temptime, tempnum;
  int x, currtime;

  currtime = time(0);

  for (x = 0; x < MAX_INTRO; x++) {
    /* go through, 0 out old ones */
    if ((currtime - ch->player_specials->saved.introd_times[x]) > 604800) {
      ch->player_specials->saved.introd_list[x] = 0;
      ch->player_specials->saved.introd_times[x] = 0;
    }
  }

  /* now bubblesort */
  swaps = 1;
  while (swaps) {
    swaps = 0;
    for (x = 0; x < MAX_INTRO - 1; x++) {
      if (ch->player_specials->saved.introd_list[x] > ch->player_specials->saved.introd_list[x + 1]) {
	/* swap */
	swaps = 1;
	temptime = ch->player_specials->saved.introd_times[x];
	tempnum = ch->player_specials->saved.introd_list[x];
	ch->player_specials->saved.introd_times[x] = ch->player_specials->saved.introd_times[x + 1];
	ch->player_specials->saved.introd_list[x] = ch->player_specials->saved.introd_list[x + 1];
	ch->player_specials->saved.introd_times[x + 1] = temptime;
	ch->player_specials->saved.introd_list[x + 1] = tempnum;
      }
    }
  }
}

ACMD(do_introduce)
{
  char buf[MAX_STRING_LENGTH];
  struct char_data *vict;
  int x;

  if (IS_TRUSTED(ch) && (*argument == '\0')) {
    for (x = 0; x < MAX_INTRO; x++)
      if (ch->player_specials->saved.introd_list[x]) {
	sprintf(buf, "%ld\n\r", ch->player_specials->saved.introd_list[x]);
	send_to_char(buf, ch);
      }
    return;
  }
  vict = get_char_vis(ch, argument);
  if (!vict) {
    send_to_char("Introduce yourself to whom?\n\r", ch);
    return;
  }
  if (IS_NPC(ch) || IS_NPC(vict))
    return;
  if (ch->in_room != vict->in_room) {
    send_to_char("That person doesn't seem to be here.\r\n", ch);
    return;
  }
  if (is_introd(vict, ch)) {
    send_to_char("They already are familiar with you.\n\r", ch);
    return;
  }
  if (!CAN_SEE(vict, ch)) {
    send_to_char("How can you introduce yourself when they can't see you?\r\n", ch);
    return;
  }
  /* add ch to victs list */
  add_intro(ch, vict);
  send_to_char("They now know your name.\n\r", ch);
  act("$n has introduced $mself to you!", FALSE, ch, 0, vict, TO_VICT);
  return;

}

#define CAN_SEE_Z_CORD(vict, ch) 1

char *PERS(struct char_data *ch, struct char_data *vict, int short_d)
{
  static char buf1[80];

  if (!CAN_SEE_Z_CORD(vict, ch) || !CAN_SEE(vict, ch)) {
    strcpy(buf1, "someone");
    return buf1;
  }
  if (!is_introd(vict, ch)) {
    sprintf(buf1, "%s %s", ((GET_RACE(ch) == RACE_MIND_FLAYER) ||
	     (GET_RACE(ch) == RACE_ORC) || (GET_RACE(ch) == RACE_OGRE)) ?
	    "An" : "A", race_types[(int) GET_RACE(ch)]);
    return buf1;
  }
  if (IS_DARK(ch->in_room) && !IS_TRUSTED(vict)) {
    if (IS_AFFECTED(vict, AFF_INFRAVISION)) {
      strcpy(buf1, "a red shape");
      return buf1;
    } else {
      strcpy(buf1, "someone");
      return buf1;
    }
  }
  if (IS_NPC(ch))
    return (ch->player.short_descr);
  else {
    if (is_introd(vict, ch))
      return (ch->player.name);
    if (short_d)
      return ch->player.short_descr;
    sprintf(buf1, "%s %s", ((GET_RACE(ch) == RACE_ORC) || (GET_RACE(ch) == RACE_OGRE)) ?
	    "An" : "A", race_types[(int) GET_RACE(ch)]);
    return buf1;
  }
}

#undef CAN_SEE_Z_CORD(vict, ch)

/* room is the originating room, or the room you want the offset from.
   from_dir is the direction of the room you want the offset of from
   the originating room.
   Example: Want the offset (or my equivelent x_cord) in the room north
   of me. I would call GetXOffset(my_room_no, NORTH)
 */
int GetXOffset(room_rnum room, int to_dir)
{
  int x_dif = 0;
  int to_room = 0;

  if (world[room].x_size >= 3 && world[room].y_size >= 3) {
    if (to_dir > -1) {
      if (world[room].dir_option[to_dir]) {
	if (world[room].dir_option[to_dir]->x_cord != -1) {
	  to_room = world[room].dir_option[to_dir]->to_room;
	  if (world[to_room].dir_option[rev_dir[to_dir]] &&
	      world[room].dir_option[to_dir])
	    x_dif = world[to_room].dir_option[rev_dir[to_dir]]->x_cord -
		world[room].dir_option[to_dir]->x_cord;
	}
      }
    }
  }
  return x_dif;
}
int GetYOffset(room_rnum room, int to_dir)
{
  int y_dif = 0;
  room_rnum to_room;
  if (world[room].x_size >= 3 && world[room].y_size >= 3) {
    if (to_dir > -1) {
      if (world[room].dir_option[to_dir]) {
	if (world[room].dir_option[to_dir]->y_cord != -1) {
	  to_room = world[room].dir_option[to_dir]->to_room;
	  if (world[to_room].dir_option[rev_dir[to_dir]] &&
	      world[room].dir_option[to_dir])
	    y_dif = world[to_room].dir_option[rev_dir[to_dir]]->y_cord -
		world[room].dir_option[to_dir]->y_cord;
	}
      }
    }
  }
  return y_dif;
}
int GetZOffset(room_rnum room, int to_dir)
{
  int z_dif = 0;
  room_rnum to_room;
  if (world[room].x_size >= 3 && world[room].y_size >= 3) {
    if (to_dir > -1) {
      if (world[room].dir_option[to_dir]) {
	if (world[room].dir_option[to_dir]->z_cord != -1) {
	  to_room = world[room].dir_option[to_dir]->to_room;
	  if (world[to_room].dir_option[rev_dir[to_dir]] &&
	      world[room].dir_option[to_dir])
	    z_dif = world[to_room].dir_option[rev_dir[to_dir]]->z_cord -
		world[room].dir_option[to_dir]->x_cord;
	}
      }
    }
  }
  return z_dif;
}

/*Checks if victim is within distance d of character */
byte victWithinRangeofChar(struct char_data * ch, struct char_data * vict,
			   int d)
{
  int dr = -1;
  if (d < 0)
    return 0;
  return (WithinProximity(ch, NULL, vict, NULL, NULL, -1, &d, &dr, FALSE));
}

/*checks if victim is within distance d of room location */
byte victWithinRangeofLoc(struct location_data * loc, struct char_data * vict,
			  int d)
{
  int dr = -1;
  return (WithinProximity(NULL, loc, vict, NULL, NULL, -1, &d, &dr, FALSE));
}

/*checks if object is within distance d of character */
byte objWithinRangeofChar(struct char_data * ch, struct obj_data * obj,
			  int d)
{
  int dr = -1;
  return (WithinProximity(ch, NULL, NULL, NULL, obj, -1, &d, &dr, FALSE));
}

/*checks if object is within distance d of room location */
byte objWithinRangeofLoc(struct location_data * l, struct obj_data * obj,
			 int d)
{
  int dr = -1;
  return (WithinProximity(NULL, l, NULL, NULL, obj, -1, &d, &dr, FALSE));
}

/*check if character is within distance d of door to direction door */
byte charWithinRangeofDoor(struct char_data * ch, int door,
			   int d)
{
  int dr = -1;
  return (WithinProximity(ch, NULL, NULL, NULL, NULL, door, &d, &dr, FALSE));
}

/*check if two locations are within distance d of each other */
byte locWithinRangeofLoc(struct location_data * s, struct location_data * f,
			 int d)
{
  int dr = -1;
  return (WithinProximity(NULL, s, NULL, f, NULL, -1, &d, &dr, FALSE));
}
/*get the cardinal direction (NESW) between character and victim */
int angleBetweenCharAndVict(struct char_data *ch, struct char_data *vict)
{
  return (face_angle(angle_offset(ch, NULL, vict, NULL)));
}

/*return the cardinal direction (NESW) between character and object */
int angleBetweenCharAndObj(struct char_data *ch, struct obj_data *o)
{
  int d = -1;
  int dr = -1;
  WithinProximity(ch, NULL, NULL, NULL, o, -1, &d, &dr, TRUE);
  return dr;
}

/*get the cardinal direction (NEWS) between character and location */
int angleBetweenCharAndLoc(struct char_data *ch, struct location_data *l)
{
  return (face_angle(angle_offset(ch, NULL, NULL, l)));
}

/* the cardinal direction (NESW) between character and door. */
int angleBetweenCharAndDoor(struct char_data *ch, int door)
{
  int d = -1;
  int dr = -1;
  WithinProximity(ch, NULL, NULL, NULL, NULL, door, &d, &dr, TRUE);
  return dr;
}

/*get the cardinal direction (NESW) between location and character.
   this is the inverse of the above value. */
int angleBetweenLocAndChar(struct location_data *l, struct char_data *ch)
{
  return (face_angle(angle_offset(NULL, l, ch, NULL)));
}

/*get the cardinal direction (NESW) between two locations. */
int angleBetweenLocAndLoc(struct location_data *s, struct location_data *f)
{
  return (face_angle(angle_offset(NULL, s, NULL, f)));
}



/* returns distance between character and victim */
int distBetweenCharAndVict(struct char_data *ch, struct char_data *vict)
{
  int d = -1;
  int dr = -1;
  WithinProximity(ch, NULL, vict, NULL, NULL, -1, &d, &dr, TRUE);
  return d;
}

/* returns distance between character and location */
int distBetweenCharAndLoc(struct char_data *ch, struct location_data *loc)
{
  int d = -1;
  int dr = -1;
  WithinProximity(ch, NULL, NULL, loc, NULL, -1, &d, &dr, TRUE);
  return d;
}

/* returns distance between character and object */
int distBetweenCharAndObj(struct char_data *ch, struct obj_data *obj)
{
  int d = -1;
  int dr = -1;
  WithinProximity(ch, NULL, NULL, NULL, obj, -1, &d, &dr, TRUE);
  return d;
}
 /* returns distance between character and door */
int distBetweenCharAndDoor(struct char_data *ch, int door)
{
  int d = -1;
  int dr = -1;
  WithinProximity(ch, NULL, NULL, NULL, NULL, door, &d, &dr, TRUE);
  return d;
}

/* returns distance between location and object */
int distBetweenLocAndObj(struct location_data *l, struct obj_data *obj)
{
  int d = -1;
  int dr = -1;
  WithinProximity(NULL, l, NULL, NULL, obj, -1, &d, &dr, TRUE);
  return d;
}

/* returns distance between location and location */
int distBetweenLocAndLoc(struct location_data *s, struct location_data *f)
{
  int d = -1;
  int dr = -1;
  WithinProximity(NULL, s, NULL, f, NULL, -1, &d, &dr, TRUE);
  return d;
}

/* returns true if the character is within a certain distance from the target */
byte WithinProximity(struct char_data * ch, struct location_data
* start, struct char_data * target, struct location_data * finish, struct
    obj_data * obj, int door, int *distance, int *direction, byte change)
{
  int c_x, c_y, c_z, t_x = 0, t_y = 0, t_z = 0, i, dis_bet, dir = -1;
  int tar_in_room = -1, char_in_room = -1;
  int t_alter, c_alter;
  char buf[MAX_STRING_LENGTH];

  if (ch) {
    char_in_room = ch->in_room;
    c_x = GET_X_CORD(ch);
    c_y = GET_Y_CORD(ch);
    c_z = GET_Z_CORD(ch);
  } else {
    char_in_room = start->real_room;
    c_x = start->x_cord;
    c_y = start->y_cord;
    c_z = start->z_cord;
  }

  if (target) {
    tar_in_room = target->in_room;
    t_x = GET_X_CORD(target);
    t_y = GET_Y_CORD(target);
    t_z = GET_Z_CORD(target);
  } else if (door >= 0) {
    if (ch) {
      if (world[ch->in_room].dir_option[door]) {
	tar_in_room = ch->in_room;
	t_x = world[ch->in_room].dir_option[door]->x_cord;
	t_y = world[ch->in_room].dir_option[door]->y_cord;
	t_z = world[ch->in_room].dir_option[door]->z_cord;
      }
    } else if (start) {
      if (world[start->real_room].dir_option[door]) {
	tar_in_room = start->real_room;
	t_x = world[start->real_room].dir_option[door]->x_cord;
	t_y = world[start->real_room].dir_option[door]->y_cord;
	t_z = world[start->real_room].dir_option[door]->z_cord;
      }
    } else {
      return 0;
    }
  } else if (finish) {
    tar_in_room = finish->real_room;
#if 1
    sprintf(buf, "WithinProx():tar_in_room: %d", tar_in_room);
    log(buf);
#endif
    t_x = finish->x_cord;
    t_y = finish->y_cord;
    t_z = finish->z_cord;
  } else {
    tar_in_room = obj->in_room;
    t_x = GET_OBJ_X_CORD(obj);
    t_y = GET_OBJ_Y_CORD(obj);
    t_z = GET_OBJ_Z_CORD(obj);
  }

  if (char_in_room != tar_in_room) {
    for (i = 0; i < NUM_OF_DIRS; i++) {
      if (world[char_in_room].dir_option[i])
	if (world[char_in_room].dir_option[i]->to_room == tar_in_room)
	  break;
    }
    c_alter = (world[tar_in_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
    t_alter = (world[char_in_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
    switch (i) {
    case NORTH:
      c_x += world[tar_in_room].x_size - c_alter;
      c_y += GetYOffset(char_in_room, NORTH);
      c_z += GetZOffset(char_in_room, NORTH);
      break;
    case EAST:
      t_y += world[char_in_room].y_size - t_alter;
      c_x += GetXOffset(char_in_room, EAST);
      c_z += GetZOffset(char_in_room, EAST);
      break;
    case SOUTH:
      t_x += world[char_in_room].x_size - t_alter;
      c_y += GetYOffset(char_in_room, SOUTH);
      c_z += GetZOffset(char_in_room, SOUTH);
      break;
    case WEST:
      c_y += world[tar_in_room].y_size - c_alter;
      c_x += GetXOffset(char_in_room, WEST);
      c_z += GetZOffset(char_in_room, WEST);
      break;
    case UP:
      c_x += GetXOffset(char_in_room, UP);
      c_y += GetYOffset(char_in_room, UP);
      t_z += GET_Z_SIZE(char_in_room);
      break;
    case DOWN:
      c_x += GetXOffset(char_in_room, DOWN);
      c_y += GetYOffset(char_in_room, DOWN);
      c_z += GET_Z_SIZE(tar_in_room);
      break;
    case NORTHEAST:
      c_x += world[tar_in_room].x_size - c_alter;
      t_y += world[char_in_room].y_size - t_alter;
      c_z += GetZOffset(char_in_room, NORTHEAST);
      break;
    case NORTHWEST:
      c_x += world[tar_in_room].x_size - c_alter;
      c_y += world[tar_in_room].y_size - c_alter;
      c_z += GetZOffset(char_in_room, NORTHWEST);
      break;
    case SOUTHEAST:
      t_y += world[char_in_room].y_size - t_alter;
      t_x += world[char_in_room].x_size - t_alter;
      c_z += GetZOffset(char_in_room, SOUTHEAST);
      break;
    case SOUTHWEST:
      t_x += world[char_in_room].x_size - t_alter;
      c_y += world[tar_in_room].y_size - c_alter;
      c_z += GetZOffset(char_in_room, SOUTHWEST);
      break;
    }
  }
  if (c_x > t_x) {
    if (c_y == t_y)
      dir = NORTH;
    if (c_y < t_y)
      dir = NORTHEAST;
    if (c_y > t_y)
      dir = NORTHWEST;
  } else if (c_x == t_x) {
    if (c_y == t_y)
      dir = -1;
    if (c_y < t_y)
      dir = EAST;
    if (c_y > t_y)
      dir = WEST;
  } else if (c_x < t_x) {
    if (c_y == t_y)
      dir = SOUTH;
    if (c_y < t_y)
      dir = SOUTHEAST;
    if (c_y > t_y)
      dir = SOUTHWEST;
  }
  dis_bet = sqrt(pow((c_x - t_x), 2) + pow((c_y - t_y), 2)
		 + pow((c_z - t_z), 2));
  if ((dis_bet <= *distance) || (*distance == -1)) {
    if (change)
      *distance = dis_bet;
    if (*direction == -1) {
      *direction = dir;
      return 1;
    } else if (dir == *direction)
      return 1;
  }
  return 0;
}

byte WithinRange(struct char_data * ch, struct location_data * location, int
		 distance, int *act_dis)
{
  int c_x, c_y, c_z, t_x, t_y, t_z, i, dis_bet;
  int t_alter, c_alter;

  if (!ch || !location)
    return 0;
  c_x = GET_X_CORD(ch);
  c_y = GET_Y_CORD(ch);
  c_z = GET_Z_CORD(ch);
  t_x = location->x_cord;
  t_y = location->y_cord;
  t_z = location->z_cord;
  if (ch->in_room != location->real_room) {
    for (i = 0; i <= NUM_OF_DIRS; i++) {
      if (i == NUM_OF_DIRS)
	return 0;
      if (world[ch->in_room].dir_option[i])
	if (world[ch->in_room].dir_option[i]->to_room == location->real_room)
	  break;
    }
    c_alter = (world[location->real_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
    t_alter = (world[ch->in_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
    switch (i) {
    case NORTH:
      c_x += world[location->real_room].x_size - c_alter;
      c_y += GetYOffset(ch->in_room, NORTH);
      c_z += GetZOffset(ch->in_room, NORTH);
      break;
    case EAST:
      t_y += world[ch->in_room].y_size - t_alter;
      c_x += GetXOffset(ch->in_room, EAST);
      c_z += GetZOffset(ch->in_room, EAST);
      break;
    case SOUTH:
      t_x += world[ch->in_room].x_size - t_alter;
      c_y += GetYOffset(ch->in_room, SOUTH);
      c_z += GetZOffset(ch->in_room, SOUTH);
      break;
    case WEST:
      c_y += world[location->real_room].y_size - c_alter;
      c_x += GetXOffset(ch->in_room, WEST);
      c_z += GetZOffset(ch->in_room, WEST);
      break;
    case UP:
      c_x += GetXOffset(ch->in_room, UP);
      c_y += GetYOffset(ch->in_room, UP);
      t_z += GET_Z_SIZE(ch->in_room);
      break;
    case DOWN:
      c_x += GetXOffset(ch->in_room, DOWN);
      c_y += GetYOffset(ch->in_room, DOWN);
      c_z += GET_Z_SIZE(location->real_room);
      break;
    case NORTHEAST:
      c_x += world[location->real_room].x_size - c_alter;
      t_y += world[ch->in_room].y_size - t_alter;
      break;
    case NORTHWEST:
      c_x += world[location->real_room].x_size - c_alter;
      c_y += world[location->real_room].y_size - c_alter;
      break;
    case SOUTHEAST:
      t_y += world[ch->in_room].y_size - t_alter;
      t_x += world[ch->in_room].x_size - t_alter;
      break;
    case SOUTHWEST:
      t_x += world[ch->in_room].x_size - t_alter;
      c_y += world[location->real_room].y_size - c_alter;
      break;
    }
  }
  dis_bet = sqrt(pow((c_x - t_x), 2) + pow((c_y - t_y), 2)
		 + pow((c_z - t_z), 2));
  *act_dis = dis_bet;
  if (dis_bet <= distance)
    return 1;
  return 0;
}

void call_obj_specs(void)
{
  extern struct descriptor_data *descriptor_list;
  struct descriptor_data *d;
  extern struct index_data *obj_index;
  struct obj_data *i;
  int j;

  for (d = descriptor_list; d; d = d->next) {
    for (j = 0; j < NUM_WEARS; j++)
      if (d->character && GET_EQ(d->character, j) &&
	  GET_OBJ_SPEC(GET_EQ(d->character, j)))
	if (GET_OBJ_SPEC(GET_EQ(d->character, j)) (d->character,
			GET_EQ(d->character, j), 0, "", "", PROC_NORMAL))
	  continue;
  }
  for (d = descriptor_list; d; d = d->next) {
    if (d->character)
      for (i = d->character->carrying; i; i = i->next_content)
	if (GET_OBJ_SPEC(i) != NULL)
	  if (GET_OBJ_SPEC(i) (d->character, i, 0, "", "", PROC_NORMAL))
	    continue;
  }
}

void call_room_specs(void)
{
  extern struct descriptor_data *descriptor_list;
  struct descriptor_data *d;

  for (d = descriptor_list; d; d = d->next) {
    if (d->character && d->character->in_room && GET_ROOM_SPEC(d->character->in_room) != NULL)
      if (GET_ROOM_SPEC(d->character->in_room) (d->character, world + d->character->in_room, 0, "", "", PROC_NORMAL))
	continue;
  }
}

void vlogf(char *fmt,...)
{
  char Gbuf[2 * MAX_STRING_LENGTH];
  va_list args;
  va_start(args, fmt);
  vsprintf(Gbuf, fmt, args);
  va_end(args);

  log(Gbuf);
}

char *strip_color(const char *string)
{
  static char colorless[MAX_STRING_LENGTH];
  int x, y = 0;

  for (x = 0; x <= strlen(string); x++) {
    if (string[x] == '&') {
      if (string[x + 1] == '+') {
	colorless[y++] = string[x + 3];
	x += 3;
      } else {
	colorless[y++] = string[x + 2];
	x += 2;
      }
    } else {
      colorless[y++] = string[x];
    }
  }
  return colorless;
}

#if 1
// determine how much noise a person makes (in yards)
int char_noise_distance(struct char_data *ch)
{
  int armor_modifier;
  int i, j;

  // moving silently?  then you don't make any noise, do you?
  if (IS_AFFECTED4(ch, AFF4_SLNT_MOVE))
    return 0;

  armor_modifier = -1;		// no armor gives a bonus

  // range of 1-20 yards, based on eq.  some eq slots make more noise
  // then others.

  for (i = 0; i < NUM_WEARS; i++)
    if (GET_EQ(ch, i) && (GET_OBJ_TYPE(GET_EQ(ch, i)) == ITEM_ARMOR)) {
      // _any_ armor removes bonus
      armor_modifier = MAX(armor_modifier, 0);
      switch (i) {
      case WEAR_BODY:
	// have to compensate as there are different tables for
	// full body and just body.  blah.
	if (IS_SET(GET_OBJ_EXTRA(GET_EQ(ch, i)), ITEM_WHOLE_BODY))
	  j = 1;
	else
	  j = 0;
	if (GET_OBJ_VAL(GET_EQ(ch, i), 0) >= (j ? 4 : 5))
	  armor_modifier += 12;	// plate

	else if (GET_OBJ_VAL(GET_EQ(ch, i), 0) >= (j ? 2 : 3))
	  armor_modifier += 4;	// chain/ring

	break;
      case WEAR_LEGS:
      case WEAR_ARMS:		// limb_types[]
	// actually, this table looks perfect for my needs...

	armor_modifier += GET_OBJ_VAL(GET_EQ(ch, i), 0);

      case WEAR_FEET:
	armor_modifier += 3;	// armor on feet make noise walking

	break;
      }
      // armor_modifier should end up being between 0 and 20.
    }
  armor_modifier = BOUNDED(-1, armor_modifier, 20);

  // okay.. everyone makes a certain amount of noise.  Lets call it
  // '3', meaing 9 feet.
  armor_modifier += DIST_NOISE_NORM;

  // adjust for size?  sure..
  armor_modifier += (GET_SIZE(ch) - SIZE_MEDIUM) / 2;

  // any other adjustments?

  return BOUNDED(1, armor_modifier, DIST_NOISE_MAX);
}

int char_hearing_modifier(struct char_data *ch)
{
  int mod = 0;

  // if deaf, they can't hear ANYTHING.  However, if something has
  // enough sound that it can move 100 yards, then it would probably
  // be felt (vibration) by a deaf person.  Note that because this is
  // a vibration, other things won't modify it, so just return
  if (IS_AFFECTED4(ch, AFF4_DEAF))
    return -100;

  if (!AWAKE(ch))
    mod -= 20;			// sleeping "dulls" hearing alot

  if (GET_EQ(ch, WEAR_HEAD))
    mod -= 2;

  // this should actually be "is there fighting nearby".
  if (IS_FIGHTING(ch))
    mod -= 2;

  // add code to deal with races that have good hearing, and give
  // them plusses.

  return mod;

}
int ac_can_hear(struct char_data *sub, struct char_data *obj)
{
  int noise_prox;
  int closeness;

  if (!AWAKE(sub) || IS_AFFECTED4(sub, AFF4_DEAF))
    return 0;

  noise_prox = char_noise_distance(obj);
  if (!noise_prox)
    return 0;
  noise_prox = MAX(0, (noise_prox + char_hearing_modifier(sub)));
  closeness = distBetweenCharAndVict(sub, obj);

  // note that we should also take into account things like 'walls'
  // between sub and obj.

  if (closeness <= noise_prox)
    return 1;
  return 0;
}

/* Determine visibility of a person */

/*
 * this function is called about a billion times a minute
 * (ok, so I exaggerated a little), don't even THINK about
 * doing anything to it that would make it slower, even a
 * LITTLE bit.  JAB
 */

int ac_can_see(struct char_data *sub, struct char_data *obj, char who)
{
  /* minor detail, sleeping chars can't see squat! */
  if (!AWAKE(sub))
    return 0;

  if (GET_INVIS_LEV(obj) > GET_LEVEL(sub))
    return 0;

  if (IS_TRUSTED(sub))
    return 1;

#if 0
  /* Flyers */
  if (GET_Z_CORD(sub) != GET_Z_CORD(obj))
    return 0;
#endif


  /* can't see behind you */
  /* arghh, this thing makes for goofy looks sometimes.
     if (!who && !WithinAngle(sub, NULL, obj, NULL, 90, 90) && (sub != obj))
     return 0;
   */
  /* Object is invisible and subject does not have detect invis */
  if (IS_AFFECTED(obj, AFF_INVISIBLE) && !IS_DETECTING(sub, DETECTING_INVIS)) {
    return 0;
  }				/* end sub invis */
  /* Object is invis to animals, and subject is an animal */
  if (affected_by_spell(obj, SPELL_INVIS_ANIMALS) && (GET_INT(sub) < 6) &&
      !IS_DETECTING(sub, DETECTING_INVIS)) {
    return 0;
  }
  /* Object is invis to undead, and subject is undead */
  if (affected_by_spell(obj, SPELL_INVIS_UNDEAD) && IS_UNDEAD(sub) &&
      !IS_DETECTING(sub, DETECTING_INVIS)) {
    return 0;
  }
  /* Subject is blinded */
  if (IS_AFFECTED(sub, AFF_BLIND))
    return 0;

  if (IS_AFFECTED(obj, AFF_HIDE) && (obj != sub))
    return 0;

  // Illogical, but hey it really helps them out
  if (IS_NPC(sub))
    return 1;

  /* room is dark - do infra checks */
  if (IS_DARK(IN_ROOM(obj)) && IS_AFFECTED((sub), AFF_INFRAVISION)) {
    if (IS_UNDEAD(obj) ||
	(GET_RACE(obj) == RACE_GOLEM) ||
	(GET_RACE(obj) == RACE_SNAKE) ||
	(GET_RACE(obj) == RACE_FISH))
      return 0;
    else
      return 1;			/* will see 'red shape' */
  } else if (IS_DARK(IN_ROOM(sub)))
    return 0;


  /* room isnt dark, so from here on its a line of sight question */
#if 0
  /* too many things cant be seen it semms */
  /* standing next to is a guaranteed */
  if ((GET_X_CORD(sub) <= GET_X_CORD(obj) + 1 && GET_X_CORD(sub) >= GET_X_CORD(obj) - 1)
      && (GET_Y_CORD(sub) <= GET_Y_CORD(obj) + 1 && GET_Y_CORD(sub) >= GET_Y_CORD(obj) - 1)
      && (GET_Z_CORD(sub) <= GET_Z_CORD(obj) + 1 && GET_Z_CORD(sub) >= GET_Z_CORD(obj) - 1))
    return 1;
#endif

  /*Should return 1 here since all cases where he couldnt see person
     are checked for already. */
  return 1;
}

#if 0
/* Determine visibility of an object */
int ac_can_see_obj(struct char_data *sub, struct obj_data *obj)
{
  int rroom;

  /* sub is flying, obj isn't */
  if (OBJ_ROOM(obj) && sub->specials.z_cord != obj->z_cord)
    return 0;

  /* Immortal can see anything */
  if (IS_TRUSTED(sub))
    return 1;

  /* minor detail, sleeping chars can't see squat! */
  if (!AWAKE(sub))
    return 0;

  if (IS_NOSHOW(obj))
    return 0;
#if 0
  /* item will be shown strung to person using */
  if (GET_OBJ_TYPE(obj) == ITEM_FURNITURE && (obj)->people)
    )
	return 0;
#endif
  /* Check to see if object is invis */
/*
   if (IS_SET(obj->extra_flags, ITEM_INVISIBLE) &&
   !IS_DETECTING(sub, DETECTING_INVISIBLE))
   return 0;
 */
  /* Check if subject is blind */
  if (IS_AFFECTED(sub, AFF_BLIND))
    return 0;

  if (IS_SET((obj)->extra_flags, ITEM_SECRET))
    return 0;

  if (IS_SET((obj)->extra_flags, ITEM_BURIED))
    return 0;

  /* Room is magically dark */
  if (IS_SET(world[sub->in_room].room_flags, MAGIC_DARK))
    return 0;

  while (OBJ_INSIDE(obj))
    obj = obj->loc.inside;

  if (OBJ_ROOM(obj))
    rroom = obj->loc.room;
  else if (OBJ_WORN(obj))
    rroom = obj->loc.wearing->in_room;
  else if (OBJ_CARRIED(obj))
    rroom = obj->loc.carrying->in_room;
  else if (OBJ_NOWHERE(obj))
    rroom = sub->in_room;
  else
    return IS_TRUSTED(sub) ? 1 : 0;


  if (GET_RACE(sub) == RACE_ORC)
    return 1;

  if (GET_RACE(sub) == RACE_ILLITHID)
    return 1;

  if (IS_THRIKREEN(sub) || IS_MINOTAUR(sub))
    return 1;

  return 0;
}
#endif
int coin_type(char *s) {
  if (LOWER(*s) == 'c')
    return 0;
  else if (LOWER(*s) == 's')
    return 1;
  else if (LOWER(*s) == 'g')
    return 2;
  else if (LOWER(*s) == 'p')
    return 3;
  else
    return -1;
} char *coin_stringv(int amount) {
  int p = 0, g = 0, e = 0, s = 0;
  static char Gbuf[300];

   Gbuf[0] = '\0';

  if (!amount) {
    strcat(Gbuf, "no money!");
    return Gbuf;
  }
  p = amount / 10000;
  amount -= p * 10000;
  g = amount / 1000;
  amount -= g * 1000;
  e = amount / 100;
  amount -= e * 100;
  s = amount / 10;
  amount -= s * 10;

  if (p) {
    sprintf(Gbuf, "%d &+Wplatinum&N", p);
  }
  if (g) {
    if (p && !s && !e && !amount)
      sprintf(Gbuf + strlen(Gbuf), ", and ");
    else if (p)
      sprintf(Gbuf + strlen(Gbuf), ", ");
    sprintf(Gbuf + strlen(Gbuf), "%d &+Ygold&N", g);
  }
  if (e) {
    if ((p || g) && !s && !amount)
      sprintf(Gbuf + strlen(Gbuf), ", and ");
    else if (p || g)
      sprintf(Gbuf + strlen(Gbuf), ", ");
    sprintf(Gbuf + strlen(Gbuf), "%d &+Lelectrum&n", e);
  }
  if (s) {
    if ((p || g || e) && !amount)
      sprintf(Gbuf + strlen(Gbuf), ", and ");
    else if (p || g || e)
      sprintf(Gbuf + strlen(Gbuf), ", ");
    sprintf(Gbuf + strlen(Gbuf), "%d silver", s);
  }
  if (amount) {
    if (p || g || e || s)
      sprintf(Gbuf + strlen(Gbuf), ", and ");
    sprintf(Gbuf + strlen(Gbuf), "%d &+ycopper&N", amount);
  }
  return Gbuf;
}

/*
 * adds flat amount (in copper), to ch, using smallest number of coins
 */
int ADD_MONEY(struct char_data *ch, int amount) {
  int t = 0;
  int t2 = 0;

  if (amount < 0) {
    log("ADD_MONEY: negative amount");
    raise(SIGSEGV);
  }
  if (amount == 0)
     return 0;

  /* plat is a bit rarer, and thus is returned as change less often */
  if (amount > 9999) {
    t = amount / 10000;
    t2 = number(0, t / 2);
    create_and_give_money(ch, t2, MONEY_PLATINUM);
    amount -= t2 * 10000;
  }
  if (amount > 999) {
    t = amount / 1000;
    t2 = number(0, t);
    create_and_give_money(ch, t2, MONEY_GOLD);
    amount -= t2 * 1000;
  }
  if (amount > 99) {
    t = amount / 100;
    create_and_give_money(ch, t, MONEY_ELECTRUM);
    amount -= t * 100;
  }
  if (amount > 9) {
    t = amount / 10;
    create_and_give_money(ch, t, MONEY_SILVER);
    amount -= t * 10;
  }
  if (amount)
    create_and_give_money(ch, t, MONEY_COPPER);

  return 1;			/* prolly useless, but might care someday */
}


int SUB_MONEY(struct char_data *ch, int amount) {
  int amount2 = 0;

  if (amount <= 0)
     return 0;
  if (amount > GET_MONEY(ch))
     return 0;

   amount2 = CoinAmount(ch, MONEY_COPPER);
  if (amount > amount2) {
    amount -= CoinAmount(ch, MONEY_COPPER);
    remove_and_destroy_money(ch, amount2, MONEY_COPPER);
  } else {
    remove_and_destroy_money(ch, amount, MONEY_COPPER);
    return 0;
  }
  if (amount > 0) {
    amount2 = (CoinAmount(ch, MONEY_SILVER) * 10);
    if (amount >= amount2) {
      amount -= amount2;
    } else {
      amount2 = (int) (amount / 10) + 1;
      amount -= amount2 * 10;
    }
    remove_and_destroy_money(ch, amount2, MONEY_SILVER);
  }
  if (amount > 0) {
    amount2 = (CoinAmount(ch, MONEY_ELECTRUM) * 100);
    if (amount >= amount2) {
      amount -= amount2;
    } else {
      amount2 = (int) (amount / 100) + 1;
      amount -= amount2 * 100;
    }
    remove_and_destroy_money(ch, amount2, MONEY_ELECTRUM);
  }
  if (amount > 0) {
    amount2 = (CoinAmount(ch, MONEY_GOLD) * 1000);
    if (amount >= amount2) {
      amount -= amount2;
    } else {
      amount2 = (int) (amount / 1000) + 1;
      amount -= amount2 * 1000;
    }
    remove_and_destroy_money(ch, amount2, MONEY_GOLD);
  }
  if (amount > 0) {
    amount2 = (CoinAmount(ch, MONEY_PLATINUM) * 10000);
    if (amount >= amount2) {
      amount -= amount2;
    } else {
      amount2 = (int) (amount / 10000) + 1;
      amount -= amount2 * 10000;
    }
    remove_and_destroy_money(ch, amount2, MONEY_PLATINUM);
  }
  if (amount < 0)
    ADD_MONEY(ch, -(amount));

  return 1;			/* prolly useless, but might care someday */
}

int GET_MONEY(struct char_data *ch) {
  int total = 0;
   total += CoinAmount(ch, MONEY_COPPER);
   total += (CoinAmount(ch, MONEY_SILVER) * 10);
   total += (CoinAmount(ch, MONEY_ELECTRUM) * 100);
   total += (CoinAmount(ch, MONEY_GOLD) * 1000);
   total += (CoinAmount(ch, MONEY_PLATINUM) * 10000);
   return total;
} /* returns amount of coins carried in requested amount */ int CoinAmount(struct char_data *ch, int type) {
  struct obj_data *obj, *next_obj;
  int amount = 0;

  for (obj = ch->carrying; obj; obj = next_obj) {
    next_obj = obj->next_content;
    if (CAN_SEE_OBJ(ch, obj))
      if (GET_OBJ_TYPE(obj) == ITEM_MONEY)
	if (GET_OBJ_VAL(obj, 1) == type)
	  amount += GET_OBJ_VAL(obj, 0);
  }
  return amount;
}

#endif

/*Given a start point(player or location_data) and: either an end 
   point(ditto) or a distance and angle, returns if there is a clear path 
   between the two. Checks for objects will go here as well. If a 
   location_data structure is passed as the end, you MUST specify if you 
   want this changed. This is where the final (ie. point of blockage) 
   unobstructed point is. */

byte unobstructed_line(struct char_data * ch, struct location_data * start,
    struct char_data * target, struct location_data * stop, int distance,
		       int angle, byte change) {
  float x = 0, y = 0, z = 0;
  float xc = 0, yc = 0, zc = 0, fd;
  int alter = 100, door_dir = -1, door_dis = 1, direc = -1;
  double fangle;
  struct location_data *end;
  byte near = 0;

  CREATE(end, struct location_data, 1);

  if (ch) {
    end->real_room = ch->in_room;
    xc = (float) GET_X_CORD(ch);
    yc = (float) GET_Y_CORD(ch);
    zc = (float) GET_Z_CORD(ch);
  } else {
    end->real_room = start->real_room;
    xc = (float) start->x_cord;
    yc = (float) start->y_cord;
    zc = (float) start->z_cord;
  }

  if (distance)
    fd = (float) distance;
  else {
    near = WithinProximity(ch, start, target, stop, NULL, -1, &alter, &door_dir,
			   TRUE);
    if (near)
      fd = (float) alter;
    else {
      if (change && stop) {
	stop->real_room = end->real_room;
	stop->x_cord = (int) xc;
	stop->y_cord = (int) yc;
	stop->z_cord = (int) zc;
      }
      FREE(end);
      return 0;
    }
  }

  alter = (world[end->real_room].shape == ROOM_SHAPE_WILDERNESS) ? 0 : 1;

  if (near) {
    angle = angle_offset(ch, start, target, stop);
  }
  angle %= 360;
  fangle = (double) (angle * PI / 180.0);
  x = -sin(fangle);
  y = cos(fangle);

  while (fd > 0) {
    if (xc + x < 1) {
      if (yc + y >= 1)
	door_dir = NORTH;
      if (yc + y < 1)
	door_dir = NORTHWEST;
      if ((yc + y) >= (world[end->real_room].y_size - alter))
	door_dir = NORTHEAST;
    } else if ((xc + x) > (world[end->real_room].x_size - alter)) {
      if ((yc + y) <= (world[end->real_room].y_size - alter))
	door_dir = SOUTH;
      if (yc + y < 1)
	door_dir = SOUTHWEST;
      if ((yc + y) >= (world[end->real_room].y_size - alter))
	door_dir = SOUTHEAST;
    } else {
      if (yc + y < 1)
	door_dir = WEST;
      if ((yc + y) > (world[end->real_room].y_size - alter))
	door_dir = EAST;
    }
    /* sanity check... GLD */
    if (!world[end->real_room].dir_option[door_dir])
      door_dir = -1;

    if (door_dir != -1) {
      end->x_cord = (int) xc;
      end->y_cord = (int) yc;
      end->z_cord = (int) zc;
#ifdef DEUB_GRANOR
      sprintf(buf, "unobstr: door_dir = %d,room = %d, door_dis = %d, direc = %d",
	      door_dir, end->real_room, door_dis, direc);
      log(buf);
#endif
      if (!WithinProximity(NULL, end, NULL, NULL, NULL,
			   door_dir, &door_dis, &direc, FALSE)
	  || IS_SET(world[end->real_room].dir_option[door_dir]->exit_info,
		    EX_CLOSED)) {
	if (change && stop) {
	  stop->real_room = end->real_room;
	  stop->x_cord = end->x_cord;
	  stop->y_cord = end->y_cord;
	  stop->z_cord = end->z_cord;
	}
	FREE(end);
	return 0;
      }
    } else {
      fd -= sqrt(pow(x, 2) + pow(y, 2) + pow(z, 2));
      xc += x;
      yc += y;
    }

    if (door_dir != -1) {
      end->real_room = world[end->real_room].dir_option[door_dir]->to_room;
      alter = (world[end->real_room].shape == ROOM_SHAPE_WILDERNESS) ? 0 : 1;
      xc = (float) world[end->real_room].dir_option[rev_dir[door_dir]]->x_cord;
      yc = (float) world[end->real_room].dir_option[rev_dir[door_dir]]->y_cord;
      zc = (float) world[end->real_room].dir_option[rev_dir[door_dir]]->z_cord;
      door_dir = -1;
    }
  }

  end->x_cord = (int) xc;
  end->y_cord = (int) yc;
  end->z_cord = (int) zc;

  end->x_cord = BOUNDED(1, end->x_cord, world[end->real_room].x_size - alter);
  end->y_cord = BOUNDED(1, end->y_cord, world[end->real_room].y_size - alter);
  end->z_cord = BOUNDED(0, end->z_cord, world[end->real_room].z_size);

  if (change && stop) {
    stop->real_room = end->real_room;
    stop->x_cord = end->x_cord;
    stop->y_cord = end->y_cord;
    stop->z_cord = end->z_cord;
  }
  FREE(end);
  return 1;
}

/* Given a starting point (either a character or location_data) and an 
   end point (ditto), return the angle (in degres) between the two. There
   is a bit of math at the bottom that puts it in the right quadrent.
   If in same block, return -1;
 */
int angle_offset(struct char_data *ch, struct location_data *start, struct
		 char_data *vict, struct location_data *end) {
  int x_dif, y_dif;
  double rad_val;
  int char_in_room, tar_in_room, c_x, c_y, c_z, t_x, t_y, t_z;
  int c_alter, t_alter, i;

  if (!ch && !start)
     return 0;
  if (!vict && !end)
     return 0;

  if (ch) {
    char_in_room = ch->in_room;
    c_x = GET_X_CORD(ch);
    c_y = GET_Y_CORD(ch);
    c_z = GET_Z_CORD(ch);
  } else {
    char_in_room = start->real_room;
    c_x = start->x_cord;
    c_y = start->y_cord;
    c_z = start->z_cord;
  }

  if (vict) {
    tar_in_room = vict->in_room;
    t_x = GET_X_CORD(vict);
    t_y = GET_Y_CORD(vict);
    t_z = GET_Z_CORD(vict);
  } else {
    tar_in_room = end->real_room;
    t_x = end->x_cord;
    t_y = end->y_cord;
    t_z = end->z_cord;
  }


  if (char_in_room != tar_in_room) {
    for (i = 0; i < NUM_OF_DIRS; i++) {
      if (world[char_in_room].dir_option[i])
	if (world[char_in_room].dir_option[i]->to_room == tar_in_room)
	  break;
    }
    c_alter = (world[tar_in_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
    t_alter = (world[char_in_room].shape == ROOM_SHAPE_WILDERNESS) ? 1 : 2;
    switch (i) {
    case NORTH:
      c_x += world[tar_in_room].x_size - c_alter;
      c_y += GetYOffset(char_in_room, NORTH);
      c_z += GetZOffset(char_in_room, NORTH);
      break;
    case EAST:
      t_y += world[char_in_room].y_size - t_alter;
      c_x += GetXOffset(char_in_room, EAST);
      c_z += GetZOffset(char_in_room, EAST);
      break;
    case SOUTH:
      t_x += world[char_in_room].x_size - t_alter;
      c_y += GetYOffset(char_in_room, SOUTH);
      c_z += GetZOffset(char_in_room, SOUTH);
      break;
    case WEST:
      c_y += world[tar_in_room].y_size - c_alter;
      c_x += GetXOffset(char_in_room, WEST);
      c_z += GetZOffset(char_in_room, WEST);
      break;
    case UP:
      c_x += GetXOffset(char_in_room, UP);
      c_y += GetYOffset(char_in_room, UP);
      t_z += GET_Z_SIZE(char_in_room);
      break;
    case DOWN:
      c_x += GetXOffset(char_in_room, DOWN);
      c_y += GetYOffset(char_in_room, DOWN);
      c_z += GET_Z_SIZE(tar_in_room);
      break;
    case NORTHEAST:
      c_x += world[tar_in_room].x_size - c_alter;
      t_y += world[char_in_room].y_size - t_alter;
      c_z += GetZOffset(char_in_room, NORTHEAST);
      break;
    case NORTHWEST:
      c_x += world[tar_in_room].x_size - c_alter;
      c_y += world[tar_in_room].y_size - c_alter;
      c_z += GetZOffset(char_in_room, NORTHWEST);
      break;
    case SOUTHEAST:
      t_y += world[char_in_room].y_size - t_alter;
      t_x += world[char_in_room].x_size - t_alter;
      c_z += GetZOffset(char_in_room, SOUTHEAST);
      break;
    case SOUTHWEST:
      t_x += world[char_in_room].x_size - t_alter;
      c_y += world[tar_in_room].y_size - c_alter;
      c_z += GetZOffset(char_in_room, SOUTHWEST);
      break;
    }
  }
  x_dif = -(t_x - c_x);
  y_dif = t_y - c_y;

  if (!x_dif && !y_dif)
    return -1;

  if (y_dif)
    rad_val = atan((double) x_dif / (double) y_dif);
  else
    rad_val = (PI / 2.0) * ((x_dif >= 0) ? 1 : -1);

  if (rad_val > 0)
    if (x_dif < 0)
      rad_val += PI;
  if (rad_val < 0)
    if (x_dif > 0)
      rad_val += PI;
  if (rad_val == 0)
    if (y_dif < 0)
      rad_val += PI;

  return (MOD((int) ((rad_val * 180.0) / PI), 360));
}

/*returns the proper facing angle, given an angle (in degrees) */
int face_angle(int angle) {
  int i = -1;

   angle = MOD(angle, 360);

  if (angle < 23 || angle > 337)
     i = EAST;
  else if (angle >= 23 && angle < 68)
     i = NORTHEAST;
  else if (angle >= 68 && angle < 113)
     i = NORTH;
  else if (angle >= 113 && angle < 158)
     i = NORTHWEST;
  else if (angle >= 158 && angle < 213)
     i = WEST;
  else if (angle >= 213 && angle < 248)
     i = SOUTHWEST;
  else if (angle >= 248 && angle < 303)
     i = SOUTH;
  else
     i = SOUTHEAST;

   return i;
} int angle_facing(int face_dir) {
  int j = 0;
  switch (face_dir) {
  case EAST:
    j = 0;
    break;
    case NORTHEAST:
    j = 45;
    break;
    case NORTH:
    j = 90;
    break;
    case NORTHWEST:
    j = 135;
    break;
    case WEST:
    j = 180;
    break;
    case SOUTHWEST:
    j = 225;
    break;
    case SOUTH:
    j = 270;
    break;
    case SOUTHEAST:
    j = 315;
    break;
    default:
  }
  return j;
}

int positive_angle(int orig_angle) {
  return MOD(orig_angle, 360);
} byte WithinAngle(struct char_data * ch, struct location_data * start,
   struct char_data * vict, struct location_data * end, int ang_to_right,
		   int ang_to_left) {
  int angle_between, start_angle, beg_angle = 100, end_angle = -1;

  angle_between = angle_offset(ch, start, vict, end);

  if (angle_between == -1)
    return 1;

  if (ch)
    start_angle = angle_facing(GET_FACING(ch));
  else {
    start_angle = start->angle;
  }

  beg_angle = MOD((start_angle - ang_to_right), 360);
  end_angle = MOD((start_angle + ang_to_left), 360);

  if (start_angle < beg_angle) {
    start_angle += 360;
    end_angle += 360;
    angle_between += 360;
  } else if (start_angle > end_angle) {
    start_angle -= 360;
    beg_angle -= 360;
    angle_between -= 360;
  }
  return ((angle_between >= beg_angle) && (angle_between <= end_angle))
      ? 1 : 0;
}

char *FirstWord(char *namelist) {
  static char holder[30];
  register char *point;

  if (!namelist)
     return (NULL);

  while (*namelist == ' ')
     namelist++;
  for (point = holder; isalpha(*namelist); namelist++, point++)
    *point = *namelist;

  *point = '\0';

   return (holder);
} int MOD(int value, int mod) {
  int quotient = 0, remainder = 0;

   quotient = value / mod;
  if (value < 0)
     quotient--;
   remainder = value - (quotient * mod);

   return remainder;
}
byte makes_save(struct char_data * ch, int savetype, int modifier) {
  byte makes = 0;

  if (((GET_RACE(ch) == RACE_HALFLING) || (GET_RACE(ch) == RACE_DWARF)
       || (GET_RACE(ch) == RACE_GNOME)) &&
      (savetype == SAVING_SPELL))
    modifier += (int) (GET_CON(ch) / 3.5);
  makes = (number(1, 20) > (GET_SAVE(ch, savetype) - modifier));
  return makes;
}


int distance_from_shore(int room, int newx, int newy) {
  int distance, dir;

  if (world[room].map == NULL)
     return 0;
  if (newy > GET_Y_SIZE(room) || newx > GET_X_SIZE(room) ||
      newy < 0 || newx < 0)
     return 0;

  for (distance = 1; distance < 26; distance++)
    for (dir = 0; dir < NUM_OF_DIRS; dir++)
      switch (dir) {
      case NORTH:
	if (IS_WATER_CORD(room, newx - distance, newy))
	  return distance;
	break;
	case NORTHEAST:
	if (IS_WATER_CORD(room, newx - distance, newy + distance))
	  return distance;
	break;
	case EAST:
	if (IS_WATER_CORD(room, newx, newy + distance))
	  return distance;
	break;
	case SOUTHEAST:
	if (IS_WATER_CORD(room, newx + distance, newy + distance))
	  return distance;
	break;
	case SOUTH:
	if (IS_WATER_CORD(room, newx + distance, newy))
	  return distance;
	break;
	case SOUTHWEST:
	if (IS_WATER_CORD(room, newx + distance, newy - distance))
	  return distance;
	break;
	case WEST:
	if (IS_WATER_CORD(room, newx, newy - distance))
	  return distance;
	break;
	case NORTHWEST:
	if (IS_WATER_CORD(room, newx - distance, newy - distance))
	  return distance;
	break;
      }
  return distance;
}


/*changes aiming location to wear location */
int parse_aiming(int location) {
  int wear = WEAR_BODY;

  switch (location) {
  case LOCATION_VITALS:
  case LOCATION_TORSO:
    wear = WEAR_BODY;
    break;
    case LOCATION_BRAIN:
    wear = WEAR_HEAD;
    break;
    case LOCATION_FACE:
    wear = WEAR_FACE;
    break;
    case LOCATION_WEAPON_ARM:
    case LOCATION_SHIELD_ARM:
    wear = WEAR_ARMS;
    break;
    case LOCATION_WEAPON_HAND:
    case LOCATION_SHIELD_HAND:
    wear = WEAR_HANDS;
    break;
    case LOCATION_RIGHT_LEG:
    case LOCATION_LEFT_LEG:
    wear = WEAR_LEGS;
    break;
    case LOCATION_LEFT_FOOT:
    case LOCATION_RIGHT_FOOT:
    wear = WEAR_FEET;
    break;
    default:
    wear = WEAR_BODY;
  }
  return wear;
}

int ac_to_pd(int ac) {
  int pd = 0;

   pd = (10 - ac) / 3;

   return pd;
} int ac_to_dr(int ac) {
  int dr = 0;

   dr = (10 - ac) / 2;

  if (ac < 0)
     dr -= ac;

   return dr;
} /* this one returns 0% for no load, 100% for max load */ int carried_load_percent(struct char_data *ch) {
  int x, i;
   x = IS_CARRYING_W(ch) ? IS_CARRYING_W(ch) : 1;
   i = CAN_CARRY_W(ch) ? CAN_CARRY_W(ch) : 1;

   return (int) ((100 * x) / i);
} /*checks for a successful magic resistance roll */ byte resists_spell(struct char_data *ch) {
  int val = number(1, 100);

   return ((val < ch->char_specials.magic_resistance) ? 1 : 0);
} /*check for person within range of minor globe */ byte IS_MINOR_GLOBED(struct char_data * ch) {
  int room = ch->in_room;
  struct room_affected_type *hjp = NULL, *next = NULL;
  char buf[150];


  for (hjp = world[room].spell_affects; hjp; hjp = next) {
    next = hjp->next;
    if (hjp->type == SPELL_MINOR_GLOBE_OF_INVULNERABILITY) {
      sprintf(buf, "minor globe: %d %d %d %d", hjp->loc->real_room, hjp->loc->x_cord, hjp->loc->y_cord, hjp->loc->z_cord);
      log(buf);
      if (victWithinRangeofLoc(hjp->loc, ch, 2))
	return 1;
    }
  }
  return 0;
}

void search_replace(char *string, const char *find, const char *replace) {
  char final[MAX_INPUT_LENGTH], temp[2];
  size_t start, end, i;

  while (strstr(string, find) != NULL) {

    final[0] = '\0';
    start = strstr(string, find) - string;
    end = start + strlen(find);

    temp[1] = '\0';

    strncat(final, string, start);

    strcat(final, replace);

    for (i = end; string[i] != '\0'; i++) {
      temp[0] = string[i];
      strcat(final, temp);
    }
    sprintf(string, final);

  }
  return;
}

int view_distance(struct char_data *i) {
  int distance = 7, zon;
  struct obj_data *obj;

  if (GET_LEVEL(i) < 31 || IS_TRUSTED(i))
     return 12;

  if (IS_SUNLIT(i->in_room))
     return 17;

  if ((IS_DARK(i->in_room) && !CAN_SEE_IN_DARK(i)) || IS_AFFECTED(i, AFF_BLIND))
     return 4;			/* size of border only */

  /* one cord per 5 ambient    */
   zon = world[i->in_room].zone;
  if (!ROOM_FLAGGED(i->in_room, ROOM_INDOORS))
     distance += (zone_table[zon].conditions.ambient_light / 5);

  /* rm->light */

  /* personal torches */
  if (i->equipment[WEAR_HOLD]) {
    obj = i->equipment[WEAR_HOLD];
    if (GET_OBJ_TYPE(obj) == ITEM_LIGHT && obj->obj_flags.value[3]) {
      distance = (GET_OBJ_VAL(obj, 1) / 5) + .5;
    }
  }
  if (i->equipment[WEAR_HOLD2]) {
    obj = i->equipment[WEAR_HOLD2];
    if (GET_OBJ_TYPE(obj) == ITEM_LIGHT && obj->obj_flags.value[3]) {
      distance = (GET_OBJ_VAL(obj, 1) / 5) + .5;
    }
  }
  /* GET_OBJ_VAL(obj, 1) = 1-100 level */
  /* GET_OBJ_VAL(obj, 1) = 1 sphere 2 cone 3 line */

  return distance;
}

char IS_CRIPPLED(struct char_data *ch, int location) {
  char is = 0;

  switch (location) {
  case LOCATION_WEAPON_ARM:
    if (GET_HIT_WA(ch) <= HIT_INCAP)
      is = 1;
    break;
    case LOCATION_WEAPON_HAND:
    if (GET_HIT_WH(ch) <= HIT_INCAP)
      is = 1;
    break;
    case LOCATION_SHIELD_ARM:
    if (GET_HIT_SA(ch) <= HIT_INCAP)
      is = 1;
    break;
    case LOCATION_SHIELD_HAND:
    if (GET_HIT_SH(ch) <= HIT_INCAP)
      is = 1;
    break;
    case LOCATION_LEFT_LEG:
    if (GET_HIT_LL(ch) <= HIT_INCAP)
      is = 1;
    break;
    case LOCATION_LEFT_FOOT:
    if (GET_HIT_LF(ch) <= HIT_INCAP)
      is = 1;
    break;
    case LOCATION_RIGHT_LEG:
    if (GET_HIT_RL(ch) <= HIT_INCAP)
      is = 1;
    break;
    case LOCATION_RIGHT_FOOT:
    if (GET_HIT_RF(ch) <= HIT_INCAP)
      is = 1;
    break;
    default:
  }
  return is;
}

char *FathomMobIntelligence(struct char_data *mob, int num) {
  static char buf[MAX_STRING_LENGTH];

  if (mob)
    switch (GET_INT(mob)) {
    case 0:
      return int_ratings[0];
      case 1:
      return int_ratings[1];
      case 2:
      case 3:
      case 4:
      return int_ratings[2];
      case 5:
      case 6:
      case 7:
      return int_ratings[3];
      case 8:
      case 9:
      case 10:
      return int_ratings[4];
      case 11:
      case 12:
      return int_ratings[5];
      case 13:
      case 14:
      return int_ratings[6];
      case 15:
      case 16:
      return int_ratings[7];
      case 17:
      case 18:
      return int_ratings[8];
      case 19:
      case 20:
      return int_ratings[9];
      default:
      return int_ratings[10];
  } else {
    if (num == 0)
      strcat(buf, "0");
    else if (num == 1)
      strcat(buf, "1");
    else if (num >= 2 || num <= 4)
      sprintf(buf, "%d", number(2, 4));
    else if (num >= 5 || num <= 7)
      sprintf(buf, "%d", number(5, 7));
    else if (num >= 8 || num <= 10)
      sprintf(buf, "%d", number(8, 10));
    else if (num >= 11 || num <= 12)
      sprintf(buf, "%d", number(11, 12));
    else if (num >= 13 || num <= 14)
      sprintf(buf, "%d", number(13, 14));
    else if (num >= 15 || num <= 16)
      sprintf(buf, "%d", number(14, 16));
    else if (num >= 17 || num <= 18)
      sprintf(buf, "%d", number(15, 18));
    else if (num >= 19 || num <= 20)
      sprintf(buf, "%d", number(17, 20));
    else
      sprintf(buf, "%d", number(21, 25));
    return buf;
  }
}

void change_facing(struct char_data *ch, int new_dir) {

  if (new_dir == UP || new_dir == DOWN)
    return;

  GET_FACING(ch) = new_dir;
  if (ch->watching)
    if (!CAN_SEE(ch, ch->watching->vict))
      stop_watcher(ch);
  if (ch->watchers) {
    struct follow_type *k, *j;
    char buf[256];
     sprintf(buf, "You notice $N turn to the %s", dirs[new_dir]);
    for (k = ch->watchers; k; k = j) {
      j = k->next;
      // make sure that k can still see ch
      if (CAN_SEE(k->follower, ch) &&
	  victWithinRangeofChar(k->follower, ch, 9) &&
	  WithinAngle(k->follower, NULL, ch, NULL, 95, 95))
	act(buf, FALSE, k->follower, 0, ch, TO_CHAR);
    }				// let watchers know..
  } else {
    SET_FACING(ch, new_dir);
  }
}

void stop_watcher(struct char_data *ch) {
  struct follow_type *j, *k;

   assert(ch->watching);

  // if ch->watching->ch is null, we're being called from the event
  // handler, so things are kinda different...

  if (ch->watching->ch)
     act("You stop watching $N.", FALSE, ch, 0, ch->watching->vict,
	 TO_CHAR);
  else
     act("You lost track of $N.", FALSE, ch, 0, ch->watching->vict,
	 TO_CHAR);

   k = ch->watching->vict->watchers;
  if (k->follower == ch) {
    ch->watching->vict->watchers = k->next;
    FREE(k);
  } else {
    for (; k && (k->next->follower != ch); k = k->next);
    if (k) {
      j = k->next;
      k->next = j->next;
      FREE(j);
    }
  }
  // don't cancel if we're called from the handler.  the handler must
  // take care of it.
  if (ch->watching->ch) {
    event_cancel(ch->watching->event);
    ch->watching = NULL;
  }
}

void die_watcher(struct char_data *ch) {
  struct follow_type *j, *k;

  if (ch->watching)
     stop_watcher(ch);

  for (k = ch->watchers; k; k = j) {
    j = k->next;
    stop_watcher(k->follower);
  }
} EVENTFUNC(check_watcher) {
  struct watch_data *event = (struct watch_data *) (event_obj);
  struct char_data *ch;
  char buf[256];

  if (!event || !event->ch) {
    mudlog("watch event with null data or no ch!", NRM, LVL_GOD, TRUE);
    FREE(event);
    return 0;
  }
  ch = event->ch;
  if (event != ch->watching) {
    sprintf(buf, "watch event on %s doesn't match watch_data!",
	    GET_NAME(ch));
    mudlog(buf, NRM, LVL_GOD, TRUE);
    FREE(event);
    return 0;
  }
  if (!event->vict) {
    sprintf(buf, "watch event on %s with no vict!", GET_NAME(ch));
    mudlog(buf, NRM, LVL_GOD, TRUE);
    FREE(event);
    ch->watching = NULL;
    return 0;
  }
  if (!CAN_SEE(ch, event->vict) ||
      !victWithinRangeofChar(ch, event->vict, 9) ||
      !WithinAngle(ch, NULL, event->vict, NULL, 95, 95)) {
    // set event->ch to null so that stop_watcher doesn't cancel the
    // event. We have to cancel it here, in the handler.
    event->ch = NULL;
    stop_watcher(ch);
    ch->watching = NULL;
    return 0;
  }
  return (3 RL_SEC);
}

void watch_movement(struct char_data *ch) {
  struct char_data *vict;
  // ch is the person doing the watching.
  if (!ch->watching)
     return;
   vict = ch->watching->vict;
  if (CAN_SEE(ch, vict) &&
      victWithinRangeofChar(ch, vict, 9) &&
      WithinAngle(ch, NULL, vict, NULL, 95, 95)) {
    int i;
     i = face_angle(angle_offset(ch, NULL, vict, NULL));
    if (i != GET_FACING(ch)) {
      char buf[256];
       sprintf(buf, "You turn to the %s to keep watching $N.", dirs[i]);
       act(buf, FALSE, ch, 0, vict, TO_CHAR);
       SET_FACING(ch, i);
    }
  } else {
    // heh.. they lost 'em
    ch->watching->ch = NULL;
    stop_watcher(ch);
    event_cancel(ch->watching->event);
    ch->watching = NULL;
  }
}

void add_watcher(struct char_data *ch, struct char_data *leader) {
  struct follow_type *k;
  struct watch_data *event;
   assert(!ch->watching);

//    if (GET_ACTION(ch)) {
  //       // heh, they were doing something else.  searching, or memming or
  //       // something.  can't watch while do something else.  sorry.
  //       send_to_char("You're too busy doing something else.\r\n", ch);
  //       return;
  //    }

  // need to create a new event...
   CREATE(event, struct watch_data, 1);
   CREATE(k, struct follow_type, 1);

   ch->watching = event;
   event->ch = ch;
   event->vict = leader;

   k->follower = ch;
   k->next = leader->watchers;
   leader->watchers = k;

   act("You're now watching $N.", FALSE, ch, 0, leader, TO_CHAR);
   event->event = event_create(check_watcher, event, 1 RL_SEC);
} char GET_WIELDED_STYLE(struct char_data *ch) {
  int skill_style = SKILL_STYLE_ONE_HANDED_WEAPON, which = WEAR_WIELD;
  struct obj_data *wielded = NULL;

   wielded = ((ch->char_specials.attacks_left == 1) && GET_EQ(ch, WEAR_WIELD2)) ?
   GET_EQ(ch, WEAR_WIELD2) : GET_EQ(ch, WEAR_WIELD);
  if (wielded == GET_EQ(ch, WEAR_WIELD2))
     which = WEAR_WIELD2;
  if (wielded == GET_EQ(ch, WEAR_WIELD))
     which = WEAR_WIELD;

  if (wielded && (which == WEAR_WIELD)) {
    if (GET_EQ(ch, WEAR_SHIELD))	//weapon and shield

      skill_style = SKILL_STYLE_WEAPON_SHIELD;

    else if (GET_EQ(ch, WEAR_WIELD2))	// two weapons

      skill_style = SKILL_STYLE_TWO_WEAPON;

    else if (IS_OBJ_STAT(wielded, ITEM_TWOHANDS))
      skill_style = SKILL_STYLE_TWO_HANDED_WEAPON;

    else
      skill_style = SKILL_STYLE_ONE_HANDED_WEAPON;
  } else if (which == WEAR_WIELD2) {
    skill_style = SKILL_STYLE_TWO_WEAPON;
  }
  return skill_style;
}

byte IS_NEARBY(struct char_data * ch, struct char_data * ch1) {
  /* this is only used for visuals, so we're not going to check z cord */
  if ((GET_X_CORD(ch) <= GET_X_CORD(ch1) + 1) && (GET_X_CORD(ch) >= GET_X_CORD(ch1) - 1))
    if ((GET_Y_CORD(ch) <= GET_Y_CORD(ch1) + 1) && (GET_Y_CORD(ch) >= GET_Y_CORD(ch1) - 1))
      return YES;

  return NO;
}

/* dont use this, use GET_ALIGNMENT(ch) */
sbyte
ac_get_alignment(struct char_data * ch) {
  if (ch->points.align_ge > 350) {
    if (ch->points.align_lc > 350)
      return ALIGN_LAWFUL_GOOD;
    else if (ch->points.align_lc < -350)
      return ALIGN_CHAOTIC_GOOD;
    else
      return ALIGN_NEUTRAL_GOOD;
  } else if (ch->points.align_ge < -350) {
    if (ch->points.align_lc > 350)
      return ALIGN_LAWFUL_EVIL;
    else if (ch->points.align_lc < -350)
      return ALIGN_CHAOTIC_EVIL;
    else
      return ALIGN_NEUTRAL_EVIL;
  } else {
    if (ch->points.align_lc > 350)
      return ALIGN_LAWFUL_NEUTRAL;
    else if (ch->points.align_lc < -350)
      return ALIGN_CHAOTIC_NEUTRAL;
    else
      return ALIGN_NEUTRAL_NEUTRAL;
  }
/* error! */
  return ALIGN_NONE;
}

void
 ac_set_alignment(struct char_data *ch, int i) {
  switch (i) {
  case ALIGN_NONE:
    ch->points.align_lc = 0;
    ch->points.align_ge = 0;
    break;
    case ALIGN_LAWFUL_GOOD:
    ch->points.align_lc = number(400, 1000);
    ch->points.align_ge = number(400, 1000);
    break;
    case ALIGN_NEUTRAL_GOOD:
    ch->points.align_lc = number(-300, 300);
    ch->points.align_ge = number(400, 1000);
    break;
    case ALIGN_CHAOTIC_GOOD:
    ch->points.align_lc = number(-1000, -400);
    ch->points.align_ge = number(400, 1000);
    break;
    case ALIGN_LAWFUL_NEUTRAL:
    ch->points.align_lc = number(400, 1000);
    ch->points.align_ge = number(-300, 300);
    break;
    case ALIGN_NEUTRAL_NEUTRAL:
    ch->points.align_lc = number(-300, 300);
    ch->points.align_ge = number(-300, 300);
    break;
    case ALIGN_CHAOTIC_NEUTRAL:
    ch->points.align_lc = number(-1000, -400);
    ch->points.align_ge = number(-300, 300);
    break;
    case ALIGN_LAWFUL_EVIL:
    ch->points.align_lc = number(400, 1000);
    ch->points.align_ge = number(-1000, -400);
    break;
    case ALIGN_NEUTRAL_EVIL:
    ch->points.align_lc = number(-300, 300);
    ch->points.align_ge = number(-1000, -400);
    break;
    case ALIGN_CHAOTIC_EVIL:
    ch->points.align_lc = number(-1000, -400);
    ch->points.align_ge = number(-1000, -400);
    break;
  }
} void
 teleport_to(struct char_data *ch, int to_room) {
  act("$n slowly fades out of existence.", FALSE, ch, 0, 0, TO_ROOM);
  char_from_room(ch);
  char_to_room(ch, to_room, 0);
  act("$n slowly fades into existence.", FALSE, ch, 0, 0, TO_ROOM);

  if (IS_SET(world[to_room].room_flags, ROOM_DEATH) && !IS_TRUSTED(ch)) {
    extract_char(ch);
  }
}
